#include "BasicCS.h"
#include "BasicTypes.h"
#include "PtgOutput.h"
#include "ptg_gen.h"
#include "err.h"

#include <stdlib.h>

void Constraint_VLInt_Value(PTG_OUTPUT_FILE f, void *ptr)
{
        char buffer[80];

        sprintf(buffer, "%d", ((VLInt *)ptr)->getInt());
        PTG_OUTPUT_STRING(f, buffer);
}

void Constraint_VLInt_Name(PTG_OUTPUT_FILE f, void *ptr)
{
        char buffer[80];

        sprintf(buffer, "ptr%p.i", ptr);
        PTG_OUTPUT_STRING(f, buffer);
}

void Constraint_VLPoint_ValueX(PTG_OUTPUT_FILE f, void *ptr)
{
        char buffer[80];

        sprintf(buffer, "%d", ((VLPointArrayPtr)ptr)->getX());
        PTG_OUTPUT_STRING(f, buffer);
}

void Constraint_VLPoint_ValueY(PTG_OUTPUT_FILE f, void *ptr)
{
        char buffer[80];

        sprintf(buffer, "%d", ((VLPointArrayPtr)ptr)->getY());
        PTG_OUTPUT_STRING(f, buffer);
}

void Constraint_VLPoint_NameX(PTG_OUTPUT_FILE f, void *ptr)
{
        char buffer[80];

        sprintf(buffer, "ptr%p.x", ptr);
        PTG_OUTPUT_STRING(f, buffer);
}

void Constraint_VLPoint_NameY(PTG_OUTPUT_FILE f, void *ptr)
{
        char buffer[80];

        sprintf(buffer, "ptr%p.y", ptr);
        PTG_OUTPUT_STRING(f, buffer);
}

void Constraint_VLPoint_Name(PTG_OUTPUT_FILE f, void *ptr)
{
        char buffer[80];

        sprintf(buffer, "%p", ptr);
        PTG_OUTPUT_STRING(f, buffer);
}

void PtgCSPrintString(PtgFilePtr file, char *string)
{
        do {

                char *nl = strchr(string, '\n');
                char *next;
                int len;        
                if (nl) 
                {       // There is a newline. Send the sequence before to the solver.
                        len = (int)(nl - string);   // Length of String without \n
                        next = nl + 1;
                }
                else 
                {       // There is no newline. Accumulate until there is.
                        len = strlen(string);
                        next = (char *)0;
                }

                if (len > 0) 
                {
                        if (file->destinfo)          
                                Tcl_AppendToObj((Tcl_Obj *)file->destinfo, string, len);
                        else
                                file->destinfo = Tcl_NewStringObj (string, len);
                }

                if (next && file->destinfo) 
                {
                        Solver_Interact ((Tcl_Obj *)file->destinfo);
                        file->destinfo = (void *)0;
                }

                string = next;
        } while (string);
}

void PtgCSPrintChar(PtgFilePtr file, char c)
{
        if (c == '\n')
        {
                if (file->destinfo) {
                        Solver_Interact ((Tcl_Obj *)file->destinfo);
                        file->destinfo = (void *)0;
                }
        } 
        else 
        {
                if (file->destinfo)          
                        Tcl_AppendToObj((Tcl_Obj *)file->destinfo, &c, 1);
                else
                        file->destinfo = Tcl_NewStringObj (&c, 1);
        }
}       

int Solver_Interact(char *string) 
{
        return Solver_Interact (Tcl_NewStringObj(string, -1));
}

int Solver_Interact(Tcl_Obj *obj)
{
        Tcl_Obj *cmd[2], *cmdlst, *result;
        int ret;

        Tcl_IncrRefCount(obj);
        cmd[0] = Tcl_NewStringObj("::visual::solver_interact", -1);
        cmd[1] = obj;

        cmdlst = Tcl_NewListObj(2, cmd);
        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);
        Tcl_DecrRefCount(obj);

        if (ret != TCL_OK)
        {
                ReportTclError();
                return -1;
        }

        result = Tcl_GetObjResult(TclInterp);
        Tcl_GetIntFromObj((Tcl_Interp *)0, result, &ret);
        return ret;
}       

PTGNode PTGSolverOut(PTGNode root)
{
/*        printf("Entering PTGSolverOut\n"); */
        PtgFilePtr ptgfile = NewPtgFile((Tcl_Obj *)0, PtgCSPrintString);
        ptgfile->PrintChar = PtgCSPrintChar;
        
        PTGProcess(ptgfile, root);
        FreePtgFile(ptgfile);

/*        printf("Leaving PTGSolverOut\n"); */
        return root;
}

static Tcl_Obj *new_args = (Tcl_Obj *)0;
static Tcl_Obj *val_args = (Tcl_Obj *)0;
int solver_new_values = 0;

void Solver_Value_Changed(VLSolverValue *ptr)
{
    // Signal vom User-Interface, dass der angegebene Wert sich geaendert hat.
    // Diese Aenderung ist vom User-Interface angestossen.

   if (solver_new_values) {
       // Ist inzwischen der Solver gelaufen? Dann setze die Werte zurueck.
 
       if (new_args) {
           Tcl_DecrRefCount(new_args);
           new_args = (Tcl_Obj *)0;
       }
       if (val_args) {
           Tcl_DecrRefCount(val_args);
           val_args = (Tcl_Obj *)0;
       }
       solver_new_values = 0;
   }
   
   // Haenge den neuen Wert an die alten.
   char buffer[256];
   if (new_args) {
       Tcl_AppendToObj(new_args, " ", 1);
       Tcl_AppendToObj(new_args, ptr->getNewArgs(buffer), -1);
   }
   else
   {
       new_args = Tcl_NewStringObj(ptr->getNewArgs(buffer), -1);
       Tcl_IncrRefCount(new_args);
   }
   
   if (val_args) 
   {
       Tcl_AppendToObj (val_args, " ", 1);
       Tcl_AppendToObj (val_args, ptr->getValArgs(buffer), -1);
   }
   else 
   {
       val_args = Tcl_NewStringObj(ptr->getValArgs(buffer), -1);
       Tcl_IncrRefCount(val_args);
   }
}

void Solver_New()
{
    int status = 0;
    if (!new_args || !val_args || solver_new_values) {
        status = Solver_Interact("new");
        Solver_Interact("VL clear-variables");  // Magic command. Prevents deleted
                                                // attributes from being assigned.
        solver_new_values = 1;
        return;
    }
    
    Tcl_Obj *new_command, *val_command;
    
    new_command = Tcl_NewStringObj("new ", -1);
    Tcl_AppendToObj (new_command, Tcl_GetStringFromObj(new_args, (int *)0), -1);
    Tcl_IncrRefCount(new_command);
    status = Solver_Interact(new_command);
    Tcl_DecrRefCount(new_command);
    
    if (status != -42) { // timeout 
        val_command = Tcl_NewStringObj("val ", -1);
        Tcl_AppendToObj (val_command, Tcl_GetStringFromObj(val_args, (int *)0), -1);
        Tcl_IncrRefCount(val_command);
        Solver_Interact(val_command);
        Tcl_DecrRefCount(val_command);
    }
    
    solver_new_values = 1;
    
    Solver_Interact("VL clear-variables");      // Magic command. Prevents deleted
                                                // attributes from being assigned.
}

