#include "BasicCheck.h"
#include "tcl.h"
#include "tcl_vpt.h"
#include "ptg_gen.h"
#include "PtgOutput.h"
#include "err.h"

void BasicCheckPrintNodeptr(PTG_OUTPUT_FILE f, void *p)
{
        Tcl_Obj *ptrobj = TclVP_NewVoidPointerObj(p);
        Tcl_IncrRefCount(ptrobj);
        PTG_OUTPUT_STRING(f, Tcl_GetStringFromObj(ptrobj, (int *)0));
        Tcl_DecrRefCount(ptrobj);
}

void PtgBasicCheckPrintString(PtgFilePtr file, char *string)
{
        if (file->destinfo)
                Tcl_AppendToObj((Tcl_Obj *)file->destinfo, string, -1);
        else
                file->destinfo = Tcl_NewStringObj (string, -1);
}

void PtgBasicCheckPrintChar(PtgFilePtr file, char c)
{
        if (file->destinfo)
                Tcl_AppendToObj((Tcl_Obj *)file->destinfo, &c, 1);
        else
                file->destinfo = Tcl_NewStringObj (&c, 1);
}

void PTGBasicCheckExecute(int round, PTGNode root)
{
        Tcl_Obj *ob = Tcl_NewStringObj("", 0);
        Tcl_IncrRefCount(ob);

        PtgFilePtr ptgfile = NewPtgFile(ob, PtgBasicCheckPrintString);
        ptgfile->PrintChar = PtgBasicCheckPrintChar;
        PTGProcess(ptgfile, root);
        
        Tcl_Obj *cmd[3], *cmdlst;
        int ret;

        cmd[0] = Tcl_NewStringObj("::treecheck::changetree", -1);
        cmd[1] = Tcl_NewIntObj(round);
        cmd[2] = (Tcl_Obj *)ptgfile->destinfo;

        cmdlst = Tcl_NewListObj(3, cmd);
        Tcl_IncrRefCount(cmdlst);

        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
                
        if (ret != TCL_OK)
                ReportTclError();
        
        Tcl_DecrRefCount(cmdlst);
        Tcl_DecrRefCount((Tcl_Obj *)ptgfile->destinfo);
        FreePtgFile(ptgfile);
}       
