#include "BasicObjects.h"
#include "err.h"
#include "tcl.h"
#include "tcl_vpt.h"
#include <assert.h>

char VLStringBuffer[2000];


VLString VLSymName(NODEPTR p)
{
  VLList l;
  l.append("::vl::getCode");
  l.append(VLNodePtr(p));
  VLList l2;
  l2.append("::vl::lhs");
  l2.append(l.eval());
  VLList l3;
  l3.append("::vl::symbname");
  l3.append(l2.eval());
  return l3.eval();
}

void VLSetWindowTitle(VLCharPtr title)
{
  VLList l;
  l.append("::visual::setWindowTitle");
  l.append(title);
  l.eval();
}

int VLObjectCreate(NODEPTR context, VLCharPtr partname, VLCharPtr type, int depth, 
        VLPointArray *coords, VLListPtr attributes)
{
        if (attributes) {
                Tcl_Obj *attrs = attributes->getObj();
                return VLObjectCreate(context, partname, type, depth, coords, attrs);
        } else
                return VLObjectCreate(context, partname, type, depth, coords, (Tcl_Obj *)0);
}

int VLObjectCreate(NODEPTR context, VLCharPtr partname, VLCharPtr type, int depth, 
        VLPointArray *coords, VLCharPtr attributes)
{
        Tcl_Obj *attrs = (Tcl_Obj*)0;
        if (attributes)
        {
                attrs = Tcl_NewStringObj(attributes, -1);
                Tcl_IncrRefCount(attrs);
        }

        int ret = VLObjectCreate(context, partname, type, depth, coords, attrs);
        
        if (attrs) 
        {
                Tcl_DecrRefCount(attrs);
        }
        return ret;
}

static int ObjectDepthCounter=0;
static int InspointDepthCounter = 0;

void VLResetDepthCounter()
{
        ObjectDepthCounter = 0;
        InspointDepthCounter = 0;
}

int VLObjectCreate(NODEPTR context, VLCharPtr partname, VLCharPtr type, int depth, 
        VLPointArray *coords, Tcl_Obj *attrs)
{
        Tcl_Obj *cmd[6], *cmdlst, *result;
        int obid = 0;
        int ret;

        depth = (10000*depth)+ObjectDepthCounter++;
        // printf("Tiefe: %i\n", depth);
        
        cmd[0] = Tcl_NewStringObj("::visual::create_object", -1);
        cmd[1] = TclVP_NewVoidPointerObj(context);
        cmd[2] = Tcl_NewStringObj(partname, -1);
        cmd[3] = Tcl_NewStringObj(type, -1);
        cmd[4] = Tcl_NewIntObj(depth);
        cmd[5] = coords->getObj();
        cmdlst = Tcl_NewListObj(6, cmd);
        Tcl_IncrRefCount(cmdlst);

        if (attrs) {
            Tcl_ListObjAppendList(TclInterp, cmdlst, attrs);
        }
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);

        if (ret != TCL_OK) 
        {
                ReportTclError();
                return 0;
        }
        
        result = Tcl_GetObjResult(TclInterp);
        Tcl_GetIntFromObj((Tcl_Interp *)0, result, &obid);
        return obid;
}

static char thePartname[256];
VLCharPtr VLFormatPartname(VLCharPtr partname, int id)
{
        sprintf(thePartname, "%s%d", partname, id);
        return thePartname;
}       



int VLFixInspointCreate(NODEPTR context, VLCharPtr kind, VLPointArray *coords, 
        int where, VLCharPtr partname, char action, VLListPtr posattr) {
        Tcl_Obj *cmd[8], *cmdlst, *result;
        int obid = 0;
        int ret;

        cmd[0] = Tcl_NewStringObj("::visual::create_fix_inspoint", -1);
        cmd[1] = TclVP_NewVoidPointerObj(context);
        cmd[2] = Tcl_NewStringObj(kind, -1);
        cmd[3] = coords->getObj();
        cmd[4] = Tcl_NewIntObj(where);
        cmd[5] = partname ? Tcl_NewStringObj(partname, -1) : Tcl_NewObj();
        cmd[6] = Tcl_NewStringObj(&action, 1);
        cmd[7] = posattr ? posattr->getObj() : Tcl_NewObj();
        cmdlst = Tcl_NewListObj(8, cmd);

        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);

        if (ret != TCL_OK) 
        {
                ReportTclError();
                return 0;
        }

        result = Tcl_GetObjResult(TclInterp);
        Tcl_GetIntFromObj((Tcl_Interp *)0, result, &obid);
        return obid;
}       

int VLMovableInspointCreate(NODEPTR context, VLCharPtr kind, VLPointArray *coords, 
        int where, VLCharPtr partname, VLPointArray *bbox, int depth, char action, VLCharPtr posattr)
{
    Tcl_Obj *cmd[10], *cmdlst, *result;
    int obid = 0;
    int ret;

    depth = (10000 * depth) + ++InspointDepthCounter;
    
    cmd[0] = Tcl_NewStringObj("::visual::create_movable_inspoint", -1);
    cmd[1] = TclVP_NewVoidPointerObj(context);
    cmd[2] = Tcl_NewStringObj(kind, -1);
    cmd[3] = coords->getObj();
    cmd[4] = bbox ? bbox->getObj() : Tcl_NewObj();
    cmd[5] = Tcl_NewIntObj(where);
    cmd[6] = partname ? Tcl_NewStringObj(partname, -1) : Tcl_NewObj();
    cmd[7] = Tcl_NewIntObj(depth);
    cmd[8] = Tcl_NewStringObj(&action, 1);
    cmd[9] = posattr ? Tcl_NewStringObj(posattr, -1): Tcl_NewObj();
    cmdlst = Tcl_NewListObj(10, cmd);

    Tcl_IncrRefCount(cmdlst);
    ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
    Tcl_DecrRefCount(cmdlst);
    if (ret != TCL_OK) 
    {
        ReportTclError();
        return 0;
    }

    result = Tcl_GetObjResult(TclInterp);
    Tcl_GetIntFromObj((Tcl_Interp *)0, result, &obid);
    return obid;
}       

int VLRepositionCreate(NODEPTR context, VLCharPtr kind, VLPointArray *coords, VLListPtr symbolnames,
                      VLCharPtr partname, VLPointArrayPtr bbox, int depth, VLCharPtr posattr,
                      VLPointArrayPtr origin)
{
    Tcl_Obj *cmd[10], *cmdlst, *result;
    int obid = 0;
    int ret;

    depth = (10000 * depth) + ++InspointDepthCounter;
    
    cmd[0] = Tcl_NewStringObj("::visual::create_reposition", -1);
    cmd[1] = TclVP_NewVoidPointerObj(context);
    cmd[2] = Tcl_NewStringObj(kind, -1);
    cmd[3] = coords->getObj();
    cmd[4] = bbox ? bbox->getObj() : Tcl_NewObj();
    cmd[5] = partname ? Tcl_NewStringObj(partname, -1) : Tcl_NewObj();
    cmd[6] = Tcl_NewIntObj(depth);
    cmd[7] = symbolnames->getObj();
    cmd[8] = posattr ? Tcl_NewStringObj(posattr, -1): Tcl_NewObj();
    cmd[9] = origin ? origin->getObj() : Tcl_NewObj();
    
    cmdlst = Tcl_NewListObj(10, cmd);

    Tcl_IncrRefCount(cmdlst);
    ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
    Tcl_DecrRefCount(cmdlst);
    if (ret != TCL_OK) 
    {
        ReportTclError();
        return 0;
    }

    result = Tcl_GetObjResult(TclInterp);
    Tcl_GetIntFromObj((Tcl_Interp *)0, result, &obid);
    return obid;
}


void VLSetInspointProc(int inspointid, VLCharPtr insertcmd, VLListPtr args)
{
    Tcl_Obj *cmd[4], *cmdlst;
    int ret;
    
    cmd[0] = Tcl_NewStringObj("::visual::set_inspoint_proc", -1);
    cmd[1] = Tcl_NewIntObj(inspointid);
    cmd[2] = Tcl_NewStringObj(insertcmd, -1);
    cmd[3] = args ? args->getObj() : Tcl_NewObj();
    
    cmdlst = Tcl_NewListObj(4, cmd);
    Tcl_IncrRefCount(cmdlst);
    ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
    Tcl_DecrRefCount(cmdlst);
    if (ret != TCL_OK) 
        ReportTclError();
}


void VLSetPosAttrProc(int inspointid, VLCharPtr posattrproc, VLListPtr args)
{
  VLList l;
  l.append("::visual::setPosAttrProc");
  l.append(inspointid);
  l.append(posattrproc);
  l.append(*args);
  l.eval();
}



void VLConstrainInspoint(int inspointid, NODEPTR root, int tree) 
{
    Tcl_Obj *cmd[4], *cmdlst;
    int ret;
    
    cmd[0] = Tcl_NewStringObj("::visual::constrain_inspoint", -1);
    cmd[1] = Tcl_NewIntObj(inspointid);
    cmd[2] = TclVP_NewVoidPointerObj(root);
    cmd[3] = Tcl_NewIntObj(tree);
    
    cmdlst = Tcl_NewListObj(4, cmd);
    Tcl_IncrRefCount(cmdlst);
    ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
    Tcl_DecrRefCount(cmdlst);
    if (ret != TCL_OK) 
        ReportTclError();
}

int VLDragCreateEx(NODEPTR context, VLCharPtr partname, VLCharPtr type, VLPointArray *coords, VLCharPtr kind, VLCharPtr attrname, 
                 VLCharPtr associatecmd, VLList *args)
{
        Tcl_Obj *cmd[9], *cmdlst, *result;
        int obid = 0;
        int ret;
        
        cmd[0] = Tcl_NewStringObj("::visual::create_drag", -1);
        cmd[1] = TclVP_NewVoidPointerObj(context);
        cmd[2] = Tcl_NewStringObj(partname, -1);
        cmd[3] = Tcl_NewStringObj(type, -1);
        cmd[4] = Tcl_NewStringObj(kind, -1);
        cmd[5] = Tcl_NewStringObj(attrname, -1);
        cmd[6] = coords->getObj();
        cmd[7] = associatecmd ? Tcl_NewStringObj(associatecmd, -1) : Tcl_NewObj();
        cmd[8] = args ? args->getObj() : Tcl_NewObj();
        
        cmdlst = Tcl_NewListObj(9, cmd);

        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);
        if (ret != TCL_OK) 
        {
                ReportTclError();
                return 0;
        }

        result = Tcl_GetObjResult(TclInterp);
        Tcl_GetIntFromObj((Tcl_Interp *)0, result, &obid);
        return obid;
}       

void VLSetDrop(int obid, VLCharPtr kind, VLPropertyPtr property, VLList *args)
{
        Tcl_Obj *cmd[5], *cmdlst;
        int ret;
        
        cmd[0] = Tcl_NewStringObj("::visual::droptarget", -1);
        cmd[1] = TclVP_NewVoidPointerObj(property);
        cmd[2] = Tcl_NewIntObj(obid);
        cmd[3] = Tcl_NewStringObj(kind, -1);
        cmd[4] = args ? args->getObj() : Tcl_NewObj();

        cmdlst = Tcl_NewListObj(5, cmd);
        
        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);
        if (ret != TCL_OK) 
        {
                ReportTclError();
        }
}       



void VLAddEditContext(VLNodePtr visualContext, VLNodePtr editContext)
{
        Tcl_Obj *cmd[3], *cmdlst, *result;
        int obid = 0;
        int ret;
        
        cmd[0] = Tcl_NewStringObj("::visual::addEditContext", -1);
        /*
        cmd[1] = TclVP_NewVoidPointerObj(visualContext);
        cmd[2] = TclVP_NewVoidPointerObj(editContext);
        */
        cmd[1] = visualContext.getObj();
        cmd[2] = editContext.getObj();

        cmdlst = Tcl_NewListObj(3, cmd);

        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);
        if (ret != TCL_OK) 
                ReportTclError();

        return;
}       


void VLContextedit(NODEPTR context, VLCharPtr label, VLCharPtr attribute, VLValuePtr value)
{
        Tcl_Obj *cmd[5], *cmdlst, *result;
        int obid = 0;
        int ret;
        
        cmd[0] = Tcl_NewStringObj("::visual::do_contextedit", -1);
        cmd[1] = TclVP_NewVoidPointerObj(context);
        cmd[2] = Tcl_NewStringObj(label, -1);
        cmd[3] = Tcl_NewStringObj(attribute, -1);
        cmd[4] = value ? value->getObj() : Tcl_NewStringObj("1", -1);
        
        cmdlst = Tcl_NewListObj(5, cmd);

        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);
        if (ret != TCL_OK) 
                ReportTclError();

        return;
}       

void VLContextedit(NODEPTR context, VLCharPtr label, VLListPtr command)
{
        Tcl_Obj *cmd[4], *cmdlst, *result;
        int obid = 0;
        int ret;
        
        cmd[0] = Tcl_NewStringObj("::visual::do_contexteditcmd", -1);
        cmd[1] = TclVP_NewVoidPointerObj(context);
        cmd[2] = Tcl_NewStringObj(label, -1);
        cmd[3] = command ? command->getObj() : 
                Tcl_NewStringObj("::log::log \"No Command Specified\"", -1);
        
        cmdlst = Tcl_NewListObj(4, cmd);

        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);
        if (ret != TCL_OK) 
                ReportTclError();

        return;
}       



VLPointArray *VLMeasureText(char *text, char *fontname)
{
        Tcl_Obj *cmd[3], *cmdlst, *result;
        VLPointArray *dimensions;
        int ret;
        
        cmd[0] = Tcl_NewStringObj("::font::measure", -1);
        cmd[1] = Tcl_NewStringObj(fontname ? fontname : (char *)"default", -1);
        cmd[2] = Tcl_NewStringObj(text, -1);

        cmdlst = Tcl_NewListObj(3, cmd);
        
        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);
        if (ret != TCL_OK) 
        {
                ReportTclError();
                return new VLPointArray(0,0);
        }

        result = Tcl_GetObjResult(TclInterp);
        dimensions = new VLPointArray();
        dimensions->setObj(result);
        return dimensions;
}       


VLPointArray *VLMeasureImage(char *name)
{
        Tcl_Obj *result;
        VLPointArray *dimensions;
        int ret;
        char buffer[300];
        sprintf(buffer,"list [image width $::cfg::img(%s)] [image height $::cfg::img(%s)]",
                name,name);
        
        ret = Tcl_GlobalEval(TclInterp,buffer);
        if (ret != TCL_OK) 
        {
                ReportTclError();
                return new VLPointArray(0,0);
        }

        result = Tcl_GetObjResult(TclInterp);
        dimensions = new VLPointArray();
        dimensions->setObj(result);
        return dimensions;
}       



VLString VLGetImageId(char *name)
{
  char buffer[100];
  sprintf(buffer,"::cfg::img(%s)",name);
  char *id=Tcl_GetVar(TclInterp,buffer,0); 
  assert(id);
  return id;
}

void VLSetResizer(int objectid, int corner, VLCharPtr attrname)
{
        Tcl_Obj *cmd[4], *cmdlst;
        int ret;
        
        cmd[0] = Tcl_NewStringObj("::visual::set_resizer", -1);
        cmd[1] = Tcl_NewIntObj(objectid);
        cmd[2] = Tcl_NewIntObj(corner);
        cmd[3] = Tcl_NewStringObj(attrname, -1);
        
        cmdlst = Tcl_NewListObj(4, cmd);

        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);
        if (ret != TCL_OK) 
                ReportTclError();
}


void VLCreateHandle(int objectid, VLCharPtr procname, VLListPtr param)
{
  VLList l;
  l.append("::visual::createHandle");
  l.append(objectid);
  l.append(procname);
  l.append(*param);
  l.eval();
}


void VLSetEditable(int objectid, VLCharPtr attrname)
{
        Tcl_Obj *cmd[3], *cmdlst;
        int ret;
        
        cmd[0] = Tcl_NewStringObj("::visual::set_editable", -1);
        cmd[1] = Tcl_NewIntObj(objectid);
        cmd[2] = Tcl_NewStringObj(attrname, -1);
        
        cmdlst = Tcl_NewListObj(3, cmd);

        Tcl_IncrRefCount(cmdlst);
        ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
        Tcl_DecrRefCount(cmdlst);
        if (ret != TCL_OK) 
                ReportTclError();
}

VLString VLFile(VLCharPtr filename)
{
        Tcl_Obj *result;
        int ret;
        char *buffer = new char[40 + strlen(filename)];

        sprintf(buffer,"file join $::cfg::libdir %s", filename);
        
        ret = Tcl_GlobalEval(TclInterp,buffer);
        if (ret != TCL_OK) 
        {
                ReportTclError();
                return VLString();
        }

        result = Tcl_GetObjResult(TclInterp);
        return VLString(result);
}       

void *SaveModuleBasicObjects()
{
        return (void *)0;
}

void RestoreModuleBasicObjects(void *base)
{
        VLResetDepthCounter();
}       
