
#include "VisAutoPatternCode.h"


int VisTileDrawing::expandX(float fLeft,float fRight,int minSize)
{
  int left=int(floor(fLeft));
  int right=int(ceil(fRight));
  //Suche erste und letzte dragLine
  int i,first,end, currSize, staticSize, dynamicalSize;
  first=0;
  while(first<dragLineX.getSize() && dragLineX[first].pos<left) first++;
  end=first;
  while(end<dragLineX.getSize() && dragLineX[end].pos<right) end++;
  if(minSize!=0) assert(first<end);
  //berechne aktuelle Groesse
  staticSize=sumX[right]-sumX[left];
  dynamicalSize=0;
  for(i=first; i<end; i++) dynamicalSize+=dragLineX[i].getRepSize();
  currSize=staticSize+dynamicalSize;
  int delta=max(0,minSize-currSize);

  //Suche drag-Lines mit hoechster Prioritaet
  int maxpri=0;
  for(i=first; i<end; i++) maxpri=max(maxpri,dragLineX[i].priority);
  int countmaxpri=0;
  for(i=first; i<end; i++) if(dragLineX[i].priority==maxpri) countmaxpri++;

  //vergroessere moeglichst gleichmaessig
  i=first;
  while(delta>0)
  {
    assert(i<end);
    if(dragLineX[i].priority==maxpri)
    {
      int delta2=delta/(countmaxpri--);
      int deltaRep=(delta2+dragLineX[i].size-1)/dragLineX[i].size;
      dragLineX[i].repetitions+=deltaRep;
      delta-=deltaRep*dragLineX[i].size;
    }
    i++;
  }
  return currSize-delta;
}


int VisTileDrawing::expandY(float fLeft,float fRight,int minSize)
{
  int left=int(floor(fLeft));
  int right=int(ceil(fRight));
  //Suche erste und letzte dragLine
  int i,first,end, currSize, staticSize, dynamicalSize;
  first=0;
  while(first<dragLineY.getSize() && dragLineY[first].pos<left) first++;
  end=first;
  while(end<dragLineY.getSize() && dragLineY[end].pos<right) end++;
  if(minSize!=0) assert(first<end);
  //berechne aktuelle Groesse
  staticSize=sumY[right]-sumY[left];
  dynamicalSize=0;
  for(i=first; i<end; i++) dynamicalSize+=dragLineY[i].getRepSize();
  currSize=staticSize+dynamicalSize;
  int delta=max(0,minSize-currSize);

  //Suche drag-Lines mit hoechster Prioritaet
  int maxpri=0;
  for(i=first; i<end; i++) maxpri=max(maxpri,dragLineY[i].priority);
  int countmaxpri=0;
  for(i=first; i<end; i++) if(dragLineY[i].priority==maxpri) countmaxpri++;

  //vergroessere moeglichst gleichmaessig
  i=first;
  while(delta>0)
  {
    assert(i<end);
    if(dragLineY[i].priority==maxpri)
    {
      int delta2=delta/(countmaxpri--);
      int deltaRep=(delta2+dragLineY[i].size-1)/dragLineY[i].size;
      dragLineY[i].repetitions+=deltaRep;
      delta-=deltaRep*dragLineY[i].size;
    }
    i++;
  }
  return currSize-delta;
}


VisDrawingTrafo *VisTileDrawing::getDrawingTrafo(void)
{
  if(sizeX.getSize()==0)
  {
    initDrawing();
    int size, i, j;
    char *name;
    // Bestimme die groessen der einzelnen Kacheln in x-Richtung
    for(i=0; i<tileCountX; i++)
    {
      size=1;
      for(j=0; j<tileCountY; j++)
      {
        name=tileName[i+tileCountX*j];
        if(name) { size=VLMeasureImage(name)->getX(); break; }
      }
      sizeX.add(size);
    }
    // das gleiche in y-Richtung
    for(j=0; j<tileCountY; j++)
    {
      size=1;
      for(i=0; i<tileCountX; i++)
      {
        name=tileName[i+tileCountX*j];
        if(name) { size=VLMeasureImage(name)->getY(); break; }
      }
      sizeY.add(size);
    }
    // Nun Summen bilden und Groessen der dragLines berechnen
    for(i=0; i<dragLineX.getSize(); i++) 
      { dragLineX[i].size=sizeX[dragLineX[i].pos]; sizeX[dragLineX[i].pos]=0; }
    for(i=0; i<dragLineY.getSize(); i++) 
      { dragLineY[i].size=sizeY[dragLineY[i].pos]; sizeY[dragLineY[i].pos]=0; }
    sumX.add(0); for(i=0; i<tileCountX; i++) sumX.add(sumX[i]+sizeX[i]);
    sumY.add(0); for(i=0; i<tileCountY; i++) sumY.add(sumY[i]+sizeY[i]);
    for(i=0; i<dragLineX.getSize(); i++) sizeX[dragLineX[i].pos]=dragLineX[i].size; 
    for(i=0; i<dragLineY.getSize(); i++) sizeY[dragLineY[i].pos]=dragLineY[i].size; 
  }
  return new VisTileDrawing(*this);
}

    
VisContainerInterface2 *VisTileDrawing::expand(void)
{
  //printf("expanding TileDrawing...\n");
  int i;
  for(i=0; i<container.getSize(); i++)
    if(container[i].interface)
    {
      if(container[i].align.x==VisAlignBuoy)
      {
        expandX(container[i].p1.x, container[i].buoy.x, 
          container[i].interface->alignmentPoint.x);
        expandX(container[i].buoy.x, container[i].p2.x, 
          container[i].interface->preferedSize.x-container[i].interface->alignmentPoint.x);
      }
      else expandX(container[i].p1.x, container[i].p2.x, container[i].interface->preferedSize.x);

      if(container[i].align.y==VisAlignBuoy)
      {
        expandY(container[i].p1.y, container[i].buoy.y, 
          container[i].interface->alignmentPoint.y);
        expandY(container[i].buoy.y, container[i].p2.y, 
          container[i].interface->preferedSize.y-container[i].interface->alignmentPoint.y);
      }
      else expandY(container[i].p1.y, container[i].p2.y, container[i].interface->preferedSize.y);
    }
  containerInterface.preferedSize=
   VisIntVector(expandX(0,tileCountX,0),expandY(0,tileCountY,0));
  containerInterface.alignmentPoint=
   VisIntVector(expandX(0,alignmentPosition.x,0),expandY(0,alignmentPosition.y,0));
  containerInterface.nodePtr=nodePtr;
  return &containerInterface;
}


void VisTileDrawing::draw(int derivpos)
{
  //printf("drawing TileDrawing...\n");
  //An Containergroesse anpassen
  if(containerInterface.align.x==VisAlignScale) expandX(0,tileCountX,containerInterface.size.x);
  else containerInterface.xFit();
  if(containerInterface.align.y==VisAlignScale) expandY(0,tileCountY,containerInterface.size.y);
  else containerInterface.yFit();

  //kacheln zeichnen
  VisIntVector tile,rep,nextDragLine,coords;
  coords.x=containerInterface.position.x;
  nextDragLine.x=0;
  for(tile.x=0; tile.x<tileCountX; tile.x++)
  {
    if(nextDragLine.x < dragLineX.getSize() && dragLineX[nextDragLine.x].pos==tile.x)
    { rep.x=dragLineX[nextDragLine.x].repetitions; nextDragLine.x++; }
    else rep.x=1;
    while(rep.x-->0)
    {
      //das ganze in y-Richtung:
      coords.y=containerInterface.position.y;
      nextDragLine.y=0;
      for(tile.y=0; tile.y<tileCountY; tile.y++)
      {
        if(nextDragLine.y < dragLineY.getSize() && dragLineY[nextDragLine.y].pos==tile.y)
        { rep.y=dragLineY[nextDragLine.y].repetitions; nextDragLine.y++; }
        else rep.y=1;
        while(rep.y-->0)
        {
          char *name=tileName[tile.x+tileCountX*tile.y];
          if(name)
          {
            char tclObjName[100];
            char tclObjParam[100];
            sprintf(tclObjName, "tile_%i-%i_/_%i-%i_(derivpos_%i)",tile.x,rep.x,tile.y,rep.y,
                    derivpos);
            sprintf(tclObjParam,
                    "-anchor nw -image %s",(char *)VLGetImageId(name));
            VLObjectCreate(nodePtr, tclObjName, "image", 1, coords, tclObjParam);
          }
          coords.y+=sizeY[tile.y];
        }
      }
      coords.x+=sizeX[tile.x];
    }
  }

  // Einfuegestellen
  for(int i=0; i<insertionLine.getSize(); i++)
  {
    VisTileDrawingInsertionLine *l=&insertionLine[i];
    NODEPTR afterNodePtr = listRootNodePtr;
    if(l->after)
    {
      int id=getContainerId(l->after);
      assert(id>=0);
      // assert(container[id].interface); *** muss wieder eingebaut werden
      if(container[id].interface)
      afterNodePtr=container[id].interface->nodePtr;
    } 
    insPointProc.assign(VLFixInspointCreate(afterNodePtr,"line",
      VLREGION(getPosX(l->p1.x),getPosY(l->p1.y),getPosX(l->p2.x),getPosY(l->p2.y)),
      l->after ? INSPOINT_BELOW : INSPOINT_SON
    ));
  }

  //Positionen der Unterobjekte berechnen
  for(int i=0; i<container.getSize(); i++)
    if(container[i].interface)
      {
        container[i].interface->position=
          containerInterface.position
          +VisIntVector(expandX(0,container[i].p1.x,0),expandY(0,container[i].p1.y,0));
        container[i].interface->size=
          containerInterface.position
          +VisIntVector(expandX(0,container[i].p2.x,0),expandY(0,container[i].p2.y,0))
          -container[i].interface->position;
        container[i].interface->align=container[i].align;
        container[i].interface->buoy=
          containerInterface.position
          +VisIntVector(expandX(0,container[i].buoy.x,0),expandY(0,container[i].buoy.y,0))
          -container[i].interface->position;
      }
}

void VisTileDrawing::setSubElement(VisContainerInterface2 *elem,char *name)
{
  int posId=getContainerId(name);
  if(posId<0)
  {
    printf("Container \"%s\" nicht gefunden! Vorhanden sind:\n",name);
    for(int i=0; i<containerNames.getSize(); i++) 
      printf("%s\n",containerNames[i]);
    assert(0); return;
  }
  assert(posId < container.getSize());
  container[posId].interface=elem;
}



int VisVectorDrawingCoord::min(void)
{
  int m=INT_MAX;
  for(int i=0; i<getSize(); i++) if(value(i)<m) m=value(i);
  return m;
}

int VisVectorDrawingCoord::max(void)
{
  int m=INT_MIN;
  for(int i=0; i<getSize(); i++) if(value(i)>m) m=value(i);
  return m;
}

void VisVectorDrawingCoord::stretch(int left, int right, int delta, int quantum)
{ 
  delta=quantum*((delta+quantum-1)/quantum);
  double factor=1+delta/double(right-left);
  for(int i=0; i<getSize(); i++)
    if(value(i)>left)
      if(value(i)<right) value(i) = left + roundToInt(factor*(value(i)-left));
      else value(i) += delta;
}


void VisVectorDrawingCoord::scale(int left, int right, double factor,int quantum)
{
  int delta=roundToInt((factor-1)*(right-left));
  stretch(left,right,delta,quantum);
}



class VisVectorDrawingTrafo; // forward-Deklaration

VisDrawingTrafo *VisVectorDrawing::getDrawingTrafo(void)
{ 
  if(!inited)
  {
    draw();
    // berechne fuer jedes Container-Intervall das erste 
    // Stretch-Intervall, das dessen Groesse beeinflusst
    int contId2;
    startintId2X.setSize(contPosX.getSize()/2);
    endintId2X.setSize(contPosX.getSize()/2);
    for(contId2=0; contId2<contPosX.getSize(); contId2+=2)
    {
      int contLeft=contPosX[contId2];
      int contRight=contPosX[contId2+1];
      int startintId2=0;
      int endintId2=0;
      while(startintId2<intPosX.getSize() && intPosX[startintId2+1]<=contLeft)
        startintId2+=2;
      startintId2X[contId2/2]=startintId2;
      endintId2=startintId2;
      while(endintId2<intPosX.getSize() && intPosX[endintId2]<contRight)
        endintId2+=2;
      assert(startintId2<endintId2);
      endintId2X[contId2/2]=endintId2-2;
    }
    // das gleiche fuer Y:
    startintId2Y.setSize(contPosY.getSize()/2);
    endintId2Y.setSize(contPosY.getSize()/2);
    for(contId2=0; contId2<contPosY.getSize(); contId2+=2)
    {
      int contLeft=contPosY[contId2];
      int contRight=contPosY[contId2+1];
      int startintId2=0;
      int endintId2=0;
      while(startintId2<intPosY.getSize() && intPosY[startintId2+1]<=contLeft)
        startintId2+=2;
      startintId2Y[contId2/2]=startintId2;
      endintId2=startintId2;
      while(endintId2<intPosY.getSize() && intPosY[endintId2]<contRight)
        endintId2+=2;
      assert(startintId2<endintId2);
      endintId2Y[contId2/2]=endintId2-2;
    }
    if(!boundingBox)
    {
      boundPosX.add(min(min(contPosX.min(),intPosX.min()),figurePosX.min()));
      boundPosX.add(max(max(contPosX.max(),intPosX.max()),figurePosX.max()));
      boundPosY.add(min(min(contPosY.min(),intPosY.min()),figurePosY.min()));
      boundPosY.add(max(max(contPosY.max(),intPosY.max()),figurePosY.max()));
      boundPosX.add(0);
      boundPosY.add(0);
    }
    inited=true; 
  }
  VisVectorDrawingTrafo *trafo=new VisVectorDrawingTrafo();
  trafo->init(*this);
  return trafo;
}





void VisVectorDrawingTrafo::init(VisVectorDrawing &d)
{
  drawing=&d; 
  contPosX=d.contPosX; contPosY=d.contPosY; 
  intPosX=d.intPosX; intPosY=d.intPosY;
  figurePosX=d.figurePosX; figurePosY=d.figurePosY;
  boundPosX=d.boundPosX; boundPosY=d.boundPosY;
  insertionLineX=d.insertionLineX; insertionLineY=d.insertionLineY;
  contXPreferedSize.setSize(contPosX.getSize()/2);
  contYPreferedSize.setSize(contPosY.getSize()/2);
  interface.setSize(drawing->contProp.getSize());
  int i;
  for(i=0; i<contXPreferedSize.getSize(); i++) contXPreferedSize[i]=0;
  for(i=0; i<contYPreferedSize.getSize(); i++) contYPreferedSize[i]=0;
  for(i=0; i<interface.getSize(); i++) interface[i]=NULL;
}   


void VisVectorDrawingTrafo::scaleX(int intId2, double factor)
{
  int left=intPosX[intId2];
  int right=intPosX[intId2+1];
  int quantum=drawing->intQuantumX[intId2/2];
  contPosX.scale(left,right,factor,quantum);
  intPosX.scale(left,right,factor,quantum);
  figurePosX.scale(left,right,factor,quantum);
  boundPosX.scale(left,right,factor,quantum);
  insertionLineX.scale(left,right,factor,quantum);
}

void VisVectorDrawingTrafo::scaleY(int intId2, double factor)
{
  int left=intPosY[intId2];
  int right=intPosY[intId2+1];
  int quantum=drawing->intQuantumY[intId2/2];
  contPosY.scale(left,right,factor,quantum);
  intPosY.scale(left,right,factor,quantum);
  figurePosY.scale(left,right,factor,quantum);
  boundPosY.scale(left,right,factor,quantum);
  insertionLineY.scale(left,right,factor,quantum);
}


void VisVectorDrawingTrafo::stretchX(int intId2, int delta)
{
  int left=intPosX[intId2];
  int right=intPosX[intId2+1];
  int quantum=drawing->intQuantumX[intId2/2];
  contPosX.stretch(left,right,delta,quantum);
  intPosX.stretch(left,right,delta,quantum);
  figurePosX.stretch(left,right,delta,quantum);
  boundPosX.stretch(left,right,delta,quantum);
  insertionLineX.stretch(left,right,delta,quantum);
}


void VisVectorDrawingTrafo::stretchY(int intId2, int delta)
{
  int left=intPosY[intId2];
  int right=intPosY[intId2+1];
  int quantum=drawing->intQuantumY[intId2/2];
  contPosY.stretch(left,right,delta,quantum);
  intPosY.stretch(left,right,delta,quantum);
  figurePosY.stretch(left,right,delta,quantum);
  boundPosY.stretch(left,right,delta,quantum);
  insertionLineY.stretch(left,right,delta,quantum);
}

void VisVectorDrawingTrafo::evaluateInterfaces(void)
{
  for(int i=0; i<interface.getSize(); i++)
  {
    VisContainerInterface2 *p=interface[i];    
    if(!p) continue;
    p->align.x=drawing->contProp[i].alignX;
    p->align.y=drawing->contProp[i].alignY;
    if(drawing->contProp[i].alignX==VisAlignBuoy)
    {
      contXPreferedSize[drawing->contProp[i].idX]=p->alignmentPoint.x;
      contXPreferedSize[drawing->contProp[i].idX+1]=p->preferedSize.x-p->alignmentPoint.x;
    }
    else contXPreferedSize[drawing->contProp[i].idX]=p->preferedSize.x;

    if(drawing->contProp[i].alignY==VisAlignBuoy)
    {
      contYPreferedSize[drawing->contProp[i].idY]=p->alignmentPoint.y;
      contYPreferedSize[drawing->contProp[i].idY+1]=p->preferedSize.y-p->alignmentPoint.y;
    }
    else contYPreferedSize[drawing->contProp[i].idY]=p->preferedSize.y;
  }
}


void VisVectorDrawingTrafo::fit(void)
{
  //printf("expanding VectorDrawing...\n");
  int contId;
  // gehe alle Container durch
  // faktoren: Wie viel Prozent ueberdeckt der Container das Intervall?
  for(contId=0; contId<contXPreferedSize.getSize(); contId++)
  {
    int contLeft=contPosX[2*contId];
    int contRight=contPosX[2*contId+1];
    int contSize=contRight-contLeft;
    if(contXPreferedSize[contId]<=contSize) continue;
    int firstintId2=drawing->startintId2X[contId];
    int lastintId2=drawing->endintId2X[contId];

    // nachtraeglich eingefuegte Intervall-Prioritaeten:
    int i2,maxpri=0;
    for(i2=firstintId2; i2<=lastintId2; i2+=2)
      maxpri=max(maxpri,drawing->intPriorityX[i2/2]);
    while(drawing->intPriorityX[firstintId2/2]!=maxpri) firstintId2+=2;
    while(drawing->intPriorityX[lastintId2/2]!=maxpri) lastintId2-=2;
    int nonmaxpricount=0;
    for(i2=firstintId2; i2<=lastintId2; i2+=2) 
      if(drawing->intPriorityX[i2/2]!=maxpri) nonmaxpricount++;

    int stickOutLeft=max(0, contLeft-intPosX[firstintId2]);
    int stickOutRight=max(0, intPosX[lastintId2+1]-contRight);
    if(firstintId2 == lastintId2)
    { // Container schneidet nur ein Intervall
      double factor = 1 - (stickOutLeft+stickOutRight) 
                        / double(intPosX[firstintId2+1]-intPosX[firstintId2]);
      isometricalStretchX(firstintId2,roundToInt((contXPreferedSize[contId]-contSize)/factor));
    }
    else
    { // beruecksichtige alle geschnittenen Intervalle
      double factor1  = 1 - stickOutLeft
                          / double(intPosX[firstintId2+1]-intPosX[firstintId2]);
      double factorN  = 1 - stickOutRight
                          / double(intPosX[lastintId2+1]-intPosX[lastintId2]);
      double stretch  = (contXPreferedSize[contId]-contSize)
                  / (factor1*factor1+factorN*factorN+(lastintId2-firstintId2-2)/2-nonmaxpricount);
      isometricalStretchX(firstintId2,roundToInt(factor1*stretch));
      for(int intId2=firstintId2+2; intId2<lastintId2; intId2+=2)
        if(drawing->intPriorityX[intId2/2]==maxpri)
          isometricalStretchX(intId2,roundToInt(stretch));
      isometricalStretchX(lastintId2,roundToInt(factorN*stretch));
    }
  }
  // fuer Y analog
  for(contId=0; contId<contYPreferedSize.getSize(); contId++)
  {
    int contLeft=contPosY[2*contId];
    int contRight=contPosY[2*contId+1];
    int contSize=contRight-contLeft;
    if(contYPreferedSize[contId]<=contSize) continue;
    int firstintId2=drawing->startintId2Y[contId];
    int lastintId2=drawing->endintId2Y[contId];

    // nachtraeglich eingefuegte Intervall-Prioritaeten:
    int i2,maxpri=0;
    for(i2=firstintId2; i2<=lastintId2; i2+=2)
      maxpri=max(maxpri,drawing->intPriorityY[i2/2]);
    while(drawing->intPriorityY[firstintId2/2]!=maxpri) firstintId2+=2;
    while(drawing->intPriorityY[lastintId2/2]!=maxpri) lastintId2-=2;
    int nonmaxpricount=0;
    for(i2=firstintId2; i2<=lastintId2; i2+=2) 
      if(drawing->intPriorityY[i2/2]!=maxpri) nonmaxpricount++;

    int stickOutLeft=max(0, contLeft-intPosY[firstintId2]);
    int stickOutRight=max(0, intPosY[lastintId2+1]-contRight);
    if(firstintId2 == lastintId2)
    { // Container schneidet nur ein Intervall
      double factor = 1 - (stickOutLeft+stickOutRight) 
                        / double(intPosY[firstintId2+1]-intPosY[firstintId2]);
      isometricalStretchY(firstintId2,roundToInt((contYPreferedSize[contId]-contSize)/factor));
    }
    else
    { // beruecksichtige alle geschnittenen Intervalle
      double factor1  = 1 - stickOutLeft
                          / double(intPosY[firstintId2+1]-intPosY[firstintId2]);
      double factorN  = 1 - stickOutRight
                          / double(intPosY[lastintId2+1]-intPosY[lastintId2]);
      double stretch  = (contYPreferedSize[contId]-contSize)
                  / (factor1*factor1+factorN*factorN+(lastintId2-firstintId2-2)/2-nonmaxpricount);
      isometricalStretchY(firstintId2,roundToInt(factor1*stretch));
      for(int intId2=firstintId2+2; intId2<lastintId2; intId2+=2)
        if(drawing->intPriorityY[intId2/2]==maxpri)
          isometricalStretchY(intId2,roundToInt(stretch));
      isometricalStretchY(lastintId2,roundToInt(factorN*stretch));
    }
  }
}


void VisVectorDrawingTrafo::draw(int derivpos)
{
#ifdef DEBUG_COORD
  printf("drawing VectorDrawing...\n");
  printf("Interf: x:%i y:%i\n",containerInterface.position.x,containerInterface.position.y);
#endif

  // Groesse an den Container anpassen (evtl. expandieren):
  if(containerInterface.align.x != VisAlignScale) containerInterface.xFit();
  else
  {
    int maxpri=0; 
    for(int i=0; i<drawing->intPriorityX.getSize(); i++) 
      maxpri=max(maxpri,drawing->intPriorityX[i]);
    int currSize=0;
    int intId2;
    // currSize: wie gross ist die Summe alle Stretch-Intervalle (mit max. prioritaet) ?
    for(intId2=0; intId2<intPosX.getSize(); intId2+=2)
      if(drawing->intPriorityX[intId2/2]==maxpri) 
        currSize+=intPosX[intId2+1]-intPosX[intId2];
    // wantedSize: wie gross soll die Summe aller Stretch-Intervalle werden?
    int wantedSize=containerInterface.size.x-((boundPosX[1]-boundPosX[0])-currSize);
    for(intId2=0; intId2<intPosX.getSize(); intId2+=2)
      if(drawing->intPriorityX[intId2/2]==maxpri)
      {
        double factor=wantedSize/double(currSize);
        currSize-=intPosX[intId2+1]-intPosX[intId2];
        scaleX(intId2,factor);
        wantedSize-=intPosX[intId2+1]-intPosX[intId2];
      }
  }
  // fuer y analog:
  if(containerInterface.align.y != VisAlignScale) containerInterface.yFit();
  else
  {
    int maxpri=0; 
    for(int i=0; i<drawing->intPriorityY.getSize(); i++) 
      maxpri=max(maxpri,drawing->intPriorityY[i]);
    int currSize=0;
    int intId2;
    // currSize: wie gross ist die Summe alle Stretch-Intervalle (mit max. prioritaet) ?
    for(intId2=0; intId2<intPosY.getSize(); intId2+=2)
      if(drawing->intPriorityY[intId2/2]==maxpri) 
        currSize+=intPosY[intId2+1]-intPosY[intId2];
    // wantedSize: wie gross soll die Summe aller Stretch-Intervalle werden?
    int wantedSize=containerInterface.size.y-((boundPosY[1]-boundPosY[0])-currSize);
    for(intId2=0; intId2<intPosY.getSize(); intId2+=2)
      if(drawing->intPriorityY[intId2/2]==maxpri)
      {
        double factor=wantedSize/double(currSize);
        currSize-=intPosY[intId2+1]-intPosY[intId2];
        scaleY(intId2,factor);
        wantedSize-=intPosY[intId2+1]-intPosY[intId2];
      }
  }

  //Grafische Objekte zeichnen
  char buffer[100];
  int dx=containerInterface.position.x-boundPosX[0];
  int dy=containerInterface.position.y-boundPosY[0];
  int i;
  for(i=0; i<drawing->figureProp.getSize(); i++)
  {
    VisVectorDrawingFigureProp *p=&(drawing->figureProp[i]);
    VLPointArrayPtr coords=new VLPointArray;
    for(int j=0; j<p->points; j++)
      coords->setXY(j,figurePosX[j+p->posId]+dx, figurePosY[j+p->posId]+dy);
    sprintf(buffer,"figure_%i_%s_(derivpos_%i)",i,p->figure,derivpos);
    VLObjectCreate(nodePtr, buffer, p->figure, 1, coords, p->figureStyle);
  }
  // InsertionLines einfuegen

  for(i=0; i<drawing->insertionLineAfter.getSize(); i++)
  {
    NODEPTR afterNodePtr = listRootNodePtr;
    if(drawing->insertionLineAfter[i])
    {
      int id=drawing->getContainerId(drawing->insertionLineAfter[i]);
      assert(id>=0);
      // assert(interface[id]); *** muss wieder eingebaut werden
      if(interface[id])
      afterNodePtr=interface[id]->nodePtr;
    } 
    VLPointArrayPtr coords=VLREGION(insertionLineX[2*i+0]+dx,insertionLineY[2*i+0]+dy,
                                    insertionLineX[2*i+1]+dx,insertionLineY[2*i+1]+dy);
    sprintf(buffer,"insertionLine_%i_(derivpos_%i)",i,derivpos);
    insPointProc.assign(VLFixInspointCreate(afterNodePtr,"line",coords,
      drawing->insertionLineAfter[i] ? INSPOINT_BELOW : INSPOINT_SON));
  }

  // Unterpositionen festlegen
  for(i=0; i<interface.getSize(); i++)
  { 
    VisContainerInterface2 *p=interface[i];
    if(!p) continue;
    VisVectorDrawingContainerProp *c=&(drawing->contProp[i]);
    p->position=VisIntVector(contPosX[c->idX*2]+dx, contPosY[c->idY*2]+dy);
    p->size=VisIntVector(contPosX[c->idX*2+ ((c->alignX==VisAlignBuoy)?3:1)]-contPosX[c->idX*2], 
                         contPosY[c->idY*2+ ((c->alignY==VisAlignBuoy)?3:1)]-contPosY[c->idY*2]);
    p->buoy=VisIntVector(contPosX[c->idX*2+1]-contPosX[c->idX*2], 
                         contPosY[c->idY*2+1]-contPosY[c->idY*2]);
    p->align.x=c->alignX;
    p->align.y=c->alignY;
  }
}


void VisVectorDrawingTrafo::setSubElement (VisContainerInterface2 *elem,char *name)
{
  int posId=drawing->getContainerId(name);
  if(posId<0)
  {
    printf("Container \"%s\" nicht gefunden! Vorhanden sind:\n",name);
    for(int i=0; i<drawing->containerNames.getSize(); i++) 
      printf("%s\n",drawing->containerNames[i]);
    assert(0); return;
  }
  assert(posId < interface.getSize());
  interface[posId]=elem;
}


VisContainerInterface2 * VisVectorDrawingTrafo::expand(void)
{
  evaluateInterfaces();
  fit();
  containerInterface.preferedSize=preferedSize();
  containerInterface.alignmentPoint=
    VisIntVector(boundPosX[2]-boundPosX[0],boundPosY[2]-boundPosY[0]);
  containerInterface.nodePtr=nodePtr;
  return &containerInterface;
}




VisNullDrawing visNullDrawing;
VisNullDrawingTrafo visNullDrawingTrafo;



VisDefaultFormDrawing::VisDefaultFormDrawing(VisContainerPosition2PtrList list, char* rule)
{
  char buffer[200];
  const int sizex = 150;
  sprintf(buffer,"-text {rule %s} -anchor nw -fill gray",rule);
  figure(1,"text",VLDYNSTRING(buffer)); point(5,5);
  int y=40;
  while(list != NULLVisContainerPosition2PtrList)
  {
    VisContainerPosition2Ptr elem=HeadVisContainerPosition2PtrList(list);
    list=TailVisContainerPosition2PtrList(list);
    sprintf(buffer,"-text {%s} -anchor sw -fill gray",elem->containerName);
    figure(1,"text",VLDYNSTRING(buffer)); point(5,y);
    container(elem->containerName, 10,y,sizex-10,y+10,VisAlignScale, VisAlignScale);
    figure(2,"rectangle","-outline gray"); point(10,y); point(sizex-10,y+10);
    stretchY(y,y+10);
    y+=30;
  }
  figure(2,"rectangle",""); point(0,0); point(sizex,y);
  stretchX(10,sizex-10);
}




VisContainerInterface2Ptr VisRecursiveList::calcSize(void)
{ 
  // Erstelle Ableitungsbaum und fuege die Listenelemente ein
  if(optRoot->containerExists("listelement"))
  {
    assert(listElements); // Dies ist eine Plausibilitaetspruefung !?
    if(listElements)
    {
      optRoot->setSubElement(
       HeadVisContainerInterfacePtrList(listElements)->toType2(),"listelement");
      listElements=TailVisContainerInterfacePtrList(listElements);
    }
  }
  deriv.add(optRoot);
  assert(listElements || !optTerm->containerExists("listelement"));
  if(listElements)
  {
    while(TailVisContainerInterfacePtrList(listElements))
    {
      deriv.add(optRec->getDrawingTrafo());
      deriv.last()->setSubElement(
        HeadVisContainerInterfacePtrList(listElements)->toType2(),"listelement");
      listElements=TailVisContainerInterfacePtrList(listElements);
    }
    if(optTerm->containerExists("listelement"))
      optTerm->setSubElement(
        HeadVisContainerInterfacePtrList(listElements)->toType2(),"listelement");
    else
    {
      deriv.add(optRec->getDrawingTrafo());
      deriv.last()->setSubElement(
        HeadVisContainerInterfacePtrList(listElements)->toType2(),"listelement");
    }
  }
  deriv.add(optTerm);

  //Fuege die Formularelemente ein
  while(formElements)
  {
    VisContainerPosition2Ptr pos=HeadVisContainerPosition2PtrList(formElements);
    formElements=TailVisContainerPosition2PtrList(formElements);
    if(optRoot->containerExists(pos->containerName))
      optRoot->setSubElement(pos->element,pos->containerName);
    else
    {
      assert(optTerm->containerExists(pos->containerName));
      optTerm->setSubElement(pos->element,pos->containerName);
    }
  }

  // setze die Knotenzeiger
  for(int i=0; i<deriv.getSize(); i++) 
  {
    deriv[i]->nodePtr=nodePtr;
    deriv[i]->listRootNodePtr=listRootNodePtr;
    deriv[i]->insPointProc=insPointProc;
  }

  //berechne Groessen
  for(int i=deriv.getSize()-1; i>1; i--)
    deriv[i-1]->setSubElement(deriv[i]->expand(),"sublist");
  if(optRoot->containerExists("sublist")) optRoot->setSubElement(deriv[1]->expand(),"sublist");
  else 
  { 
    assert(optRoot->containerExists("default"));
    optRoot->setSubElement(deriv[1]->expand(),"default");
  }
  return optRoot->expand();
}


void VisRecursiveList::draw(void)
{
  //printf("drawing RecursiveList\n");
  for(int i=0; i<deriv.getSize()-1; i++) 
    deriv[i]->draw(i);
  deriv[deriv.getSize()-1]->draw(VisDerivpos_term);
}


VLPointArrayPtr VisSimpleList::VLRegion(int x1,int y1, int x2, int y2)
{
  if(yDirection(direction)) return VLREGION(x1,y1,x2,y2);
  else return VLREGION(y1,x1,y2,x2);
}


void VisSimpleList::setDefaultValues(void)
{
  frameStyle=NULL;
  separatorStyle=NULL;
  baselineStyle=NULL;
  insertionPointType="line"; 
  inspointProc=NULL;
  inspointParam=VLLIST();
  direction=visSouth;
  alignInLength=VisAlignCenter;
  alignInWidth=VisAlignCenter;
  borderDimensions=0;
  elementDistance=0;
  minLength=0;
  minWidth1=0;
  minWidth2=0;
}

VisContainerInterface2Ptr VisSimpleList::calcSize(void)
{
  width1=minWidth1;
  width2=minWidth2;
  if(alignInWidth!=VisAlignBuoy) { width2+=width1; width1=0; }
  VisContainerInterfacePtrList list=subElements;
  int length=0;
  subElementNumber=0;
  while(list!=NULLVisContainerInterfacePtrList)
  {
    subElementNumber++;
    VisContainerInterface2 *p=HeadVisContainerInterfacePtrList(list)->toType2();
    list=TailVisContainerInterfacePtrList(list);
    if(alignInWidth!=VisAlignBuoy) p->alignmentPoint=VisIntVector(0,0);
    p->align.c1(direction)=alignInWidth;
    p->align.c2(direction)=alignInLength;
    visEnlarge(width1, p->alignmentPoint.c1(direction));
    visEnlarge(width2, p->preferedSize.c1(direction)-p->alignmentPoint.c1(direction));
    length += p->preferedSize.c2(direction)+elementDistance;
  }
  reallyPreferedLength=length;
  visEnlarge(length, minLength);
  containerInterface.preferedSize.c1(direction)=width1+width2;
  containerInterface.preferedSize.c2(direction)=length;
  containerInterface.alignmentPoint.c1(direction)=width1;
  containerInterface.alignmentPoint.c2(direction)=0;
  containerInterface.addBorder(borderDimensions);  
  return &containerInterface;
}


void VisSimpleList::draw()
{
  if(containerInterface.align.x!=VisAlignScale) containerInterface.xFit();
  if(containerInterface.align.y!=VisAlignScale) containerInterface.yFit();

  drawFrame(containerInterface.position, 
            containerInterface.position+containerInterface.size);
  containerInterface.subBorder(borderDimensions);
  width1=containerInterface.buoy.c1(direction);
  width2=containerInterface.size.c1(direction)-width1;
  left=containerInterface.position.c1(direction);
  right=left+containerInterface.size.c1(direction);
  int pos=containerInterface.position.c2(direction);
  //       +containerInterface.buoy.c2(direction);
  int seppos;
  int separatorId=0;

  int currSize=reallyPreferedLength-subElementNumber*elementDistance;
  int wantedSize=containerInterface.size.c2(direction)-subElementNumber*elementDistance;

  if(subElements==NULLVisContainerInterfacePtrList 
    && containerInterface.align.c2(direction)==VisAlignScale)
    pos+=containerInterface.size.c2(direction)/2; 

  int inspointId = VLFixInspointCreate(listRootNodePtr,insertionPointType,
      VLRegion(left-borderDimensions.left,pos,right+borderDimensions.right,pos),
      INSPOINT_SON);
  if(inspointProc) VLSetInspointProc(inspointId, inspointProc, inspointParam);
  if(inspointValidity) VLConstrainInspoint(inspointId, inspointValidity);
  pos+=elementDistance/2;
  VisContainerInterfacePtrList list=subElements;
  while(list!=NULLVisContainerInterfacePtrList)
  {
    VisContainerInterface2 *p=HeadVisContainerInterfacePtrList(list)->toType2();
    list=TailVisContainerInterfacePtrList(list);    
    p->position = VisIntVector(left,pos,direction);
    if(currSize)
      p->size=VisIntVector(width1+width2, 
                    p->preferedSize.c2(direction)*wantedSize/currSize, direction);
    else
      p->size=VisIntVector(width1+width2, p->preferedSize.c2(direction), direction);
    currSize-=p->preferedSize.c2(direction);
    wantedSize-=p->size.c2(direction);
    p->buoy.c1(direction)=width1;
    pos +=  p->size.c2(direction) + elementDistance;
    seppos = pos-elementDistance/2;

    if(list!=NULLVisContainerInterfacePtrList) drawSeparator(seppos,separatorId++);

    inspointId = VLFixInspointCreate(p->nodePtr, insertionPointType, 
      VLRegion(left-borderDimensions.left,seppos,right+borderDimensions.right,seppos),
      INSPOINT_BELOW);
    if(inspointProc) VLSetInspointProc(inspointId, inspointProc, inspointParam);
    if(inspointValidity) VLConstrainInspoint(inspointId, inspointValidity);
  }
}


void VisSimpleList::drawFrame(VisIntVector p1, VisIntVector p2)
{
  if(frameStyle)
    VLObjectCreate(nodePtr,"frame_01","rectangle", 1, VLREGION(p1.x,p1.y,p2.x,p2.y),
      frameStyle);

  /* Basislinie sollte nur zwischen den Elementen gezeichnet werden! */
  if(baselineStyle)
    if(subElements==NULLVisContainerInterfacePtrList)
    {
      // printf("buoy: %i\n", containerInterface.buoy.c1(direction));
      p1.c1(direction)+=containerInterface.buoy.c1(direction);
      p2.c1(direction)=p1.c1(direction);
      VLObjectCreate(nodePtr,"frame_02","line", 1, VLREGION(p1.x,p1.y,p2.x,p2.y),
        baselineStyle);
    }
    else
    {
      int horpos= containerInterface.buoy.c1(direction)+containerInterface.position.c1(direction);
      int coord= containerInterface.position.c2(direction);
      VLObjectCreate(nodePtr,"x1","line", 1, 
         VLRegion(horpos,coord,horpos,coord+elementDistance/2),
         separatorStyle);
      coord+= containerInterface.size.c2(direction);
      VLObjectCreate(nodePtr,"x2","line", 1, 
         VLRegion(horpos,coord,horpos,coord-elementDistance/2),
         separatorStyle);
    }
}

void VisSimpleList::drawSeparator(int coord, int separatorId)
{
  char buffer[100];
  if(separatorStyle)
  {
    sprintf(buffer,"seprarator_%i",separatorId);
    VLObjectCreate(nodePtr,buffer,"line", 1, 
      VLRegion(left-borderDimensions.left,coord,right+borderDimensions.right,coord),
      separatorStyle);
  }
  if(baselineStyle)
  {
    sprintf(buffer,"baseline_%i",separatorId);
    int horpos= containerInterface.buoy.c1(direction)+containerInterface.position.c1(direction);

    VLObjectCreate(nodePtr,buffer,"line", 1, 
      VLRegion(horpos,coord-elementDistance/2,horpos,coord+elementDistance/2),
      separatorStyle);
  }
}



void VisAbstractMatrix::calcSize(void)
{
  int x,y;
  deltaX.setSize(2*sizeX,1); 
  deltaY.setSize(2*sizeY,1);
  for(x=0; x<sizeX; x++)
    for(y=0; y<sizeY; y++)
    {
      VisContainerInterface2* c=container(x,y);
      if(c)
      {
        if(c->align.x!=VisAlignBuoy) c->alignmentPoint.x=0;
        if(c->align.y!=VisAlignBuoy) c->alignmentPoint.y=0;
        visEnlarge(deltaX[2*x+0], c->alignmentPoint.x);
        visEnlarge(deltaY[2*y+0], c->alignmentPoint.y);
        visEnlarge(deltaX[2*x+1], c->preferedSize.x - c->alignmentPoint.x);
        visEnlarge(deltaY[2*y+1], c->preferedSize.y - c->alignmentPoint.y);
      }
    }

  //Mindestgroesse einhalten:
  for(x=0; x<sizeX; x++)
  {
    int currSize=deltaX[2*x]+deltaX[2*x+1];
    if(currSize<minCellSize.x) 
    { 
      deltaX[2*x]=deltaX[2*x]*minCellSize.x/currSize;
      deltaX[2*x+1]=deltaX[2*x+1]*minCellSize.x/currSize;
    }
  }
  for(y=0; y<sizeY; y++)
  {
    int currSize=deltaY[2*y]+deltaY[2*y+1];
    if(currSize<minCellSize.y) 
    { 
      deltaY[2*y]=deltaY[2*y]*minCellSize.y/currSize;
      deltaY[2*y+1]=deltaY[2*y+1]*minCellSize.y/currSize;
    }
  }
}


VisContainerInterface2* VisAbstractMatrix::calcInterface(void)
{
  calcSize();
  containerInterface.preferedSize=VisIntVector(0,0);
  int x,y;
  for(x=0; x<2*sizeX; x++) containerInterface.preferedSize.x+=deltaX[x];
  for(y=0; y<2*sizeY; y++) containerInterface.preferedSize.y+=deltaY[y];
  containerInterface.preferedSize.x+=sizeX*elementDistance.x;
  containerInterface.preferedSize.y+=sizeY*elementDistance.y;
  containerInterface.alignmentPoint=VisIntVector(0,0);
  //containerInterface.addBorder(borderDimensions);  
  return &containerInterface;
}


void VisAbstractMatrix::resizeMatrix(void)
{
  int x,y;

  //Groesse anpassen (x-Richtung):
  if(containerInterface.align.x!=VisAlignScale) containerInterface.xFit();
  else
  { int stretchSize=containerInterface.preferedSize.x-sizeX*elementDistance.x;
    int wantedSize=containerInterface.size.x-sizeX*elementDistance.x;
    for(x=0; x<sizeX; x++) if(!scalePriorityX(x)) stretchSize+=(deltaX[2*x]+deltaX[2*x+1]);
    for(x=0; x<2*sizeX; x++)
      if(scalePriorityX(x/2))
      { int newDelta=deltaX[x]*wantedSize/stretchSize;
        stretchSize-=deltaX[x];
        wantedSize-=newDelta;
        deltaX[x]=newDelta;
      }
  }
  //das gleiche in y-Richtung:
  if(containerInterface.align.y!=VisAlignScale) containerInterface.yFit();
  else
  { int stretchSize=containerInterface.preferedSize.y-sizeY*elementDistance.y;
    int wantedSize=containerInterface.size.y-sizeY*elementDistance.y;
    for(y=0; y<sizeY; y++) if(!scalePriorityY(y)) stretchSize+=(deltaY[2*y]+deltaY[2*y+1]);
    for(y=0; y<2*sizeY; y++)
      if(scalePriorityY(y/2))
      { int newDelta=deltaY[y]*wantedSize/stretchSize;
        stretchSize-=deltaY[y];
        wantedSize-=newDelta;
        deltaY[y]=newDelta;
      }
  }

  //Absolute Positionen berechnen
  VisIntVector absPos;
  absPos.x=containerInterface.position.x;
  for(x=0; x<sizeX; x++)
  {  
    absPositionX.add(absPos.x);
    absPos.x+=deltaX[2*x]+deltaX[2*x+1]+elementDistance.x;
  }
  absPositionX.add(absPos.x);
  absPos.y=containerInterface.position.y;
  for(y=0; y<sizeY; y++)
  {  
    absPositionY.add(absPos.y);
    absPos.y+=deltaY[2*y]+deltaY[2*y+1]+elementDistance.y;
  }
  absPositionY.add(absPos.y);
}



void VisAbstractMatrix::drawMatrix(void)
{
  int x,y;

  //Container positionieren
  for(x=0; x<sizeX; x++)
    for(y=0; y<sizeY; y++)
      if(container(x,y))
      {
        container(x,y)->position=VisIntVector(absPositionX[x]+elementDistance.x/2,
                                              absPositionY[y]+elementDistance.y/2);
        container(x,y)->buoy=VisIntVector(deltaX[2*x],deltaY[2*y]);
        container(x,y)->size=VisIntVector(deltaX[2*x]+deltaX[2*x+1],deltaY[2*y]+deltaY[2*y+1]);
      }

  //Verzierungen
   if(gridStyle)
  {
    char tclObjName[100];
    for(x=0; x<=sizeX; x++)
      {
        sprintf(tclObjName,"gridline_X=%i",x);
        VLObjectCreate(nodePtr, tclObjName, "line", 1, getAbsPosition(x,0,x,sizeY),gridStyle);
      }
    for(y=0; y<=sizeY; y++)
      {
        sprintf(tclObjName,"gridline_Y=%i",y);
        VLObjectCreate(nodePtr, tclObjName, "line", 1, getAbsPosition(0,y,sizeX,y),gridStyle);
      }
  }
}


void VisAbstractMatrix::drawInsLine(bool horr, int pos, NODEPTR nodePtr, int where)
{
  int x1,y1,x2,y2;
  if(horr) { x1=0; x2=sizeX; y1=y2=pos; }
  else     { y1=0; y2=sizeY; x1=x2=pos; }
  insPointProc.assign(VLFixInspointCreate(nodePtr,insPointType,
    VLREGION(absPositionX[x1],absPositionY[y1],absPositionX[x2],absPositionY[y2]), where));
}



void VisTable::draw(void)
{
  char tclObjName[100];
  resizeMatrix();
  if(headBackground)
     VLObjectCreate(nodePtr, "headBackgrnd", "rectangle", 1, 
       getAbsPosition(0,0,virtSizeX,1),headBackground);
  drawMatrix();
  int pos= contList ? 1:0;

  if(pos==1) 
  // Zeichne Kopfzeile, gleicher Code wie untern bis auf "p->nodePtr"
  {
    if(inDirectionLineStyle)
      for(int x=1; x<virtSizeX; x++)
      {
        sprintf(tclObjName,"fieldSeparator_%i_%i",x,pos);
        VLObjectCreate(nodePtr, tclObjName, "line", 1, getAbsPosition(x,pos-1,x,pos),
         inDirectionLineStyle);
      }
    if(inDirectionBorderLineStyle)
    {
      sprintf(tclObjName,"inDirectionLine_%i_%i",0,pos);
      VLObjectCreate(nodePtr, tclObjName, "line", 1, getAbsPosition(0,pos-1,0,pos),
       inDirectionBorderLineStyle);
      if(virtSizeX)
      { sprintf(tclObjName,"inDirectionLine_%i_%i",virtSizeX,pos);
        VLObjectCreate(nodePtr, tclObjName, "line", 1, 
          getAbsPosition(virtSizeX,pos-1,virtSizeX,pos), inDirectionBorderLineStyle);
      }
    }
  }

  VisComplexElementPtrList remCompList = compList;
  drawInsLine(yDirection(direction),pos++,listRootNodePtr, INSPOINT_SON);
  while(remCompList!=NULLVisComplexElementPtrList)
  {
    VisComplexElementPtr p=HeadVisComplexElementPtrList(remCompList);
    remCompList=TailVisComplexElementPtrList(remCompList);
    if(inDirectionLineStyle)
      for(int x=1; x<virtSizeX; x++)
      {
        sprintf(tclObjName,"fieldSeparator_%i_%i",x,pos);
        VLObjectCreate(p->nodePtr, tclObjName, "line", 1, getAbsPosition(x,pos-1,x,pos),
         inDirectionLineStyle);
      }
    if(inDirectionBorderLineStyle)
    {
      sprintf(tclObjName,"inDirectionLine_%i_%i",0,pos);
      VLObjectCreate(p->nodePtr, tclObjName, "line", 1, getAbsPosition(0,pos-1,0,pos),
       inDirectionBorderLineStyle);
      if(virtSizeX)
      { sprintf(tclObjName,"inDirectionLine_%i_%i",virtSizeX,pos);
        VLObjectCreate(p->nodePtr, tclObjName, "line", 1, 
          getAbsPosition(virtSizeX,pos-1,virtSizeX,pos), inDirectionBorderLineStyle);
      }
    }
    drawInsLine(yDirection(direction),pos++, p->nodePtr, INSPOINT_BELOW);
  }

  // Linien orthogonal zu Direction:
  if(orthDirectionLineStyle)
    for(int y=1; y<virtSizeY; y++)
      {
        sprintf(tclObjName,"orthDirectionLine_%i",y);
        VLObjectCreate(nodePtr, tclObjName, "line", 1, getAbsPosition(0,y,virtSizeX,y),
          orthDirectionLineStyle);
      }
  if(orthDirectionBorderLineStyle)
  {
    sprintf(tclObjName,"orthDirectionLine_%i",0);
    VLObjectCreate(nodePtr, tclObjName, "line", 1, getAbsPosition(0,0,virtSizeX,0),
      orthDirectionBorderLineStyle);
    if(virtSizeY)
    { sprintf(tclObjName,"orthDirectionLine_%i",virtSizeY);
      VLObjectCreate(nodePtr, tclObjName, "line", 1, 
        getAbsPosition(0,virtSizeY,virtSizeX,virtSizeY), orthDirectionBorderLineStyle);
    }
  }
}

void VisTable::insertElements(VLListPtr fieldList, VisContainerPosition2PtrList aContList,
                              VisComplexElementPtrList aCompList)
{
  compList=aCompList;
  contList=aContList;
  //Bestimme die Namen der Container
  if(fieldList)
  {
    for(int i=0; i<fieldList->getLength(); i++)
      containerNames.add(fieldList->getIndex(i));
  }
  else
  {
    VisContainerPosition2PtrList l=
      contList ? contList : 
      compList ? HeadVisComplexElementPtrList(compList)->contList :
      NULLVisContainerPosition2PtrList;
    while(l!=NULLVisContainerPosition2PtrList)
    {
      VisContainerPosition2Ptr h=HeadVisContainerPosition2PtrList(l);
      l=TailVisContainerPosition2PtrList(l);
      containerNames.add(h->containerName);
    }
  }

  //Erzeuge die Matrix
  int listLength=LengthVisComplexElementPtrList(compList);
  if(contList) listLength++;
  initMatrix(containerNames.getSize(),listLength);

  //Fuelle die Matrix
  VisComplexElementPtrList remCompList = compList;
  int listElementNo=0;
  if(aContList) //fuege Kopfzeile ein
  {
    while(aContList!=NULLVisContainerPosition2PtrList)
    {
      VisContainerPosition2Ptr h=HeadVisContainerPosition2PtrList(aContList);
      aContList=TailVisContainerPosition2PtrList(aContList);
      container(getContainerId(h->containerName),0) = h->element;
    }
    listElementNo++;
  }

  while(remCompList!=NULLVisComplexElementPtrList)
  {
    VisContainerPosition2PtrList l=HeadVisComplexElementPtrList(remCompList)->contList;
    remCompList=TailVisComplexElementPtrList(remCompList);
    while(l!=NULLVisContainerPosition2PtrList)
    {
      VisContainerPosition2Ptr h=HeadVisContainerPosition2PtrList(l);
      l=TailVisContainerPosition2PtrList(l);
      int contId=getContainerId(h->containerName);
      ASSERT_INFO(contId!=-1,
        printf("Container %s nicht gefunden. Gueltige Container sind:\n",h->containerName);
        for(int i=0; i<containerNames.getSize(); i++) printf("%s\n",containerNames[i]);
      )
      container(contId,listElementNo) = h->element;
      //h->element->align=VisIntVector(VisAlignLeft, VisAlignCenter);
    }
    listElementNo++;
  }    
}





void VisTerminal::setDefaultValues(void)
{
  size=VisIntVector(50,50);
  alignmentPoint=VisIntVector(0,0);
  figure="oval";
  figureStyle="";  
  borderDimensions=0;
}

VisContainerInterface2Ptr VisTerminal::calcSize(void)
{
  containerInterface.preferedSize=size;
  containerInterface.alignmentPoint=alignmentPoint;
  containerInterface.addBorder(borderDimensions);
  return &containerInterface;
}

void VisTerminal::draw(void)
{
  containerInterface.xFit();
  containerInterface.yFit();
  containerInterface.subBorder(borderDimensions);
  VisIntVector start=containerInterface.position;
  VisIntVector end=start+size;
  VLObjectCreate(nodePtr, "elem01", figure,1, VLREGION(start.x,start.y,end.x,end.y),
   figureStyle);
}


