
#ifndef VISAUTOPATTERNCODE_H
#define VISAUTOPATTERNCODE_H
#include <limits.h>
#include "VisBasicCode.h"
#include "VisBasicCode.head"
#include "DynamicalArray.h"
#include "VisComplexElementPtrList.h"


#define ASSERT_INFO(assertion,info) \
  if(!(assertion)) { printf("ASSERT_INFO:\n"); info; assert(0); }

class VisRegion2: public VisObject
{
public:
  VisRegion2(NODEPTR x) : VisObject(x) { containerInterface.nodePtr=x; }
  VisContainerInterface2 containerInterface;
};


class VisInsPointProc
{
public:
  char *name;
  VLListPtr args;
  NODEPTR validity;
  VisInsPointProc() { name=NULL; args=VLLIST(); validity=NULL; }
  VisInsPointProc(char *aName, VLListPtr aArgs, NODEPTR aValidity) 
  { name=aName; args=aArgs; validity=aValidity; }

  void assign(int insPointId) 
  { 
    if(name) VLSetInspointProc(insPointId, name, args); 
    if(validity) VLConstrainInspoint(insPointId, validity);
  }
};



enum { VisDerivpos_term=-1, VisDerivpos_frame=-2 };

typedef class VisDrawingTrafo
{
public:
  virtual ~VisDrawingTrafo() {}

  NODEPTR nodePtr;
  NODEPTR listRootNodePtr;
  VisInsPointProc insPointProc;
  void setNodePtr(NODEPTR x) { nodePtr=x; }
  virtual void draw(int derivpos=0)=0;
  virtual VisContainerInterface2 *expand(void)=0;
  virtual void setSubElement (VisContainerInterface2 *, char *contName)=0;
  virtual int containerExists(char *name)=0;
  void setSubElements(VisContainerPosition2PtrList list)
  {
    while(list!=NULLVisContainerPosition2PtrList)
    {
      VisContainerPosition2 *p=HeadVisContainerPosition2PtrList(list);
      list=TailVisContainerPosition2PtrList(list);
      setSubElement(p->element, p->containerName);
    }
  }
} *VisDrawingTrafoPtr;


typedef class VisDrawing
{
public:
  virtual ~VisDrawing() {}


  virtual VisDrawingTrafo *getDrawingTrafo(void)=0;
  DynamicalArray<char *> containerNames;
  int getContainerId(char *name) 
  { 
    for(int i=0; i<containerNames.getSize(); i++) if(!strcmp(containerNames[i],name)) return i;
    return -1;
  }
} *VisDrawingPtr;



struct VisTileDrawingContainer
{
  VisContainerInterface2 *interface;
  VisFloatVector p1,p2,buoy;
  VisIntVector align;
  VisTileDrawingContainer(void) {}
  VisTileDrawingContainer(VisFloatVector aP1, VisFloatVector aP2,
    VisAlignmentDefinition alignX, VisAlignmentDefinition alignY)
    { p1=aP1; p2=aP2; align.x=alignX.type; align.y=alignY.type; 
      buoy.x=alignX.value; buoy.y=alignY.value;
      interface=NULL;
    }
};

struct VisTileDrawingDragLine
{
  int pos;
  int repetitions;
  int size;
  int priority;
  VisTileDrawingDragLine(int aPos, int aPriority, int aRepetitions) 
  { pos=aPos; priority=aPriority; repetitions=aRepetitions; }
  VisTileDrawingDragLine(void) {}
  int getRepSize(void) { return repetitions*size; }
};

struct VisTileDrawingInsertionLine
// Position einer Einfuegesetelle (eine Linie)
{
  VisFloatVector p1,p2;
  char *after;
  VisTileDrawingInsertionLine(void) {}
  VisTileDrawingInsertionLine(VisFloatVector aP1, VisFloatVector aP2, char *aAfter) :
    p1(aP1), p2(aP2), after(aAfter) {}
};

class VisTileDrawing: public VisDrawing, public VisDrawingTrafo
{
public:
  //Optionen:
  int tileCountX, tileCountY;
  char **tileName; //Array der Groesse tileCountX*tileCountY
  DynamicalArray<int> sumX;
  DynamicalArray<int> sumY;
  DynamicalArray<int> sizeX;
  DynamicalArray<int> sizeY;
  VisContainerInterface2 containerInterface;
  DynamicalArray<VisTileDrawingContainer> container;
  DynamicalArray<VisTileDrawingDragLine> dragLineX;
  DynamicalArray<VisTileDrawingDragLine> dragLineY;
  DynamicalArray<VisTileDrawingInsertionLine> insertionLine;
  VisFloatVector alignmentPosition;
  VisTileDrawing(void) { alignmentPosition=VisFloatVector(0.0,0.0); }

  int expandX(float fleft, float fright, int minSize);
  int expandY(float fleft, float fright, int minSize);
  int getPosX(float i) { return getPosX((int)i); }
  int getPosY(float i) { return getPosY((int)i); }
  int getPosX(int i) { return expandX(0,i,0)+containerInterface.position.x; }
  int getPosY(int i) { return expandY(0,i,0)+containerInterface.position.y; }
  VisDrawingTrafo *getDrawingTrafo(void);
  VisContainerInterface2 *expand(void);
  void draw(int derivpos);
  void setSubElement (VisContainerInterface2 *, char *name);
  int containerExists(char *name) { return getContainerId(name)!=-1; }
  virtual void initDrawing(void) { assert(0); }

  //Zeichenfunktionen
  void addContainer(char *name, float x1, float y1, float x2, float y2, 
                         VisAlignmentDefinition alignX, VisAlignmentDefinition alignY)
    { container.add(
       VisTileDrawingContainer(VisFloatVector(x1,y1),VisFloatVector(x2,y2),alignX,alignY)); 
      containerNames.add(name);
    }
  void addDragLineX(int x,int priority=0, int mincnt=0) 
   { dragLineX.add(VisTileDrawingDragLine(x,priority,mincnt)); }
  void addDragLineY(int y,int priority=0, int mincnt=0) 
  { dragLineY.add(VisTileDrawingDragLine(y,priority,mincnt)); }
  void addInsertionLine(float x1, float y1, float x2, float y2, char *after)
    { insertionLine.add(VisTileDrawingInsertionLine(
        VisFloatVector(x1,y1),VisFloatVector(x2,y2),after)); }
};



class VisVectorDrawingCoord: public DynamicalArray<int>
{
public: 
  void scale(int left, int right, double factor, int quantum);
  void stretch(int left, int right, int delta, int quantum);
  int min(void);
  int max(void);
};



struct VisVectorDrawingContainerProp
{
  int idX; // posX[2*idX],... und contXPreferedSize[idX],... gehoeren zu diesem Kontainer
  int idY;
  VisAlignmentAdvice alignX;
  VisAlignmentAdvice alignY;
  VisVectorDrawingContainerProp(void) {}
  VisVectorDrawingContainerProp(int aIdX, int aIdY, 
     VisAlignmentAdvice aAlignX, VisAlignmentAdvice aAlignY) 
  { idX=aIdX; idY=aIdY; alignX=aAlignX; alignY=aAlignY; }
};



class VisVectorDrawingFigureProp
{
public:
  int posId;
  int points;
  VLCharPtr figure;
  VLCharPtr figureStyle;
  VisVectorDrawingFigureProp(void) {}
  VisVectorDrawingFigureProp(int aPosId, int aPoints, 
               VLCharPtr aFigure, VLCharPtr aFigureStyle)
  { posId=aPosId; points=aPoints; figure=aFigure; figureStyle=aFigureStyle; }
};



class VisVectorDrawing: public VisDrawing
{
public:
  VisVectorDrawing(void) { boundingBox=0; inited=false; stretchIsometrical=VisIntVector(-1,-1); }
  VisVectorDrawingCoord contPosX; // linke, rechte Seite eines Intervalls, das
  VisVectorDrawingCoord contPosY; //           eine bestimmte Groesse benoetigt
  VisVectorDrawingCoord boundPosX;
  VisVectorDrawingCoord boundPosY;
  DynamicalArray<int> startintId2X;
  DynamicalArray<int> startintId2Y;
  DynamicalArray<int> endintId2X;
  DynamicalArray<int> endintId2Y;
  DynamicalArray<VisVectorDrawingContainerProp> contProp;
  VisVectorDrawingCoord intPosX;   // linke, rechte Seite eines Intervalls, das
  VisVectorDrawingCoord intPosY;   //                        gedehnt werden kann
  DynamicalArray<int> intQuantumX; // intQuantumX[i] is Vergroesserungsquantum 
  DynamicalArray<int> intQuantumY; // des Intervalls intPosX[2*i]...intPosX[2*i+1]
  DynamicalArray<int> intPriorityX; // intPriorityX[i] is Vergroesserungsprioritaet 
  DynamicalArray<int> intPriorityY; // des Intervalls intPosX[2*i]...intPosX[2*i+1]
  VisVectorDrawingCoord figurePosX;
  VisVectorDrawingCoord figurePosY;
  VisVectorDrawingCoord insertionLineX;     // Koordinaten einer Einfuegesetelle.
  VisVectorDrawingCoord insertionLineY;     // insertionLineAfter: Hinter dem entspr.
  DynamicalArray<char*> insertionLineAfter; // Containerelement wird eingefuegt.
  VisIntVector stretchIsometrical;
  DynamicalArray<VisVectorDrawingFigureProp> figureProp;
  int boundingBox;
  bool inited;

  // Zeichenfunktionen  
  void figure(int points, VLCharPtr aFigure, VLCharPtr aStyle)
  { figureProp.add(VisVectorDrawingFigureProp(figurePosX.getSize(),points,aFigure,aStyle)); }
  void point(int x, int y)
  { figurePosX.add(x); figurePosY.add(y); }
  void container(char *name,int x1, int y1, int x2, int y2, 
                         VisAlignmentDefinition alignX, VisAlignmentDefinition alignY)
  { contProp.add(VisVectorDrawingContainerProp(contPosX.getSize()/2,contPosY.getSize()/2,
                                                            alignX.type, alignY.type));
    contPosX.add(x1); 
    if(alignX.type==VisAlignBuoy) { contPosX.add(alignX.value); contPosX.add(alignX.value); }
    contPosX.add(x2);
    contPosY.add(y1); 
    if(alignY.type==VisAlignBuoy) { contPosY.add(alignY.value); contPosY.add(alignY.value); }
    contPosY.add(y2);
    containerNames.add(name);
  }
  void insertionLine(int x1, int y1, int x2, int y2, char *after)
  { 
    insertionLineX.add(x1); insertionLineX.add(x2);
    insertionLineY.add(y1); insertionLineY.add(y2);
    insertionLineAfter.add(after);
  }
  void stretchX(int left, int right, int priority=0, int quantum=1) 
  { intPosX.add(left); intPosX.add(right); 
    intQuantumX.add(quantum); intPriorityX.add(priority); }
  void stretchY(int left, int right, int priority=0, int quantum=1) 
  { intPosY.add(left); intPosY.add(right); 
    intQuantumY.add(quantum); intPriorityY.add(priority); }

  // andere Methoden
  VisDrawingTrafo *getDrawingTrafo(void);
  virtual void draw(void) {}
};



class VisVectorDrawingTrafo: public VisDrawingTrafo
{
public:
  VisContainerInterface2 containerInterface;
  VisVectorDrawingCoord contPosX; // linke, rechte Seite eines Intervalls, das
  VisVectorDrawingCoord contPosY; //           eine bestimmte Groesse benoetigt
  VisVectorDrawingCoord intPosX;  // linke, rechte Seite eines Intervalls, das
  VisVectorDrawingCoord intPosY;  //                        gedehnt werden kann
  VisVectorDrawingCoord figurePosX;
  VisVectorDrawingCoord figurePosY;
  VisVectorDrawingCoord boundPosX;
  VisVectorDrawingCoord boundPosY;
  VisVectorDrawingCoord insertionLineX;
  VisVectorDrawingCoord insertionLineY;
  DynamicalArray<int> contXPreferedSize;
  DynamicalArray<int> contYPreferedSize;
  DynamicalArray<VisContainerInterface2*> interface;
  VisVectorDrawing *drawing;

  VisVectorDrawingTrafo(void) { drawing=NULL; }
  void init(VisVectorDrawing &d);
  void scaleX(int intId2, double factor);
  void scaleY(int intId2, double factor);
  void stretchX(int intId2, int delta);
  void stretchY(int intId2, int delta);

  void isometricalStretchX(int intId2, int delta)
  {
    double factor=1+delta/double(intPosX[intId2+1]-intPosX[intId2]);
    scaleX(intId2,factor);
    if(2*drawing->stretchIsometrical.x==intId2) 
    { assert(drawing->stretchIsometrical.y>=0); scaleY(drawing->stretchIsometrical.y,factor); }
  }
  void isometricalStretchY(int intId2, int delta)
  {
    double factor=1+delta/double(intPosY[intId2+1]-intPosY[intId2]);
    scaleY(intId2,factor);
    if(2*drawing->stretchIsometrical.y==intId2) 
    { assert(drawing->stretchIsometrical.x>=0); scaleX(drawing->stretchIsometrical.x,factor); }
  }


  void evaluateInterfaces(void);
  void fit(void);
  VisIntVector preferedSize(void) 
  { return VisIntVector(boundPosX[1]-boundPosX[0], boundPosY[1]-boundPosY[0]); }

  void setSubElement(VisContainerInterface2 *elem,char *name);
  int containerExists(char *name) { return drawing->getContainerId(name)!=-1; }
  virtual void draw(int derivpos);
  virtual VisContainerInterface2 *expand(void);
};



#define DECL_TILEDRAWING(x) class T##x: public VisTileDrawing { void initDrawing(void); }; \
                            extern T##x x;
#define IMPL_TILEDRAWING(x) T##x x; \
                            void T##x::initDrawing(void)

#define DECL_VECTORDRAWING(x) class T##x: public VisVectorDrawing { void draw(void); }; \
                              extern T##x x;
#define IMPL_VECTORDRAWING(x) T##x x; \
                              void T##x::draw(void)



extern class VisNullDrawingTrafo: public VisDrawingTrafo
{
public:
  void setNodePtr(NODEPTR x) {}
  void draw(int derivpos) {}
  VisContainerInterface2 *expand(void) 
  { return new VisContainerInterface2(VisIntVector(0,0),NULL); }
  void setSubElement (VisContainerInterface2 *, char *contName) { assert(0); }
  int containerExists(char *name) { return 0; }
} visNullDrawingTrafo;


extern class VisNullDrawing: public VisDrawing
{
public:
  VisDrawingTrafo *getDrawingTrafo(void) { return &visNullDrawingTrafo; }
} visNullDrawing;




// VisDefaultFormDrawing
// ---------------------
// Eine generische Zeichnung, die defaultmaessig fuer
// die Darstellung eines Formulars benutzt wird 
//

class VisDefaultFormDrawing: public VisVectorDrawing
{
  public: VisDefaultFormDrawing(VisContainerPosition2PtrList list, char* rule);
};



typedef class VisRecursiveList: public VisRegion2
{
public:
  VisRecursiveList(NODEPTR x) : VisRegion2(x) {}
  VisDrawingTrafoPtr optRoot, optTerm;
  VisDrawingPtr optRec;
  VisInsPointProc insPointProc;
  DynamicalArray<VisDrawingTrafoPtr> deriv;
  VisContainerInterfacePtrList listElements;
  VisContainerPosition2PtrList formElements;
  NODEPTR listRootNodePtr;

  VisContainerInterface2Ptr calcSize(void);
  void draw();
} *VisRecursiveListPtr;



typedef class VisSimpleList: public VisRegion2
{
public:
  VisSimpleList(NODEPTR x) : VisRegion2(x) { setDefaultValues(); }

  // Optionen
  VLCharPtr frameStyle; // Optionen fuer "rectangle"
  VLCharPtr separatorStyle; // Optionen fuer "line"
  VLCharPtr baselineStyle; // Optionen fuer "line"
  VLCharPtr insertionPointType; 
  VLCharPtr inspointProc;
  VLListPtr inspointParam;
  VisBorderDimensions borderDimensions;
  VisDirection direction;
  VisAlignmentAdvice alignInWidth;
  VisAlignmentAdvice alignInLength;
  int elementDistance;
  int minLength, minWidth1, minWidth2;
  NODEPTR inspointValidity;

  // andere Werte
  VisContainerInterfacePtrList subElements;
  int width1, width2, left, right;
  int subElementNumber;
  int reallyPreferedLength;
  NODEPTR listRootNodePtr;

  void setSubElements(VisContainerInterfacePtrList x) { subElements=x; }
  void setDefaultValues(void);
  VisContainerInterface2Ptr calcSize(void);
  void draw(void);
  virtual void drawFrame(VisIntVector p1, VisIntVector p2);
  virtual void drawSeparator(int coord, int separatorId);
  VLPointArrayPtr VLRegion(int x1,int y1, int x2, int y2);
} *VisSimpleListPtr;



class VisAbstractMatrix: public VisRegion2
{
public:
  //Struktur:
  int sizeX;
  int sizeY;
  DynamicalArray<VisContainerInterface2*> containerArray;
  DynamicalArray<int> deltaX;
  DynamicalArray<int> deltaY;
  DynamicalArray<int> absPositionX;
  DynamicalArray<int> absPositionY;

  //Zeichenoptionen:
  VisIntVector elementDistance;
  VisInsPointProc insPointProc;
  VLCharPtr insPointType;
  VLCharPtr gridStyle;
  VisIntVector minCellSize;

  VisAbstractMatrix(NODEPTR nodePtr) : VisRegion2(nodePtr) 
  { gridStyle=NULL; elementDistance=VisIntVector(0,0); insPointType=NULL; 
    minCellSize=VisIntVector(10,10); }

  virtual ~VisAbstractMatrix() {}

  void initMatrix(int aSizeX, int aSizeY)
  { sizeX=aSizeX; sizeY=aSizeY; containerArray.setSize(sizeX*sizeY,NULL); }
  VisContainerInterface2* &container(int x, int y) 
  { assert(0<=x && x<sizeX); assert(0<=y && y<sizeY); return containerArray[x+y*sizeX]; }
  void  calcSize(void);
  VisContainerInterface2* calcInterface(void);
  void resizeMatrix(void);
  void drawMatrix(void);
  void drawInsLine(bool horr, int pos, NODEPTR nodePtr, int where);
  virtual int scalePriorityX(int x) { return 1; };
  virtual int scalePriorityY(int y) { return 1; };
  VLPointArrayPtr getAbsPosition(int x1, int y1, int x2, int y2) 
  { return VLREGION(absPositionX[x1],absPositionY[y1],absPositionX[x2],absPositionY[y2]); }
};



typedef class VisTable: public VisAbstractMatrix
{
public:
  int virtSizeX, virtSizeY;
  VisDirection direction;
  VLCharPtr headBackground;

  VLCharPtr inDirectionLineStyle,inDirectionBorderLineStyle,
            orthDirectionLineStyle,orthDirectionBorderLineStyle;

  NODEPTR listRootNodePtr;
  VisComplexElementPtrList compList;
  VisContainerPosition2PtrList contList;
  DynamicalArray<char *> containerNames;
  VisTable(NODEPTR nodePtr) : VisAbstractMatrix(nodePtr) 
  {
    inDirectionLineStyle=inDirectionBorderLineStyle=
    orthDirectionLineStyle=orthDirectionBorderLineStyle=NULL;
  }

  int getContainerId(char *name) 
  { 
    for(int i=0; i<containerNames.getSize(); i++) if(!strcmp(containerNames[i],name)) return i;
    return -1;
  }
  void insertElements(VLListPtr fieldList,VisContainerPosition2PtrList,VisComplexElementPtrList);
  void draw(void);

  //Funktionen fuer die Koordinatentransformation
  void coordtrafo(int &x, int &y, int sub)
  { if(negDirection(direction)) y=virtSizeY-y-sub;
    if(xDirection(direction)) { int buffer=x; x=y; y=buffer; }
  }  
  void initMatrix(int aSizeX, int aSizeY)
  { if(yDirection(direction)) VisAbstractMatrix::initMatrix(aSizeX, aSizeY); 
    else VisAbstractMatrix::initMatrix(aSizeY, aSizeX);  
    virtSizeX=aSizeX; virtSizeY=aSizeY;
  }
  VisContainerInterface2* &container(int x, int y)
  { coordtrafo(x,y,1); return VisAbstractMatrix::container(x,y); }
  VLPointArrayPtr getAbsPosition(int x1, int y1, int x2, int y2)
  { 
    coordtrafo(x1,y1,0); coordtrafo(x2,y2,0); 
    return VisAbstractMatrix::getAbsPosition(x1,y1,x2,y2); }

} *VisTablePtr;




class VisTerminal: public VisRegion2
{
public:
  // Optionen
  VisIntVector size;
  VisIntVector alignmentPoint;
  VLCharPtr figure;
  VLCharPtr figureStyle;  
  VisBorderDimensions borderDimensions;

  VisTerminal(NODEPTR x) : VisRegion2(x) { setDefaultValues(); }
  void setDefaultValues(void);
  VisContainerInterface2Ptr calcSize(void);
  void draw(void);
};


typedef VisTerminal *VisTerminalPtr;


#endif
