
#ifndef VISBASICCODE_H
#define VISBASICCODE_H
#include "ptg_gen.h"
#include "PtgCommon.h"
#include "BasicCS.h"


#include "BasicObjects.h"
#include "BasicTypes.h"
#include <assert.h>
#include <math.h>
#include "ptg_gen.h"
#include "PtgOutFPtr.h"
#include "assert.h"

#define NEW(x) (new x)
#define SETOPT(obj,opt,val) obj->opt=val
#define ASSIGN(a,b) a=b

#define MAX(a,b) ((a)>=(b) ? (a) : (b))

#include "tcl.h"
extern Tcl_Interp *TclInterp;


inline int roundToInt(double d) { return (int)rint(d); }

inline void visEnlarge(int &a, int b) { if(a<b) a=b; }
inline void visReduce(int &a, int b) { if(a>b) a=b; }

template <class T>
inline T max(const T &a, const T&b) { return a>=b ? a : b; }

template <class T>
inline T min(const T &a, const T&b) { return a<=b ? a : b; }




/////////////////////////////////////////////////////////////// VisDirection

enum VisDirection { visEast=0, visSouth, visWest, visNorth };
inline bool xDirection(VisDirection d) { return !(d & 1); }
inline bool yDirection(VisDirection d) { return (d & 1); }
inline bool negDirection(VisDirection d) { return d>=visWest; }
inline bool posDirection(VisDirection d) { return d<visWest; }
inline int signDirection(VisDirection d, int i) { return d<visWest ? i : -i; }


////////////////////////////////////////////////// VisIntVector, VisFloatVector




class VisFloatVector
{
public:
  float x, y;

  //Konstruktoren
  VisFloatVector(void) {}
  VisFloatVector(float  aX, float aY) { x=aX; y=aY; }
  VisFloatVector(float aX, float aY, VisDirection d) 
    { if(yDirection(d)) { x=aX; y=aY; } else { x=aY; y=aX; }}
  VisFloatVector(VLPointArrayPtr p) { x=p->getX(); y=p->getY(); }
  VisFloatVector(VLPointArray a, int pos=0) { x=a.getX(pos); y=a.getY(pos); }
  VisFloatVector(VLPointArray a, float defx, float defy) { x=a.getX(0,(int)defx); y=a.getY(0,(int)defy); }

  void setX(float aX) { x=aX; }
  void setY(float aY) { y=aY; }
  void setXY(float aX, float aY) { x=aX; y=aY; }
  float &c1(VisDirection d) { return yDirection(d) ? x : y; }
  float &c2(VisDirection d) { return yDirection(d) ? y : x; }
  //void swap(void) { float buf=x; x=y; y=buf; }

  void print(void) { printf("(%f,%f)\n",x,y); }
  void print(char *s) { printf("%s = (%f,%f)\n",s,x,y); }
  operator VLPointArrayPtr(void) { return VLPOINT((int)x,(int)y); }
  void operator =(VLPointArrayPtr p) { x=p->getX(); y=p->getY(); }
  void move(float dx, float dy) { x+=dx; y+=dy; }
  void move(VisFloatVector d) { x+=d.x; y+=d.y; }

  double length() { return sqrt(double(x)*x+double(y)*y); }
  VisFloatVector norm() { double fak=1/length(); return VisFloatVector(fak*x,fak*y); }
  VisFloatVector rightnormal() const { return VisFloatVector(y,-x).norm(); }
};


inline float scalprod(VisFloatVector a, VisFloatVector b)
{
  return a.x*b.x + a.y*b.y;
}

inline VisFloatVector operator -(VisFloatVector a)
{
  return VisFloatVector(-a.x, -a.y);
}


inline VisFloatVector operator /(VisFloatVector a, float b)
{
  return VisFloatVector(a.x/b, a.y/b);
}


inline VisFloatVector operator *(float fak, VisFloatVector v)
{
  return VisFloatVector(fak*v.x, fak*v.y);
}

inline VisFloatVector operator+(VisFloatVector a, VisFloatVector b)
{
  return VisFloatVector(a.x+b.x, a.y+b.y);
}

inline VisFloatVector operator-(VisFloatVector a, VisFloatVector b)
{
  return VisFloatVector(a.x-b.x, a.y-b.y);
}

typedef VisFloatVector *VisFloatVectorPtr;




class VisIntVector
{
public:
  int x, y;

  //Konstruktoren
  VisIntVector(void) {}
  VisIntVector(int  aX, int aY) { x=aX; y=aY; }
  VisIntVector(int aX, int aY, VisDirection d) 
    { if(yDirection(d)) { x=aX; y=aY; } else { x=aY; y=aX; }}
  VisIntVector(VLPointArrayPtr p) { x=p->getX(); y=p->getY(); }
  VisIntVector(VLPointArray a, int pos=0) { x=a.getX(pos); y=a.getY(pos); }
  VisIntVector(VLPointArray a, int defx, int defy) { x=a.getX(0,(int)defx); y=a.getY(0,(int)defy); }

  void setX(int aX) { x=aX; }
  void setY(int aY) { y=aY; }
  void setXY(int aX, int aY) { x=aX; y=aY; }
  int &c1(VisDirection d) { return yDirection(d) ? x : y; }
  int &c2(VisDirection d) { return yDirection(d) ? y : x; }
  //void swap(void) { int buf=x; x=y; y=buf; }

  void print(void) { printf("(%i,%i)\n",x,y); }
  void print(char *s) { printf("%s = (%i,%i)\n",s,x,y); }
  operator VLPointArrayPtr(void) { return VLPOINT((int)x,(int)y); }
  void operator =(VLPointArrayPtr p) { x=p->getX(); y=p->getY(); }
  void move(int dx, int dy) { x+=dx; y+=dy; }
  void move(VisIntVector d) { x+=d.x; y+=d.y; }

  double length() { return sqrt(double(x)*x+double(y)*y); }
  VisFloatVector norm() { double fak=1/length(); return VisFloatVector(fak*x,fak*y); }
  VisFloatVector rightnormal() const { return VisFloatVector(y,-x).norm(); }
};


inline int scalprod(VisIntVector a, VisIntVector b)
{
  return a.x*b.x + a.y*b.y;
}

inline VisIntVector operator -(VisIntVector a)
{
  return VisIntVector(-a.x, -a.y);
}


inline VisIntVector operator /(VisIntVector a, int b)
{
  return VisIntVector(a.x/b, a.y/b);
}


inline VisIntVector operator *(int fak, VisIntVector v)
{
  return VisIntVector(fak*v.x, fak*v.y);
}

inline VisIntVector operator+(VisIntVector a, VisIntVector b)
{
  return VisIntVector(a.x+b.x, a.y+b.y);
}

inline VisIntVector operator-(VisIntVector a, VisIntVector b)
{
  return VisIntVector(a.x-b.x, a.y-b.y);
}

typedef VisIntVector *VisIntVectorPtr;




/////////////////////////////////////////////////////////////// VisIntInterval
/*
class VisIntInterval
{
public:
  int left, right;
  VisIntInterval(void) {}
  VisIntInterval(int aLeft, int aRight) { left=aLeft; right=aRight; }
};
*/
  
/////////////////////////////////////////////////////////////// VisBorderDimensions

class VisBorderDimensions
{
public:
  int left, right, top, bottom;
  VisBorderDimensions(void) { left=0; right=0; top=0; bottom=0; }
  VisBorderDimensions(int b) { left=b; right=b; top=b; bottom=b; }
  VisBorderDimensions(int x, int y) { left=x; right=x; top=y; bottom=y; }
  VisBorderDimensions(int l,int r,int t, int b) { left=l; right=r; top=t; bottom=b; }
  int xSum(void) { return left+right; }
  int ySum(void) { return top+bottom; }
};


/////////////////////////////////////////////////////////// VisAlignmentAdvice

enum VisAlignmentAdvice 
{ 
  VisAlignLeft=-1,  // nicht aendern!
  VisAlignTop=-1,   // nicht aendern!
  VisAlignCenter=0, // nicht aendern!
  VisAlignRight=1,  // nicht aendern!
  VisAlignBottom=1, // nicht aendern!
  VisAlignScale=2,
  VisAlignBuoy=3
};

class VisAlignmentDefinition
{
public:
  VisAlignmentAdvice type;
  int value;
  VisAlignmentDefinition(void) { type=VisAlignCenter; value=0; }
  VisAlignmentDefinition(VisAlignmentAdvice t) { type=t; assert(t!=VisAlignBuoy); value=0; }
  VisAlignmentDefinition(VisAlignmentAdvice t, int v) { type=t; value=v; }
};

inline VisAlignmentDefinition visAlignBuoy(int v) 
{ 
  return VisAlignmentDefinition(VisAlignBuoy, v); 
}

/////////////////////////////////////////////////////////////// VisConstraintVariable

/* by cs: Ist nur eine Huelle fuer VLPoint-Zeiger, die Hilfsfunktionen zur Generierung
   von Constraint-Strings implementiert. Es werden spezielle Namenskonventionen verwendet,
   auf denen die Auswertung der Constraintsolver-Ergebnisse basiert! */

class VisConstraintVariable
  {
 public:
  void *point;
  PTGNode nameX,nameY;

  VisConstraintVariable(void)
    { setVariable(VLPOINT(30,30)); }
  
  VisConstraintVariable(void *p)
    { 
    setVariable(p); 
    }
  
  void setVariable(void *p)
    {
    point = p;
    nameX = PTGC_Name_VLPointX(p);
    nameY = PTGC_Name_VLPointY(p);
    }
    
  int getValueX(void) { assert(point); return ((VLPointArrayPtr)point)->getX(); }
  int getValueY(void) { assert(point); return ((VLPointArrayPtr)point)->getY(); }

  /* by cs: */
  PTGNode setValue(VisIntVector p)
    {
    assert(point);
    ((VLPointArrayPtr)point)->setXY(0,p.x,p.y);
    return PTGC_Set_VLPoint(point);
    }


  PTGNode setValue(void)
    {
    assert(point);
    return PTGC_Set_VLPoint(point);
    }


  PTGNode getID_X(void) { return getID('x'); } 
  PTGNode getID_Y(void) { return getID('y'); } 

  PTGNode getID(char c) 
    {
    assert(point);
    if(c=='x')      return nameX;
    else if(c=='y') return nameY;
    else            return PTGC_HelpName(PTGC_Name_VLPoint(point),c);
    }

  PTGNode getCommonID(char c, void *point2)
    {
    assert(point);
    return PTGC_HelpDoubleName(PTGC_Name_VLPoint(point), 
                               PTGC_Name_VLPoint(point2),c);
    }
  };

typedef VisConstraintVariable *VisConstraintVariablePtr;


/////////////////////////////////////////////////////////////// VisContainerInterface

class VisContainerInterface1;
class VisContainerInterface2;

class VisContainerInterface
{
public:
  // Abhaengigkeit: INCLUDING ContainerOwner.ContainerPositionsDefined
  int depth; // Tiefe, in der Container-Inhalt gezeichnet werden soll
  NODEPTR nodePtr;
  VisIntVector position; 
     // linke obere Ecke des Containtes

  const int interfaceType; // 1 oder 2
  VisContainerInterface(int type) : interfaceType(type) {}
  VisContainerInterface1 *toType1(void) 
  { assert(interfaceType==1); return (VisContainerInterface1*)this; }
  VisContainerInterface2 *toType2(void) 
  { assert(interfaceType==2); return (VisContainerInterface2*)this; }
  virtual void topElement(void)=0;
  virtual int getXPosition(void)=0;
  virtual int getYPosition(void)=0;
  virtual int getXSize(void)=0;
  virtual int getYSize(void)=0;
  virtual VLPointArrayPtr getRegion(void)=0;

};



class VisContainerInterface1: public VisContainerInterface
  {
 public:
  // Abhaengigkeit: keine
  VisConstraintVariable size;

  VisContainerInterface1() : VisContainerInterface(1) {}
  virtual void topElement(void) { position=VisIntVector(10,10); }

  int getXSize(void) { return size.getValueX(); }
  int getYSize(void) { return size.getValueY(); }
  int getXPosition(void) { return position.x; }
  int getYPosition(void) { return position.y; }

  VLPointArrayPtr getRegion(void) 
    { 
    return VLREGION(getXPosition(),getYPosition(),getXPosition()+getXSize(),getYPosition()+getYSize());
    }
  };

typedef VisContainerInterface1 *VisContainerInterface1Ptr;



class VisContainerInterface2: public VisContainerInterface
{
public:
  // Abhaengigkeit folgender Attr.: keine
  VisIntVector preferedSize;    
     // Groesse, die mindestens erforderlich ist
  VisIntVector alignmentPoint; 
     // Position innerhalb des geforderten Bereichs, die evtl. zur Ausrichtung des
     // Objekts benutzt wird. alignmentPoint.? wird evtl. ueberschrieben, wenn 
     // align.? ungleich VisAlignBuoy ist.

  // Abhaengigkeit folgender Attr.: INCLUDING ContainerOwner.ContainerPositionsDefined
  VisIntVector size; 
     // Tatsaechliche Groesse des Containers. Es wird immer der gesamte zur
     // Verfuegung stehen Platz angegeben, damit sich das Objekt evtl. vergroessern kann
  VisIntVector buoy; 
     // eine relative Position innerhalb des Containers, die zur Ausrichtung dient.
     // buoy.? ist nur dann definiert, wenn der entsprechende Wert 
     // align.? gleich VisAlignBuoy ist.
  VisIntVector align;
     // Vorschlag fuer die Ausrichtung innerhalb des Containers.
     // Die beiden int's sind Element von VisAlignmentAdvice.
     // Ist align.X gleich VisAlignBuoy, dann kann das Objekt innerhalb des
     // Containers in X-Richtung so positioniert werden, dass alignmentPoint.X
     // auf buoy.X zu liegen kommt.

  VisContainerInterface2() : VisContainerInterface(2) {}
  VisContainerInterface2(VisIntVector aPreferedSize, NODEPTR aNodePtr) : VisContainerInterface(2) 
  { preferedSize=aPreferedSize; alignmentPoint=VisIntVector(0,0); nodePtr=aNodePtr; }

  /* void setDefaults(void) { align=VisIntVector(VisAlignCenter,VisAlignCenter); } */

  void xFit(void);
  void yFit(void);
    // verkleinert den Container in X-Richtung auf die bevorzugte Groesse, wobei
    // der Wert align.X beruecksichtigt wird. Dabei wird auch der Wert
    // buoy.X validiert.
  void addBorder(VisBorderDimensions b);
    // vergroessert den geforderten Bereich um einen zusaetzlichen Rand
  void subBorder(VisBorderDimensions b);
    // bereinigt den Container von dem zusaetzlich geforderten Rand

  void topElement(void) 
    { size=preferedSize; 
      position=VisIntVector(10,10); 
      align=VisIntVector(0,0); depth=1; 
    }
  
  int getXPosition(void) { return position.x; }
  int getYPosition(void) { return position.y; }
  int getXSize(void)     { return preferedSize.x; /*size.x;*/ }
  int getYSize(void)     { return preferedSize.y; }

  VLPointArrayPtr getRegion(void) 
  { 
    return VLREGION(position.x,position.y,position.x+size.x,position.y+size.y);
  }

  void print(void)
  {
    printf("x: %i, y:%i\n",position.x, position.y);
  }
};

typedef VisContainerInterface *VisContainerInterfacePtr;
typedef VisContainerInterface2 *VisContainerInterface2Ptr;

inline void VisContainerInterface2::xFit(void)
{
  switch(align.x) {
    case -1: 
      break;
    case VisAlignScale:
    case 0: 
      position.x+=(size.x-preferedSize.x) / 2;
      break;
    case 1: 
      position.x+=size.x-preferedSize.x;
      break;
    case VisAlignBuoy: 
      position.x+=buoy.x-alignmentPoint.x;
      break;
    default: 
      assert(0);
  }
  size.x=preferedSize.x;
  buoy.x=alignmentPoint.x;
}

inline void VisContainerInterface2::yFit(void)
{ // analog zu xFit!
  switch(align.y) {
    case -1: 
      break;
    case VisAlignScale:
    case 0: 
      position.y+=(size.y-preferedSize.y) / 2;
      break;
    case 1: 
      position.y+=size.y-preferedSize.y;
      break;
    case VisAlignBuoy: 
      position.y+=buoy.y-alignmentPoint.y;
      break;
    default: 
      assert(0);
  }
  size.y=preferedSize.y;
  buoy.y=alignmentPoint.y;
}


inline void VisContainerInterface2::addBorder(VisBorderDimensions b)
{
      preferedSize.x+=b.xSum();
      preferedSize.y+=b.ySum();
      alignmentPoint.x+=b.left;
      alignmentPoint.y+=b.top;
}

inline void VisContainerInterface2::subBorder(VisBorderDimensions b)
{
      preferedSize.x-=b.xSum();
      preferedSize.y-=b.ySum();
      alignmentPoint.x-=b.left;
      alignmentPoint.y-=b.top;
      position.x+=b.left;
      position.y+=b.top;
      size.x-=b.xSum();
      size.y-=b.ySum();
      buoy.x-=b.left;
      buoy.y-=b.top;
}


////////////////////////////////////////////////////////// VisContainerPosition2

class VisContainerPosition2
{
public:
  VisContainerPosition2(VisContainerInterfacePtr aElement, char *aContainerName)
    : element((VisContainerInterface2 *)aElement), containerName(aContainerName) {}
  VisContainerInterface2 *element;
  char *containerName;
};

typedef VisContainerPosition2 *VisContainerPosition2Ptr;

///////////////////////////////////////////////////////////////////// VisObject


class VisObject
{
public:
  NODEPTR nodePtr;
  VisObject(NODEPTR x) { nodePtr=x; }
};


/////////////////////////////////////////////////////////////////// fuer Listen
class   VisRelationType;
typedef VisRelationType *VisRelationTypePtr; 

class   VisRelationLabel;
typedef VisRelationLabel *VisRelationLabelPtr; 

class   VisRelationTypeReference;
typedef VisRelationTypeReference *VisRelationTypeReferencePtr; 

class   VisMovable;
typedef VisMovable *VisMovablePtr; 

class   VisCrossArea;
typedef VisCrossArea *VisCrossAreaPtr;


class VisSolver
  {
 public:
  void solve(PTGNode c);
  };

typedef VisSolver *VisSolverPtr;

  void VisRoundedRectangle(VLPointArrayPtr,int,int,int,int,int);


VisFloatVector linecoord2abscoord(VisFloatVector linestart, VisFloatVector lineend, 
   VisFloatVector boxsize, VisFloatVector relpos);

VisFloatVector linecoord2abscoord(VisFloatVector linestart, VisFloatVector lineend, VisFloatVector relpos);


#endif
