
#ifndef _VisComplexElementPtrLIST_H
#define _VisComplexElementPtrLIST_H

/* include header file defining VisComplexElementPtr if VisComplexElement is set: */
#define EMPTYVisComplexElementHDR
#ifndef EMPTYHDR
#include "VisComplexElement.h"
#endif
#undef EMPTYVisComplexElementHDR

#include "VisComplexElementPtrFunctionTypes.h"

typedef struct _VisComplexElementPtrLE *VisComplexElementPtrList;
typedef VisComplexElementPtrList    *VisComplexElementPtrListPtr;

struct _VisComplexElementPtrLE {
VisComplexElementPtr            head;
VisComplexElementPtrList        tail;
};

#define NULLVisComplexElementPtrList            ((VisComplexElementPtrList)0)
#define NullVisComplexElementPtrList()  ((VisComplexElementPtrList)0)
#define SingleVisComplexElementPtrList(e)       (ConsVisComplexElementPtrList((e),NULLVisComplexElementPtrList))

#if defined(__STDC__) || defined(__cplusplus)

extern void FinlVisComplexElementPtrList(void);
   /* Deallocates all VisComplexElementPtrLists. */
   /* Dynamic storage is allocated on first need */

extern VisComplexElementPtrList ConsVisComplexElementPtrList (VisComplexElementPtr e, VisComplexElementPtrList l);
   /* Constructs a VisComplexElementPtrList of an element e and a given tail l.
    * e is the first element of the list. */

extern VisComplexElementPtr HeadVisComplexElementPtrList (VisComplexElementPtrList l);
   /* Returns the first element of the list l.
    * The list l must not be empty. */

extern VisComplexElementPtrList TailVisComplexElementPtrList (VisComplexElementPtrList l);
   /* Returns the tail of the list l.
    * If l is empty, an empty list is returned. */

extern int LengthVisComplexElementPtrList (VisComplexElementPtrList l);
   /* Returns the number of elements in the list l. */

extern VisComplexElementPtr IthElemVisComplexElementPtrList (VisComplexElementPtrList l, int i);
   /* Returns the i-th element of the List l. The head of l
    * is referred to as 1. If the value of
    * i is greater than the length of the list, an error
    * is reported and the program exits.
    */

extern VisComplexElementPtrList CopyVisComplexElementPtrList (VisComplexElementPtrList l, VisComplexElementPtrMapFct cp);
   /* Copies the list l. Elements are copied by calls of cp. */

extern VisComplexElementPtrList AppVisComplexElementPtrList (VisComplexElementPtrList l1, VisComplexElementPtrList l2);
   /* Concatenates two lists l1 and l2. The resulting list contains l2 at
    * the end of a copy of list l1. */

extern VisComplexElementPtrList AppElVisComplexElementPtrList (VisComplexElementPtrList l, VisComplexElementPtr e);
   /* Appends an element e to the list l.
    * The list l is not copied, it is modified as a side-effect
    * of this function. */

extern void InsertAfterVisComplexElementPtrList (VisComplexElementPtrList l, VisComplexElementPtr e);
   /* This function requires a non-empty list l. The element e is inserted
    * just after the first element of l. 
    * The list l is modified as a side-effect of this function. */

extern VisComplexElementPtrList OrderedInsertVisComplexElementPtrList (VisComplexElementPtrList l, VisComplexElementPtr e, VisComplexElementPtrCmpFctType fcmp);
   /* Inserts the element e into the list l maintaining l in
    * ascending order with respect to the compare fcmp. 
    * The list l is modified as a side-effect of this function. */

extern VisComplexElementPtrListPtr RefEndConsVisComplexElementPtrList (VisComplexElementPtrListPtr addr, VisComplexElementPtr e);
   /* Appends an element e to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern VisComplexElementPtrListPtr RefEndAppVisComplexElementPtrList (VisComplexElementPtrListPtr addr, VisComplexElementPtrList l);
   /* Appends a list l to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern int ElemInVisComplexElementPtrList (VisComplexElementPtr e, VisComplexElementPtrList l, VisComplexElementPtrCmpFctType cmpfct);
   /* This function returns true (1) iff the element e is in the List l.
    * List elements are compared by the function cmpfct. */

extern VisComplexElementPtrList AddToSetVisComplexElementPtrList (VisComplexElementPtr e, VisComplexElementPtrList l, VisComplexElementPtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. */

extern VisComplexElementPtrList AddToOrderedSetVisComplexElementPtrList 
       (VisComplexElementPtr e, VisComplexElementPtrList l, VisComplexElementPtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. l is assumed to be ordered increasingly in the sense of
    * cmpfct. */

extern VisComplexElementPtrList MapVisComplexElementPtrList (VisComplexElementPtrList l, VisComplexElementPtrMapFct f);
   /* Returns a new VisComplexElementPtrList obtained by applying f to each element of l. */

extern int CompVisComplexElementPtrList (VisComplexElementPtrList l1, VisComplexElementPtrList l2, VisComplexElementPtrCmpFctType f);
   /* Compares the lists l1 and l2 lexicographically by applying f
    * to the corresponding elements. */

extern VisComplexElementPtr SumVisComplexElementPtrList (VisComplexElementPtrList l, VisComplexElementPtrSumFct f, VisComplexElementPtr a);
   /* Applies the binary function f to the elements of the List:
    *   f( f(... f(a, e1), e2, ...), en)
    * If l is empty a is returned. */

#else

extern void FinlVisComplexElementPtrList ();
extern VisComplexElementPtrList ConsVisComplexElementPtrList ();
extern VisComplexElementPtr HeadVisComplexElementPtrList ();
extern VisComplexElementPtrList TailVisComplexElementPtrList ();
extern int LengthVisComplexElementPtrList ();
extern VisComplexElementPtr IthElemVisComplexElementPtrList ();
extern VisComplexElementPtrList CopyVisComplexElementPtrList ();
extern VisComplexElementPtrList AppVisComplexElementPtrList ();
extern VisComplexElementPtrList AppElVisComplexElementPtrList ();
extern void InsertAfterVisComplexElementPtrList ();
extern VisComplexElementPtrList OrderedInsertVisComplexElementPtrList ();
extern VisComplexElementPtrListPtr RefEndConsVisComplexElementPtrList ();
extern VisComplexElementPtrListPtr RefEndAppVisComplexElementPtrList ();
extern int ElemInVisComplexElementPtrList ();
extern VisComplexElementPtrList AddToSetVisComplexElementPtrList ();
extern VisComplexElementPtrList MapVisComplexElementPtrList ();
extern int CompVisComplexElementPtrList ();
extern VisComplexElementPtr SumVisComplexElementPtrList ();

#endif

#ifdef MONITOR
#define DAPTO_RESULTVisComplexElementPtrList(l) DAPTO_RESULT_PTR(l)
#define DAPTO_ARGVisComplexElementPtrList(l)    DAPTO_ARG_PTR (l, VisComplexElementPtrList)
#endif

#endif
