
#ifndef VISCONNECTIONPATTERNCODE_H
#define VISCONNECTIONPATTERNCODE_H
#include <limits.h>
#include "VisBasicCode.h"
#include "VisBasicCode.head"

 
// VisRelationLabel
//////////////////////////////////////////////////////////////////////
// Klasse mit Daten fuer die Labels von Relationen

class VisRelationLabel{
 public:
  VLListPtr textPtr;                  // Labeltext
  VLCharPtr textAttr;                 // Selbsteditierend ?
  int       x,y;                      // Labelposition
  int       ox,oy;                    // Labelposition-Offset
  NODEPTR   nodePtr;                  // Zeiger auf diesen Baumknoten

  VisRelationLabel(NODEPTR,VLListPtr,VLCharPtr,int,int,int,int);
  void paint(int,int);                // Zeichenfunktion  
  };

typedef VisRelationLabel *VisRelationLabelPtr;
 
// VisEndPointLabelsObject
//////////////////////////////////////////////////////////////////////
// Klasse die alle Labels an einem Endpunkt verwaltet und zeichnet

class VisEndPointLabelsObject{
 public:                              // Zeiger auf Relationslisten
  VisRelationTypeReferencePtrList fromRels, toRels;
  int borderdXY;                      // Welche Ausdehnung ist begrenzt? 0=x 1=y
  int maxSize;                        // maximale Groessen (Beschraenkung)
  int sizeX,sizeY;                    // Ausdehnung aller Referenzen
  int optimalSize;                    // Optimale Groesse (1=ja)
  int count;                          // Anzahl der Objekte
  int maxX, maxY;                     // maximale Groessen (eines Objekts)
  int sumX, sumY;                     // Summe der Groessen (aller Objekte)

  VisEndPointLabelsObject(VisRelationTypeReferencePtrList,
                          VisRelationTypeReferencePtrList,int,int,int);
  void calcLabelSize(void);           // Groesse aller Labels zusammen berechnen
  int  getLabelSizeX(void);           // Groesse ausgeben
  int  getLabelSizeY(void);
  void paint(int,int);                // Zeichenfunktion  
  };

typedef VisEndPointLabelsObject *VisEndPointLabelsObjectPtr;
 
// VisCrossArea 
//////////////////////////////////////////////////////////////////////
// Klasse mit Daten fuer Querbereiche (orthogonale Linien)
// Enthaelt auch Hilfsfunktionen fuer die Verwaltung der hier verlaufenden
// Linien


class VisRelationTypeOrthogonal;
typedef VisRelationTypeOrthogonal *VisRelationTypeOrthogonalPtr; 

class VisCrossAreaLineDef{
 public:
  int row;                             // Ebene der n-ten Linie
  VisRelationTypeOrthogonalPtr linePtr;// Zeiger auf Linienklasse
  int pointOffset;                     // Punkteoffset
  int exitDirections[2];               // Ausgangsrichtungen der Endpunkte
  int ex;                              // Start und Ende vertauscht?
  int fromTo[2];                       // Start- und Endkoordinaten
  };

typedef VisCrossAreaLineDef *VisCrossAreaLineDefPtr;


class VisCrossArea
  {
 public:
  VisContainerInterfacePtr container;  // Zeiger auf Rahmen des Objekts
  int direction;                       // Ausbreitungsrichtung der Querlinien
  int distance;                        // Abstand zwischen den einzelnen Linien
                                       // -1 = passend

  VisCrossAreaLineDefPtr lines[100];   // Zeiger auf Linien
  int count;                           // Anzahl der hier laufenden Linien

  VisCrossArea(VisContainerInterfacePtr c,int dir,int dis);
  void addLine(VisRelationTypeOrthogonalPtr,int,int,int);
  void calcConnections(void);          // Verbindungen der Linien berechnen
  };
  

typedef VisCrossArea *VisCrossAreaPtr;
 

// VisRelationType 
//////////////////////////////////////////////////////////////////////
// Virtuelle Klasse zum Malen 

class VisRelationTypeDirect;
class VisRelationTypeOrthogonal;
class VisRelationTypePolyline;
class VisRelationTypeReference;

class VisRelationType:public VisObject 
  { 
 public: 
  VisIntVectorPtr points[2];          // Quell- und Zielposition (abs. Koordinaten)
  VLPointArrayPtr coordsPtr;          // Zeiger auf Speicher der Koordinaten
  int objShape[2];                    // Form der Objekte der Endpunkte
  int offset[2][2];                   // rel. Offset (fuer ToPoint-Linien)
  int objKind[2];                     // Art der Einfuegestellen (Form und evt. nur der Rand)
  int direction[2];                   // Ausgangsrichtungen (fuer orthogonale Linien)
  int toTheEnd[2];                    // Soll die Linie bis zum Ende gehen ?
  int target[2];                      // Wohin Zielt die Linie ? 
                                      // 0=Mittelpunkt, 1=andere Pos, 2=relative Position
  VLPointArrayPtr area[4];            // Ausmasse der Objekte bei den Zielpunkten
  VisRelationLabelPtrList labels;     // Liste aller Labels

  VisRelationType(NODEPTR x,int type) : VisObject(x), relationType(type) 
    { 
    points[0]=new VisIntVector; 
    points[1]=new VisIntVector;
    }

  virtual ~VisRelationType() {}


  void virtual draw(VisCrossAreaPtrList)=0; 
  void setValues(VLPointArrayPtr,int,int,int,int,int,int,int,int,int,
                 int,int,int,int,int,VLPointArrayPtr ,VLPointArrayPtr,VisRelationLabelPtrList);
  void calcRealTarget(void);
  void setCoordValues(int,int,int,int);


  const int relationType;              // 1, 2, 3 oder 4

  VisRelationTypeDirect     *toDirect(void)     { return (VisRelationTypeDirect *)this;     }
  VisRelationTypeOrthogonal *toOrthogonal(void) { return (VisRelationTypeOrthogonal *)this; }
  VisRelationTypePolyline   *toPolyline(void)   { return (VisRelationTypePolyline *)this;   }
  VisRelationTypeReference  *toReference(void)  { return (VisRelationTypeReference *)this;  }
  }; 
 
typedef VisRelationType *VisRelationTypePtr; 
 
 

class VisRelationTypeDirect:public VisRelationType 
  { 
 public: 
  VisRelationTypeDirect(NODEPTR x) : VisRelationType(x,1)  
    { setDefaultValues(); }
 
  // Optionen 
  VLCharPtr lineType; 
  VLCharPtr arrowType; 
   
  void draw(VisCrossAreaPtrList);   
  void setDefaultValues(void); 
  }; 
 
typedef VisRelationTypeDirect *VisRelationTypeDirectPtr; 
 
 

class VisRelationTypeDirect2:public VisRelationType 
  { 
 public: 
  VisRelationTypeDirect2(NODEPTR x) : VisRelationType(x,1)  
    { setDefaultValues(); }
 
  // Optionen 
  VLCharPtr lineType; 
  VLCharPtr arrowType; 

  // Kopien von Lido-Attributen:
  VLPointArrayPtr intermediatePoints;  
  VLPointArrayPtr fromDif, toDif;
  VLCharPtr intermediatePointsAttrName;

  void calcEndpoint(int i, VisIntVector &target, VisIntVector startPoint);
  void draw(VisCrossAreaPtrList); 
  void setDefaultValues(void); 
  }; 
 
typedef VisRelationTypeDirect2 *VisRelationTypeDirect2Ptr; 
 
 
// VisRelationTypeOrthogonal
///////////////////////////////////////////////////////////////////// 
// eine orthogonale Linie 

class VisRelationTypeOrthogonal:public VisRelationType 
  { 
 public: 
  VisCrossAreaPtr areas[3];                 // Bereiche (rechts,mitte,links)
  int minDist[3];                           // Minimale Abstaende zu den Punkten
  VLPointArray p;                           // Ziellinie
  int ldirection;                           // Linienrichtung   
  int ex;                                   // Sind Start und Ziel vertauscht ?

  // Optionen 
  VLCharPtr lineType;                       // Linientyp
  VLCharPtr arrowType;                      // Pfeiltyp
  int simplify;                             // Vereinfachen der Linien ?
  int reDistance;                           // Linienlaenge in Ausgangsrichtung
 
  VisRelationTypeOrthogonal(NODEPTR x) : VisRelationType(x,2)  
    { setDefaultValues(); }
 
  void setXY(int,VLPointArray *,int,int,int,int,int,int);
  int dif(int,int);
  void draw(VisCrossAreaPtrList);           // Das malen (nur) vorbereiten
  void drawReady(void);                     // Entgueltig malen
  void setDefaultValues(void); 

  }; 
 
typedef VisRelationTypeOrthogonal *VisRelationTypeOrthogonalPtr; 
 
 
 
// VisRelationTypePolyline
///////////////////////////////////////////////////////////////////// 
// eine Polygonlinie Linie 

class VisRelationTypePolyline:public VisRelationType 
  { 
 public: 
  // Optionen 
  VLCharPtr lineType;                       // Linientyp
  VLCharPtr arrowType;                      // Pfeiltyp
  int spline;                               // Splinelinie (1=ja)
 
  VisRelationTypePolyline(NODEPTR x) : VisRelationType(x,3)  
    { }
 
  void draw(VisCrossAreaPtrList);           // Das malen 
  }; 
 
typedef VisRelationTypePolyline *VisRelationTypePolylinePtr;
 
// VisRelationTypeReference
///////////////////////////////////////////////////////////////////////////
// Eine Referenz (Label oder Farben)

class VisRelationTypeReference: public VisRelationType
  { 
 public: 
  // Eigener Baumknoten
  NODEPTR nodePtr;

  // Groesse des Objekts
  VLPointArrayPtr sizePtr;
 
  // Groesse des Objekts
  VLPointArrayPtr frameSizePtr;
 
  // Erstellen dieses Objekts und uebergeben der Daten
  VisRelationTypeReference(
    NODEPTR         aNodePtr,
    VLPointArrayPtr aSize,
    VLPointArrayPtr aFrameSize)
    :VisRelationType(aNodePtr,4)
    {
    nodePtr      = aNodePtr;
    sizePtr      = aSize;
    frameSizePtr = aFrameSize;
    }

  // malen eines Endpunkts
  virtual void drawAtEndpoint(int,int,int)=0;

  // malen beider Endpunkte (falls nicht verbunden)
  virtual void draw(VisCrossAreaPtrList)=0;

  // berechnen der Ausdehnung einer Label
  int getXSize(void);
  int getYSize(void);
  }; 
 
typedef VisRelationTypeReference *VisRelationTypeReferencePtr; 
 
// VisRelationTypeLabelreference
///////////////////////////////////////////////////////////////////////////
// Eine Referenz (Label oder Farben)

class VisRelationTypeLabelreference: public VisRelationTypeReference
  { 
 public: 
  // Text und dessen Attribute
  VLListPtr finalAttr;
  
  VisRelationTypeLabelreference(
       NODEPTR aNode, VLPointArrayPtr as, VLPointArrayPtr afs, VLListPtr afAttr):
       VisRelationTypeReference(aNode,as,afs)
    {
    finalAttr=afAttr;
    }

  void draw(VisCrossAreaPtrList);
  void drawAtEndpoint(int,int,int);
  }; 
 
typedef VisRelationTypeLabelreference *VisRelationTypeLabelreferencePtr; 
 

// VisSimpleRelations
///////////////////////////////////////////////////////////////////////////
// Einfache Linienrelationen

class VisSimpleRelations:public VisObject 
  { 
 public: 
  // Liste alle zugehoerigen Linien
  VisRelationTypePtrList subRelations; 
 
  VisSimpleRelations(NODEPTR x): VisObject(x) {} 
  void setSubRelations(VisRelationTypePtrList x) { subRelations=x; } 
  void draw(VisCrossAreaPtrList); 
  }; 
 
typedef VisSimpleRelations *VisSimpleRelationsPtr; 


#endif
