
#ifndef VISCONSTRAINTPATTERNCODE_H
#define VISCONSTRAINTPATTERNCODE_H
#include "VisBasicCode.h"
#include "VisBasicCode.head"
#include "ptg_gen.h"
#include "PtgCommon.h"
#include "BasicCS.h"

 



// VisConnectedSets
///////////////////////////////////////////////////////////////////////////
// Mehrere Mengen sollen mit Constraints verbunden werden

class VisConnectedSets
  { 
 public:
  /* Berechnung der Constraints                                 */
  PTGNode calcConstraints(VisConstraintVariablePtrList list,int,int);
  };

typedef VisConnectedSets *VisConnectedSetsPtr;
 

// VisSet
///////////////////////////////////////////////////////////////////////////
// Einfache freie Menge

class VisSet:public VisObject
  { 
 public: 
  // Liste alle zugehoerigen Linien
  VisMovablePtrList subItems; 

  // Optionen fuer einen Rechteckigen Rahmen
  VLCharPtr frameStyle; 
  VLCharPtr frameShape;

  // Groesse des Objekts
  VisIntVector size;
  VisConstraintVariable csize;
  VisIntVector startSize;
 
  // Ausrichtung
  VisIntVector alignmentPoint;

  // Rahmengroesse
  VisBorderDimensions borderDimensions;

  // Zeiger auf das eigene Containerinterface
  VisContainerInterfacePtr containerInterfacePtr;

  // Form dieses Objekts
  // 0=rechteck    1=kreis
  int shape;

  // Art der Constaints fuer diese Klasse
  // 0 = nur kein Ueberlappen ueber den Rand dieses Objekts
  // 1 = Disjunkte Constraints mit Mindestabstand
  // 2 = Hoechstabstand
  int constraintKind;
  int distanceMin;
  int distanceMax;       

  // Die Constraints als Text
  PTGNode constraints;


  VisSet(NODEPTR x,int CKind,void *psize):VisObject(x)
    { 
    setDefaultValues(); 

    csize.setVariable(psize);
    if(CKind==2)
      containerInterfacePtr=new VisContainerInterface2();
    else
      {
      containerInterfacePtr=new VisContainerInterface1();
      containerInterfacePtr->toType1()->size.setVariable(psize);
      }
    containerInterfacePtr->nodePtr=x;
    } 

  void setDefaultValues();
  PTGNode calcConstraints(VisMovablePtrList x);
  VisContainerInterfacePtr calcSize(void);
  void drawFrame(VisIntVector p1, VisIntVector p2);
  void draw(void); 
  PTGNode getConstraints() { return constraints; }
  }; 
 
typedef VisSet *VisSetPtr; 
 



// VisMovable
///////////////////////////////////////////////////////////////////////////
// Ein Element, dessen relative Position durch den Benutzer und von
// dem Constraint-Solver festgelegt wird

class VisMovable:public VisObject
  { 
 public: 
  // Relative Position dieses Objekts
  VisConstraintVariable relPos;

  // Containter des zugehoerigen Objects
  VisContainerInterfacePtr containerInterfacePtr;

  // Form dieses Objekts
  // 0=rechteck    1=kreis
  int shape;

  VisMovable(NODEPTR x, VisContainerInterfacePtr c, 
              void *pos, int f): VisObject(x) 
    {  
    relPos.setVariable(pos);    
    containerInterfacePtr=c;
    shape=f;
    } 

  int getRelativePositionX(void) { return relPos.getValueX(); }
  int getRelativePositionY(void) { return relPos.getValueY(); }
  }; 
 
typedef VisMovable *VisMovablePtr; 

#endif
