
#ifndef _VisCrossAreaPtrLIST_H
#define _VisCrossAreaPtrLIST_H

/* include header file defining VisCrossAreaPtr if VisBasicCode is set: */
#define EMPTYVisBasicCodeHDR
#ifndef EMPTYHDR
#include "VisBasicCode.h"
#endif
#undef EMPTYVisBasicCodeHDR

#include "VisCrossAreaPtrFunctionTypes.h"

typedef struct _VisCrossAreaPtrLE *VisCrossAreaPtrList;
typedef VisCrossAreaPtrList    *VisCrossAreaPtrListPtr;

struct _VisCrossAreaPtrLE {
VisCrossAreaPtr         head;
VisCrossAreaPtrList     tail;
};

#define NULLVisCrossAreaPtrList         ((VisCrossAreaPtrList)0)
#define NullVisCrossAreaPtrList()       ((VisCrossAreaPtrList)0)
#define SingleVisCrossAreaPtrList(e)    (ConsVisCrossAreaPtrList((e),NULLVisCrossAreaPtrList))

#if defined(__STDC__) || defined(__cplusplus)

extern void FinlVisCrossAreaPtrList(void);
   /* Deallocates all VisCrossAreaPtrLists. */
   /* Dynamic storage is allocated on first need */

extern VisCrossAreaPtrList ConsVisCrossAreaPtrList (VisCrossAreaPtr e, VisCrossAreaPtrList l);
   /* Constructs a VisCrossAreaPtrList of an element e and a given tail l.
    * e is the first element of the list. */

extern VisCrossAreaPtr HeadVisCrossAreaPtrList (VisCrossAreaPtrList l);
   /* Returns the first element of the list l.
    * The list l must not be empty. */

extern VisCrossAreaPtrList TailVisCrossAreaPtrList (VisCrossAreaPtrList l);
   /* Returns the tail of the list l.
    * If l is empty, an empty list is returned. */

extern int LengthVisCrossAreaPtrList (VisCrossAreaPtrList l);
   /* Returns the number of elements in the list l. */

extern VisCrossAreaPtr IthElemVisCrossAreaPtrList (VisCrossAreaPtrList l, int i);
   /* Returns the i-th element of the List l. The head of l
    * is referred to as 1. If the value of
    * i is greater than the length of the list, an error
    * is reported and the program exits.
    */

extern VisCrossAreaPtrList CopyVisCrossAreaPtrList (VisCrossAreaPtrList l, VisCrossAreaPtrMapFct cp);
   /* Copies the list l. Elements are copied by calls of cp. */

extern VisCrossAreaPtrList AppVisCrossAreaPtrList (VisCrossAreaPtrList l1, VisCrossAreaPtrList l2);
   /* Concatenates two lists l1 and l2. The resulting list contains l2 at
    * the end of a copy of list l1. */

extern VisCrossAreaPtrList AppElVisCrossAreaPtrList (VisCrossAreaPtrList l, VisCrossAreaPtr e);
   /* Appends an element e to the list l.
    * The list l is not copied, it is modified as a side-effect
    * of this function. */

extern void InsertAfterVisCrossAreaPtrList (VisCrossAreaPtrList l, VisCrossAreaPtr e);
   /* This function requires a non-empty list l. The element e is inserted
    * just after the first element of l. 
    * The list l is modified as a side-effect of this function. */

extern VisCrossAreaPtrList OrderedInsertVisCrossAreaPtrList (VisCrossAreaPtrList l, VisCrossAreaPtr e, VisCrossAreaPtrCmpFctType fcmp);
   /* Inserts the element e into the list l maintaining l in
    * ascending order with respect to the compare fcmp. 
    * The list l is modified as a side-effect of this function. */

extern VisCrossAreaPtrListPtr RefEndConsVisCrossAreaPtrList (VisCrossAreaPtrListPtr addr, VisCrossAreaPtr e);
   /* Appends an element e to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern VisCrossAreaPtrListPtr RefEndAppVisCrossAreaPtrList (VisCrossAreaPtrListPtr addr, VisCrossAreaPtrList l);
   /* Appends a list l to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern int ElemInVisCrossAreaPtrList (VisCrossAreaPtr e, VisCrossAreaPtrList l, VisCrossAreaPtrCmpFctType cmpfct);
   /* This function returns true (1) iff the element e is in the List l.
    * List elements are compared by the function cmpfct. */

extern VisCrossAreaPtrList AddToSetVisCrossAreaPtrList (VisCrossAreaPtr e, VisCrossAreaPtrList l, VisCrossAreaPtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. */

extern VisCrossAreaPtrList AddToOrderedSetVisCrossAreaPtrList 
       (VisCrossAreaPtr e, VisCrossAreaPtrList l, VisCrossAreaPtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. l is assumed to be ordered increasingly in the sense of
    * cmpfct. */

extern VisCrossAreaPtrList MapVisCrossAreaPtrList (VisCrossAreaPtrList l, VisCrossAreaPtrMapFct f);
   /* Returns a new VisCrossAreaPtrList obtained by applying f to each element of l. */

extern int CompVisCrossAreaPtrList (VisCrossAreaPtrList l1, VisCrossAreaPtrList l2, VisCrossAreaPtrCmpFctType f);
   /* Compares the lists l1 and l2 lexicographically by applying f
    * to the corresponding elements. */

extern VisCrossAreaPtr SumVisCrossAreaPtrList (VisCrossAreaPtrList l, VisCrossAreaPtrSumFct f, VisCrossAreaPtr a);
   /* Applies the binary function f to the elements of the List:
    *   f( f(... f(a, e1), e2, ...), en)
    * If l is empty a is returned. */

#else

extern void FinlVisCrossAreaPtrList ();
extern VisCrossAreaPtrList ConsVisCrossAreaPtrList ();
extern VisCrossAreaPtr HeadVisCrossAreaPtrList ();
extern VisCrossAreaPtrList TailVisCrossAreaPtrList ();
extern int LengthVisCrossAreaPtrList ();
extern VisCrossAreaPtr IthElemVisCrossAreaPtrList ();
extern VisCrossAreaPtrList CopyVisCrossAreaPtrList ();
extern VisCrossAreaPtrList AppVisCrossAreaPtrList ();
extern VisCrossAreaPtrList AppElVisCrossAreaPtrList ();
extern void InsertAfterVisCrossAreaPtrList ();
extern VisCrossAreaPtrList OrderedInsertVisCrossAreaPtrList ();
extern VisCrossAreaPtrListPtr RefEndConsVisCrossAreaPtrList ();
extern VisCrossAreaPtrListPtr RefEndAppVisCrossAreaPtrList ();
extern int ElemInVisCrossAreaPtrList ();
extern VisCrossAreaPtrList AddToSetVisCrossAreaPtrList ();
extern VisCrossAreaPtrList MapVisCrossAreaPtrList ();
extern int CompVisCrossAreaPtrList ();
extern VisCrossAreaPtr SumVisCrossAreaPtrList ();

#endif

#ifdef MONITOR
#define DAPTO_RESULTVisCrossAreaPtrList(l) DAPTO_RESULT_PTR(l)
#define DAPTO_ARGVisCrossAreaPtrList(l)    DAPTO_ARG_PTR (l, VisCrossAreaPtrList)
#endif

#endif
