
#ifndef _VisMovablePtrLIST_H
#define _VisMovablePtrLIST_H

/* include header file defining VisMovablePtr if VisBasicCode is set: */
#define EMPTYVisBasicCodeHDR
#ifndef EMPTYHDR
#include "VisBasicCode.h"
#endif
#undef EMPTYVisBasicCodeHDR

#include "VisMovablePtrFunctionTypes.h"

typedef struct _VisMovablePtrLE *VisMovablePtrList;
typedef VisMovablePtrList    *VisMovablePtrListPtr;

struct _VisMovablePtrLE {
VisMovablePtr           head;
VisMovablePtrList       tail;
};

#define NULLVisMovablePtrList           ((VisMovablePtrList)0)
#define NullVisMovablePtrList() ((VisMovablePtrList)0)
#define SingleVisMovablePtrList(e)      (ConsVisMovablePtrList((e),NULLVisMovablePtrList))

#if defined(__STDC__) || defined(__cplusplus)

extern void FinlVisMovablePtrList(void);
   /* Deallocates all VisMovablePtrLists. */
   /* Dynamic storage is allocated on first need */

extern VisMovablePtrList ConsVisMovablePtrList (VisMovablePtr e, VisMovablePtrList l);
   /* Constructs a VisMovablePtrList of an element e and a given tail l.
    * e is the first element of the list. */

extern VisMovablePtr HeadVisMovablePtrList (VisMovablePtrList l);
   /* Returns the first element of the list l.
    * The list l must not be empty. */

extern VisMovablePtrList TailVisMovablePtrList (VisMovablePtrList l);
   /* Returns the tail of the list l.
    * If l is empty, an empty list is returned. */

extern int LengthVisMovablePtrList (VisMovablePtrList l);
   /* Returns the number of elements in the list l. */

extern VisMovablePtr IthElemVisMovablePtrList (VisMovablePtrList l, int i);
   /* Returns the i-th element of the List l. The head of l
    * is referred to as 1. If the value of
    * i is greater than the length of the list, an error
    * is reported and the program exits.
    */

extern VisMovablePtrList CopyVisMovablePtrList (VisMovablePtrList l, VisMovablePtrMapFct cp);
   /* Copies the list l. Elements are copied by calls of cp. */

extern VisMovablePtrList AppVisMovablePtrList (VisMovablePtrList l1, VisMovablePtrList l2);
   /* Concatenates two lists l1 and l2. The resulting list contains l2 at
    * the end of a copy of list l1. */

extern VisMovablePtrList AppElVisMovablePtrList (VisMovablePtrList l, VisMovablePtr e);
   /* Appends an element e to the list l.
    * The list l is not copied, it is modified as a side-effect
    * of this function. */

extern void InsertAfterVisMovablePtrList (VisMovablePtrList l, VisMovablePtr e);
   /* This function requires a non-empty list l. The element e is inserted
    * just after the first element of l. 
    * The list l is modified as a side-effect of this function. */

extern VisMovablePtrList OrderedInsertVisMovablePtrList (VisMovablePtrList l, VisMovablePtr e, VisMovablePtrCmpFctType fcmp);
   /* Inserts the element e into the list l maintaining l in
    * ascending order with respect to the compare fcmp. 
    * The list l is modified as a side-effect of this function. */

extern VisMovablePtrListPtr RefEndConsVisMovablePtrList (VisMovablePtrListPtr addr, VisMovablePtr e);
   /* Appends an element e to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern VisMovablePtrListPtr RefEndAppVisMovablePtrList (VisMovablePtrListPtr addr, VisMovablePtrList l);
   /* Appends a list l to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern int ElemInVisMovablePtrList (VisMovablePtr e, VisMovablePtrList l, VisMovablePtrCmpFctType cmpfct);
   /* This function returns true (1) iff the element e is in the List l.
    * List elements are compared by the function cmpfct. */

extern VisMovablePtrList AddToSetVisMovablePtrList (VisMovablePtr e, VisMovablePtrList l, VisMovablePtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. */

extern VisMovablePtrList AddToOrderedSetVisMovablePtrList 
       (VisMovablePtr e, VisMovablePtrList l, VisMovablePtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. l is assumed to be ordered increasingly in the sense of
    * cmpfct. */

extern VisMovablePtrList MapVisMovablePtrList (VisMovablePtrList l, VisMovablePtrMapFct f);
   /* Returns a new VisMovablePtrList obtained by applying f to each element of l. */

extern int CompVisMovablePtrList (VisMovablePtrList l1, VisMovablePtrList l2, VisMovablePtrCmpFctType f);
   /* Compares the lists l1 and l2 lexicographically by applying f
    * to the corresponding elements. */

extern VisMovablePtr SumVisMovablePtrList (VisMovablePtrList l, VisMovablePtrSumFct f, VisMovablePtr a);
   /* Applies the binary function f to the elements of the List:
    *   f( f(... f(a, e1), e2, ...), en)
    * If l is empty a is returned. */

#else

extern void FinlVisMovablePtrList ();
extern VisMovablePtrList ConsVisMovablePtrList ();
extern VisMovablePtr HeadVisMovablePtrList ();
extern VisMovablePtrList TailVisMovablePtrList ();
extern int LengthVisMovablePtrList ();
extern VisMovablePtr IthElemVisMovablePtrList ();
extern VisMovablePtrList CopyVisMovablePtrList ();
extern VisMovablePtrList AppVisMovablePtrList ();
extern VisMovablePtrList AppElVisMovablePtrList ();
extern void InsertAfterVisMovablePtrList ();
extern VisMovablePtrList OrderedInsertVisMovablePtrList ();
extern VisMovablePtrListPtr RefEndConsVisMovablePtrList ();
extern VisMovablePtrListPtr RefEndAppVisMovablePtrList ();
extern int ElemInVisMovablePtrList ();
extern VisMovablePtrList AddToSetVisMovablePtrList ();
extern VisMovablePtrList MapVisMovablePtrList ();
extern int CompVisMovablePtrList ();
extern VisMovablePtr SumVisMovablePtrList ();

#endif

#ifdef MONITOR
#define DAPTO_RESULTVisMovablePtrList(l) DAPTO_RESULT_PTR(l)
#define DAPTO_ARGVisMovablePtrList(l)    DAPTO_ARG_PTR (l, VisMovablePtrList)
#endif

#endif
