static char RCSid[] = "$Id: err.c,v 3.6 2000/08/08 11:31:03 mjung Exp $";
/* Copyright 1989, The Regents of the University of Colorado */

/* This file is part of the Eli Module Library.

The Eli Module Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The Eli Module Library is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the Eli Module Library; see the file COPYING.LIB.
If not, write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA.  */

/* As a special exception, when this file is copied by Eli into the
   directory resulting from a :source derivation, you may use that
   created file as a part of that directory without restriction. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "err.h"
#include "source.h"
#include "tcl.h"
#include "tcl_vpt.h"
#include "node.h"

	/* Variables exported by the Error Module */

int ErrorCount[] = {		/* Counts at each severity level */
  0, 0, 0, 0, 0
  };
int LineNum = 1;		/* Index of the current line
				   in the total source text */

int err_dummyline = 0;
int err_dummycol = 0;

// POSITION NoCoord = { (NODEPTR)0 }; /* The NULL coordinate */

POSITION curpos;		/* Position variable for general use */
jmp_buf on_deadly;              /* Where to jump on deadly errors. */
Tcl_Interp *TclInterp = (Tcl_Interp *)0;

static int GrammarLine = 0;	/* 1 to print AG line number */
static int ErrorLimit = 0;	/* 1 to abort after too many errors */


void
#ifdef PROTO_OK
ErrorInit(Tcl_Interp *interp, int AGout, int ErrLimit)
#else
ErrorInit(interp, AGout, ErrLimit) Tcl_Interp *interp; int AGout, ErrLimit;
#endif
/* Initialize the error module
 *    On entry-
 *       interp=Tcl-Intepreter that issues the Compiler-Call
 *       AGout=1 to print AG line number on error reports
 *       ErrLimit=1 to limit the number of errors reported
 ***/
{
  TclInterp = interp;
  GrammarLine = AGout; 
  ErrorLimit = ErrLimit;
}

static void
IssueMessage(NODEPTR currn, char *editview, char *editattr, int severity, char *Msgtext)
{
    Tcl_Obj *cmd[3], *msg[5];
    Tcl_Obj *cmdlst;
    Tcl_Obj *cmdvar;
    int len = 0;

    /* First part: Expand command once */
    cmd[0] = Tcl_NewStringObj("eval", -1);
    
    cmdvar = Tcl_NewStringObj("::vl::onError", -1);
    Tcl_IncrRefCount(cmdvar);
    cmd[1] = Tcl_ObjGetVar2(TclInterp, cmdvar, 0, TCL_GLOBAL_ONLY);
    Tcl_DecrRefCount(cmdvar);
    Tcl_GetStringFromObj(cmd[1], &len);  /* need side effect of setting len */
    
    msg[0] = currn ? TclVP_NewVoidPointerObj(currn) : Tcl_NewObj();
    msg[1] = editview ? Tcl_NewStringObj(editview, -1) : Tcl_NewObj();
    msg[2] = editattr ? Tcl_NewStringObj(editattr, -1) : Tcl_NewObj();
    msg[3] = Tcl_NewIntObj(severity);
    msg[4] = Tcl_NewStringObj(Msgtext, -1);
    cmd[2] = Tcl_NewListObj(5, msg);
    
    cmdlst = Tcl_NewListObj(3, cmd);
    Tcl_IncrRefCount(cmdlst);

    if (len) 
    {
	if (Tcl_GlobalEvalObj(TclInterp, cmdlst) != TCL_OK)
	{
	    printf ("Error during Error callback: %s\n", Tcl_GetStringResult(TclInterp));
	}
    }
    
    Tcl_DecrRefCount(cmdlst);
}
      
void ReportTclError(char *msg)
{
    Tcl_Obj *cmd[3];
    Tcl_Obj *cmdlst;
    Tcl_Obj *cmdvar;
    int len = 0;

    /* First part: Expand command once */
    cmd[0] = Tcl_NewStringObj("eval", -1);
    
    cmdvar = Tcl_NewStringObj("::vl::onLog", -1);
    Tcl_IncrRefCount(cmdvar);
    cmd[1] = Tcl_ObjGetVar2(TclInterp, cmdvar, 0, TCL_GLOBAL_ONLY);
    Tcl_DecrRefCount(cmdvar);

    Tcl_GetStringFromObj(cmd[1], &len);  /* Only need side effect of setting len */
    
    if (msg)
	cmd[2] = Tcl_NewStringObj(msg, -1);
    else
	cmd[2] = Tcl_NewStringObj("$::errorInfo", -1);
    
    cmdlst = Tcl_NewListObj(3, cmd);
    Tcl_IncrRefCount(cmdlst);

    if (len) 
    {
	if (Tcl_GlobalEvalObj(TclInterp, cmdlst) != TCL_OK)
	{
	    printf ("Error during Error callback: %s\n", Tcl_GetStringResult(TclInterp));
	}
    }
    
    Tcl_DecrRefCount(cmdlst);
}

void
#ifdef PROTO_OK
message(int severity, char *Msgtext, int grammar, NODEPTR source, char *view, char *attr)
#else
message(severity, Msgtext, grammar, source, view, attr)
int severity; char *Msgtext; int grammar; NODEPTR source;
char *view, *attr;
#endif
/* Report an error
 *    On entry-
 *      severity=error severity
 *      Msgtext=message text
 *      grammar=identification of the test that failed
 *      source=source coordinates at which the error was detected
 ***/
{ 

  if (severity < NOTE || severity > DEADLY) {
    severity = DEADLY;
  }

  IssueMessage(source, view, attr, severity == DEADLY ? FATAL : severity, Msgtext);

  ErrorCount[severity]++;


  if (severity == DEADLY) {
    longjmp(on_deadly, 1);
  }
  
  if(ErrorLimit && ErrorCount[ERROR] > 20) {
    message(DEADLY, "Too many ERRORs", 0, (POSITION*)0);
  }
}
