#ifndef ERR_H
#define ERR_H

/* $Id: err.h,v 3.4 2000/05/15 09:42:15 mjung Exp $ */
/* Copyright 1997, The Regents of the University of Colorado */

/* This file is part of the Eli Module Library.

The Eli Module Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The Eli Module Library is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the Eli Module Library; see the file COPYING.LIB.
If not, write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA.  */

/* As a special exception, when this file is copied by Eli into the
   directory resulting from a :source derivation, you may use that
   created file as a part of that directory without restriction. */

#include "eliproto.h"
#include "nodeptr.h"

#ifdef PROTO_OK
#include <stdio.h>
#include "tcl.h"
#endif
#include <setjmp.h>

	/* Error report classification */

#define NOTE	0	/* Nonstandard construct */
#define COMMENT	0	/* Obsolete */
#define WARNING	1	/* Repairable error */
#define ERROR	2	/* Unrepairable error */
#define FATAL	3	/* Obsolete */
#define DEADLY	4	/* Error that makes continuation impossible */


	/* Types exported by the Error Module */

typedef struct NODEPTR_struct POSITION;
typedef POSITION *CoordPtr;

extern int err_dummyline;
extern int err_dummycol;

#define NoPosition	((POSITION *)0)
#define LineOf(pos)     (err_dummyline)
#define ColOf(pos)      (err_dummycol)
#ifdef RIGHTCOORD
#define RLineOf(pos)    (err_dummyline)
#define RColOf(pos)     (err_dummycol)
#endif
#define NoView          ((char *)0)
#define NoAttr          ((char *)0) 

	/* Variables exported by the Error Module */

extern int ErrorCount[];
extern int LineNum;	   /* Index of the current line in the total source text */
// extern POSITION NoCoord;   /* The NULL coordinate */
extern POSITION curpos;	   /* Position variable for general use */
extern jmp_buf on_deadly;  /* Where to jump on deadly errors */
extern Tcl_Interp *TclInterp;

	/* Routines exported by the Error Module */

extern void
ErrorInit ELI_ARG((Tcl_Interp *interp, int AGout, int ErrLimit));
/* Initialize the error module 
 *    On entry- 
 *       interp=Tcl-Intepreter that issues the Compiler-Call
 *       AGout=1 to print AG line number on error reports 
 *       ErrLimit=1 to limit the number of errors reported
 ***/

extern void
message ELI_ARG((int severity, char *Msgtext, int grammar, NODEPTR source = NoPosition, char *view = NoView, char *attr = NoAttr));
/* Report an error
 *    On entry-
 *      severity=error severity
 *      Msgtext=message text
 *      grammar=identification of the test that failed
 *      source=source coordinates at which the error was detected
 ***/

extern void 
ReportTclError ELI_ARG((char *msg = (char *)0));
/* Reports error message assigned to Tcl-Interpreter to the Log */

#endif
