# ----------------------------------------------------------------------
# Module Error
# ----------------------------------------------------------------------
# Procedures to perform Error-Handling
#
# $Id: visual.tcl,v 3.9 1999/11/05 09:06:45 mjung Exp $

# Create our namespace.
namespace eval ::error {
}

proc ::error::init {} {
    set ::vl::onError ::error::message
    # ::event::add_listener tree_delete "::error::clearnodeerrors 0"
}

###### Interaktion mit Baumberechnungen ######

proc ::error::initCheckRun {id} {
    variable err
    _clearErrors $id
    set err(treerun-errorListId) $id
}

proc ::error::doneCheckRun {} {
    variable err
    #updateerrors $err(treerun-errorListId)
    unset err(treerun-errorListId)

}

proc ::error::initProcessorRun {file errorWindowName} {
    variable err
    set err(treerun-errorListId) {} 
    set err(treerun-errorWindowName) $errorWindowName
    set err(treerun-file) $file
}

proc ::error::doneProcessorRun {} {
    variable err
    if { $err(treerun-errorListId) != "" } {
	open_window $err(treerun-errorListId)
    }
    unset err(treerun-errorListId)
}

proc ::error::getCurrentErrorListId {} {
    oop::private

    variable err
    if { $err(treerun-errorListId) == "" } {
	set err(treerun-errorListId) \
		[createErrorList $err(treerun-file) $err(treerun-errorWindowName)]
    }
    return $err(treerun-errorListId)
}

######## Fenster-Methoden ################

proc ::error::wincmd_save {win} {
    variable err
    ::file::save $err($win-fileId)
}

proc ::error::wincmd_saveAs {win} {
    variable err
    ::file::saveAs $data($win-fileId)
}


proc ::error::wincmd_process {win processor} {
    variable err
    ::file::process $err($win-fileId) $processor 1
}


proc ::error::wincmd_showErrWindow {id} {
    # Do nothing. This is already the error-window!
}

proc ::error::wincmd_showAbstree {id} {
    variable err
    if { "$err($id-fileId)" != "" } {
	::file::activateMainView $err($id-fileId)
    } else {
	::CDlg::Information "Don't know which file!"
    }
}

###########################################

proc ::error::doneErrorList {id} {
    file::forgetWindow $err($id-fileId) $id
}


proc ::error::createErrorList {fileId {title ""}} {
    variable err

    if { ![info exists err(namecounter)] } {
	set err(namecounter) 1
    }
    set id "errorList$err(namecounter)"
    incr err(namecounter)
    oop::setClass $id "error"

    set err($id-status) "closed"
    set err($id-fileId) $fileId
    set err($id-title) $title
    set err($id-messages) ""
    set err($id-sevlabel,0) "NOTE"
    set err($id-sevlabel,1) "WARNING"
    set err($id-sevlabel,2) "ERROR"
    set err($id-sevlabel,3) "FATAL"
    set err($id-predeftag,NOTE) 1
    set err($id-predeftag,WARNING) 1
    set err($id-predeftag,ERROR) 1
    set err($id-predeftag,FATAL) 1
    set err($id-predeftag,CURRENT) 1
    set err($id-update_required) 0

    # Das Fenster dem file zuordnen
    file::addWindow $err($id-fileId) $id

    return $id
}

proc ::error::createWorkspace {window} {
    frame $window.err

    text $window.err.text -relief sunken \
	    -font "Helvetica -14" \
	    -yscrollcommand [list $window.err.sb set] \
	    -width 60 -height 3 -wrap none
    scrollbar $window.err.sb -orient vertical -command "$window.err.text yview" \
	    -takefocus 0
    pack $window.err.sb -fill y -side right
    pack $window.err.text -expand true -fill both -side left

    #frame $window.ctl -relief sunken
    #pack $window.ctl  -side bottom -fill x
    pack $window.err  -expand true -fill both

    #button $window.ctl.clear -text "Clear" -command "::error::clearerrors"
    #pack $window.ctl.clear -side left -expand true

    # Set tags for errors, warning and deadly messages
    #window.err.text tag configure NOTE -background "#80ff80"
    $window.err.text tag configure ERROR -background "#ffc080"
    $window.err.text tag configure WARNING -background "#ffff80"
    $window.err.text tag configure FATAL -background "#ff8080"
    $window.err.text tag configure CURRENT -font "Helvetica -14 bold"

    set width [expr [font measure "Helvetica -14 bold" "WARNING"] + 5]
    $window.err.text configure -tabs "$width right"

    # remove Text from bindtags
    set btags [bindtags $window.err.text]
    set i [lsearch $btags Text]
    if {$i >= 0} {
	set btags [lreplace $btags $i $i]
    }
    bindtags $window.err.text $btags
}

proc ::error::getTitle {id} {
    variable err

    if { "$err($id-title)" != "" } {
	return $err($id-title)
    }
    return "Error messages ([file::modifiedMarker $err($id-fileId)][file::getFilename $err($id-fileId)])"
}


proc ::error::updateTitle {id} {
    variable err
    if {$err($id-status) == "open"} {
	::MDIFrame::SetTitle $err($id-client) [getTitle $id]
    }
}


proc ::error::closeWindow {id} {
    variable err
    if {$err($id-status) == "open"} {
	::MDIFrame::CloseClient $err($id-client) 1
    }
}


proc ::error::open_window {id} {
    variable err

    if {$err($id-status) == "open"} {
	::MDIFrame::MakeVisible $err($id-client)
	return
    } 
    
    set total [::MDIFrame::Totalsize]
    set total_x [lindex $total 0]
    set total_y [lindex $total 1]
    
    set window [::MDIFrame::New_Window "dummy title" "" \
	    -x 0 -y [expr $total_y - 120] -width [expr $total_x / 2] -height 120]

    set err($id-client) [::MDIFrame::GetClient $window]

    # Das Fenster anderen bekannt machen
    oop::setAlias $id $err($id-client)
    
    pack $window -expand true -fill both

    createWorkspace $window

    bind $window <Destroy> "if \{\"%W\" == \"$window\"\} \{::error::on_destroy $id\}"

    set err($id-status) "open"
    set err($id-data) $window.err.text


    # Munge the messages
    set err($id-update_required) 1
    updateTitle $id
    updateerrors $id
}

proc ::error::on_destroy {id} {
    variable err
    set err($id-status) "closed"
}

proc ::error::errorlevel {id} {
    variable err
    set level 0
    foreach msg $err($id-messages) {
	set severity [lindex $msg 1]
	if { $severity > $level } {
	    set level $severity
	}
    }
    return $level
}

proc ::error::message {node editview editattr severity message} {
    # severity could be one of 
    # 0     NOTE
    # 1     WARNING
    # 2     ERROR
    # 3     FATAL

    variable err
    set id [getCurrentErrorListId]

    if {$err($id-status) == "closed"} {
	lappend err($id-messages) [list [list $node $editview $editattr] $severity $message]
    } else {
	lappend err($id-messages) [list [list $node $editview $editattr] $severity $message]
	appendmessage $id [list $node $editview $editattr] $severity $message
    }
}

proc ::error::appendmessage {id node severity message} {

    variable err

    set line [$err($id-data) index "end - 1 char"]
    $err($id-data) insert $line "\t$err($id-sevlabel,$severity) $message\n"
    $err($id-data) tag add $err($id-sevlabel,$severity) $line "$line + 1 line"
    $err($id-data) tag add "line$line" $line "$line + 1 line"
    $err($id-data) tag bind "line$line" <ButtonPress-1> \
	    "::error::editerror $id $line [list $node]"
}

proc ::error::on_treeModified {id node} {
    # called by file automatically
    variable err

    set prevrequired $err($id-update_required)
    set filteredmessages ""
    set fileRoot [::file::getRoot $err($id-fileId)]
    foreach msg $err($id-messages) {
	set msgnode [lindex $msg 0]
	set keep 0
	if {$msgnode == ""} {
	    set keep 1
	} elseif {[::vl::isAncestor $fileRoot $msgnode]} {
	    set keep 1
	}
	if {$keep} {
	    lappend filteredmessages $msg
	} else {
	    set err($id-update_required) 1
	}
    }
    set err($id-messages) $filteredmessages

    if {$err($id-update_required) && !$prevrequired} {
	after idle "::error::updateerrors $id"
    }
}


proc ::error::clearerrors {id} {
    return

    variable err

    set err($id-messages) ""
    if {$err($id-status) == "open"} {
	$err($id-data) delete 1.0 end
	set tags [$err($id-data) tag names]
	foreach tag $tags {
	    if {[info exists err($id-predeftag,$tag)] == 0} {
		$err($id-data) tag delete $tag
	    }
	}
    }
}

proc ::error::_clearErrors {id} {
    variable err

    set err($id-messages) ""
    set err($id-update_required) 1

    after idle "::error::updateerrors $id"
}


proc ::error::clearnodeerrors_old {subtree nodeId} {

    # Removes all errors that are reported to $nodeId or some son of $nodeId.
    variable err

    set prevrequired $err(update_required)
    set filteredmessages ""
    foreach msg $err(messages) {
	set msgnode [lindex $msg 0]
	set keep 0
	if {$msgnode == ""} {
	    set keep 1
	} elseif {$subtree} {
	    if {[treeutil::exists $msgnode] && ! [::vl::isAncestor $nodeId $msgnode]} {
		set keep 1
	    }
	} else {
	    if {$nodeId != $msgnode} {
		set keep 1
	    }
	}

	if {$keep} {
	    lappend filteredmessages $msg
	} else {
	    set err(update_required) 1
	}
    }

    set err(messages) $filteredmessages

    if {$err(update_required) && !$prevrequired} {
	after idle "::error::updateerrors $id"
    }

	    
}

proc ::error::updateerrors {id} {
    variable err
    # Call this after a series of clearnodeerrors-Operations

    if {$err($id-update_required) && $err($id-status) == "open"} {
	$err($id-data) delete 1.0 end
	set tags [$err($id-data) tag names]
	foreach tag $tags {
	    if {[info exists err($id-predeftag,$tag)] == 0} {
		$err($id-data) tag delete $tag
	    }
	}
	foreach error $err($id-messages) {
	    eval appendmessage $id $error
	}
    }
    set err($id-update_required) 0
}

proc ::error::editerror {id line node} {
    variable err

#    $err(data) tag add CURRENT $line "$line + 1 line"
    set n [lindex $node 0]
    set v [lindex $node 1]
    set a [lindex $node 2]

    if {$n == ""} {
	::CDlg::Information "No position information available." "Edit Error"
	return
    }

    if {$v != ""} {
	set editor ""
	set rootnode $n
	while {$editor == ""} {
	    foreach ed [::view::get_editors $rootnode] {
		if {[lindex $ed 2] == $v} {
		    set editor $ed
		}
	    }

	    if {$editor == ""} {
		set rootnode [::vl::getFather $rootnode]
		if {$node == ""} {
		    ::CDlg::Information "A Node for View $v could not be found." "Edit Error"
		    return
		}
	    }
	}
    } else {

	set editor ""
	set rootnode $node
	while {$editor == ""} {
	    set editor [lindex [::view::get_editors $rootnode] 0]
	    if {$editor == ""} {
		set rootnode [::vl::getFather $rootnode]
		if {$node == ""} {
    		    ::CDlg::Information "No editable node could be found." "Edit Error"
		    return
		}
	    }
	}
    }

    foreach window [::MDIFrame::GetWindowList] {
	if {[::MDIFrame::GetRootId $window] == $rootnode} {
	    # Already a window open that shows the desired context
	    set selectcommand [::MDIFrame::GetSelectcommand $window]
	    if {$selectcommand != ""} {
		eval $selectcommand $n
	    }

	    ::MDIFrame::MakeVisible $window
	    return
	}
    }

    # No Window open. Open a new one.
    [lindex $editor 0] $rootnode -selectnode $n -selectattr $a
}
