# ----------------------------------------------------------------------
# Module event
# ----------------------------------------------------------------------
# $Id: event.tcl,v 3.5 2000/04/18 14:14:51 cschmidt Exp $
#
# User Event Handling Routines
#
# Event-name       Arguments
# ----------------------------------------------------------------------
# tree_changed     id       id of a node that roots the nodes that has
#                           been inserted or extracted or those attributes
#                           have changed
# tree_delete      id       id of a node that is to be deleted

namespace eval ::event {
    variable event_listeners
}

proc ::event::add_listener {event_name command} {
    variable event_listeners
    
    if {[info exists event_listeners($event_name)]} {
	if {[lsearch -exact event_listeners($event_name) $command] >= 0} {
	    return
	}
    }
    lappend event_listeners($event_name) $command
}

proc ::event::post {event_name immediate args} {
    
    variable event_listeners
    global errorInfo
    
    if {[info exists event_listeners($event_name)]} {
	if {$event_listeners($event_name) != ""} {
	    if {$immediate} {
		foreach command $event_listeners($event_name) {
		    if {[catch "$command $args" err]} {
			log::log "Error during event-processing: $err"
			log::log $errorInfo
			bgerror "Error during event-processing: $err"
		    }
		}
	    } else {
		after idle [format {\
		    set args {%s}
		    foreach command "%s" {
			if {[catch "$command $args" err]} {
			    
			    log::log "Error during event-processing: $err"
			    log::log $errorInfo
			    bgerror "Error during event-processing: $err"
			}
		    }
		} $args $event_listeners($event_name)]
	    }
	}
    }
}

proc ::event::init {} {
}	
	
    



