
namespace eval ::file {
    variable data
    variable nextid 0
}


set filetypes {
    { {Editor Data} {.$::cfg::extension}   }
    { {All Files}   *                      }
}


proc ::file::init {} {
    variable data
    set data(filelist) ""
    ::event::add_listener tree_delete "::file::classMethod_on_treeModified"
    ::event::add_listener tree_changed "::file::classMethod_on_treeModified"
}

proc ::file::classMethod_on_treeModified {node} {
    variable data
    foreach id $data(filelist) {
	on_treeModified $id $node
    }
}

proc ::file::on_treeModified {id node} {
    variable data
    set data($id-modified) 1

    if { ![::vl::isAncestor $data($id-rootnode) $node]} {
        return
    }

    foreach win $data($id-windows) {
        ::oop::msg $win on_treeModified $node
    }
    updateTitle $id
}

proc ::file::modified {id} {
    variable data
    return $data($id-modified)
}

proc ::file::modifiedMarker {id} {
    if [file::modified $id] {
        return "*"
    } else {
        return ""
    }    
}

proc ::file::new {} {
    variable nextid
    variable data

    set id [incr nextid]
    set data($nextid-rootnode)   [::vl::create $::vl::rootrule]
    set data($nextid-filename)   "unnamed$nextid.$::cfg::extension"
    set data($nextid-modified)   0
    set data($nextid-unnamed)    1
    set data($nextid-mainwin)    ""
    set data($nextid-windows)    ""

    ::file::displayfile $id
    return $id
}


proc ::file::open {filename} {

    set rootnode [::file::read $filename]

    variable nextid
    variable data

    set id [incr nextid]
    set data($nextid-rootnode)   $rootnode
    set data($nextid-filename)   $filename
    set data($nextid-modified)   0
    set data($nextid-unnamed)    0
    set data($nextid-mainwin)    ""
    set data($nextid-windows)    ""

    ::file::displayfile $id
    return $id
}


proc ::file::openDialog {} {
    global filetypes
    set filename [tk_getOpenFile -filetypes $filetypes -defaultextension .$::cfg::extension]
    if {$filename == ""} return ""
    return [open $filename]
}


proc ::file::prepareClose {id} {
    # Executes nessasary operations before file termination.
    # Returns 1, if the file should be closed, 0 otherwise.

    variable data

    if { ! $data($id-modified) } {return 1}

    set msg ""
    append msg "[getPath $id]\n\n"
    append msg "This file contains unsaved changes. Save them before closing?"

    set res [tk_messageBox -title "Save changes?" -message $msg -icon question -type yesnocancel]
    switch $res {
        yes {
            return [save $id]
        }
        no {
            return 1
        }
        cancel {
            return 0
	}  
    }
}


proc ::file::closeAfterConfirmation {id} {
    if [prepareClose $id] {
        close $id
    }
}


proc ::file::close {id} {
    # Wird aufgerufen, wenn das Strukturbaum-Fenster geschlossen wird
    variable data
    foreach win $data($id-windows) {
        ::oop::msg $win closeWindow
    }

    ::vl::delete $data($id-rootnode)
    ldelete data(filelist) $id
}


proc ::file::save {id} {
    # Returns 1, if file was saved, 0 otherwise

    variable data

    if $data($id-unnamed) {
        return [::file::saveAs $id]
    }
    set res [::file::write $id $data($id-filename)]
    
    updateTitle $id
    # Sternchen zur Kennzeichnung eines ungesichterten Zustandes entfernen

    return $res
}


proc ::file::updateTitle {id} {
   variable data

    foreach win $data($id-windows) {
        ::oop::msg $win updateTitle
    }
}

proc ::file::saveAs {id} {
    # Returns 1, if file was saved, 0 otherwise

    global filetypes
    variable data

    set filename [tk_getSaveFile -filetypes $filetypes -defaultextension .$::cfg::extension]
    if {$filename == ""} {return 0}

    set data($id-unnamed) 0
    set data($id-filename) $filename
    ::file::write $id $data($id-filename)
    updateTitle $id

    return 1
}

proc ::file::getFilename {id} {
    variable data

    set name $data($id-filename)
    return [file tail $name]
}


proc ::file::getPath {id} {
    variable data

    set name $data($id-filename)
    return $name
}


proc ::file::getFileId {treeNode} {
    variable data

    set root [::vl::getRoot $treeNode]

    foreach n [array names data "*-rootnode"] {
        if { $data($n) == $root } {
            set name $n
            break
        }
    }
    regexp "(.*)-rootnode" $name dummy id
    return $id
}


proc ::file::removeMe_getFileId2 {} {
    # $::visual::currVarname ist nur whrend der Ausfhrung eines
    # Attributauswerters gltig. Dieser Code mu korrigiert werden.

  upvar #0 $::visual::currVarname data
  set root [::vl::getRoot $data(rootid)]
  return [getFileId $root]
}


proc ::file::getRoot {id} {
    variable data

    set root $data($id-rootnode)
    return $root
}


proc ::file::getErrorListId {id} {
    variable data
    return $data($id-errorlist)
}

proc ::file::openErrorList {id} {
    variable data
    ::error::open_window $data($id-errorlist)
}

proc ::file::displayfile {id} {
    variable data

    set data($id-errorlist) [::error::createErrorList $id]

    if {$::cfg::mainview != ""} {
	$::cfg::mainview [getRoot $id]
	set data($id-mainwin) [::abstree::open $id]
	#lappend data($id-windows) $data($id-mainwin)
	tkwait visibility [::MDIFrame::GetWindowPath $data($id-mainwin)]
	::MDIFrame::Minimize $data($id-mainwin)
    } else {
	set data($id-mainwin) [::abstree::open $id]
	#lappend data($id-windows) $data($id-mainwin)
    }
    lappend data(filelist) $id
}

proc ::file::activateMainView {id} {
    variable data
    ::MDIFrame::MakeVisible $data($id-mainwin)
    #::MDIFrame::Minimize $data($id-mainwin)
}


proc ::file::saveExit {} {
    exit
}


    
proc ::file::write {id filename} {
    variable data

    set root $data($id-rootnode)
    set fileid [::open $filename w]

    puts $fileid "# $::cfg::apptitle - Data File"
    puts $fileid "# Editor Version X.X"
    puts $fileid "# The Editor was generated by VLEli Version 1.0"

    set liste [::treeutil::serializeSubtree $root]
    # Ich moechte, dass in der Datei die einzelnen Listenelemente
    # durch newlines getrennt sind, damit man die Datei besser lesen kann.
    foreach x $liste {
        puts $fileid $x
    }
    puts $fileid "# END OF FILE"
    ::close $fileid
    ::CDlg::Information "File '$filename' saved!!"
    set data($id-modified) 0

    return 1
}


proc ::file::read {filename} {
    ::log::info execution "Loading '$filename' ..."
    set fileid [::open $filename r]
    set file [::read $fileid]
    set res [::treeutil::unserializeSubtree [list $file]]
    ::log::info execution "File '$filename' loaded!"
    return $res
    # Anmerkung: Die gespeicherte Liste wird beim Laden nicht
    # restauriert, statt dessen wird die Datei als einelementige
    # Liste an unserializeSubtree uebergeben. 
    # Funktioniert aber trotzdem !-)
}


proc ::file::addWindow {id win} {
    # Bei einer Namensaenderung der Datei wird die virtuelle Methode updateTitle aufgerufen.
    # Beim Schliessen der Datei wird die virtuelle Methode destroy aufgerufen.
    variable data
    lappend data($id-windows) $win
}


proc ::file::forgetWindow {id win} {
    variable data
    ldelete data($id-windows) $win
}

proc ::file::process {id processor {maxErrorlevel 0}} {
    variable data

    if { [error::errorlevel $data($id-errorlist)] <= $maxErrorlevel } {
	::error::initProcessorRun $id "Messages of $processor-processor"
	::log::info execution "Executing processor $processor"
	::processing::execute $processor $data($id-rootnode)
	::log::info execution "Processor $processor ready!"
	::error::doneProcessorRun
    } else {
	error::open_window $data($id-errorlist)
    }
}


# Folgende Funktion musste irgendwo untergebracht werden...

proc ldelete {listname elem} {
    upvar $listname list
    set index [lsearch -exact $list $elem]
    set list [lreplace $list $index $index]
}
