#ifndef GEN_TREEIO_H
#define GEN_TREEIO_H
#include "nodeptr.h"
#include "eliproto.h"
#include "tcl.h"
#include "nodecode.h"

#define VL_ROOT RULERule_4
typedef struct _ruleinfo {
	char *rulename;              /* Name of the Rule */
	int ruletype;                /* 0: Normal Rule, 1: List Root, 2: Inner List node */
	NODEPTR (*create)(void);     /* Creates a new subtree by application */
	                             /* of specified rule at the root */
	void (*delrule)(NODEPTR);    /* Deletes the given subtree */
	Tcl_Obj* (*getdesc)(NODEPTR, int);
	                             /* Returns Descendants of Rule, 0 means all */
	                             /* 2nd arg is ignored for lists. */
	Tcl_Obj *(*getattrs)(NODEPTR);
	                             /* Returns List with attribute names and values */
	                             /* suitable for array get-command */
	Tcl_Obj *(*getvlpropertyids)(NODEPTR);
	                             /* Returns List with attribute names and deftable-key-ids */
	                             /* suitable for array get-command */
	Tcl_Obj *(*getvlvalues)(NODEPTR);
	                             /* Returns List with attribute names and deftable-key-ids */
	                             /* suitable for array get-command */
	void (*setattr)(NODEPTR, char*, Tcl_Obj*);
	                             /* Sets a new value for a named attribute */
	int (*insert)(NODEPTR,NODEPTR,int,int);
	                             /* Inserts second in pos. of first int into first. */
	                             /* if second int is 1, replaces. Ret. 0 on success */
	void (*extract)(NODEPTR,NODEPTR);
	                             /* Un-inserts second node from first. */
	void (*getminmax)(NODEPTR, NODEPTR, int *, int *);
	                             /* Returns minimum and maximum number of a given subtree node */
} Ruleinfo;

#define RI_LEN 26
extern Ruleinfo RI[];
void InitRuleTable ELI_ARG((void));

typedef struct _symbinfo {
	char *symbname;              /* Name of the Symbol */
	int listelement;             /* Symbol id of list element symbol */
	int defaultprod;             /* Default Production for a symbol */
} Symbinfo;

#define SI_LEN 21
extern Symbinfo SI[];
void InitSymbolTable ELI_ARG((void));


#endif
