# ----------------------------------------------------------------------
# Module Hierarchical
# ----------------------------------------------------------------------
# Routines to display tree structured data in a tree view.
# 
# $Id: hierarchical.tcl,v 3.1.1.1 1999/08/31 21:21:59 mjung Exp $

namespace eval ::hierlist {
}

# Initialisiert dieses Modul

proc ::hierlist::init {} {
    variable hierInfo

    if {[info exists hierInfo(sideArrow)]} {
	# Already initialized.
	return
    }

    option add *Hierlist.activeBackground "#000080" widgetDefault
    option add *Hierlist.activeForeground white widgetDefault
    option add *Hierlist.indent 15 widgetDefault
    option add *Hierlist.hbox.background white widgetDefault
    option add *Hierlist.hbox.width 40 widgetDefault
    option add *Hierlist.hbox.height 10 widgetDefault
    option add *Hierlist.hbox.cursor top_left_arrow widgetDefault
    option add *Hierlist.hbox.font \
	    -*-lucida-medium-r-normal-sans-*-120-* widgetDefault

    set hierInfo(sideArrow) $::cfg::img(plus)
    set hierInfo(downArrow) $::cfg::img(minus)

    menu .hierlistmenu -tearoff 0
}

# Extrahiert den Namen eines Knotens

proc ::hierlist::nodename {var node} {
    upvar #0 ::hierlist::$var data
    
    set name [eval "$data(nodename) $data($node-coords)"]
    return $name
}

proc ::hierlist::nodeicon {var node} {
    upvar #0 ::hierlist::$var data
    
    if {$data(nodeicon) != ""} {
	return [eval "$data(nodeicon) $data($node-coords)"]
    } else {
	return ""
    }
}

# Extrahiert eine Liste mit den Koordinaten der Shne eines
# Knotens.

proc ::hierlist::nodechildren {var node} {
    upvar #0 ::hierlist::$var data
    
    set name [eval "$data(nodechildren) $data($node-coords)"]
    return $name
}

# Prft, ob ein Knoten Shne hat oder nicht

proc ::hierlist::nodehaschildren {var node} {
    upvar #0 ::hierlist::$var data

    if {$data(nodehaschildren) != ""} {
	return [eval "$data(nodehaschildren) $data($node-coords)"]
    } else {
	return [expr {[nodechildren $var $node] != {}}]
    }
}

# Fgt zu einem Knoten die Koordinaten der Shne dieses Knotens
# ein und ermittelt jeweils den Namen des Knotens

proc ::hierlist::data_add {var node} {
    upvar #0 ::hierlist::$var data
    
    # Clean up subtree.
    set nodecoords $data($node-coords)
    foreach index [array names data $node-*] {
	unset data($index)
    }

    # Re-Enter current data.
    set data($node-coords) $nodecoords
    set data($node-label) [nodename $var $node]
    set data($node-icon) [nodeicon $var $node]
    set data($node-children) ""

    set num 0
    
    foreach rec [nodechildren $var $node] {
	set subnode "$node-[incr num]"
	lappend data($node-children) $subnode
	set data($subnode-coords) $rec
	set data($subnode-label) [nodename $var $subnode]
	set data($subnode-icon) [nodeicon $var $subnode]
    }
}

proc ::hierlist::data { win rootcoord procnames {category "world"}} {
    variable hierInfo
    upvar $procnames procs

    if {[info exists hierInfo($win-data)]} {
	set var $hierInfo($win-data)
	upvar #0 ::hierlist::$var data
	unset data
    } else {
	set counter 0
	while {1} {
	    set var "hierData#[incr counter]"
	    upvar #0 ::hierlist::$var data
	    if {![info exists data]} {
		break
	    }
	}
	set hierInfo($win-data) $var
	lappend hierInfo(category-$category) $win

	foreach field "nodename nodechildren" {
	    if {[info exists procs($field)]} {
		set data($field) $procs($field)
	    } else {
		error "Required field missing: $procnames($field)"
	    }
	}
	foreach field "nodemenu nodeicon nodeselect nodeedit nodehaschildren" {
	    if {[info exists procs($field)]} {
		set data($field) $procs($field)
	    } else {
		set data($field) ""
	    }
	}
	set data(category) $category
	set data(root-coords) $rootcoord
	set data(selectednode) ""
    }

}

proc ::hierlist::create {win} {

    # Create main frame, scrollbar and text pane.
    frame $win -class Hierlist
    scrollbar $win.sbar -command "$win.hbox yview"
    pack $win.sbar -side right -fill y
    text $win.hbox -wrap none -takefocus 0 \
	    -yscrollcommand "$win.sbar set"
    pack $win.hbox -side left -expand yes -fill both

    # Configure tabs.
    set tabsize [option get $win indent Indent]
    set tabsize [winfo pixels $win $tabsize]
    set tabs "15"
    for {set i 1} {$i < 20} {incr i} {
	lappend tabs [expr $i * $tabsize + 15]
    }
    $win.hbox configure -tabs $tabs

    # remove Text from bindtags
    set btags [bindtags $win.hbox]
    set i [lsearch $btags Text]
    if {$i >= 0} {
	set btags [lreplace $btags $i $i]
    }
    bindtags $win.hbox $btags
    
    # Arrange for cleanup when window is closed.
    bind $win <Destroy> "::hierlist::destroy $win"
    return $win
}

proc ::hierlist::destroy {win} {
    variable hierInfo
    if {[info exists hierInfo($win-data)]} {
	set var $hierInfo($win-data)
	upvar #0 ::hierlist::$var data
	set category $data(category)
	set windex [lsearch -exact $hierInfo(category-$category) $win]
	set hierInfo(category-$category) \
		[lreplace $hierInfo(category-$category) $windex $windex]
	unset data
	unset hierInfo($win-data)
    }
}

proc ::hierlist::insert {win node} {
    variable hierInfo

    set var $hierInfo($win-data)
    data_add $var $node
    upvar #0 ::hierlist::$var data
    lappend data(open_nodes) $node
    set imagebg [option get $win.hbox background Color]

    set indent ""
    set count 0
    foreach digit [split $node "-"] {
	incr count
    }
    while {$count > 1} {
	append indent "\t"
	incr count -1
    }

    $win.hbox mark set pos "$node:start"

    foreach subnode $data($node-children) {
	$win.hbox insert pos \
		"$indent" 
	if {[nodehaschildren $var $subnode]} {
	    set arrow "$win.hbox.arrow-$subnode"
	    label $arrow -image $hierInfo(sideArrow) \
		    -borderwidth 0
	    bind $arrow <ButtonPress-1> \
		    "::hierlist::expand $win $subnode"
	    $win.hbox window create pos -window $arrow
	}
	if {$data($subnode-icon) != ""} {
	    set icon $win.hbox.icon-$subnode
	    label $icon -image $data($subnode-icon) \
		    -background $imagebg -borderwidth 0
	    $win.hbox insert pos \
		    "\t" 
	    $win.hbox window create pos -window $icon
	}
	$win.hbox insert pos \
		"\t" 
	$win.hbox insert pos \
		"$data($subnode-label)" $subnode
	$win.hbox insert pos \
		"\n" 
	$win.hbox tag bind $subnode <ButtonPress-1> \
		"::hierlist::select $win $subnode"
	$win.hbox tag bind $subnode <Double-ButtonPress-1> \
		"::hierlist::open $win $subnode"
	$win.hbox tag bind $subnode <ButtonPress-3> \
		"::hierlist::contextmenu $win $subnode %X %Y"
	
	$win.hbox mark set "$subnode:start" pos
	$win.hbox mark gravity "$subnode:start" left
    }
    $win.hbox mark set "$node:end" pos
}

proc ::hierlist::select {win node} {
    variable hierInfo

    set var $hierInfo($win-data)
    upvar #0 ::hierlist::$var data

    if {$data(selectednode) == $node} {
	# Already selected.
	return
    }
    set activebg [option get $win activeBackground Color]
    set activefg [option get $win activeForeground Color]

    if {$data(selectednode) != ""} {
	$win.hbox tag configure $data(selectednode) -background {} -foreground {}
	if {$data(nodeselect) != ""} {
	    eval "$data(nodeselect) unselect $data($node-coords)"
	}
	set data(selectednode) ""
    }

    $win.hbox tag configure $node -background $activebg -foreground $activefg
    if {$data(nodeselect) != ""} {
	eval "$data(nodeselect) select $data($node-coords)"
    }

    set data(selectednode) $node
    set data(selectedcoord) $data($node-coords)
}

proc ::hierlist::open {win node} {
    variable hierInfo

    set var $hierInfo($win-data)
    upvar #0 ::hierlist::$var data

    if {$data(nodeedit) != ""} {
	eval "$data(nodeedit) invoke $data($node-coords)"
    }
}

proc ::hierlist::contextmenu {win node X Y} {

    variable hierInfo

    set var $hierInfo($win-data)
    upvar #0 ::hierlist::$var data

    if {$data(nodemenu) == ""} {
	# No contextmenu desired.
	return
    }

    select $win $node

    .hierlistmenu delete 0 last
    foreach win [winfo children .hierlistmenu] {
	::destroy $win
    }
    set id $data($node-coords)
    if {$data(nodeedit) != ""} {
	if {[eval "$data(nodeedit) inquire $id"]} {
	    .hierlistmenu add command -label "Edit" -command \
		    "$data(nodeedit) invoke $id"
	}
    }

    $data(nodemenu) $id .hierlistmenu

    if {[.hierlistmenu index last] != "none"} {
	tk_popup .hierlistmenu $X $Y
    }
}

proc ::hierlist::remove {win node} {
    variable hierInfo

    set var $hierInfo($win-data)
    upvar #0 ::hierlist::$var data

    set left_open ""
    set nodelen [expr [string length $node] - 1]

    foreach n $data(open_nodes) {
	if {[string range $n 0 $nodelen] != $node} {
	    lappend left_open $n
	}
    }
    
    set data(open_nodes) $left_open
    $win.hbox delete "$node:start" "$node:end"
}

proc ::hierlist::update {coord {category "world"}} {
    variable hierInfo

    foreach win $hierInfo(category-$category) {
	update_win $win $coord
    }
}

proc ::hierlist::update_win {win coord} {
    variable hierInfo

    set var $hierInfo($win-data)
    upvar #0 ::hierlist::$var data

    # Search node that has been updated
    set updatednode ""
    foreach node $data(open_nodes) {
	if {"$data($node-coords)" == "$coord"} {
	    set updatednode $node
	    break
	}   
    }
    if {$updatednode == ""} {
	# Nothing to update (not open)
	return
    }
    
    # Speichere liste von offenen Knoten des 
    # geupdateten Knotens ab in der Form:
    #    [list coord fathernode]

    set new_open ""
    set l [expr [string length $updatednode] - 1]

    foreach node $data(open_nodes) {
	if {[string length $node] > $l && \
		$updatednode == [string range $node 0 $l] } {
	    # Offener Sohn  des geupdateten Knotens
	    set father [string range $node 0 [expr [string last - $node] - 1]]
	    lappend new_open [list $data($node-coords) $father]
	}
    }
    set new_open [lsort -index 1 $new_open]

    # Schliee und ffne den geupdateten Knoten einmal.

    remove $win $updatednode
    insert $win $updatednode

    # ffne nun alle vorher offenen _Koordinaten_
    # Aus diesem Grunde sind in obiger Liste auer den Koordinaten noch der
    # Knotenpfad des Vaterknotens gespeichert.

    foreach pair $new_open {
	set coord [lindex $pair 0]
	set fathernode [lindex $pair 1]

	# At this point fathernode must have been opened, because
	# the list new_open is sorted by the fathernode and because
	# the father of each open node must also be open. 
	# The only exception: An open node has been deleted.

	if {! [info exists data($fathernode-children)]} {
	    continue
	}
	
	foreach index $data($fathernode-children) {
	    if {$data($index-coords) == $coord \
		    && [lsearch -exact $data(open_nodes) $index] < 0} {
		if {[nodehaschildren $var $index]} {
		    # Important: Also change bindings and images.
		    expand $win $index
		}
	    }
	}
    }

    set removeselection 0
    if {[catch {
	if {$data(selectednode) != ""} {
	    if {$data(selectedcoord) != $data($data(selectednode)-coords)} {
		# Update changed selected node
		set removeselection 1
	    }
	}
    }]} {
	# Something weird occurred. Must have been that evaluation of
	# $data($data(selectednode)-coords) 
	# caused an error. If that is the case, the selected node disappeard.
	# Tell this to the associated hierarchical application.
	set removeselection 1
    }
    
    if {$removeselection} {
	$win.hbox tag configure $data(selectednode) -background {} -foreground {}
	if {$data(nodeselect) != ""} {
	    eval "$data(nodeselect) unselect $data(selectedcoord)"
	}
	set data(selectednode) ""
    }
    
}


proc ::hierlist::display {win} {
    variable hierInfo
    set var $hierInfo($win-data)

    $win.hbox delete 1.0 end
    $win.hbox mark set pos 1.0
    $win.hbox insert pos "[nodename $var root]" root
    $win.hbox insert pos "\n"
    $win.hbox tag bind root <ButtonPress-1> "::hierlist::select $win root"
    $win.hbox tag bind root <Double-ButtonPress-1> "::hierlist::open $win root"
    $win.hbox tag bind root <ButtonPress-3> "::hierlist::contextmenu $win root %X %Y"

    $win.hbox mark set "root:start" 2.0
    $win.hbox mark gravity "root:start" left
    insert $win "root"
}


proc ::hierlist::expand {win node} {
    variable hierInfo

    set arrow "$win.hbox.arrow-$node"
    set image [$arrow cget -image]
    if {$image == $hierInfo(sideArrow)} {
	$arrow configure -image $hierInfo(downArrow)
	bind $arrow <ButtonPress-1> \
		"::hierlist::collapse $win $node"
	insert $win $node
    }
}

proc ::hierlist::collapse {win node} {
    variable hierInfo

    set arrow "$win.hbox.arrow-$node"
    set image [$arrow cget -image]
    if {$image == $hierInfo(downArrow)} {
	$arrow configure -image $hierInfo(sideArrow)
	bind $arrow <ButtonPress-1> \
		"::hierlist::expand $win $node"

	remove $win $node
    }
}

