# ----------------------------------------------------------------------
# Modul LogWindow
# ----------------------------------------------------------------------
# Procedures to maintain a log window
#
# $Id: logwindow.tcl,v 3.5 2001/01/08 11:51:55 cschmidt Exp $

namespace eval ::log {
    variable log ""
    variable win ""
    variable ticket ""
}

proc ::log::open {} {
    variable win
    variable log
    variable ticket

    set ticket ""

    set title "Log Window"

    # Prevent multiple open log windows.
    set client [::MDIFrame::FindWindow $title]
    if {$client != ""} {
	::MDIFrame::MakeVisible $client
	return
    }

    set total [::MDIFrame::Totalsize]
    set total_x [lindex $total 0]
    set total_y [lindex $total 1]

    set window [::MDIFrame::New_Window $title "" -closecmd "::log::on_close" \
	    -x     [expr $total_x / 2] -y      [expr $total_y - 120] \
	    -width [expr $total_x / 2] -height 120]
    set client [::MDIFrame::FindWindow $title]

    pack $window -expand true -fill both

    frame $window.log

    text $window.log.text -relief sunken \
	    -font -*-courier-medium-r-normal-*-12-*-*-*-*-*-*-* \
	    -yscrollcommand [list $window.log.sb set] \
	    -width 60 -height 5 -wrap none
    scrollbar $window.log.sb -orient vertical -command "$window.log.text yview" \
	    -takefocus 0
    pack $window.log.sb -fill y -side right
    pack $window.log.text -expand true -fill both -side left
    
    frame $window.ctl -relief sunken
    pack $window.ctl -fill x -side bottom
    pack $window.log -expand true -fill both

    button $window.ctl.clear -text "Clear" -command "$window.log.text delete 1.0 end"
    pack $window.ctl.clear -side left -expand true

    set win $window
    
    if {$log != ""} {
	$window.log.text insert 1.0 $log
	set log ""
    }
    oop::setClass [::MDIFrame::GetClient $window] log
}

proc ::log::on_close {} {
    variable log
    variable win

    if {$win != ""} {
	set log [$win.log.text get 1.0 end]
	set win ""
    }
    return 1
}


proc ::log::info {infoClass infoMsg} {
    # Hierhin werden alle Informationen gemeldet, die der Programmierer und/oder Anwender
    # evlt. sehen moechte. Es wird erst hier entschieden, wie sie tatsaechlich
    # ausgegeben werden.
    #
    # Bekannte infoClasses:
    # error, selection, execution, timing, treeModification, processIO, solver

    if { "$infoClass" == "execution" } {
      ::MDIFrame::SetStatus $infoMsg
    }
    puts "...info about $infoClass: $infoMsg"
}

proc ::log::log {args} {
    # Diese Funktion dient zur Meldung  leichter Fehler. Um diese zu erkennen, wird
    # das Log-Fenster automatisch geoeffnet. Die Meldung wird zusaetzlich von
    # der info-Routine bearbeitet.

    variable log
    variable win
    variable ticket

    set line "[join $args { }]\n"

    if {$win != ""} {
	$win.log.text insert end $line
	$win.log.text see end
    } else {
	set log "$log$line"
    }

    if {$ticket == ""} {
	set ticket [after idle {
	    update idletasks
	    ::log::open}]
    }

    info error $line
}

	

