#!/usr/local/bin/wish8.0

puts "Tcl-Version: $tcl_patchLevel Tk-Version: $tk_patchLevel"

proc sourcedir {} {
    set scriptname [info script]
    set scriptdir [file dirname $scriptname]

    # make it an absolute dirname
    set currdir [pwd]
    cd $scriptdir
    set fulldir [pwd]
    cd $currdir

    return $fulldir
}

proc call_init {args} {
    global initfunc

    lappend initfunc $args
}

set my_dir [sourcedir]
set initfunc ""

namespace eval cfg {
    variable libdir                  "$my_dir"
    variable mdilibdir               "$my_dir"
    variable systemmenudelay         350
    variable visualscrolldelay       200
    variable defaultselectcolor      "#8080ff"
    variable defaultresizerfillcolor "#ffffff"

    # The following values may be set differently 
    # later in Function InitGeneratedCode.
    variable mainview                ""
    variable appname                 "VLEdit"                 
    variable apptitle                "VL Editor"   
    variable extension               "vle"
    variable gridsizes               64
    variable defaultgridsize         64
    variable solver_timeout          5000

    if {$::tcl_platform(platform) == "unix" && $::tcl_platform(os) == "SunOS"} {
	variable solver              $my_dir/parcon.solaris.start
    }
    if {$::tcl_platform(platform) == "unix" && $::tcl_platform(os) == "Linux"} {
	variable solver              $my_dir/parcon.linux.start
    }
    if {[info exists solver]} {
	puts "Choosing Constraint-Solver: [file tail $solver]"
    } else {
	puts "No Constraint Solver!"
    }
}
unset my_dir

# Load the generated loadable library

if {[lsearch [package names] "vl"] < 0} {
    # vl module not preloaded.

    set libs ""
    catch {set libs [glob [file join $::cfg::libdir *.vl_module.so]]}
    if {[llength $libs] != 1} {
        error "Could not find the dll"
    }
    load "$libs" vl	
}

proc tk_popup { menu x y {entry ""}} {
    # In der Bibliotheksroutine ist ein Leerzeichen zu wenig!

    global tkPriv
    global tcl_platform
    if {($tkPriv(popup) != "") || ($tkPriv(postedMb) != "")} {
	tkMenuUnpost {}
    }
    tkPostOverPoint $menu $x $y $entry
    if {$tcl_platform(platform) == "unix"} {
	tkSaveGrabInfo $menu
	grab -global $menu
	set tkPriv(popup) $menu
	tk_menuSetFocus $menu;
    }
}

# :vlEnv changes the next line to source all tcl files in :all_tcl

source $::cfg::libdir/BasicObjects.tcl
source $::cfg::libdir/VisConnectionPatternCode.tcl
source $::cfg::libdir/VisConnectionPattern.tcl
source $::cfg::libdir/VisContainerInterfacePtrList.tcl
source $::cfg::libdir/VisContainerPosition2PtrList.tcl
source $::cfg::libdir/VisRelationTypePtrList.tcl
source $::cfg::libdir/VisRelationTypeReferencePtrList.tcl
source $::cfg::libdir/VisConstraintVariablePtrList.tcl
source $::cfg::libdir/VisMovablePtrList.tcl
source $::cfg::libdir/VisCrossAreaPtrList.tcl
source $::cfg::libdir/VisRelationLabelPtrList.tcl
source $::cfg::libdir/VisComplexElementPtrList.tcl
source $::cfg::libdir/gen_dialog.tcl
source $::cfg::libdir/gen_process.tcl
source $::cfg::libdir/gen_view.tcl
source $::cfg::libdir/statechart.specs.inlinedgif.tcl
source $::cfg::libdir/balloonhelp.tcl
source $::cfg::libdir/canvasitems.tcl
source $::cfg::libdir/commondlg.tcl
source $::cfg::libdir/error.tcl
source $::cfg::libdir/event.tcl
source $::cfg::libdir/file.tcl
source $::cfg::libdir/font.tcl
source $::cfg::libdir/hierarchical.tcl
source $::cfg::libdir/iconmanager.tcl
source $::cfg::libdir/logwindow.tcl
source $::cfg::libdir/mdiframe.tcl
source $::cfg::libdir/menuset.tcl
source $::cfg::libdir/oop.tcl
source $::cfg::libdir/output.tcl
source $::cfg::libdir/process.tcl
source $::cfg::libdir/property.tcl
source $::cfg::libdir/solver.tcl
source $::cfg::libdir/treecheck.tcl
source $::cfg::libdir/treeutil.tcl
source $::cfg::libdir/visual_drag.tcl
source $::cfg::libdir/visual_insert.tcl
source $::cfg::libdir/visual_inspoint.tcl
source $::cfg::libdir/visual_move.tcl
source $::cfg::libdir/visual.tcl
source $::cfg::libdir/vlview.tcl

source $::cfg::mdilibdir/mdi.tcl

MDIFrame::Setup
CDlg::Init
hierlist::init
visual::init
event::init
view::init
font::init
treecheck::init
error::init
processing::init
file::init
balloon::init

# Perform Initialization of TCL-Files from Specification
foreach fct $initfunc {
    eval $fct
}
unset initfunc

set ::vl::onLog "::log::log"

tkwait visibility .

if { $argc > 0 } {
  ::file::open [lindex $argv 0]
}

