namespace eval MDIFrame {
}

proc ::MDIFrame::Setup {} {
    
    # First, make a Menu
    
    set mid [Menu::Create .menu]
    variable mainMenu $mid
    
    Menu::Menu $mid &File
    Menu::Command $mid File "&New" {::file::new}
    Menu::Command $mid File "&Open" {::file::openDialog}
    Menu::Separator $mid File
    Menu::Command $mid File "&Save" {::MDIFrame::clientcommand wincmd_save}
    Menu::Command $mid File "Save &as" {::MDIFrame::clientcommand wincmd_saveAs}
    Menu::Command $mid File &Print {
	::CDlg::Information "Can't print whole files! Use view->print to print views."}
    Menu::Separator $mid File
    Menu::Command $mid File "Activate E&rror view" {::MDIFrame::clientcommand wincmd_showErrWindow}
    Menu::Command $mid File "Activate AS&T view" {::MDIFrame::clientcommand wincmd_showAbstree}
    Menu::Separator $mid File
    #Menu::Command $mid File &Exit {::file::saveExit}
    Menu::Command $mid File "&Exit" {::MDIFrame::exit}
    Menu::Command $mid File "Kill application" {exit}
    
    Menu::Menu $mid &Edit
    
    Menu::Menu $mid &View
    Menu::Command $mid View "Print" {::MDIFrame::clientcommand wincmd_print}
    Menu::Command $mid View "Show solver interaction" \
	    {::MDIFrame::clientcommand wincmd_showSolverInteraction}

    Menu::Menu $mid &Process

    Menu::Menu $mid &Window
    Menu::Command $mid Window "Log Window" {::log::open}
    Menu::Command $mid Window "&Minimize all" {::MDIFrame::Minimize_all}
    Menu::Command $mid Window "&Arrange Icons" {::MDIFrame::Arrange_Icons}
    Menu::Command $mid Window "&Restore all" {::MDIFrame::Restore_all}
    Menu::Command $mid Window "&Next" {::MDIFrame::NextWindow}
    Menu::Command $mid Window "&Previous" {::MDIFrame::PrevWindow}
    Menu::Separator $mid Window

    # Make a Status Bar

    frame .mainmessagebar -relief sunken -borderwidth 1
    label .mainmessagebar.text -text " " -foreground black
    pack .mainmessagebar.text -side left -fill both
    frame .mainmessagebar.indicator -width 30 -relief sunken
    label .mainmessagebar.indicator.text -text " " -foreground black
    pack .mainmessagebar.indicator.text -fill both
    pack .mainmessagebar.indicator -side right -fill y
    pack .mainmessagebar -side bottom -fill x

    frame .mdi 
    pack .mdi -side top -fill both -expand yes

    # Initialize MDI Module

    variable master [::MDI::master mdi .mdi -menupath .menu]
    variable nextslave 0

    # Set width / height / position of Frame

    wm geometry . 909x763+1+1

    # For now, create two dummy windows

    #	set slave1 [::MDI::slave  slave1 $master -height 500 -width 700 -closecmd foo]
    #	set slave2 [::MDI::slave  slave2 $master -closecmd bar]

    frame .toolbar -relief raised -borderwidth 1

    button .toolbar.newfile -image $::cfg::img(button_newfile) \
	    -command {::file::new}
    pack .toolbar.newfile -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.newfile "Creates new file"

    button .toolbar.open -image $::cfg::img(button_open) \
	    -command {::file::openDialog}
    pack .toolbar.open -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.open "Loads file from disc"

    button .toolbar.save -image $::cfg::img(button_save) \
	    -command {::MDIFrame::clientcommand wincmd_save}
    pack .toolbar.save -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.save "Saves current file to disc"

    button .toolbar.print -image $::cfg::img(button_print) \
	    -command {::MDIFrame::clientcommand wincmd_print}
    pack .toolbar.print -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.print \
      "Saves content of current view to postscript file"

    button .toolbar.showAbstree -image $::cfg::img(button_tree) \
	    -command {::MDIFrame::clientcommand wincmd_showAbstree}
    pack .toolbar.showAbstree -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.showAbstree \
      "Activates the `abstract structure tree' window of current file"

    button .toolbar.showErrWindow -image $::cfg::img(button_errors) \
	    -command {::MDIFrame::clientcommand wincmd_showErrWindow}
    pack .toolbar.showErrWindow -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.showErrWindow \
      "Activates error window of current file"
  
    pack .toolbar -after .menu -side top -fill x
    SetMainTitle
}

proc ::MDIFrame::exit {} {
    variable master
    ::MDI::closemaster ::${master}::
}


proc ::MDIFrame::clientcommand {cmdname args} {
    variable master
    set client [lindex [GetWindowList] 0]
    if { "$client" == "" } {
	::CDlg::Information "This command needs a current window"
	return
    }
    if { ![oop::understandsMessage $client $cmdname] } {
	::CDlg::Information "Current window cannot handle this command"
	return
    }
    oop::msgArglist $client $cmdname $args
}


proc ::MDIFrame::deleteme_clientcommand {cmdname args} {
    variable master
    set client [lindex [GetWindowList] 0]
    upvar #0 ::${master}::${client} cstruct 

    if {$cstruct(callback) != ""} {
	eval $cstruct(callback) $cmdname $args
    }
}

proc ::MDIFrame::deleteme_SetClientcommand {client cmdname} {
    variable master
    upvar #0 ::${master}::${client} cstruct 
    
    set cstruct(callback) $cmdname
}

proc ::MDIFrame::SetMainTitle {} {
    tk appname $::cfg::appname
    wm title . $::cfg::apptitle
}

proc ::MDIFrame::SetStatus { text } {
    .mainmessagebar.text configure -text $text
}

proc ::MDIFrame::SetIndicator { on {num ""} } {
    if {$on} {
	set text "CS"
	if {$num != ""} {
	    set text "CS: $num"
	}
    } else {
	set text ""
    }
	
    .mainmessagebar.indicator.text configure -text $text
}


proc ::MDIFrame::Totalsize {} {
    variable master

    upvar #0 ::${master}:: mstruct 
    return [list [winfo width $mstruct(path)] [winfo height $mstruct(path)]]
}

proc ::MDIFrame::New_Window { {title "Untitled"} root args} {

    variable master
    variable nextslave

    set clientname "slave[incr nextslave]"
    set newslave [eval {::MDI::slave $clientname $master \
	    -title [MakeTitleUnique $title] -userargs [list $root ""] \
	    -closecmd [list ::MDIFrame::on_close $clientname]} $args]
#      after idle "
#          update idletasks
#          ::MDIFrame::MakeVisible $clientname
#      "
    UpdateWindowMenu

    return $newslave
}


proc ::MDIFrame::on_close { client } {
    
    if [oop::understandsMessage $client "wincmd_closeAfterConfirmation"] {
        if ![oop::msg $client "wincmd_closeAfterConfirmation"] {return 0}
    }

    after idle {
	::MDIFrame::UpdateWindowMenu
    }
    return 1
}

proc ::MDIFrame::GetWindowList { } {
    # Reihenfolge entspricht der Reihenfolge der Fenster.
    variable master
    
    return [::MDI::clients ::${master}::]
}

proc ::MDIFrame::GetSortedWindowList { } {
    variable master
    
    set wlist [::MDI::clients ::${master}::]
    set slist ""
    foreach client $wlist {
	lappend slist [list $client [GetTitle $client]]
    }
    set slist [lsort -index 1 -dictionary $slist]
    set wlist ""
    foreach clienttitle $slist {
	lappend wlist [lindex $clienttitle 0]
    }
    return $wlist
}


proc ::MDIFrame::Arrange_Icons { } {
    variable master

    ::MDI::arrall ::${master}::
}

proc ::MDIFrame::Minimize_all { } {
    variable master

    ::MDI::minall ::${master}::
}

proc ::MDIFrame::Minimize { client } {
    variable master
    
    ::MDI::minclient $client ::${master}::
}

proc ::MDIFrame::Restore_all { } {
    variable master

    ::MDI::resall ::${master}::
}

proc ::MDIFrame::NextWindow { } {
    variable master

    ::MDI::next ::${master}::
}

proc ::MDIFrame::PrevWindow { } {
    variable master

    ::MDI::prev ::${master}::
}

proc ::MDIFrame::GetTitle { client } {
    variable master

    ::MDI::gettitle ::${master}:: $client
}

proc ::MDIFrame::SetTitle { client title } {
    variable master

    ::MDI::settitle ::${master}:: $client $title
    UpdateWindowMenu
}

proc ::MDIFrame::GetCoords { client } {
    variable master
    upvar #0 ::${master}::${client} cstruct

    return [list $cstruct(rootx) $cstruct(rooty) [expr $cstruct(rootx) + $cstruct(width)] [expr $cstruct(rooty) + $cstruct(height)]]
}

proc ::MDIFrame::SetClientpath { client path } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    set cstruct(clientpath) $path
}

proc ::MDIFrame::SetSelectcommand { client command } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    if {$cstruct(userargs) != ""} {
	set cstruct(userargs) [lreplace $cstruct(userargs) 1 1 $command]
    }
}

proc ::MDIFrame::GetRootId { client } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    return [lindex $cstruct(userargs) 0]
}

proc ::MDIFrame::GetSelectcommand { client } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    return [lindex $cstruct(userargs) 1]
}


proc ::MDIFrame::SetDropcommand { client command } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    set cstruct(dropcommand) $command
}

proc ::MDIFrame::GetDropcommand { client } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    if {[info exists cstruct(dropcommand)]} {
	return $cstruct(dropcommand)
    } else {
	return ""
    }
}

proc ::MDIFrame::FindDropTarget { X Y } {
    variable master

    set client [GetClient [winfo containing $X $Y]]
    set cmd [GetDropcommand $client]
    if {$cmd != "" && [IsInside $client $X $Y]} {
	return $cmd
    }
   
}

proc ::MDIFrame::IsInside { client X Y } {
    variable master
    upvar #0 ::${master}:: mstruct ::${master}::${client} cstruct 

    set clx [winfo rootx $cstruct(clientpath)]
    set cly [winfo rooty $cstruct(clientpath)]
    set clw [winfo width $cstruct(clientpath)]
    set clh [winfo height $cstruct(clientpath)]

    return [expr $X >= $clx && $X <= ($clx + $clw) \
	    &&   $Y >= $cly && $Y <= ($cly + $clh)]
}
    
proc ::MDIFrame::GetWindowPath { client } {
    variable master
    upvar #0 ::${master}:: mstruct ::${master}::${client} cstruct 

    return $cstruct(path)
}


proc ::MDIFrame::GetUserpath { client } {
    variable master

    ::MDI::getuserpath ::${master}:: $client
}


proc ::MDIFrame::CloseWindow { client } {
    variable master

    ::MDI::closeclient ::${master}:: $client
}

proc ::MDIFrame::MakeVisible { client } {
    variable master

    if {[::MDI::validclient ::${master}:: $client]} {
	::MDI::mclient $client ::${master}:: norm
	::MDI::setcurrent $client ::${master}::
    }
}

proc ::MDIFrame::UpdateWindowMenu { } {
    variable master
    variable mainMenu

    ::Menu::Delete $mainMenu Window 5
    ::Menu::Separator $mainMenu Window
    
    foreach client [GetSortedWindowList] {
	set title [GetTitle $client]
	::Menu::Command $mainMenu Window $title "::MDIFrame::MakeVisible $client"
    }
}

proc ::MDIFrame::FindWindow { title } {
    
    foreach client [GetWindowList] {
	set window_title [GetTitle $client]
	if {"$title" == "$window_title"} {
	    return $client
	}
    }

    return ""
}

proc ::MDIFrame::GetClient { win } {
    if {[regexp {\.([^.]*)\.([^.]*)\.([^.]*)\..*} $win match mdi1 mdi2 client]} {
	if {$mdi1 == "mdi" && $mdi2 == "mdi"} {
	    return $client
	} else {
	    return ""
	}
    } else {
	return ""
    }
}

proc ::MDIFrame::FindClient { win } {
    
    foreach client [GetWindowList] {
	set userpath [GetUserpath $client]
	if {"$userpath" == "$win"} {
	    return $client
	}
    }

    return ""
}

proc ::MDIFrame::CloseClient { client { force 0}} {
    variable master

    if {$force} {
	::MDI::destroyclient $client ::${master}::
    } else {
	::MDI::closeclient $client ::${master}::
    }
}

proc ::MDIFrame::MakeTitleUnique { title } {
    if {[FindWindow $title] == ""} {
	return $title
    }
    
    set no 1
    set new_title "$title:$no"
    while {[FindWindow $new_title] != ""} {
	incr no
	set new_title "$title:$no"
    }

    return $new_title
}

proc ::MDIFrame::GetWidth {} {
    winfo width .mdi.mdi
}

proc ::MDIFrame::GetHeight {} {
    winfo height .mdi.mdi
}

