# $Id: menuset.tcl,v 3.1.1.1 1999/07/05 14:10:18 mjung Exp $
# Ein Men-Packet aus "Praktisches Programmieren in Tcl und Tk"
# von Brent Welch
#
# Ergnzung von Matthias Jung: Hotkeys fr Menues werden durch
# vorangestelltes '&'-Zeichen spezifiziert (wie bei Windows)

# ----------------------------------------------------------------------
# MenuSetup menubar
#
# menubar:     Tk-Widget-Name of Menubar
#
namespace eval ::Menu {

    variable Menu
    set Menu(menuid) 0

    proc Create { menubar } {

	variable Menu

	set newmenu $Menu(menuid)
	incr Menu(menuid)

	frame $menubar -relief raised -borderwidth 1
	pack $menubar -side top  -fill x
	set Menu($newmenu,menubar) $menubar
	set Menu($newmenu,uid) 0
	
	return $newmenu
    }

    # ----------------------------------------------------------------------
    # MenuMakeLabel
    # 
    # label:     Name of a variable of the calling procedure deonting
    #            a menu label. From this variable the character '&'
    #            is removed.
    # Returns the index of the removed character or '-1'
    
    proc MakeLabel { label } {
	upvar $label lab

	set u [string first "&" "$lab"]
	set len_1 [expr [string length "$lab"] - 1]
	set prefix [string range "$lab" 0 [expr $u - 1]]
	set postfix [string range "$lab" [expr $u + 1] $len_1]
	set lab "$prefix$postfix"
	return $u
    }

    # ----------------------------------------------------------------------
    # Menu label
    #
    # label:       Name of Menu as displayed on screen.
    #
    
    proc Menu { mid label } {

	variable Menu

	set u [MakeLabel label]
	if [info exists Menu($mid,menu,$label)] {
	    error "Menu $label already defined"
	}

	set name $Menu($mid,menubar).mb$Menu($mid,uid)
	set menuName $name.menu
	incr Menu($mid,uid)
	set mb [menubutton $name -text $label -menu $menuName -underline $u]
	if {$label == "Help" || $label == "Hilfe"} {
	    pack $mb -side right
	} else {
	    pack $mb -side left
	}
	set menu [menu $menuName -tearoff 0]
	set Menu($mid,menu,$label) $menu
    }

    # ----------------------------------------------------------------------
    # MenuCascade menuName label
    #
    # menuName:    Name of Menu (as created with 'Menu') in which
    #              submenu should be entered.
    # label:       Name of submenu as displayed on screen
    
    proc Cascade { mid menuName label } {
	variable Menu
	
	if [catch {set Menu($mid,menu,$menuName)} menu] {
	    error "No such menu: $menuName"
	}
	
	set u [MakeLabel label]
	if [info exists Menu($mid,menu,$label)] {
	    error "Menu $label already defined"
	}
	
	set sub $menu.sub$Menu($mid,uid)
	incr Menu($mid,uid)
	menu $sub -tearoff 0
	$menu add cascade -label $label -underline $u -menu $sub
	set Menu($mid,menu,$label) $sub
    }
    
    # ----------------------------------------------------------------------
    # MenuCommand menuName label command
    #
    # menuName:    Name of Menu (as created with 'Menu') in which
    #              command should be entered.
    # label:       Name of Menuentry as displayed on screen
    # command:     command to be issued if Entry is selected
    #
    
    proc Command { mid menuName label command } {

	variable Menu
	
	if [catch {set Menu($mid,menu,$menuName)} menu] {
	    error "No such menu: $menuName"
	}
	set u [MakeLabel label]
	$menu add command -label $label -command $command -underline $u
    }

    # ----------------------------------------------------------------------
    # MenuCheck menuName label var ?command?
    #
    # menuName:    Name of Menu (as created with 'Menu') in which
    #              checkbox-entry should be entered.
    # label:       Name of Menuentry as displayed on screen
    # var:         variable that reflects the beeing checked state
    #              of the entry. This is a global variable.
    # command:     command to be issued if Entry is selected, if desired.
    #
    
    proc Check { mid menuName label var { command {} } } {
	
	variable Menu
    
	if [catch {set Menu($mid,menu,$menuName)} menu] {
	    error "No such menu: $menuName"
	}
	set u [MakeLabel label]
	$menu add check -label $label -variable $var -command $command \
		-underline $u
    }

    # ----------------------------------------------------------------------
    # MenuRadio menuName label var ?val? ?command?
    #
    # menuName:    Name of Menu (as created with 'Menu') in which
    #              radio-button submenu should be entered.
    # label:       Name of Menuentry as displayed on screen
    # var:         variable that contains the current selection
    #              of the entry. This is a global variable.
    # val:         Initial state of var, defaults to $label.
    # command:     command to be issued if Entry is selected, if desired.
    #

    proc Radio { mid menuName label var { val {} } { command {} } } {
	
	variable Menu
	
	if [catch {set Menu($mid,menu,$menuName)} menu] {
	    error "No such menu: $menuName"
	}
	if {[string length $val] == 0} {
	    set val $label
	}
	
	set u [MakeLabel label]
	
	$menu add radio -label $label -variable $var -value $val \
		-command $command -underline $u
    }

    # ----------------------------------------------------------------------
    # MenuSeparator menuName
    #
    # menuName:    Name of Menu (as created with 'Menu') in which
    #              separator should be entered.
    #
    
    proc Separator { mid menuName } {

	variable Menu
	
	if [catch {set Menu($mid,menu,$menuName)} menu] {
	    error "No such menu: $menuName"
	}
	
	$menu add separator 
    }

    # ----------------------------------------------------------------------
    # MenuBind what sequence menuName label
    # Bindet einen Menbefehl an ein Ereignis.
    #
    # what:        Tag, an das die Aktion zu binden ist.
    # sequence:    Das Ereignis, an das die Aktion gebunden werden soll.
    # menuName:    Name des Men's in dem der Meneintrag gefunden werden
    #              soll.
    # label:       Name eines Meneintrags, dessen Befehl an die Aktion
    #              gebunden werden soll.
    
    proc Bind { mid what sequence menuName label } {
	variable Menu
	
	if [catch {set Menu($mid,menu,$menuName)} menu] {
	    error "No such menu: $menuName"
	}
	
	if [catch {$menu index $label} index] {
	    error "$label not in menu $menuName"
	}
	
	set command [$menu entrycget $index -command]
	bind $what $sequence $command
	$menu entryconfigure $index -accelerator $sequence
    }

    # ----------------------------------------------------------------------
    # MenuState menuName label newstate
    # 
    # menuName:    Name of Menu (as created with 'Menu') in which
    #              radio-button submenu should be entered.
    # label:       Name of Menuentry as displayed on screen
    # newstate:    normal oder disabled
    
    proc State { mid menuName label newstate } {
	
	variable Menu
	
	if [catch {set Menu($mid,menu,$menuName)} menu] {
	    error "No such menu: $menuName"
	}
	
	if [catch {$menu index $label} index] {
	    error "$label not in menu $menuName"
	}
	
	$menu entryconfigure $index -state $newstate
    }

    proc Delete { mid menuName firstindex {lastindex "last"}} {
	
	variable Menu
	
	if [catch {set Menu($mid,menu,$menuName)} menu] {
	    error "No such menu: $menuName"
	}

	$menu delete $firstindex $lastindex
    }
}
