
# Dieses Modul verwaltet Properties vom Typ VLValue.
# Eine Besonderheit ist, dass dadurch Informationen vom Check-Attributauswerter
# zu anderen Attributauswertern transportiert werden koennen.
#
#
# Auf die Properties wird durch setProperty/getProperty zugegriffen.
#
# Der momentane Zustand wird durch die Funktionen preCheck/postCheck/preOtherRun/postOtherRun festgelegt.
#
# Jeder Check-Durchlauf startet auf einer leeren Property-Menge
# Jeder andere Durchlauf startet auf der Property-Menge des letzten updateProperties-Aufrufs
#
# Vorgesehener Aufrufsequenz:
# (  (preCheck (setProperty|getProperty)* postCheck)* 
#    update
#    (preOtherRun (setProperty|getProperty)* postOtherRun)*
# )*
#
# Durch die Funktion "update" werden Sichten zum Update vorgemerkt, die aufgrund von
# geaenderten Properties ungueltig geworden sind. Dazu wird protokolliert, auf welche
# Properties diese Sicht beim letzten Durchlauf zugrgriffen hat.
#
#
# BUG: Es wird kein Update ausgeloest, wenn durch eine
# Aenderung von Querreferenzen ein Neuzeichnen notwendig ist!



namespace eval ::property {
}

# Datenstruktur:
# savedProp(key$,$name): Property-Wert
# prop($key,$name): Property-Wert
# watching($key,$name): Liste mit Fenstern, die ggf. upgedatet werden muessen
# logmsg: Liste mit Meldungen, die nur fuer den letzten Check-Durchlauf relevant sind
# status: inCheck, inOtherRun

array set ::property::prop {}
array set ::property::savedProp {}
array set ::property::watching {}


proc ::property::preCheck {} {
    variable prop
    variable status
    variable logmsg
    array set prop {}
    set status inCheck
    set logmsg {}
}


proc ::property::postCheck {} {
    variable status
    unset status
}


proc ::property::update {} {
    variable watching
    variable prop
    variable savedProp
    variable status
    foreach x [array names watching] {
        if {    
            [info exists prop($x)] && ![info exists savedProp($x)]
            || ![info exists prop($x)] && [info exists savedProp($x)]
            || [info exists prop($x)] && [info exists savedProp($x)] && $prop($x)!=$savedProp($x) 
        } {
            foreach win $watching($x) {
                set update($win) 1
            }
        }
    }
    array set savedProp [array get prop]
    foreach win [array names update] {
        if [info exists ::visual::visInfo($win-data)] {
            ::visual::update_after_idle $win
        }
    }
    array set watching {}

    # Jetzt noch Meldungen des letzten Check-Durchlaufs ausgeben
    variable logmsg
    foreach msg $logmsg {
        ::log::log $msg
    }
}


proc ::property::preOtherRun {} {
    variable prop
    variable savedProp
    variable status
    set status inOtherRun
    array set prop [array get savedProp]
}


proc ::property::postOtherRun {} {
    variable status
    unset status
}


proc ::property::getProperty {key name args} {
    variable watching
    variable prop
    variable status

    if {"$status"=="inOtherRun"} {
        upvar #0 $::visual::currVarname data
        # by cs: bug!? Was passiert bei processor-laeufen?
        lappend watching($key,$name) $data(windowid)
    }
    if [info exists prop($key,$name)] {
        return $prop($key,$name)
    }

    # Property ist undefiniert!
    if [llength $args] {
        return [lindex $args 0]
    }
    if {"$status"=="inOtherRun"} {
        ::log::log "OtherRun: Property $key,$name is not defined!"
    } else {
        variable logmsg
        lappend logmsg "Final Check: Property $key,$name is not defined!"
    }
    return "undefined"
}


proc ::property::setProperty {key name value} {
    variable prop
    set prop($key,$name) $value
}





