#include <stdio.h>
#include <stdlib.h>

#include "ptg_gen.h"
#include "obstack.h"

/* -------------------------------------------------------- */
/*                      Memory Management                   */
/* -------------------------------------------------------- */

static Obstack _PTGObstack;
static void *_PTGFirstObj = NULL;

static void _PTGInit()
{
	if (_PTGFirstObj)
		return;
	obstack_init(&_PTGObstack);
	_PTGFirstObj = obstack_alloc(&_PTGObstack, 0);
}

void PTGFree()
{
	obstack_free(&_PTGObstack, _PTGFirstObj);
	_PTGFirstObj = obstack_alloc(&_PTGObstack, 0);
}


#if defined(__STDC__) || defined(__cplusplus)
static void *MALLOC(int size)
#else
static void *MALLOC(size) int size;
#endif
{
	if (! _PTGFirstObj)
		_PTGInit();
	return (obstack_alloc(&_PTGObstack, size));
}

/* -------------------------------------------------------- */
/*                       Output-functions                   */
/* -------------------------------------------------------- */

static PTG_OUTPUT_FILE f;
static char buffer[40];

#if PTG_OUTPUT_DEFAULT
#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGOut (PTGNode r)
#else
PTGNode PTGOut (r)
	PTGNode r;
#endif
{
	f = stdout;
	if (r) {
		(* (r->_print)) (r);
	}
	return (r);
}

#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGOutFile (char *fn, PTGNode r)
#else
PTGNode PTGOutFile (fn, r)
	char * fn; PTGNode r;
#endif
{
	if ((f = fopen(fn, "w"))  == (FILE *)NULL)
	{
		fprintf(stderr, "ERROR: PTGOutFile: output file '%s' can't be opened.\n",fn);
		exit(1);
	}
	if (r)
		(* (r->_print)) (r);
	fclose(f);
	return (r);
}

#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGOutFPtr(FILE *fptr, PTGNode r)
#else
PTGNode PTGOutFPtr(fptr, r)
	FILE *fptr; PTGNode r;
#endif
{
	if ((f = fptr) == (FILE *)NULL)
	{
		fprintf(stderr, "ERROR: PTGOutFPtr: output file not open.\n");
		exit(1);
	}
	if (r)
		(* (r->_print)) (r);
	return (r);
}

#endif

#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGProcess(PTG_OUTPUT_FILE file, PTGNode r)
#else
PTGNode PTGProcess(file, r)
	PTG_OUTPUT_FILE file; PTGNode r;
#endif
{
	f = file;
	if (r)
		(* (r->_print)) (r);
	return (r);
}
/* -------------------------------------------------------- */
/*                            PTGNULL                       */
/* -------------------------------------------------------- */

/* Define PTGNULL as a PTGNode that prints nothing. */

#if defined(__STDC__) || defined(__cplusplus)
static void _PrPTGNULL(_PPTG0 n)
#else
static void _PrPTGNULL(n)
_PPTG0 n;
#endif
{(void)n; /* function printing nothing */}

struct _SPTG0   _PTGNULL = { (_PTGProc) _PrPTGNULL };

/* -------------------------------------------------------- */
/*          Node-Construction and Print-functions           */
/* -------------------------------------------------------- */


/* Implementation of Pattern NodePtr */

typedef struct _SPTGNodePtr{
	_PTGProc _print;
	void * p1;
} * _PPTGNodePtr;

#ifdef PROTO_OK
static void _PrPTGNodePtr(_PPTGNodePtr n)
#else
static void _PrPTGNodePtr(n)
	_PPTGNodePtr n;
#endif
{
	BasicCheckPrintNodeptr(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGNodePtr(void * p1)
#else
PTGNode PTGNodePtr(p1)
void * p1;
#endif
{
	_PPTGNodePtr n;

	n = (_PPTGNodePtr)MALLOC(sizeof(struct _SPTGNodePtr));
	n->_print = (_PTGProc)_PrPTGNodePtr;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Seq */

typedef struct _SPTGSeq{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGSeq;

#ifdef PROTO_OK
static void _PrPTGSeq(_PPTGSeq n)
#else
static void _PrPTGSeq(n)
	_PPTGSeq n;
#endif
{
	n->p1->_print(n->p1);
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGSeq(PTGNode p1, PTGNode p2)
#else
PTGNode PTGSeq(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGSeq n;

	n = (_PPTGSeq)MALLOC(sizeof(struct _SPTGSeq));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGSeq;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern SetAttr */

typedef struct _SPTGSetAttr{
	_PTGProc _print;
	void * p1;
	char * p2;
	char * p3;
} * _PPTGSetAttr;

#ifdef PROTO_OK
static void _PrPTGSetAttr(_PPTGSetAttr n)
#else
static void _PrPTGSetAttr(n)
	_PPTGSetAttr n;
#endif
{
	PTG_OUTPUT_STRING(f, "set node ");
	BasicCheckPrintNodeptr(f, n->p1);
	PTG_OUTPUT_STRING(f, " \n");
	PTG_OUTPUT_STRING(f, "::treeutil::setAttr $node {");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "} {");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "} \n");
}

#ifdef PROTO_OK
PTGNode PTGSetAttr(void * p1, char * p2, char * p3)
#else
PTGNode PTGSetAttr(p1, p2, p3)
void * p1;
char * p2;
char * p3;
#endif
{
	_PPTGSetAttr n;

	n = (_PPTGSetAttr)MALLOC(sizeof(struct _SPTGSetAttr));
	n->_print = (_PTGProc)_PrPTGSetAttr;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern SetAttr2 */

typedef struct _SPTGSetAttr2{
	_PTGProc _print;
	void * p1;
	char * p2;
	PTGNode p3;
} * _PPTGSetAttr2;

#ifdef PROTO_OK
static void _PrPTGSetAttr2(_PPTGSetAttr2 n)
#else
static void _PrPTGSetAttr2(n)
	_PPTGSetAttr2 n;
#endif
{
	PTG_OUTPUT_STRING(f, "set node ");
	BasicCheckPrintNodeptr(f, n->p1);
	PTG_OUTPUT_STRING(f, " \n");
	PTG_OUTPUT_STRING(f, "::treeutil::setAttr $node {");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "} {");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "} \n");
}

#ifdef PROTO_OK
PTGNode PTGSetAttr2(void * p1, char * p2, PTGNode p3)
#else
PTGNode PTGSetAttr2(p1, p2, p3)
void * p1;
char * p2;
PTGNode p3;
#endif
{
	_PPTGSetAttr2 n;

	n = (_PPTGSetAttr2)MALLOC(sizeof(struct _SPTGSetAttr2));
	n->_print = (_PTGProc)_PrPTGSetAttr2;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern InsertNode */

typedef struct _SPTGInsertNode{
	_PTGProc _print;
	PTGNode p1;
	int p2;
	PTGNode p3;
} * _PPTGInsertNode;

#ifdef PROTO_OK
static void _PrPTGInsertNode(_PPTGInsertNode n)
#else
static void _PrPTGInsertNode(n)
	_PPTGInsertNode n;
#endif
{
	PTG_OUTPUT_STRING(f, "set node ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " \n");
	PTG_OUTPUT_STRING(f, "::treeutil::insertNode ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, " $node \n");
}

#ifdef PROTO_OK
PTGNode PTGInsertNode(PTGNode p1, int p2, PTGNode p3)
#else
PTGNode PTGInsertNode(p1, p2, p3)
PTGNode p1;
int p2;
PTGNode p3;
#endif
{
	_PPTGInsertNode n;

	n = (_PPTGInsertNode)MALLOC(sizeof(struct _SPTGInsertNode));
	n->_print = (_PTGProc)_PrPTGInsertNode;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern InsertAndConnectNode2 */

typedef struct _SPTGInsertAndConnectNode2{
	_PTGProc _print;
	PTGNode p1;
	int p2;
	PTGNode p3;
	char * p4;
	PTGNode p5;
} * _PPTGInsertAndConnectNode2;

#ifdef PROTO_OK
static void _PrPTGInsertAndConnectNode2(_PPTGInsertAndConnectNode2 n)
#else
static void _PrPTGInsertAndConnectNode2(n)
	_PPTGInsertAndConnectNode2 n;
#endif
{
	PTG_OUTPUT_STRING(f, "set node ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " \n");
	PTG_OUTPUT_STRING(f, "::treeutil::insertNode ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, " $node \n");
	PTG_OUTPUT_STRING(f, "::treeutil::setSubAttr $node {");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "} {");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "} \n");
}

#ifdef PROTO_OK
PTGNode PTGInsertAndConnectNode2(PTGNode p1, int p2, PTGNode p3, char * p4, PTGNode p5)
#else
PTGNode PTGInsertAndConnectNode2(p1, p2, p3, p4, p5)
PTGNode p1;
int p2;
PTGNode p3;
char * p4;
PTGNode p5;
#endif
{
	_PPTGInsertAndConnectNode2 n;

	n = (_PPTGInsertAndConnectNode2)MALLOC(sizeof(struct _SPTGInsertAndConnectNode2));
	n->_print = (_PTGProc)_PrPTGInsertAndConnectNode2;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern RemoveNode */

typedef struct _SPTGRemoveNode{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGRemoveNode;

#ifdef PROTO_OK
static void _PrPTGRemoveNode(_PPTGRemoveNode n)
#else
static void _PrPTGRemoveNode(n)
	_PPTGRemoveNode n;
#endif
{
	PTG_OUTPUT_STRING(f, "::treeutil::removeNode ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " \n");
}

#ifdef PROTO_OK
PTGNode PTGRemoveNode(PTGNode p1)
#else
PTGNode PTGRemoveNode(p1)
PTGNode p1;
#endif
{
	_PPTGRemoveNode n;

	n = (_PPTGRemoveNode)MALLOC(sizeof(struct _SPTGRemoveNode));
	n->_print = (_PTGProc)_PrPTGRemoveNode;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ExtractNode */

typedef struct _SPTGExtractNode{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGExtractNode;

#ifdef PROTO_OK
static void _PrPTGExtractNode(_PPTGExtractNode n)
#else
static void _PrPTGExtractNode(n)
	_PPTGExtractNode n;
#endif
{
	PTG_OUTPUT_STRING(f, "::vl::extract ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGExtractNode(PTGNode p1)
#else
PTGNode PTGExtractNode(p1)
PTGNode p1;
#endif
{
	_PPTGExtractNode n;

	n = (_PPTGExtractNode)MALLOC(sizeof(struct _SPTGExtractNode));
	n->_print = (_PTGProc)_PrPTGExtractNode;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ExtractSuperNode */

typedef struct _SPTGExtractSuperNode{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGExtractSuperNode;

#ifdef PROTO_OK
static void _PrPTGExtractSuperNode(_PPTGExtractSuperNode n)
#else
static void _PrPTGExtractSuperNode(n)
	_PPTGExtractSuperNode n;
#endif
{
	PTG_OUTPUT_STRING(f, "::treeutil::extractSuperNode ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGExtractSuperNode(PTGNode p1)
#else
PTGNode PTGExtractSuperNode(p1)
PTGNode p1;
#endif
{
	_PPTGExtractSuperNode n;

	n = (_PPTGExtractSuperNode)MALLOC(sizeof(struct _SPTGExtractSuperNode));
	n->_print = (_PTGProc)_PrPTGExtractSuperNode;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern RemoveSuperNode */

typedef struct _SPTGRemoveSuperNode{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGRemoveSuperNode;

#ifdef PROTO_OK
static void _PrPTGRemoveSuperNode(_PPTGRemoveSuperNode n)
#else
static void _PrPTGRemoveSuperNode(n)
	_PPTGRemoveSuperNode n;
#endif
{
	PTG_OUTPUT_STRING(f, "::treeutil::removeSuperNode ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGRemoveSuperNode(PTGNode p1)
#else
PTGNode PTGRemoveSuperNode(p1)
PTGNode p1;
#endif
{
	_PPTGRemoveSuperNode n;

	n = (_PPTGRemoveSuperNode)MALLOC(sizeof(struct _SPTGRemoveSuperNode));
	n->_print = (_PTGProc)_PrPTGRemoveSuperNode;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern CreateNode */

typedef struct _SPTGCreateNode{
	_PTGProc _print;
	char * p1;
} * _PPTGCreateNode;

#ifdef PROTO_OK
static void _PrPTGCreateNode(_PPTGCreateNode n)
#else
static void _PrPTGCreateNode(n)
	_PPTGCreateNode n;
#endif
{
	PTG_OUTPUT_STRING(f, "[::vl::create $::vl::prod(");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ")]");
}

#ifdef PROTO_OK
PTGNode PTGCreateNode(char * p1)
#else
PTGNode PTGCreateNode(p1)
char * p1;
#endif
{
	_PPTGCreateNode n;

	n = (_PPTGCreateNode)MALLOC(sizeof(struct _SPTGCreateNode));
	n->_print = (_PTGProc)_PrPTGCreateNode;
	n->p1 = p1;
	return (PTGNode)n;
}

/* No Implementation of Pattern Concat needed */
/* See Seq for implementation */

/* Implementation of Pattern Constraint_Equal */

typedef struct _SPTGConstraint_Equal{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGConstraint_Equal;

#ifdef PROTO_OK
static void _PrPTGConstraint_Equal(_PPTGConstraint_Equal n)
#else
static void _PrPTGConstraint_Equal(n)
	_PPTGConstraint_Equal n;
#endif
{
	PTG_OUTPUT_STRING(f, "add  +1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " = 0\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_Equal(PTGNode p1, PTGNode p2)
#else
PTGNode PTGConstraint_Equal(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGConstraint_Equal n;

	n = (_PPTGConstraint_Equal)MALLOC(sizeof(struct _SPTGConstraint_Equal));
	n->_print = (_PTGProc)_PrPTGConstraint_Equal;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_RedefineVariable */

typedef struct _SPTGConstraint_RedefineVariable{
	_PTGProc _print;
	char * p1;
	int p2;
} * _PPTGConstraint_RedefineVariable;

#ifdef PROTO_OK
static void _PrPTGConstraint_RedefineVariable(_PPTGConstraint_RedefineVariable n)
#else
static void _PrPTGConstraint_RedefineVariable(n)
	_PPTGConstraint_RedefineVariable n;
#endif
{
	PTG_OUTPUT_STRING(f, "new ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "value ");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_RedefineVariable(char * p1, int p2)
#else
PTGNode PTGConstraint_RedefineVariable(p1, p2)
char * p1;
int p2;
#endif
{
	_PPTGConstraint_RedefineVariable n;

	n = (_PPTGConstraint_RedefineVariable)MALLOC(sizeof(struct _SPTGConstraint_RedefineVariable));
	n->_print = (_PTGProc)_PrPTGConstraint_RedefineVariable;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_SetVariable */

typedef struct _SPTGConstraint_SetVariable{
	_PTGProc _print;
	PTGNode p1;
	int p2;
} * _PPTGConstraint_SetVariable;

#ifdef PROTO_OK
static void _PrPTGConstraint_SetVariable(_PPTGConstraint_SetVariable n)
#else
static void _PrPTGConstraint_SetVariable(n)
	_PPTGConstraint_SetVariable n;
#endif
{
	PTG_OUTPUT_STRING(f, "set ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_SetVariable(PTGNode p1, int p2)
#else
PTGNode PTGConstraint_SetVariable(p1, p2)
PTGNode p1;
int p2;
#endif
{
	_PPTGConstraint_SetVariable n;

	n = (_PPTGConstraint_SetVariable)MALLOC(sizeof(struct _SPTGConstraint_SetVariable));
	n->_print = (_PTGProc)_PrPTGConstraint_SetVariable;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern C_SetRadius */

typedef struct _SPTGC_SetRadius{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGC_SetRadius;

#ifdef PROTO_OK
static void _PrPTGC_SetRadius(_PPTGC_SetRadius n)
#else
static void _PrPTGC_SetRadius(n)
	_PPTGC_SetRadius n;
#endif
{
	PTG_OUTPUT_STRING(f, "add  +2 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "  = 0\n");
}

#ifdef PROTO_OK
PTGNode PTGC_SetRadius(PTGNode p1, PTGNode p2)
#else
PTGNode PTGC_SetRadius(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGC_SetRadius n;

	n = (_PPTGC_SetRadius)MALLOC(sizeof(struct _SPTGC_SetRadius));
	n->_print = (_PTGProc)_PrPTGC_SetRadius;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern ObjectCenter1 */

typedef struct _SPTGObjectCenter1{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGObjectCenter1;

#ifdef PROTO_OK
static void _PrPTGObjectCenter1(_PPTGObjectCenter1 n)
#else
static void _PrPTGObjectCenter1(n)
	_PPTGObjectCenter1 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add  +2 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -2 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " = 0\n");
}

#ifdef PROTO_OK
PTGNode PTGObjectCenter1(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGObjectCenter1(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGObjectCenter1 n;

	n = (_PPTGObjectCenter1)MALLOC(sizeof(struct _SPTGObjectCenter1));
	n->_print = (_PTGProc)_PrPTGObjectCenter1;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern ObjectCenter2 */

typedef struct _SPTGObjectCenter2{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	int p3;
} * _PPTGObjectCenter2;

#ifdef PROTO_OK
static void _PrPTGObjectCenter2(_PPTGObjectCenter2 n)
#else
static void _PrPTGObjectCenter2(n)
	_PPTGObjectCenter2 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add  +2 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -2 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " = ");
	PTG_OUTPUT_INT(f, n->p3);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGObjectCenter2(PTGNode p1, PTGNode p2, int p3)
#else
PTGNode PTGObjectCenter2(p1, p2, p3)
PTGNode p1;
PTGNode p2;
int p3;
#endif
{
	_PPTGObjectCenter2 n;

	n = (_PPTGObjectCenter2)MALLOC(sizeof(struct _SPTGObjectCenter2));
	n->_print = (_PTGProc)_PrPTGObjectCenter2;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_DisjunctRectangles1 */

typedef struct _SPTGConstraint_DisjunctRectangles1{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	int p3;
	int p4;
	PTGNode p5;
	PTGNode p6;
	int p7;
	int p8;
} * _PPTGConstraint_DisjunctRectangles1;

#ifdef PROTO_OK
static void _PrPTGConstraint_DisjunctRectangles1(_PPTGConstraint_DisjunctRectangles1 n)
#else
static void _PrPTGConstraint_DisjunctRectangles1(n)
	_PPTGConstraint_DisjunctRectangles1 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add2  +1 ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add| +1 ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add| +1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p7);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add| +1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p8);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_DisjunctRectangles1(PTGNode p1, PTGNode p2, int p3, int p4, PTGNode p5, PTGNode p6, int p7, int p8)
#else
PTGNode PTGConstraint_DisjunctRectangles1(p1, p2, p3, p4, p5, p6, p7, p8)
PTGNode p1;
PTGNode p2;
int p3;
int p4;
PTGNode p5;
PTGNode p6;
int p7;
int p8;
#endif
{
	_PPTGConstraint_DisjunctRectangles1 n;

	n = (_PPTGConstraint_DisjunctRectangles1)MALLOC(sizeof(struct _SPTGConstraint_DisjunctRectangles1));
	n->_print = (_PTGProc)_PrPTGConstraint_DisjunctRectangles1;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_DisjunctRectangles23 */

typedef struct _SPTGConstraint_DisjunctRectangles23{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
	int p7;
	int p8;
	int p9;
} * _PPTGConstraint_DisjunctRectangles23;

#ifdef PROTO_OK
static void _PrPTGConstraint_DisjunctRectangles23(_PPTGConstraint_DisjunctRectangles23 n)
#else
static void _PrPTGConstraint_DisjunctRectangles23(n)
	_PPTGConstraint_DisjunctRectangles23 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add2  +1 ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p9);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add| +1 ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p9);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add| +1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p7);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add| +1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p8);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_DisjunctRectangles23(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, int p7, int p8, int p9)
#else
PTGNode PTGConstraint_DisjunctRectangles23(p1, p2, p3, p4, p5, p6, p7, p8, p9)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
int p7;
int p8;
int p9;
#endif
{
	_PPTGConstraint_DisjunctRectangles23 n;

	n = (_PPTGConstraint_DisjunctRectangles23)MALLOC(sizeof(struct _SPTGConstraint_DisjunctRectangles23));
	n->_print = (_PTGProc)_PrPTGConstraint_DisjunctRectangles23;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	n->p9 = p9;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_DisjunctRectangles4 */

typedef struct _SPTGConstraint_DisjunctRectangles4{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
	PTGNode p7;
	PTGNode p8;
	int p9;
} * _PPTGConstraint_DisjunctRectangles4;

#ifdef PROTO_OK
static void _PrPTGConstraint_DisjunctRectangles4(_PPTGConstraint_DisjunctRectangles4 n)
#else
static void _PrPTGConstraint_DisjunctRectangles4(n)
	_PPTGConstraint_DisjunctRectangles4 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add2  +1 ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p9);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add| +1 ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p9);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add| +1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p7->_print(n->p7);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p9);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add| +1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p8->_print(n->p8);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p9);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_DisjunctRectangles4(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7, PTGNode p8, int p9)
#else
PTGNode PTGConstraint_DisjunctRectangles4(p1, p2, p3, p4, p5, p6, p7, p8, p9)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
PTGNode p7;
PTGNode p8;
int p9;
#endif
{
	_PPTGConstraint_DisjunctRectangles4 n;

	n = (_PPTGConstraint_DisjunctRectangles4)MALLOC(sizeof(struct _SPTGConstraint_DisjunctRectangles4));
	n->_print = (_PTGProc)_PrPTGConstraint_DisjunctRectangles4;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	n->p9 = p9;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_IncludingRectangles1 */

typedef struct _SPTGConstraint_IncludingRectangles1{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	int p3;
	int p4;
	int p5;
	int p6;
} * _PPTGConstraint_IncludingRectangles1;

#ifdef PROTO_OK
static void _PrPTGConstraint_IncludingRectangles1(_PPTGConstraint_IncludingRectangles1 n)
#else
static void _PrPTGConstraint_IncludingRectangles1(n)
	_PPTGConstraint_IncludingRectangles1 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  -1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p5);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p6);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_IncludingRectangles1(PTGNode p1, PTGNode p2, int p3, int p4, int p5, int p6)
#else
PTGNode PTGConstraint_IncludingRectangles1(p1, p2, p3, p4, p5, p6)
PTGNode p1;
PTGNode p2;
int p3;
int p4;
int p5;
int p6;
#endif
{
	_PPTGConstraint_IncludingRectangles1 n;

	n = (_PPTGConstraint_IncludingRectangles1)MALLOC(sizeof(struct _SPTGConstraint_IncludingRectangles1));
	n->_print = (_PTGProc)_PrPTGConstraint_IncludingRectangles1;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_IncludingRectangles2 */

typedef struct _SPTGConstraint_IncludingRectangles2{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	int p5;
	int p6;
	int p7;
	int p8;
} * _PPTGConstraint_IncludingRectangles2;

#ifdef PROTO_OK
static void _PrPTGConstraint_IncludingRectangles2(_PPTGConstraint_IncludingRectangles2 n)
#else
static void _PrPTGConstraint_IncludingRectangles2(n)
	_PPTGConstraint_IncludingRectangles2 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p5);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p6);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p7);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p8);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_IncludingRectangles2(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, int p5, int p6, int p7, int p8)
#else
PTGNode PTGConstraint_IncludingRectangles2(p1, p2, p3, p4, p5, p6, p7, p8)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
int p5;
int p6;
int p7;
int p8;
#endif
{
	_PPTGConstraint_IncludingRectangles2 n;

	n = (_PPTGConstraint_IncludingRectangles2)MALLOC(sizeof(struct _SPTGConstraint_IncludingRectangles2));
	n->_print = (_PTGProc)_PrPTGConstraint_IncludingRectangles2;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_IncludingRectangles3 */

typedef struct _SPTGConstraint_IncludingRectangles3{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	int p5;
	int p6;
	int p7;
	int p8;
} * _PPTGConstraint_IncludingRectangles3;

#ifdef PROTO_OK
static void _PrPTGConstraint_IncludingRectangles3(_PPTGConstraint_IncludingRectangles3 n)
#else
static void _PrPTGConstraint_IncludingRectangles3(n)
	_PPTGConstraint_IncludingRectangles3 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p5);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p6);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  -1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p7);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p8);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_IncludingRectangles3(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, int p5, int p6, int p7, int p8)
#else
PTGNode PTGConstraint_IncludingRectangles3(p1, p2, p3, p4, p5, p6, p7, p8)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
int p5;
int p6;
int p7;
int p8;
#endif
{
	_PPTGConstraint_IncludingRectangles3 n;

	n = (_PPTGConstraint_IncludingRectangles3)MALLOC(sizeof(struct _SPTGConstraint_IncludingRectangles3));
	n->_print = (_PTGProc)_PrPTGConstraint_IncludingRectangles3;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_IncludingRectangles4 */

typedef struct _SPTGConstraint_IncludingRectangles4{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
	int p7;
	int p8;
	int p9;
	int p10;
} * _PPTGConstraint_IncludingRectangles4;

#ifdef PROTO_OK
static void _PrPTGConstraint_IncludingRectangles4(_PPTGConstraint_IncludingRectangles4 n)
#else
static void _PrPTGConstraint_IncludingRectangles4(n)
	_PPTGConstraint_IncludingRectangles4 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p7);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p8);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p9);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add1  +1 ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p10);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_IncludingRectangles4(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, int p7, int p8, int p9, int p10)
#else
PTGNode PTGConstraint_IncludingRectangles4(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
int p7;
int p8;
int p9;
int p10;
#endif
{
	_PPTGConstraint_IncludingRectangles4 n;

	n = (_PPTGConstraint_IncludingRectangles4)MALLOC(sizeof(struct _SPTGConstraint_IncludingRectangles4));
	n->_print = (_PTGProc)_PrPTGConstraint_IncludingRectangles4;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	n->p9 = p9;
	n->p10 = p10;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_IncludingCircles12 */

typedef struct _SPTGConstraint_IncludingCircles12{
	_PTGProc _print;
	PTGNode p1;
	int p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
} * _PPTGConstraint_IncludingCircles12;

#ifdef PROTO_OK
static void _PrPTGConstraint_IncludingCircles12(_PPTGConstraint_IncludingCircles12 n)
#else
static void _PrPTGConstraint_IncludingCircles12(n)
	_PPTGConstraint_IncludingCircles12 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add  -1 ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add  ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " R= ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_IncludingCircles12(PTGNode p1, int p2, PTGNode p3, PTGNode p4, PTGNode p5)
#else
PTGNode PTGConstraint_IncludingCircles12(p1, p2, p3, p4, p5)
PTGNode p1;
int p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
#endif
{
	_PPTGConstraint_IncludingCircles12 n;

	n = (_PPTGConstraint_IncludingCircles12)MALLOC(sizeof(struct _SPTGConstraint_IncludingCircles12));
	n->_print = (_PTGProc)_PrPTGConstraint_IncludingCircles12;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_IncludingCircles34 */

typedef struct _SPTGConstraint_IncludingCircles34{
	_PTGProc _print;
	PTGNode p1;
	int p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
} * _PPTGConstraint_IncludingCircles34;

#ifdef PROTO_OK
static void _PrPTGConstraint_IncludingCircles34(_PPTGConstraint_IncludingCircles34 n)
#else
static void _PrPTGConstraint_IncludingCircles34(n)
	_PPTGConstraint_IncludingCircles34 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add  -1 ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " +1 ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add  ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " R= ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_IncludingCircles34(PTGNode p1, int p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6)
#else
PTGNode PTGConstraint_IncludingCircles34(p1, p2, p3, p4, p5, p6)
PTGNode p1;
int p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
#endif
{
	_PPTGConstraint_IncludingCircles34 n;

	n = (_PPTGConstraint_IncludingCircles34)MALLOC(sizeof(struct _SPTGConstraint_IncludingCircles34));
	n->_print = (_PTGProc)_PrPTGConstraint_IncludingCircles34;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_DisjunctCircles1 */

typedef struct _SPTGConstraint_DisjunctCircles1{
	_PTGProc _print;
	int p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
} * _PPTGConstraint_DisjunctCircles1;

#ifdef PROTO_OK
static void _PrPTGConstraint_DisjunctCircles1(_PPTGConstraint_DisjunctCircles1 n)
#else
static void _PrPTGConstraint_DisjunctCircles1(n)
	_PPTGConstraint_DisjunctCircles1 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add  +1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add  ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " R= ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_DisjunctCircles1(int p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6)
#else
PTGNode PTGConstraint_DisjunctCircles1(p1, p2, p3, p4, p5, p6)
int p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
#endif
{
	_PPTGConstraint_DisjunctCircles1 n;

	n = (_PPTGConstraint_DisjunctCircles1)MALLOC(sizeof(struct _SPTGConstraint_DisjunctCircles1));
	n->_print = (_PTGProc)_PrPTGConstraint_DisjunctCircles1;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_DisjunctCircles23 */

typedef struct _SPTGConstraint_DisjunctCircles23{
	_PTGProc _print;
	PTGNode p1;
	int p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
	PTGNode p7;
} * _PPTGConstraint_DisjunctCircles23;

#ifdef PROTO_OK
static void _PrPTGConstraint_DisjunctCircles23(_PPTGConstraint_DisjunctCircles23 n)
#else
static void _PrPTGConstraint_DisjunctCircles23(n)
	_PPTGConstraint_DisjunctCircles23 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add  +2 ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add  ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " R= ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " ");
	n->p7->_print(n->p7);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_DisjunctCircles23(PTGNode p1, int p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7)
#else
PTGNode PTGConstraint_DisjunctCircles23(p1, p2, p3, p4, p5, p6, p7)
PTGNode p1;
int p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
PTGNode p7;
#endif
{
	_PPTGConstraint_DisjunctCircles23 n;

	n = (_PPTGConstraint_DisjunctCircles23)MALLOC(sizeof(struct _SPTGConstraint_DisjunctCircles23));
	n->_print = (_PTGProc)_PrPTGConstraint_DisjunctCircles23;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	return (PTGNode)n;
}

/* Implementation of Pattern Constraint_DisjunctCircles4 */

typedef struct _SPTGConstraint_DisjunctCircles4{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	int p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
	PTGNode p7;
	PTGNode p8;
} * _PPTGConstraint_DisjunctCircles4;

#ifdef PROTO_OK
static void _PrPTGConstraint_DisjunctCircles4(_PPTGConstraint_DisjunctCircles4 n)
#else
static void _PrPTGConstraint_DisjunctCircles4(n)
	_PPTGConstraint_DisjunctCircles4 n;
#endif
{
	PTG_OUTPUT_STRING(f, "add  +1 ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -1 ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " >= ");
	PTG_OUTPUT_INT(f, n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "add  ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, " ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, " R= ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " ");
	n->p7->_print(n->p7);
	PTG_OUTPUT_STRING(f, " ");
	n->p8->_print(n->p8);
	PTG_OUTPUT_STRING(f, "\n\n");
}

#ifdef PROTO_OK
PTGNode PTGConstraint_DisjunctCircles4(PTGNode p1, PTGNode p2, int p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7, PTGNode p8)
#else
PTGNode PTGConstraint_DisjunctCircles4(p1, p2, p3, p4, p5, p6, p7, p8)
PTGNode p1;
PTGNode p2;
int p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
PTGNode p7;
PTGNode p8;
#endif
{
	_PPTGConstraint_DisjunctCircles4 n;

	n = (_PPTGConstraint_DisjunctCircles4)MALLOC(sizeof(struct _SPTGConstraint_DisjunctCircles4));
	n->_print = (_PTGProc)_PrPTGConstraint_DisjunctCircles4;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	return (PTGNode)n;
}

/* Implementation of Pattern C_HelpName */

typedef struct _SPTGC_HelpName{
	_PTGProc _print;
	PTGNode p1;
	char p2;
} * _PPTGC_HelpName;

#ifdef PROTO_OK
static void _PrPTGC_HelpName(_PPTGC_HelpName n)
#else
static void _PrPTGC_HelpName(n)
	_PPTGC_HelpName n;
#endif
{
	PTG_OUTPUT_STRING(f, "h");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ".");
	PTG_OUTPUT_CHAR(f, n->p2);
}

#ifdef PROTO_OK
PTGNode PTGC_HelpName(PTGNode p1, char p2)
#else
PTGNode PTGC_HelpName(p1, p2)
PTGNode p1;
char p2;
#endif
{
	_PPTGC_HelpName n;

	n = (_PPTGC_HelpName)MALLOC(sizeof(struct _SPTGC_HelpName));
	n->_print = (_PTGProc)_PrPTGC_HelpName;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern C_HelpDoubleName */

typedef struct _SPTGC_HelpDoubleName{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	char p3;
} * _PPTGC_HelpDoubleName;

#ifdef PROTO_OK
static void _PrPTGC_HelpDoubleName(_PPTGC_HelpDoubleName n)
#else
static void _PrPTGC_HelpDoubleName(n)
	_PPTGC_HelpDoubleName n;
#endif
{
	PTG_OUTPUT_STRING(f, "h");
	n->p1->_print(n->p1);
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ".");
	PTG_OUTPUT_CHAR(f, n->p3);
}

#ifdef PROTO_OK
PTGNode PTGC_HelpDoubleName(PTGNode p1, PTGNode p2, char p3)
#else
PTGNode PTGC_HelpDoubleName(p1, p2, p3)
PTGNode p1;
PTGNode p2;
char p3;
#endif
{
	_PPTGC_HelpDoubleName n;

	n = (_PPTGC_HelpDoubleName)MALLOC(sizeof(struct _SPTGC_HelpDoubleName));
	n->_print = (_PTGProc)_PrPTGC_HelpDoubleName;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern C_Set_VLInt */

typedef struct _SPTGC_Set_VLInt{
	_PTGProc _print;
	void * p1;
} * _PPTGC_Set_VLInt;

#ifdef PROTO_OK
static void _PrPTGC_Set_VLInt(_PPTGC_Set_VLInt n)
#else
static void _PrPTGC_Set_VLInt(n)
	_PPTGC_Set_VLInt n;
#endif
{
	PTG_OUTPUT_STRING(f, "set ");
	Constraint_VLInt_Name(f, n->p1);
	PTG_OUTPUT_STRING(f, " ");
	Constraint_VLInt_Value(f, n->p1);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGC_Set_VLInt(void * p1)
#else
PTGNode PTGC_Set_VLInt(p1)
void * p1;
#endif
{
	_PPTGC_Set_VLInt n;

	n = (_PPTGC_Set_VLInt)MALLOC(sizeof(struct _SPTGC_Set_VLInt));
	n->_print = (_PTGProc)_PrPTGC_Set_VLInt;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern C_Set_VLPoint */

typedef struct _SPTGC_Set_VLPoint{
	_PTGProc _print;
	void * p1;
} * _PPTGC_Set_VLPoint;

#ifdef PROTO_OK
static void _PrPTGC_Set_VLPoint(_PPTGC_Set_VLPoint n)
#else
static void _PrPTGC_Set_VLPoint(n)
	_PPTGC_Set_VLPoint n;
#endif
{
	PTG_OUTPUT_STRING(f, "set ");
	Constraint_VLPoint_NameX(f, n->p1);
	PTG_OUTPUT_STRING(f, " ");
	Constraint_VLPoint_ValueX(f, n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "set ");
	Constraint_VLPoint_NameY(f, n->p1);
	PTG_OUTPUT_STRING(f, " ");
	Constraint_VLPoint_ValueY(f, n->p1);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGC_Set_VLPoint(void * p1)
#else
PTGNode PTGC_Set_VLPoint(p1)
void * p1;
#endif
{
	_PPTGC_Set_VLPoint n;

	n = (_PPTGC_Set_VLPoint)MALLOC(sizeof(struct _SPTGC_Set_VLPoint));
	n->_print = (_PTGProc)_PrPTGC_Set_VLPoint;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern C_Name_VLInt */

typedef struct _SPTGC_Name_VLInt{
	_PTGProc _print;
	void * p1;
} * _PPTGC_Name_VLInt;

#ifdef PROTO_OK
static void _PrPTGC_Name_VLInt(_PPTGC_Name_VLInt n)
#else
static void _PrPTGC_Name_VLInt(n)
	_PPTGC_Name_VLInt n;
#endif
{
	Constraint_VLInt_Name(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGC_Name_VLInt(void * p1)
#else
PTGNode PTGC_Name_VLInt(p1)
void * p1;
#endif
{
	_PPTGC_Name_VLInt n;

	n = (_PPTGC_Name_VLInt)MALLOC(sizeof(struct _SPTGC_Name_VLInt));
	n->_print = (_PTGProc)_PrPTGC_Name_VLInt;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern C_Name_VLPointX */

typedef struct _SPTGC_Name_VLPointX{
	_PTGProc _print;
	void * p1;
} * _PPTGC_Name_VLPointX;

#ifdef PROTO_OK
static void _PrPTGC_Name_VLPointX(_PPTGC_Name_VLPointX n)
#else
static void _PrPTGC_Name_VLPointX(n)
	_PPTGC_Name_VLPointX n;
#endif
{
	Constraint_VLPoint_NameX(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGC_Name_VLPointX(void * p1)
#else
PTGNode PTGC_Name_VLPointX(p1)
void * p1;
#endif
{
	_PPTGC_Name_VLPointX n;

	n = (_PPTGC_Name_VLPointX)MALLOC(sizeof(struct _SPTGC_Name_VLPointX));
	n->_print = (_PTGProc)_PrPTGC_Name_VLPointX;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern C_Name_VLPointY */

typedef struct _SPTGC_Name_VLPointY{
	_PTGProc _print;
	void * p1;
} * _PPTGC_Name_VLPointY;

#ifdef PROTO_OK
static void _PrPTGC_Name_VLPointY(_PPTGC_Name_VLPointY n)
#else
static void _PrPTGC_Name_VLPointY(n)
	_PPTGC_Name_VLPointY n;
#endif
{
	Constraint_VLPoint_NameY(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGC_Name_VLPointY(void * p1)
#else
PTGNode PTGC_Name_VLPointY(p1)
void * p1;
#endif
{
	_PPTGC_Name_VLPointY n;

	n = (_PPTGC_Name_VLPointY)MALLOC(sizeof(struct _SPTGC_Name_VLPointY));
	n->_print = (_PTGProc)_PrPTGC_Name_VLPointY;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern C_Name_VLPoint */

typedef struct _SPTGC_Name_VLPoint{
	_PTGProc _print;
	void * p1;
} * _PPTGC_Name_VLPoint;

#ifdef PROTO_OK
static void _PrPTGC_Name_VLPoint(_PPTGC_Name_VLPoint n)
#else
static void _PrPTGC_Name_VLPoint(n)
	_PPTGC_Name_VLPoint n;
#endif
{
	Constraint_VLPoint_Name(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGC_Name_VLPoint(void * p1)
#else
PTGNode PTGC_Name_VLPoint(p1)
void * p1;
#endif
{
	_PPTGC_Name_VLPoint n;

	n = (_PPTGC_Name_VLPoint)MALLOC(sizeof(struct _SPTGC_Name_VLPoint));
	n->_print = (_PTGProc)_PrPTGC_Name_VLPoint;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Id */

typedef struct _SPTGId{
	_PTGProc _print;
	int p1;
} * _PPTGId;

#ifdef PROTO_OK
static void _PrPTGId(_PPTGId n)
#else
static void _PrPTGId(n)
	_PPTGId n;
#endif
{
	PtgOutId(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGId(int p1)
#else
PTGNode PTGId(p1)
int p1;
#endif
{
	_PPTGId n;

	n = (_PPTGId)MALLOC(sizeof(struct _SPTGId));
	n->_print = (_PTGProc)_PrPTGId;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern AsIs */

typedef struct _SPTGAsIs{
	_PTGProc _print;
	char * p1;
} * _PPTGAsIs;

#ifdef PROTO_OK
static void _PrPTGAsIs(_PPTGAsIs n)
#else
static void _PrPTGAsIs(n)
	_PPTGAsIs n;
#endif
{
	PTG_OUTPUT_STRING(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGAsIs(char * p1)
#else
PTGNode PTGAsIs(p1)
char * p1;
#endif
{
	_PPTGAsIs n;

	n = (_PPTGAsIs)MALLOC(sizeof(struct _SPTGAsIs));
	n->_print = (_PTGProc)_PrPTGAsIs;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Numb */

typedef struct _SPTGNumb{
	_PTGProc _print;
	int p1;
} * _PPTGNumb;

#ifdef PROTO_OK
static void _PrPTGNumb(_PPTGNumb n)
#else
static void _PrPTGNumb(n)
	_PPTGNumb n;
#endif
{
	PTG_OUTPUT_INT(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGNumb(int p1)
#else
PTGNode PTGNumb(p1)
int p1;
#endif
{
	_PPTGNumb n;

	n = (_PPTGNumb)MALLOC(sizeof(struct _SPTGNumb));
	n->_print = (_PTGProc)_PrPTGNumb;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern CString */

typedef struct _SPTGCString{
	_PTGProc _print;
	char * p1;
} * _PPTGCString;

#ifdef PROTO_OK
static void _PrPTGCString(_PPTGCString n)
#else
static void _PrPTGCString(n)
	_PPTGCString n;
#endif
{
	CPtgOutstr(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGCString(char * p1)
#else
PTGNode PTGCString(p1)
char * p1;
#endif
{
	_PPTGCString n;

	n = (_PPTGCString)MALLOC(sizeof(struct _SPTGCString));
	n->_print = (_PTGProc)_PrPTGCString;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern CChar */

typedef struct _SPTGCChar{
	_PTGProc _print;
	int p1;
} * _PPTGCChar;

#ifdef PROTO_OK
static void _PrPTGCChar(_PPTGCChar n)
#else
static void _PrPTGCChar(n)
	_PPTGCChar n;
#endif
{
	CPtgOutchar(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGCChar(int p1)
#else
PTGNode PTGCChar(p1)
int p1;
#endif
{
	_PPTGCChar n;

	n = (_PPTGCChar)MALLOC(sizeof(struct _SPTGCChar));
	n->_print = (_PTGProc)_PrPTGCChar;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern PString */

typedef struct _SPTGPString{
	_PTGProc _print;
	char * p1;
} * _PPTGPString;

#ifdef PROTO_OK
static void _PrPTGPString(_PPTGPString n)
#else
static void _PrPTGPString(n)
	_PPTGPString n;
#endif
{
	PPtgOutstr(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGPString(char * p1)
#else
PTGNode PTGPString(p1)
char * p1;
#endif
{
	_PPTGPString n;

	n = (_PPTGPString)MALLOC(sizeof(struct _SPTGPString));
	n->_print = (_PTGProc)_PrPTGPString;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern CommaSeq */

typedef struct _SPTGCommaSeq{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGCommaSeq;

#ifdef PROTO_OK
static void _PrPTGCommaSeq(_PPTGCommaSeq n)
#else
static void _PrPTGCommaSeq(n)
	_PPTGCommaSeq n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, ", ");
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGCommaSeq(PTGNode p1, PTGNode p2)
#else
PTGNode PTGCommaSeq(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGCommaSeq n;

	n = (_PPTGCommaSeq)MALLOC(sizeof(struct _SPTGCommaSeq));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGCommaSeq;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Eol */

typedef struct _SPTGEol{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGEol;

#ifdef PROTO_OK
static void _PrPTGEol(_PPTGEol n)
#else
static void _PrPTGEol(n)
	_PPTGEol n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGEol(PTGNode p1)
#else
PTGNode PTGEol(p1)
PTGNode p1;
#endif
{
	_PPTGEol n;

	n = (_PPTGEol)MALLOC(sizeof(struct _SPTGEol));
	n->_print = (_PTGProc)_PrPTGEol;
	n->p1 = p1;
	return (PTGNode)n;
}


/* -------------------------------------------------------- */
/*                  Default Output Functions                */
/* -------------------------------------------------------- */


#ifdef PROTO_OK
void _PTGPrintInt(PTG_OUTPUT_FILE file, int param)
#else
void _PTGPrintInt(file, param)
	PTG_OUTPUT_FILE file; int param;
#endif
{    /* used for short and int */
	sprintf(buffer,"%d",param);
	PTG_OUTPUT_STRING(file,buffer);
}

#ifdef PROTO_OK
void _PTGPrintLong(PTG_OUTPUT_FILE file, long param)
#else
void _PTGPrintLong(file, param)
	PTG_OUTPUT_FILE file; long param;
#endif
{
	sprintf(buffer,"%ld",param);
	PTG_OUTPUT_STRING(file,buffer);
}

#ifdef PROTO_OK
void _PTGPrintDouble(PTG_OUTPUT_FILE file, double param)
#else
void _PTGPrintDouble(file, param)
	PTG_OUTPUT_FILE file; double param;
#endif
{    /* used for float and double */
	sprintf(buffer,"%g",param);
	PTG_OUTPUT_STRING(file,buffer);
}

#ifdef PROTO_OK
void _PTGPrintChar(PTG_OUTPUT_FILE file, char param)
#else
void _PTGPrintChar(file, param)
	PTG_OUTPUT_FILE file; char param;
#endif
{
	buffer[0] = param;
	buffer[1] = 0;
	PTG_OUTPUT_STRING(file,buffer);
}

