#ifndef _PTGGEN_H
#define _PTGGEN_H

#include <stdio.h>
#include "eliproto.h"

/* Include-files specified in .ptg.phi-files */

#include "ptg.h"
/* Definition of Output-Macros */

#if !defined(PTG_OUTPUT_FILE)
#    define PTG_OUTPUT_FILE FILE *
#    define PTG_OUTPUT_DEFAULT 1
#else
#    define PTG_OUTPUT_DEFAULT 0
#endif
#if !defined(PTG_OUTPUT_STRING)
#    define PTG_OUTPUT_STRING(file,param) fputs(param, file)
#    if !defined(PTG_OUTPUT_INT)
#            define PTG_OUTPUT_INT(file,param)    fprintf(file,"%d",param)
#    endif
#    if !defined(PTG_OUTPUT_LONG)
#            define PTG_OUTPUT_LONG(file,param)   fprintf(file,"%ld",param)
#    endif
#    if !defined(PTG_OUTPUT_SHORT)
#            define PTG_OUTPUT_SHORT(file,param)  fprintf(file,"%d",(int) param)
#    endif
#    if !defined(PTG_OUTPUT_CHAR)
#            define PTG_OUTPUT_CHAR(file,param)   fputc(param, file)
#    endif
#    if !defined(PTG_OUTPUT_FLOAT)
#            define PTG_OUTPUT_FLOAT(file,param)  fprintf(file,"%g",(double)param)
#    endif
#    if !defined(PTG_OUTPUT_DOUBLE)
#            define PTG_OUTPUT_DOUBLE(file,param) fprintf(file,"%g",param)
#    endif

#else
     /* PTG_OUTPUT_STRING defined */
     extern void _PTGPrintInt ELI_ARG((PTG_OUTPUT_FILE,int));
     extern void _PTGPrintLong ELI_ARG((PTG_OUTPUT_FILE,long));
     extern void _PTGPrintChar ELI_ARG((PTG_OUTPUT_FILE,char));
     extern void _PTGPrintDouble ELI_ARG((PTG_OUTPUT_FILE,double));
#    if !defined(PTG_OUTPUT_INT)
#            define PTG_OUTPUT_INT(file,param)    _PTGPrintInt(file,param)
#    endif
#    if !defined(PTG_OUTPUT_LONG)
#            define PTG_OUTPUT_LONG(file,param)   _PTGPrintLong(file,param)
#    endif
#    if !defined(PTG_OUTPUT_SHORT)
#            define PTG_OUTPUT_SHORT(file,param)  _PTGPrintInt(file,(int)param)
#    endif
#    if !defined(PTG_OUTPUT_CHAR)
#            define PTG_OUTPUT_CHAR(file,param)   _PTGPrintChar(file,param)
#    endif
#    if !defined(PTG_OUTPUT_FLOAT)
#            define PTG_OUTPUT_FLOAT(file,param)  _PTGPrintDouble(file,(double)param)
#    endif
#    if !defined(PTG_OUTPUT_DOUBLE)
#            define PTG_OUTPUT_DOUBLE(file,param) _PTGPrintDouble(file,param)
#    endif
#endif

/* Define PTGNode Type */
#if defined(__cplusplus)
struct _SPTG0;
typedef void (* _PTGProc)(struct _SPTG0 *);
#else
typedef void (* _PTGProc)();
#endif

typedef struct _SPTG0
{
  _PTGProc _print;
} * _PPTG0;
typedef _PPTG0 PTGNode;                      /* the only exported type */

/* predefined static PTGNULL-Node */

extern struct _SPTG0 _PTGNULL;
#define PTGNULL (& _PTGNULL)
#define PTGNull() (& _PTGNULL)

/* output functions */

#if PTG_OUTPUT_DEFAULT
extern PTGNode PTGOut ELI_ARG((PTGNode root));
extern PTGNode PTGOutFile ELI_ARG((char *filename, PTGNode root));
extern PTGNode PTGOutFPtr ELI_ARG((FILE *output, PTGNode root));
#endif
extern PTGNode PTGProcess ELI_ARG((PTG_OUTPUT_FILE file, PTGNode root));

/* Memory Management functions. */

extern void PTGFree ELI_ARG((void));

/* functions for making PTG nodes */

PTGNode PTGNodePtr ELI_ARG((void * p1));
PTGNode PTGSeq ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGSetAttr ELI_ARG((void * p1, char * p2, char * p3));
PTGNode PTGSetAttr2 ELI_ARG((void * p1, char * p2, PTGNode p3));
PTGNode PTGInsertNode ELI_ARG((PTGNode p1, int p2, PTGNode p3));
PTGNode PTGInsertAndConnectNode2 ELI_ARG((PTGNode p1, int p2, PTGNode p3, char * p4, PTGNode p5));
PTGNode PTGRemoveNode ELI_ARG((PTGNode p1));
PTGNode PTGExtractNode ELI_ARG((PTGNode p1));
PTGNode PTGExtractSuperNode ELI_ARG((PTGNode p1));
PTGNode PTGRemoveSuperNode ELI_ARG((PTGNode p1));
PTGNode PTGCreateNode ELI_ARG((char * p1));
#define PTGConcat PTGSeq
PTGNode PTGConstraint_Equal ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGConstraint_RedefineVariable ELI_ARG((char * p1, int p2));
PTGNode PTGConstraint_SetVariable ELI_ARG((PTGNode p1, int p2));
PTGNode PTGC_SetRadius ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGObjectCenter1 ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGObjectCenter2 ELI_ARG((PTGNode p1, PTGNode p2, int p3));
PTGNode PTGConstraint_DisjunctRectangles1 ELI_ARG((PTGNode p1, PTGNode p2, int p3, int p4, PTGNode p5, PTGNode p6, int p7, int p8));
PTGNode PTGConstraint_DisjunctRectangles23 ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, int p7, int p8, int p9));
PTGNode PTGConstraint_DisjunctRectangles4 ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7, PTGNode p8, int p9));
PTGNode PTGConstraint_IncludingRectangles1 ELI_ARG((PTGNode p1, PTGNode p2, int p3, int p4, int p5, int p6));
PTGNode PTGConstraint_IncludingRectangles2 ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, int p5, int p6, int p7, int p8));
PTGNode PTGConstraint_IncludingRectangles3 ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, int p5, int p6, int p7, int p8));
PTGNode PTGConstraint_IncludingRectangles4 ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, int p7, int p8, int p9, int p10));
PTGNode PTGConstraint_IncludingCircles12 ELI_ARG((PTGNode p1, int p2, PTGNode p3, PTGNode p4, PTGNode p5));
PTGNode PTGConstraint_IncludingCircles34 ELI_ARG((PTGNode p1, int p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6));
PTGNode PTGConstraint_DisjunctCircles1 ELI_ARG((int p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6));
PTGNode PTGConstraint_DisjunctCircles23 ELI_ARG((PTGNode p1, int p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7));
PTGNode PTGConstraint_DisjunctCircles4 ELI_ARG((PTGNode p1, PTGNode p2, int p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7, PTGNode p8));
PTGNode PTGC_HelpName ELI_ARG((PTGNode p1, char p2));
PTGNode PTGC_HelpDoubleName ELI_ARG((PTGNode p1, PTGNode p2, char p3));
PTGNode PTGC_Set_VLInt ELI_ARG((void * p1));
PTGNode PTGC_Set_VLPoint ELI_ARG((void * p1));
PTGNode PTGC_Name_VLInt ELI_ARG((void * p1));
PTGNode PTGC_Name_VLPointX ELI_ARG((void * p1));
PTGNode PTGC_Name_VLPointY ELI_ARG((void * p1));
PTGNode PTGC_Name_VLPoint ELI_ARG((void * p1));
PTGNode PTGId ELI_ARG((int p1));
PTGNode PTGAsIs ELI_ARG((char * p1));
PTGNode PTGNumb ELI_ARG((int p1));
PTGNode PTGCString ELI_ARG((char * p1));
PTGNode PTGCChar ELI_ARG((int p1));
PTGNode PTGPString ELI_ARG((char * p1));
PTGNode PTGCommaSeq ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGEol ELI_ARG((PTGNode p1));

/* prototypes for used function call insertions */

void BasicCheckPrintNodeptr ELI_ARG((PTG_OUTPUT_FILE, void *));
void Constraint_VLInt_Name ELI_ARG((PTG_OUTPUT_FILE, void *));
void Constraint_VLInt_Value ELI_ARG((PTG_OUTPUT_FILE, void *));
void Constraint_VLPoint_NameX ELI_ARG((PTG_OUTPUT_FILE, void *));
void Constraint_VLPoint_ValueX ELI_ARG((PTG_OUTPUT_FILE, void *));
void Constraint_VLPoint_NameY ELI_ARG((PTG_OUTPUT_FILE, void *));
void Constraint_VLPoint_ValueY ELI_ARG((PTG_OUTPUT_FILE, void *));
void Constraint_VLPoint_Name ELI_ARG((PTG_OUTPUT_FILE, void *));
void PtgOutId ELI_ARG((PTG_OUTPUT_FILE, int));
void CPtgOutstr ELI_ARG((PTG_OUTPUT_FILE, char *));
void CPtgOutchar ELI_ARG((PTG_OUTPUT_FILE, int));
void PPtgOutstr ELI_ARG((PTG_OUTPUT_FILE, char *));

#ifdef MONITOR
/* Monitoring support for structured values */
#define DAPTO_RESULTPTGNode(n) DAPTO_RESULT_PTR(n)
#define DAPTO_ARGPTGNode(n) DAPTO_ARG_PTR(n, PTGNode)
#endif

#endif

