# ----------------------------------------------------------------------
# Module Visual
# ----------------------------------------------------------------------
# Procedures to visualize and edit the visualized data structure
# This file contains the fix, non-generated parts of the module.
#
# $Id: solver.tcl,v 3.16 2001/01/08 11:51:55 cschmidt Exp $

# Create our namespace.
namespace eval ::visual {
    
}

proc ::visual::solver_init {} {
    variable solver_log 0

    Menu::Check $::MDIFrame::mainMenu View "Log solver interaction" ::visual::solver_log
}

proc ::visual::solver_create {win} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    # No Solver is Running
    set data(solver) ""
    set data(solver-currformulas) ""
    set data(solver-lastformulas) ""
    set data(solver-variables) ""
    set data(solver-currblock) ""
}



proc ::visual::solver_send {varname string} {
    upvar #0 $varname data

    if {$data(solver) == ""} {
	set data(solver) [::process::open $::cfg::solver "Constraint-Solver" "::visual::_solver_respond $varname" "::visual::_solver_close $varname"]
	set data(solver-currformulas) ""
	set data(solver-lastformulas) ""
	set data(solver-variables) ""
    }

    ::process::write $data(solver) $string
    if {$::visual::solver_log} {
#	::log::log "CS<- $string"
    }
}

proc ::visual::solver_stop {varname} {
    upvar #0 $varname data
    
    if {$data(solver) != ""} {
	set solver $data(solver)

	solver_send $varname "quit"

	foreach formula [array names $varname solver-formula,*] {
	    unset data($formula)
	}

	::process::close $solver

	if {[info exists data(solver-waiting)]} {
	    set data(solver-status) -42
	}

    if {[info exists data(solver-procid)]} {
	::log::info solver "Killing $data(solver-procid)"
	catch {exec kill -HUP $data(solver-procid)}
	catch {exec kill -QUIT $data(solver-procid)}
	catch {exec kill -TERM $data(solver-procid)}
	catch {exec kill -KILL $data(solver-procid)}
	catch {exec kill -SEGV $data(solver-procid)}
	unset data(solver-procid)
    }

    }
}

proc ::visual::_solver_close {varname handle} {
    upvar #0 $varname data

    set data(solver) ""
    foreach formula [array names $varname solver-formula,*] {
	unset data($formula)
    }

    if {$::visual::solver_log} {
	::log::log "CS-> End-Of-File"
    }
    if {[info exists data(solver-waiting)]} {
	set data(solver-status) -42
    }
}
    
proc ::visual::_solver_respond {varname text} {
    upvar #0 $varname data

    if {$::visual::solver_log} {
#      ::log::log "CS-> $text"
    }
    set lead [string range $text 0 0]
#    puts "Server responds $text"
    if {$lead == "$"} {
#	puts "Solver interaction complete for $varname"
	set data(solver-status) [expr [string range $text 1 end] + 0]
    }
    if {$lead == "0"} {
	::MDIFrame::SetIndicator 1 [string range $text 3 end]
    }
    if {$lead == "~"} {
	set var_val [split [string range $text 1 end] " "]

	if {[lsearch -exact $data(solver-variables) [lindex $var_val 0]] < 0} {
	    if {$::visual::solver_log} {
		::log::log "Ignoring Variable-Set $var_val: variable not mentioned in set-command"
	    }
	} elseif {[string range $var_val 0 2] != "ptr"} {
	    if {$::visual::solver_log} {
		::log::log "Ignoring Variable-Set $var_val: Inproper built name"
	    }
	} else {
	    if {[catch {eval ::vl::poke $var_val} errmsg]} {
		::log::log $errmsg
	    }
	}
    }
    if {$lead == "="} {
	set data(solver-procid) [expr [string range $text 1 end] + 0]
	::log::info solver "Solver started under [expr [string range $text 1 end] + 0]"
    }
}

proc ::visual::solver_interact {string} {
    upvar #0 $::visual::currVarname data

    set ::solver_calctime [expr $::solver_calctime + [lindex [time {
    set returnval 0

    if {[string range $string 0 3] == "add|"} {
	if {$data(solver-currblock) == ""} {
	    ::log::log "Formula structure error in solver: add| may only follow add"
	} else {
	    lappend data(solver-currblock) $string
	}
    } else {
	if {$data(solver-currblock) != ""} {
	set normstring ""
	foreach currfrm $data(solver-currblock) {
	    set addstring [join [eval list $currfrm] ","]
	    if {$normstring != ""} {
		set normstring "$normstring;$addstring"
	    } else {
		set normstring $addstring
	    }
	}

	if {[info exists data(solver-formula,$normstring)]} {
	    # Formula already known.
	    set formula $data(solver-formula,$normstring)

	    foreach currfrm $data(solver-currblock) {
		solver_show_string $::visual::currVarname $currfrm FORMULA_KNOWN
	    }
	    solver_show_string $::visual::currVarname "# Formula Number(s) $formula" COMMENT
	} else {
	    # New formula
	    foreach currfrm $data(solver-currblock) {
		set formula [::visual::solver_interact_internal $currfrm]
		lappend data(solver-formula,$normstring) $formula
		solver_show_string $::visual::currVarname $currfrm FORMULA_UNKNOWN
	    }
	    set formula $data(solver-formula,$normstring)
	    solver_show_string $::visual::currVarname "# Formula Number(s) $formula" COMMENT
	}
	lappend data(solver-currformulas) $formula
	set data(solver-currblock) ""
	}
	if {[string range $string 0 2] == "add"} {
	    lappend data(solver-currblock) $string
        } elseif {[string range $string 0 2] == "set"} {
	    lappend data(solver-variables) [lindex [eval list $string] 1]
   	    solver_show_string $::visual::currVarname $string VALUE
	    set returnval [solver_interact_internal $string] 
        } elseif {$string == "VL clear-variables"} {
  	    set data(solver-variables) ""
        } elseif {[string range $string 0 2] == "new"} {
 	    set form_delete ""
	    set form_delete_show ""
	    foreach formula $data(solver-lastformulas) {
	        if {[lsearch -exact $data(solver-currformulas) $formula] < 0} {
		    foreach f $formula {
		        solver_interact_internal "del $f"
		        lappend form_delete_show $f
		    }
		    lappend form_delete $formula
	        }
	    }
	
	    solver_show_string $::visual::currVarname "# Deleting $form_delete_show" COMMENT

	    if {$form_delete != ""} {
	        foreach formula [array names data solver-formula,*] {
		    if {[lsearch -exact $form_delete $data($formula)] >= 0} {
		        unset data($formula)

		    }
	        }
	    }

	    set data(solver-lastformulas) $data(solver-currformulas)
	    set data(solver-currformulas) ""

            solver_show_string $::visual::currVarname $string NEW
            set returnval [solver_interact_internal $string]
        } else {
            solver_show_string $::visual::currVarname $string NEW
            set returnval [solver_interact_internal $string]
	}
    }
    }] 0]]	    

    return $returnval
}

proc ::visual::solver_interact_internal {string} {
    upvar #0 $::visual::currVarname data

    if {[info exists data(solver-status)]} {
	::log::log "*** Communication error in ::visual::solver_interact"
	unset data(solver-status)
    }

    set time [time {
    ::MDIFrame::SetIndicator 1
    ::visual::solver_send $::visual::currVarname $string
    set data(solver-waiting) 1
    set id [after $::cfg::solver_timeout {
	::log::log "Constraint Solver aborted"
	::visual::solver_stop $::visual::currVarname
    }]

#    puts "Solver-Command: $string"
#    puts "Waiting for Solver $visual::currVarname"
    tkwait variable ${::visual::currVarname}(solver-status)
#    puts "Solver replied $visual::currVarname"

    catch {after cancel $id}
    set status $data(solver-status)
    unset data(solver-status)
    unset data(solver-waiting)
    ::MDIFrame::SetIndicator 0

    if {$::visual::solver_log} {
#	::log::log "CS-> solver-status = $status"
    }
    }]"
    
    set ::solvertime [expr $::solvertime + [lindex $time 0]]
    return $status
}

#proc ::visual::solver_interrupt {varname} {
#    upvar #0 $varname data
#
#    if {[::CDlg::Confirm "Constraint-Solver did not respond." "Continue" "Wait longer"] == "Continue"} {
#	set data(solver-status) -1
#	set data(solver-interrupt) ""
#    } else {
#	set data(solver-interrupt) [after 10000 ::visual::solver_interrupt $win]
#    }
#}

proc ::visual::solver_show {win} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data
    
    if {[info exists data(solver-window)]} {
	::MDIFrame::MakeVisible $data(solver-window)
	return
    }

    set total [::MDIFrame::Totalsize]
    set frame [::MDIFrame::New_Window "Solver Interaction" "" -width 600 -height 200 \
	    -x [expr [lindex $total 0] - 600] -y [expr [lindex $total 1] - 200]]
    set data(solver-window) [::MDIFrame::GetClient $frame]

    # Use a dummy namespace to express, that the solver window can't handle any messages
    oop::setClass $data(solver-window) "solverwindow"

    frame $frame.show
    text $frame.show.text -background black -foreground white -width 60 -height 10 \
	    -font "Courier -12" -yscrollcommand [list $frame.show.sb set] \
	    -xscrollcommand [list $frame.xs set]
    scrollbar $frame.show.sb -orient vertical -command "$frame.show.text yview" \
	    -takefocus 0
    pack $frame.show.text -side left -fill both -expand true
    pack $frame.show.sb -side right -fill y
    pack $frame.show -side top -fill both -expand true
    scrollbar $frame.xs -orient horizontal -command "$frame.show.text xview" \
	    -takefocus 0
    pack $frame.xs -side bottom -fill x

    button $frame.clear -text "Clear" -command "::visual::solver_show_clear ::visual::$var"
    pack $frame.clear -side left -fill x 

    button $frame.write -text "Save" -command "::visual::solver_show_write ::visual::$var $frame.show.text"
    pack $frame.write -side left -fill x

    set data(solver-window_data) $frame.show.text

    $data(solver-window_data) tag configure FORMULA_KNOWN -foreground "#808080"
    $data(solver-window_data) tag configure FORMULA_UNKNOWN -foreground "#FFFFFF"
    $data(solver-window_data) tag configure VALUE -foreground "#00ffff"
    $data(solver-window_data) tag configure NEW -foreground "#ffff00"
    $data(solver-window_data) tag configure COMMENT -foreground "#0000ff"

    # remove Text from bindtags
    set btags [bindtags $data(solver-window_data)]
    set i [lsearch $btags Text]
    if {$i >= 0} {
	set btags [lreplace $btags $i $i]
    }
    bindtags $data(solver-window_data) $btags
    bind $frame <Destroy> "if \{\"%W\" == \"$frame\"\} \{::visual::solver_show_close ::visual::$var\}"
    
}

proc ::visual::solver_show_write {varname textwidget} {
    set filename [tk_getSaveFile -title "Save Constraints"]

    if {$filename == ""} {
	return
    }
    
    if {[file exists $filename]} {
	if {[::CDlg::Confirm "Ok to overwrite $filename?" "Yes" "No"] != "Yes"} {
	    return
	}
    }

    set channel [open $filename w]
    puts $channel [$textwidget get 1.0 end]
    close $channel
}

proc ::visual::solver_show_close {varname} {
    upvar #0 $varname data
    
    if {[info exists data(solver-window)]} {
	unset data(solver-window)
	unset data(solver-window_data)
    }
}    

proc ::visual::solver_show_string {varname string tag} {
    upvar #0 $varname data

    if {[string range $string end end] != "\n"} {
	set string "$string\n"
    }

    if {! [info exists data(solver-window)]} {
	return
    }
    
    set line [$data(solver-window_data) index "end - 1 char"]
    $data(solver-window_data) insert $line $string
    $data(solver-window_data) tag add $tag $line "$line + 1 line"
    $data(solver-window_data) see $line
}

proc ::visual::solver_show_clear {varname} {
    upvar #0 $varname data
    
    $data(solver-window_data) delete 1.0 end
}
