#include "tcl.h"
#include "HEAD.h"
#include "nodecode.h"
#include "node.h"
#include "tcl_vpt.h"
#include "gen_treeio.h"
#include "elisave.h"
#include "attrmain.h"

#include <setjmp.h>

void *state;
static Tcl_Interp *delete_interp = (Tcl_Interp *)0;

static void Eli_Init(Tcl_Interp* interp)
{
    InitRuleTable();
    InitSymbolTable();

#include "INIT.h"

    state = SaveModules();
}

void checkmin(Tcl_Interp *interp, NODEPTR rule) 
{
    int no = rule->_prod;

    delete_interp = interp;
    
    if (RI[no].getminmax) {

	/* Check if one of the descendants is a list */
	Tcl_Obj *desclist = (*RI[no].getdesc)(rule, 0);
	Tcl_IncrRefCount(desclist);

	int count = 0;
	Tcl_ListObjLength(interp, desclist, &count);
	
	for (int i = 0; i < count; i++)
	{
	    Tcl_Obj *desc = (Tcl_Obj *)0;
	    Tcl_ListObjIndex(interp, desclist, i, &desc);
	    NODEPTR descnode = NULLNODEPTR;
	    
	    int min = 0;
	    int max = 0;
	    
	    if (desc) {
		TclVP_GetVoidPointerFromObj(interp, desc, (void **)&descnode);
	
		if (RI[descnode->_prod].ruletype == 1) {
		    (*RI[no].getminmax)(rule, descnode, &min, &max);
		    if (min > 0) {

			int defaultprod = SI[SI[LHSMAP[descnode->_prod]].listelement].defaultprod;
			if (defaultprod >= 0) {
			    
			  while (min-- > 0) {
			      NODEPTR currn = (*RI[defaultprod].create)();
			      (*RI[descnode->_prod].insert)(descnode, (*RI[defaultprod].create)(), 0, 0);
			      checkmin(interp, currn);
			  }
			}
			
		    }
		    
		}
		
		if (RI[descnode->_prod].ruletype == 0) {
		    checkmin(interp, descnode);
		}
	    }
	}

	Tcl_DecrRefCount(desclist);
    }
}

static int 
Vl_version(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    Tcl_SetResult(interp,"VL-Interface Version 1.0",TCL_STATIC);
    return TCL_OK;
}

static void
Vl_callback(Tcl_Interp *interp, char *commandvar, Tcl_Obj *node)
{
    int len = 0;
    Tcl_Obj *cmd[3];
    Tcl_Obj *cmdlst;
    Tcl_Obj *cmdvar = Tcl_NewStringObj(commandvar, -1);

    cmd[0] = Tcl_NewStringObj("eval", -1);
    
    Tcl_IncrRefCount(cmdvar);
    cmd[1] = Tcl_ObjGetVar2(interp, cmdvar, 0, TCL_GLOBAL_ONLY);
    Tcl_DecrRefCount(cmdvar);
    
    Tcl_GetStringFromObj(cmd[1], &len);  /* Only need side effect of setting len */
    cmd[2] = node;

    cmdlst = Tcl_NewListObj(3, cmd);
    Tcl_IncrRefCount(cmdlst);
    
    if (len) Tcl_GlobalEvalObj(interp, cmdlst);
    Tcl_DecrRefCount(cmdlst);
}

void on_delete(NODEPTR rule)
{
    Vl_callback(delete_interp, "::vl::onDelete", TclVP_NewVoidPointerObj(rule));
}
    
static int 
Vl_rulename(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    int no = 0;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: rulename rulenumber",TCL_STATIC);
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &no) != TCL_OK)
	return TCL_ERROR;

    if (no < 0 || no > RI_LEN)
    {
	Tcl_SetResult(interp,"rulenumber out of bounds",TCL_STATIC);
	return TCL_ERROR;
    }
    
    Tcl_SetResult(interp,RI[no].rulename,TCL_STATIC);
    return TCL_OK;
}

static int 
Vl_lhs(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    int no = 0;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: lhs rulenumber",TCL_STATIC);
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &no) != TCL_OK)
	return TCL_ERROR;

    if (no < 0 || no > RI_LEN)
    {
	Tcl_SetResult(interp,"rulenumber out of bounds",TCL_STATIC);
	return TCL_ERROR;
    }
    
    Tcl_SetObjResult(interp,Tcl_NewIntObj(LHSMAP[no]));
    return TCL_OK;
}


static int 
Vl_ruletype(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    int no = 0;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: ruletype rulenumber\nreturns 0 for normal nodes, 1 for list nodes and 2 for inner list nodes",TCL_STATIC);
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &no) != TCL_OK)
	return TCL_ERROR;

    if (no < 0 || no > RI_LEN)
    {
	Tcl_SetResult(interp,"rulenumber out of bounds",TCL_STATIC);
	return TCL_ERROR;
    }
    
    Tcl_SetObjResult(interp, Tcl_NewIntObj(RI[no].ruletype));
    return TCL_OK;
}

static int 
Vl_isRemovable(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n, father;
    int ruletype;
    int result;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: isRemovable $node",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }
    
    father = n->_father;
    result = 0;
    
    if (father != NULLNODEPTR) {
	ruletype = RI[father->_prod].ruletype;
	if (ruletype == 1) {
	    NODEPTR grandfather = father->_father;
	    int min = 0;
	    int max = 0;
	    
	    if (RI[grandfather->_prod].getminmax) {
		(*RI[grandfather->_prod].getminmax)(grandfather, father, &min, &max);
	    }
	    
	    if (min > 0) {
		Tcl_Obj *allsons = (*RI[father->_prod].getdesc)(father, 0);
		Tcl_IncrRefCount(allsons);
		int count = 0;
		Tcl_ListObjLength(interp, allsons, &count);
		Tcl_DecrRefCount(allsons);
		
		result = (count > min);
	    }
	    else
		result = 1;
	}
	else if (ruletype == 0) {
	    int defaultprod = SI[LHSMAP[n->_prod]].defaultprod;
	    
	    result = (n->_prod != defaultprod);
	}
	
	
    }
    
    Tcl_SetObjResult(interp, Tcl_NewIntObj(result));    
    return TCL_OK;
}

static int 
Vl_symbname(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    int no = 0;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: symbname symbolnumber",TCL_STATIC);
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &no) != TCL_OK)
	return TCL_ERROR;

    if (no < 0 || no > SI_LEN)
    {
	Tcl_SetResult(interp,"symbolnumber out of bounds",TCL_STATIC);
	return TCL_ERROR;
    }
    
    Tcl_SetResult(interp,SI[no].symbname,TCL_STATIC);
    return TCL_OK;
}

static int 
Vl_possible_rules(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    int no = 0;
    int rule;
    Tcl_Obj *result;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: possible_rules symbolnumber",TCL_STATIC);
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &no) != TCL_OK)
	return TCL_ERROR;

    if (no < 0 || no > SI_LEN)
    {
	Tcl_SetResult(interp,"symbolnumber out of bounds",TCL_STATIC);
	return TCL_ERROR;
    }
    
    result = Tcl_NewStringObj("", 0);
    
    for (rule = 0; rule < RI_LEN; rule++)
    {
	if (LHSMAP[rule] == no) {
	    Tcl_ListObjAppendElement(interp, result, Tcl_NewIntObj(rule));
	}
    }
    
    Tcl_SetObjResult(interp, result);
    return TCL_OK;
}

static int 
Vl_listelem(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    int no = 0;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: listelem symbolnumber",TCL_STATIC);
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &no) != TCL_OK)
	return TCL_ERROR;

    if (no < 0 || no > SI_LEN)
    {
	Tcl_SetResult(interp,"symbolnumber out of bounds",TCL_STATIC);
	return TCL_ERROR;
    }
    
    if (SI[no].listelement < 0)	   /* Does not describe a list */
	Tcl_SetResult(interp, "", TCL_STATIC);
    else
	Tcl_SetObjResult(interp, Tcl_NewIntObj(SI[no].listelement));
    return TCL_OK;
}

static int 
Vl_create(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    int no = 0;
    NODEPTR rule;
    int jmp;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: create rulenumber",TCL_STATIC);
	return TCL_ERROR;
    }

    if (Tcl_GetIntFromObj(interp, objv[1], &no) != TCL_OK)
	return TCL_ERROR;

    if (no < 0 || no > RI_LEN)
    {
	Tcl_SetResult(interp,"rulenumber out of bounds",TCL_STATIC);
	return TCL_ERROR;
    }

    if (! RI[no].create) {
	Tcl_SetResult(interp, "Rules for inner list nodes are applied automatically", TCL_STATIC);
	return TCL_ERROR;
    }
    
    jmp = setjmp(on_deadly);
    if(jmp) {
	/* Error return from setjmp() */
	Tcl_SetResult(interp, "Deadly error occurred", TCL_STATIC);
	return TCL_ERROR;
    }

    ErrorInit(interp,1,0);
    rule = (RI[no].create)();

    checkmin(interp, rule);
    
    Tcl_SetObjResult(interp, TclVP_NewVoidPointerObj(rule));
    return TCL_OK;

}

static int 
Vl_getFather(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: getFather node",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {	
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    if (n->_father)
	Tcl_SetObjResult(interp, TclVP_NewVoidPointerObj(n->_father));    
    else
	Tcl_SetObjResult(interp, Tcl_NewObj());
    
    return TCL_OK;
}

static int 
Vl_getRoot(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: getRoot node",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {	
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    while (n->_father != NULLNODEPTR) 
	n = n->_father;
    
    Tcl_SetObjResult(interp, TclVP_NewVoidPointerObj(n));    
    return TCL_OK;
}

static int 
Vl_isAncestor(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR node, possibleson;
    
    if (objc != 3) {
	Tcl_SetResult(interp,"usage: isAncestor node possibleson",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&node) != TCL_OK)
	return TCL_ERROR;
    if (TclVP_GetVoidPointerFromObj(interp, objv[2], (void **)&possibleson) != TCL_OK)
	return TCL_ERROR;

    if (! TreeNodeCheck(node))
    {	
	Tcl_SetResult(interp, "node is not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    if (!TreeNodeCheck(possibleson))
    {	
	Tcl_SetResult(interp, "possibleson is not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    do {
	if (node == possibleson)
	{
	    Tcl_SetResult(interp, "1", TCL_STATIC);
	    return TCL_OK;
	}
	possibleson = possibleson->_father;
	
    } while (possibleson != NULLNODEPTR);
    
    Tcl_SetResult(interp, "0", TCL_STATIC);
    return TCL_OK;
}


static int 
Vl_getCode(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: getCode node",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    Tcl_SetObjResult(interp, Tcl_NewIntObj(n->_prod));    
    return TCL_OK;
}

static int 
Vl_delete(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: delete node",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }
    if (n->_father != NULLNODEPTR) {
	Tcl_SetResult(interp, "Node not extracted", TCL_STATIC);
	return TCL_ERROR;
    }
    

    if (! RI[n->_prod].delrule) {
	Tcl_SetResult(interp, "Rules for inner list nodes are deleted automatically", TCL_STATIC);
	return TCL_ERROR;
    }

    Vl_callback(interp, "::vl::onDelete", objv[1]);

    (*RI[n->_prod].delrule)(n);
     
    Tcl_SetObjResult(interp, Tcl_NewObj());    
    return TCL_OK;
}

static int 
Vl_insert(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    int position = 0;
    NODEPTR father = NULLNODEPTR, node = NULLNODEPTR;
    int replace = 0;
    int result = 0;

    delete_interp = interp;
    
    if (objc != 4 && objc != 5) {
	Tcl_SetResult(interp,"usage: insert father index node ?replace?",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&father) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(father))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    if (strcmp(Tcl_GetStringFromObj(objv[2], (int *)0), "end") == 0) {
	position = -1;
    } else {
	if (Tcl_GetIntFromObj(interp, objv[2], &position) != TCL_OK)
	    return TCL_ERROR;
	if (position < 0) {
	    Tcl_SetResult(interp, "index must be \"end\", 0 or positive number", TCL_STATIC);
	    return TCL_ERROR;
	}
    }
    
    if (TclVP_GetVoidPointerFromObj(interp, objv[3], (void **)&node) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(node))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    if (objc == 5) {
	if (Tcl_GetIntFromObj(interp, objv[4], &replace) != TCL_OK)
	    return TCL_ERROR;
    }

    NODEPTR ancestor = father;

    do {
	if (ancestor == node)
	{
	    Tcl_SetResult(interp, "insert would induce a circle", TCL_STATIC);
	    return TCL_ERROR;
	}
	ancestor = ancestor->_father;
    } while (ancestor != NULLNODEPTR);
    
    if (! RI[father->_prod].insert) {
	Tcl_SetResult(interp, "Rules for inner list nodes are inserted automatically", TCL_STATIC);
	return TCL_ERROR;
    }

    /*
      fprintf (stderr, "Calling insert with father=%p, node=%p, position=%d, replace=%d\n",
      father, node, position, replace);
    */
    result = (*RI[father->_prod].insert)(father, node, position, replace);
    Vl_callback(interp, "::vl::onChange", objv[1]);
     
    switch (result) {
    case 1:
      Tcl_SetResult(interp, "lhs of rule node not of expected type", TCL_STATIC);
      break;
    case 2:
      Tcl_SetResult(interp, "bad index", TCL_STATIC);
      break;
    case 3:
      Tcl_SetResult(interp, "can only replace in this node, not insert", TCL_STATIC);
      break;
    case 4:
      Tcl_SetResult(interp, "Cannot extract node from it's father", TCL_STATIC);
      break;
    default:
      ;
    }
    

    if (result == 0) {
      Tcl_SetObjResult(interp, Tcl_NewObj());    
      return TCL_OK;
    }
    else
      return TCL_ERROR;
}

static int 
Vl_extract(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n, father;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: extract node",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    father = n->_father;
    if (father == NULLNODEPTR) {
      Tcl_SetResult(interp, "node is already extracted", TCL_STATIC);
      return TCL_ERROR;
    }
    
    if (! RI[father->_prod].extract) {
	Tcl_SetResult(interp, "Rules for inner list nodes are extracted automatically", TCL_STATIC);
	return TCL_ERROR;
    }

    (*RI[father->_prod].extract)(father, n);
     
    if (n->_father != NULLNODEPTR) {
      Tcl_SetResult (interp, "Could not extract node", TCL_STATIC);
      return TCL_ERROR;
    }

    checkmin(interp, father);
    
    Tcl_Obj *fatherObj = TclVP_NewVoidPointerObj(father);
    Tcl_IncrRefCount(fatherObj);
    Vl_callback(interp, "::vl::onChange", fatherObj);
    Tcl_DecrRefCount(fatherObj);
    
    Tcl_SetObjResult(interp, Tcl_NewObj());    
    return TCL_OK;
}

static int 
Vl_getDesc(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    int which = 0;
    NODEPTR n;
    
    if (objc != 2 && objc != 3) {
	Tcl_SetResult(interp,"usage: getdesc node ?which?",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    if (objc == 3 && Tcl_GetIntFromObj(interp, objv[2], &which) != TCL_OK)
	return TCL_ERROR;
    
    if (! RI[n->_prod].getdesc) {
	Tcl_SetResult(interp, "Cannot get descendants for inner list nodes", TCL_STATIC);
	return TCL_ERROR;
    }

    Tcl_Obj *retval = (*RI[n->_prod].getdesc)(n, which);
     
    Tcl_SetObjResult(interp, retval);    
    return TCL_OK;
}
    
static int 
Vl_getAttrs(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: getAttrs node",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }
    
    if (! RI[n->_prod].getattrs) { /* Means: Has no Attributes */
	Tcl_SetObjResult(interp, Tcl_NewObj());
	return TCL_OK;
    }

    Tcl_Obj *retval = (*RI[n->_prod].getattrs)(n);
     
    Tcl_SetObjResult(interp, retval);    
    return TCL_OK;
}


/* by cs: */

static int 
Vl_getVLPropertyIds(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: getVLPropertyIds node",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }
    
    if (! RI[n->_prod].getvlpropertyids) { /* Means: Has no Attributes */
	Tcl_SetObjResult(interp, Tcl_NewObj());
	return TCL_OK;
    }

    Tcl_Obj *retval = (*RI[n->_prod].getvlpropertyids)(n);
     
    Tcl_SetObjResult(interp, retval);    
    return TCL_OK;
}

/* by cs: */

static int 
Vl_getVLValues(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n;
    
    if (objc != 2) {
	Tcl_SetResult(interp,"usage: getVLValues node",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }
    
    if (! RI[n->_prod].getvlvalues) { /* Means: Has no Attributes */
	Tcl_SetObjResult(interp, Tcl_NewObj());
	return TCL_OK;
    }

    Tcl_Obj *retval = (*RI[n->_prod].getvlvalues)(n);
     
    Tcl_SetObjResult(interp, retval);    
    return TCL_OK;
}


static int 
Vl_setAttrs(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n;
    int events = 1;
    
    if (objc != 3 && objc != 4) {
	Tcl_SetResult(interp,"usage: setAttrs node attrvaluelist ?event?",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&n) != TCL_OK)
	return TCL_ERROR;
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    Tcl_Obj *attrname = 0, *attrvalue = 0;
    int len = 0, i = 0;
    if (Tcl_ListObjLength(interp, objv[2], &len) != TCL_OK)
	return TCL_ERROR;
    
    if (objc == 4) {
	if (Tcl_GetIntFromObj(interp, objv[3], &events) != TCL_OK)
	    return TCL_ERROR;
    }

    Tcl_SetObjResult(interp, Tcl_NewObj());
    if (! RI[n->_prod].setattr) { /* Means: Has no Attributes */
	return TCL_OK;
    }

    while (i+1 < len) {
	if (Tcl_ListObjIndex(interp, objv[2], i, &attrname) != TCL_OK 
	    || Tcl_ListObjIndex(interp, objv[2], i+1, &attrvalue) != TCL_OK)
	    return TCL_ERROR;
	
	(*RI[n->_prod].setattr)(n, Tcl_GetStringFromObj(attrname, (int *)0), attrvalue);
	i += 2;
	
    }

    if (events) 
	Vl_callback(interp, "::vl::onSetAttr", objv[1]);

    return TCL_OK;
}

static int 
Vl_run(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    NODEPTR n;
    char *ea;
    int deadly;
    int mustexist = 1;
    Tcl_Obj* vals[4];
    int jmp;
    
    if (objc != 3 && objc != 4) {
	Tcl_SetResult(interp,"usage: run eaname node ?mustexist?",TCL_STATIC);
	return TCL_ERROR;
    }

    ea = Tcl_GetStringFromObj(objv[1], (int *)0);
    
    if (TclVP_GetVoidPointerFromObj(interp, objv[2], (void **)&n) != TCL_OK)
	return TCL_ERROR;

    if (objc == 4) {
	if (Tcl_GetIntFromObj(interp, objv[3], &mustexist) != TCL_OK)
	    return TCL_ERROR;
    }
    
    if (! TreeNodeCheck(n))
    {
	Tcl_SetResult(interp, "Not a valid Node Pointer", TCL_STATIC);
	return TCL_ERROR;
    }

    jmp = setjmp(on_deadly);
    if(! jmp) {
	/* normal return from setjmp() */
	RestoreModules(state);
	ErrorInit(interp,1,0);
	RUN_ATTREVAL(n, ea, mustexist);
	deadly = 0;
    }
    else
    {
	deadly = 1;
    }

    vals[0] = Tcl_NewIntObj(ErrorCount[NOTE]);
    vals[1] = Tcl_NewIntObj(ErrorCount[WARNING]);
    vals[2] = Tcl_NewIntObj(ErrorCount[ERROR]);
    vals[3] = Tcl_NewIntObj(ErrorCount[DEADLY]);
    Tcl_SetObjResult(interp,Tcl_NewListObj(4,vals));
    
    return TCL_OK;
}

static int 
Vl_poke(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    char *cp_name;
    int i_val;
    void *ptr = (void *)0;
    char tag = '\0';

    if (objc != 3) {
	Tcl_SetResult(interp,"usage: poke varname value",TCL_STATIC);
	return TCL_ERROR;
    }


    cp_name = Tcl_GetStringFromObj(objv[1], (int *)0);
    if (Tcl_GetIntFromObj(TclInterp, objv[2], &i_val) != TCL_OK)
	return TCL_ERROR;

    if (strncmp(cp_name, "ptr", 3) != 0) 
    {
	Tcl_Obj *res = Tcl_NewStringObj("varname '", -1);
	Tcl_AppendToObj(res, cp_name, -1);
	Tcl_AppendToObj(res, "' must start with sequence 'ptr'", -1);
	Tcl_SetObjResult(interp, res);
	return TCL_ERROR;
    }

    sscanf (cp_name + 3, "%x.%c", (unsigned int *) &ptr, &tag);

    if (ptr == (void *)0 || tag == '\0') 
    {
	Tcl_SetResult(interp, "varname format mismatch. Expecting ptr%x.%c", TCL_STATIC);
	return TCL_ERROR;
    }

    Tcl_SetResult(interp, "", TCL_STATIC);
    
    if (tag == 'i') 
    {
	VLInt *val = (VLInt *)ptr;
	val->setInt(i_val);
	return TCL_OK;
    }

    if (tag == 'x')
    {
	VLPointArray *val = (VLPointArray *)ptr;
	val->setXY(0, i_val, val->getY(0));
	return TCL_OK;
    }

    if (tag == 'y')
    {
	VLPointArray *val = (VLPointArray *)ptr;
	val->setXY(0, val->getX(0), i_val);
	return TCL_OK;
    }

    Tcl_SetResult(interp, "unknown tag-field", TCL_STATIC);
    return TCL_ERROR;
}

static int 
Vl_isAssociated(ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj*CONST* objv)
{
    VLPropertyPtr prop1 = (VLPropertyPtr)0, prop2 = (VLPropertyPtr)0;
    
    if (objc != 3) {
	Tcl_SetResult(interp,"usage: isAssociated prop1 prop2",TCL_STATIC);
	return TCL_ERROR;
    }

    if (TclVP_GetVoidPointerFromObj(interp, objv[1], (void **)&prop1) != TCL_OK
	|| TclVP_GetVoidPointerFromObj(interp, objv[2], (void **)&prop2) != TCL_OK) 
	return TCL_ERROR;
    
    if (prop1 != 0 && prop2 != 0 && prop1->isAssociatedWith(prop2))
	Tcl_SetResult(interp,"1",TCL_STATIC);
    else
	Tcl_SetResult(interp,"0",TCL_STATIC);
    return TCL_OK;
    
}


#ifdef __cplusplus
extern "C" 
{
#endif

#if defined (_WIN32)
    #define WINEXPORT(t) __declspec(dllexport) t
#elif defined (_WIN)
    #define WINEXPORT(t) t _export
#else
    #define WINEXPORT(t) t
#endif

#if defined (__MWERKS__)
    #pragma export on
#endif
  
WINEXPORT(int) Vl_Init(Tcl_Interp* interp)
{

    char *cmd;
    
    TclVP_registerVoidPointerType();
    
    if (Tcl_Eval(interp, "namespace eval ::vl {}") != TCL_OK) {
	Tcl_SetResult(interp,"Could not create ::vl namespace",TCL_STATIC);
	return TCL_ERROR;
    }
    Tcl_ObjSetVar2(interp, 
		   Tcl_NewStringObj("::vl::rulecount", -1), 0, Tcl_NewIntObj(RI_LEN), 
		   TCL_GLOBAL_ONLY);
    Tcl_ObjSetVar2(interp, 
		   Tcl_NewStringObj("::vl::symbolcount", -1), 0, Tcl_NewIntObj(SI_LEN), 
		   TCL_GLOBAL_ONLY);
    Tcl_ObjSetVar2(interp, 
		   Tcl_NewStringObj("::vl::rootrule", -1), 0, Tcl_NewIntObj(VL_ROOT), 
		   TCL_GLOBAL_ONLY);
    Tcl_ObjSetVar2(interp, 
		   Tcl_NewStringObj("::vl::onSetAttr", -1), 0, Tcl_NewStringObj("", -1), 
		   TCL_GLOBAL_ONLY);
    Tcl_ObjSetVar2(interp, 
		   Tcl_NewStringObj("::vl::onChange", -1), 0, Tcl_NewStringObj("", -1), 
		   TCL_GLOBAL_ONLY);
    Tcl_ObjSetVar2(interp, 
		   Tcl_NewStringObj("::vl::onDelete", -1), 0, Tcl_NewStringObj("", -1), 
		   TCL_GLOBAL_ONLY);
    Tcl_ObjSetVar2(interp, 
		   Tcl_NewStringObj("::vl::onError", -1), 0, Tcl_NewStringObj("", -1), 
		   TCL_GLOBAL_ONLY);
    Tcl_ObjSetVar2(interp, 
		   Tcl_NewStringObj("::vl::onLog", -1), 0, Tcl_NewStringObj("", -1), 
		   TCL_GLOBAL_ONLY);

    
    Tcl_CreateObjCommand(interp, "::vl::version", Vl_version, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::symbname", Vl_symbname, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::rulename", Vl_rulename, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::ruletype", Vl_ruletype, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::lhs", Vl_lhs, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::possible_rules", Vl_possible_rules, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::listelem", Vl_listelem, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::create", Vl_create, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::insert", Vl_insert, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::extract", Vl_extract, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::getFather", Vl_getFather, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::getRoot", Vl_getRoot, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::isAncestor", Vl_isAncestor, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::getCode", Vl_getCode, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::isRemovable", Vl_isRemovable, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::delete", Vl_delete, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::getDesc", Vl_getDesc, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::getAttrs", Vl_getAttrs, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::getVLPropertyIds", Vl_getVLPropertyIds, 0, 0); /* by cs */
    Tcl_CreateObjCommand(interp, "::vl::getVLValues", Vl_getVLValues, 0, 0); /* by cs */
    Tcl_CreateObjCommand(interp, "::vl::setAttrs", Vl_setAttrs, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::run", Vl_run, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::poke", Vl_poke, 0, 0);
    Tcl_CreateObjCommand(interp, "::vl::isAssociated", Vl_isAssociated, 0, 0);

    Eli_Init(interp);

    cmd = \
"for {set i 0} {$i < $::vl::rulecount} {incr i} {\n\
   set ::vl::prod([::vl::rulename $i]) $i\n\
}\n\
for {set i 0} {$i < $::vl::symbolcount} {incr i} {\n\
   set ::vl::symb([::vl::symbname $i]) $i\n\
}\n\
unset i";

    if (Tcl_Eval(interp, cmd) != TCL_OK) {
	Tcl_SetResult(interp,"Could not create prod and symb arrays",TCL_STATIC);
	return TCL_ERROR;
    }

    Tcl_PkgProvide(interp, "vl", "1.0");

    return TCL_OK;
}

WINEXPORT(int) Vl_SafeInit(Tcl_Interp* interp)
{
        return Vl_Init(interp);

}
#ifdef __cplusplus
}
#endif


