# ----------------------------------------------------------------------
# Module treecheck
# ----------------------------------------------------------------------
#

namespace eval ::treecheck {
}

proc ::treecheck::init {} {
    ::event::add_listener tree_check "::treecheck::on_check"
    set ::treecheck::nodes {}
    set ::treecheck::perform_called 0
    set ::treecheck::iterations 0

    set ::vl::onSetAttr "::treecheck::on_check"
    set ::vl::onChange  "::treecheck::on_check"
    set ::vl::onDelete  "::event::post tree_delete 1"

    ::event::add_listener tree_delete "::treecheck::on_delete"
}

proc ::treecheck::on_delete {node} {
    set nodes $::treecheck::nodes 
    set ::treecheck::nodes {}
    
    foreach n $nodes {
	if {! [::vl::isAncestor $node $n]} {
	    lappend ::treecheck::nodes $n
	}
    }

}

proc ::treecheck::on_check {node} {
    lappend ::treecheck::nodes $node
    
    if {$::treecheck::perform_called == 0} {
	set ::treecheck::perform_called 1
	after idle {::treecheck::perform}
    } 

}

proc ::treecheck::perform {} {
    variable changes

    ::log::info timing "Treecheck: [time {
    foreach prio [array names changes] {
	unset changes($prio)
    }
    incr ::treecheck::iterations

    # Free the way to be called again, but save nodelist first.
    set nodes $::treecheck::nodes
    set ::treecheck::nodes {}
    set ::treecheck::perform_called 0

    # Get the root of this tree (maybe there exist multiple files and this is one
    # of them!)

    # Assume here that all the nodes come from one source file!
    # Normally, here i must compute the roots of all the nodes and
    # accumulate them into a list of root nodes. Then, i must
    # call the check-ea once for each root. Since we have only one
    # sourcefile yet, there is no need for this.

    set roots ""
    foreach n $nodes {
	set r [::vl::getRoot $n]
	if {[lsearch -exact $roots $r] < 0} {
	    lappend roots $r
	}
    }

    ::log::info execution "Running Check #$::treecheck::iterations for $roots"
    foreach root $roots {
        ::property::preCheck
	::error::initCheckRun [::file::getErrorListId [::file::getFileId $root]]
	::vl::run check $root
	::error::doneCheckRun
        ::property::postCheck
    }

    set currentchanges ""
    set currentprio 0
    foreach prio [lsort -integer [array names changes]] {
	if {$changes($prio) != "" && $changes($prio) != "{}"} {
	    set currentchanges $changes($prio)
	    set currentprio $prio
	    break
	}
    }
    
    if {$currentchanges != ""} {
	if {$::treecheck::iterations <= $currentprio + 1} {
	    # If the tree is modified as a result of this, events are
	    # posted that call this procedure again.

	    foreach command $currentchanges {
#		::log::info treeModification "Tree Modification:\n$command"
		eval $command
	    }
	} else {
	    ::log::log "Aborting treecheck loop that seems endless. Next Command would be:"
	    foreach command $currentchanges {
		::log::log "Tree Modification:\n$command"
	    }
	}

    }

    }]"
    if {$::treecheck::nodes != ""} {
	# It seems as we will be called again, as there appeared new 
	# nodes in the do-the-updates-list
	# Simply append our nodes to it.

	eval lappend ::treecheck::nodes $nodes
    } else {

	# Must report every event i got to the others sometimes, so that
	# there are no lost updates. If much work needs to be done for update,
	# there should be a guard against too regular updates.

        ::property::update
	foreach node $nodes {
	    ::event::post tree_changed 0 $node
	}
	set ::treecheck::iterations 0
    }
}

proc ::treecheck::changetree {prio command} {
    if {$command != ""} {
	lappend ::treecheck::changes($prio) $command
    }
}

