
/* implementation of tree construction functions */

#include "node.h"

#include "nodecode.h"

#include "attrpredef.h"

#include "visitmap_check.h"

#include "treeact.h"

#ifdef MONITOR
#include "attr_mon_dapto.h"
#endif

#include <stdlib.h>

#include <stdio.h>

#include "obstack.h"

static Obstack treecon_obst;
#ifdef __cplusplus
void* NODEPTR_struct::operator new(size_t size)
{
	void *retval = malloc(size);
	obstack_make_room(&treecon_obst, sizeof(void *));
	*((void **&)obstack_next_free(&treecon_obst))++ = retval;
	if (retval) return retval;
	fprintf(stderr, "*** DEADLY: No more memory.\n");
	exit(1);
}
void NODEPTR_struct::operator delete(void *data)
{
	NODEPTR *start = (NODEPTR*)obstack_base(&treecon_obst);
	NODEPTR *end = (NODEPTR*)obstack_next_free(&treecon_obst);
	free(data);

	while (start < end)
	{
		if (*start == (NODEPTR)data) {
			obstack_next_free(&treecon_obst) = (char *)--end;
			if (start != end) *start = *end;
			return;
		}
		start++;
	}
fprintf(stderr, "*** DEADLY: Memory corrupted.\n");
}
#else
#if defined(__STDC__) || defined(__cplusplus)
char* TreeNodeAlloc(int size)
#else
char* TreeNodeAlloc(size) int size;
#endif
{
	char *retval = (char *) malloc(size);
	obstack_make_room(&treecon_obst, sizeof(void *));
	*((void **)obstack_next_free(&treecon_obst))++ = retval;
	if (retval) return retval;
	fprintf(stderr, "*** DEADLY: No more memory.\n");
	exit(1);
}
#if defined(__STDC__) || defined(__cplusplus)
void TreeNodeDelete(void *data)
#else
void TreeNodeDelete(data) char *data;
#endif
{
	NODEPTR *start = (NODEPTR*)obstack_base(&treecon_obst);
	NODEPTR *end = (NODEPTR*)obstack_next_free(&treecon_obst);
	free(data);

	while (start < end)
	{
		if (*start == (NODEPTR)data) {
			obstack_next_free(&treecon_obst) = (char *)--end;
			if (start != end) *start = *end;
			return;
		}
		start++;
	}
fprintf(stderr, "*** DEADLY: Memory corrupted.\n");
}
#endif

void InitTree() {
	obstack_init(&treecon_obst);
}

void FreeTree() { }

int TreeNodeCheck(void *data)
{
	NODEPTR *start = (NODEPTR*)obstack_base(&treecon_obst);
	NODEPTR *end = (NODEPTR*)obstack_next_free(&treecon_obst);

	while (start < end)
	{
		if (*start == data) return 1;
		start++;
	}
	return 0;
}
#ifdef MONITOR
#define _SETCOORD(node) \
        node->_coord = _coordref ? *_coordref : NoCoord;
#define _COPYCOORD(node) \
        node->_coord = _currn->_desc1->_coord;
#else
#define _SETCOORD(node)
#define _COPYCOORD(node)
#endif
#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkStateList (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkStateList (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBStateList)) return (_currn);
if (IsSymb (_currn, SYMB_List_Substate)) return (MkRule_1(_coordref, _currn));
return(NULLNODEPTR);
}/* MkStateList */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mk_List_Substate (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR Mk_List_Substate (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMB_List_Substate)) return (_currn);
return(NULLNODEPTR);
}/* Mk_List_Substate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTransitionList (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTransitionList (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTransitionList)) return (_currn);
if (IsSymb (_currn, SYMB_List_Transition)) return (MkRule_2(_coordref, _currn));
return(NULLNODEPTR);
}/* MkTransitionList */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mk_List_Transition (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR Mk_List_Transition (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMB_List_Transition)) return (_currn);
return(NULLNODEPTR);
}/* Mk_List_Transition */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkStatechart (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkStatechart (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBStatechart)) return (_currn);
return(NULLNODEPTR);
}/* MkStatechart */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkROOT (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkROOT (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBROOT)) return (_currn);
if (IsSymb (_currn, SYMBStatechart)) return (MkRule_4(_coordref, _currn));
return(NULLNODEPTR);
}/* MkROOT */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkSubstate (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkSubstate (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBSubstate)) return (_currn);
return(NULLNODEPTR);
}/* MkSubstate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkSimpleState (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkSimpleState (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBSimpleState)) return (_currn);
return(NULLNODEPTR);
}/* MkSimpleState */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkXORName (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkXORName (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBXORName)) return (_currn);
return(NULLNODEPTR);
}/* MkXORName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkXORSubstates (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkXORSubstates (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBXORSubstates)) return (_currn);
if (IsSymb (_currn, SYMB_List_Substate)) return (MkRule_7(_coordref, _currn));
return(NULLNODEPTR);
}/* MkXORSubstates */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkXORSuperstate (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkXORSuperstate (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBXORSuperstate)) return (_currn);
return(NULLNODEPTR);
}/* MkXORSuperstate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkANDName (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkANDName (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBANDName)) return (_currn);
return(NULLNODEPTR);
}/* MkANDName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkANDRegions (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkANDRegions (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBANDRegions)) return (_currn);
if (IsSymb (_currn, SYMB_List_ANDRegion)) return (MkRule_10(_coordref, _currn));
return(NULLNODEPTR);
}/* MkANDRegions */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mk_List_ANDRegion (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR Mk_List_ANDRegion (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMB_List_ANDRegion)) return (_currn);
return(NULLNODEPTR);
}/* Mk_List_ANDRegion */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkANDSuperstate (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkANDSuperstate (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBANDSuperstate)) return (_currn);
return(NULLNODEPTR);
}/* MkANDSuperstate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkANDRegion (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkANDRegion (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBANDRegion)) return (_currn);
return(NULLNODEPTR);
}/* MkANDRegion */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTransitionLabel (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTransitionLabel (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTransitionLabel)) return (_currn);
return(NULLNODEPTR);
}/* MkTransitionLabel */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTransition (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTransition (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTransition)) return (_currn);
return(NULLNODEPTR);
}/* MkTransition */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_1 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR MkRule_1 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPRule_1 _currn;
#ifdef __cplusplus
_currn = new _TPRule_1;
#else
_currn = (_TPPRule_1) TreeNodeAlloc (sizeof (struct _TPRule_1));
#endif
_currn->_prod = RULERule_1;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSP_List_Substate) Mk_List_Substate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_1: root of subtree no. 1 can not be made a _List_Substate node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_Rule_1;
#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkRule_1 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_2 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR MkRule_2 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPRule_2 _currn;
#ifdef __cplusplus
_currn = new _TPRule_2;
#else
_currn = (_TPPRule_2) TreeNodeAlloc (sizeof (struct _TPRule_2));
#endif
_currn->_prod = RULERule_2;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSP_List_Transition) Mk_List_Transition (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_2: root of subtree no. 1 can not be made a _List_Transition node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_Rule_2;
#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkRule_2 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_3 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, VLPointArray _TERM1, VLString _TERM2)
#else
NODEPTR MkRule_3 (_coordref,_desc1,_desc2, _TERM1, _TERM2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	VLPointArray _TERM1;
	VLString _TERM2;
#endif
{	_TPPRule_3 _currn;
#ifdef __cplusplus
_currn = new _TPRule_3;
#else
_currn = (_TPPRule_3) TreeNodeAlloc (sizeof (struct _TPRule_3));
#endif
_currn->_prod = RULERule_3;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSPStateList) MkStateList (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_3: root of subtree no. 1 can not be made a StateList node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionList) MkTransitionList (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_3: root of subtree no. 2 can not be made a TransitionList node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_Rule_3;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Statechart_persview0Size", _TERM1);
#endif

#ifdef MONITOR
_dapto_term_VLString(((NODEPTR)_currn), "_Term_Statechart_persName", _TERM2);
#endif

#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
_currn->_desc2->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkRule_3 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_4 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR MkRule_4 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPRule_4 _currn;
#ifdef __cplusplus
_currn = new _TPRule_4;
#else
_currn = (_TPPRule_4) TreeNodeAlloc (sizeof (struct _TPRule_4));
#endif
_currn->_prod = RULERule_4;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSPStatechart) MkStatechart (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_4: root of subtree no. 1 can not be made a Statechart node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_Rule_4;
#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkRule_4 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkpSimpleState (POSITION *_coordref, NODEPTR _desc1, VLPointArray _TERM1)
#else
NODEPTR MkpSimpleState (_coordref,_desc1, _TERM1)
	POSITION *_coordref;
	NODEPTR _desc1;
	VLPointArray _TERM1;
#endif
{	_TPPpSimpleState _currn;
#ifdef __cplusplus
_currn = new _TPpSimpleState;
#else
_currn = (_TPPpSimpleState) TreeNodeAlloc (sizeof (struct _TPpSimpleState));
#endif
_currn->_prod = RULEpSimpleState;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSPSimpleState) MkSimpleState (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pSimpleState: root of subtree no. 1 can not be made a SimpleState node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_pSimpleState;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Substate_persview0Pos", _TERM1);
#endif

#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkpSimpleState */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_5 (POSITION *_coordref, VLPointArray _TERM1, VLString _TERM2, VLPropertyPtr _TERM3)
#else
NODEPTR MkRule_5 (_coordref, _TERM1, _TERM2, _TERM3)
	POSITION *_coordref;
	VLPointArray _TERM1;
	VLString _TERM2;
	VLPropertyPtr _TERM3;
#endif
{	_TPPRule_5 _currn;
#ifdef __cplusplus
_currn = new _TPRule_5;
#else
_currn = (_TPPRule_5) TreeNodeAlloc (sizeof (struct _TPRule_5));
#endif
_currn->_prod = RULERule_5;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_SETCOORD(_currn)
_TERMACT_Rule_5;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_SimpleState_persview0SavedPos", _TERM1);
#endif

#ifdef MONITOR
_dapto_term_VLString(((NODEPTR)_currn), "_Term_SimpleState_persName", _TERM2);
#endif

#ifdef MONITOR
_dapto_term_VLPropertyPtr(((NODEPTR)_currn), "_Term_SimpleState_persEndpoint", _TERM3);
#endif

return ( (NODEPTR) _currn);
}/* MkRule_5 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_6 (POSITION *_coordref)
#else
NODEPTR MkRule_6 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPRule_6 _currn;
#ifdef __cplusplus
_currn = new _TPRule_6;
#else
_currn = (_TPPRule_6) TreeNodeAlloc (sizeof (struct _TPRule_6));
#endif
_currn->_prod = RULERule_6;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_SETCOORD(_currn)
_TERMACT_Rule_6;
return ( (NODEPTR) _currn);
}/* MkRule_6 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_7 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR MkRule_7 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPRule_7 _currn;
#ifdef __cplusplus
_currn = new _TPRule_7;
#else
_currn = (_TPPRule_7) TreeNodeAlloc (sizeof (struct _TPRule_7));
#endif
_currn->_prod = RULERule_7;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSP_List_Substate) Mk_List_Substate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_7: root of subtree no. 1 can not be made a _List_Substate node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_Rule_7;
#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkRule_7 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkpXORSuperstate (POSITION *_coordref, NODEPTR _desc1, VLPointArray _TERM1)
#else
NODEPTR MkpXORSuperstate (_coordref,_desc1, _TERM1)
	POSITION *_coordref;
	NODEPTR _desc1;
	VLPointArray _TERM1;
#endif
{	_TPPpXORSuperstate _currn;
#ifdef __cplusplus
_currn = new _TPpXORSuperstate;
#else
_currn = (_TPPpXORSuperstate) TreeNodeAlloc (sizeof (struct _TPpXORSuperstate));
#endif
_currn->_prod = RULEpXORSuperstate;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSPXORSuperstate) MkXORSuperstate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pXORSuperstate: root of subtree no. 1 can not be made a XORSuperstate node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_pXORSuperstate;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Substate_persview0Pos", _TERM1);
#endif

#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkpXORSuperstate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_8 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, VLPointArray _TERM1, VLPointArray _TERM2, VLPropertyPtr _TERM3, VLString _TERM4)
#else
NODEPTR MkRule_8 (_coordref,_desc1,_desc2, _TERM1, _TERM2, _TERM3, _TERM4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	VLPointArray _TERM1;
	VLPointArray _TERM2;
	VLPropertyPtr _TERM3;
	VLString _TERM4;
#endif
{	_TPPRule_8 _currn;
#ifdef __cplusplus
_currn = new _TPRule_8;
#else
_currn = (_TPPRule_8) TreeNodeAlloc (sizeof (struct _TPRule_8));
#endif
_currn->_prod = RULERule_8;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSPXORName) MkXORName (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_8: root of subtree no. 1 can not be made a XORName node ", 0, _coordref);
_currn->_desc2 = (_TSPXORSubstates) MkXORSubstates (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_8: root of subtree no. 2 can not be made a XORSubstates node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_Rule_8;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_XORSuperstate_persview0SavedPos", _TERM1);
#endif

#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_XORSuperstate_persview0Size", _TERM2);
#endif

#ifdef MONITOR
_dapto_term_VLPropertyPtr(((NODEPTR)_currn), "_Term_XORSuperstate_persEndpoint", _TERM3);
#endif

#ifdef MONITOR
_dapto_term_VLString(((NODEPTR)_currn), "_Term_XORSuperstate_persName", _TERM4);
#endif

#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
_currn->_desc2->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkRule_8 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_9 (POSITION *_coordref)
#else
NODEPTR MkRule_9 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPRule_9 _currn;
#ifdef __cplusplus
_currn = new _TPRule_9;
#else
_currn = (_TPPRule_9) TreeNodeAlloc (sizeof (struct _TPRule_9));
#endif
_currn->_prod = RULERule_9;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_SETCOORD(_currn)
_TERMACT_Rule_9;
return ( (NODEPTR) _currn);
}/* MkRule_9 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_10 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR MkRule_10 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPRule_10 _currn;
#ifdef __cplusplus
_currn = new _TPRule_10;
#else
_currn = (_TPPRule_10) TreeNodeAlloc (sizeof (struct _TPRule_10));
#endif
_currn->_prod = RULERule_10;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSP_List_ANDRegion) Mk_List_ANDRegion (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_10: root of subtree no. 1 can not be made a _List_ANDRegion node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_Rule_10;
#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkRule_10 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkpANDSuperstate (POSITION *_coordref, NODEPTR _desc1, VLPointArray _TERM1)
#else
NODEPTR MkpANDSuperstate (_coordref,_desc1, _TERM1)
	POSITION *_coordref;
	NODEPTR _desc1;
	VLPointArray _TERM1;
#endif
{	_TPPpANDSuperstate _currn;
#ifdef __cplusplus
_currn = new _TPpANDSuperstate;
#else
_currn = (_TPPpANDSuperstate) TreeNodeAlloc (sizeof (struct _TPpANDSuperstate));
#endif
_currn->_prod = RULEpANDSuperstate;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSPANDSuperstate) MkANDSuperstate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pANDSuperstate: root of subtree no. 1 can not be made a ANDSuperstate node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_pANDSuperstate;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Substate_persview0Pos", _TERM1);
#endif

#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkpANDSuperstate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_11 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, VLPointArray _TERM1, VLPropertyPtr _TERM2, VLString _TERM3)
#else
NODEPTR MkRule_11 (_coordref,_desc1,_desc2, _TERM1, _TERM2, _TERM3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	VLPointArray _TERM1;
	VLPropertyPtr _TERM2;
	VLString _TERM3;
#endif
{	_TPPRule_11 _currn;
#ifdef __cplusplus
_currn = new _TPRule_11;
#else
_currn = (_TPPRule_11) TreeNodeAlloc (sizeof (struct _TPRule_11));
#endif
_currn->_prod = RULERule_11;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSPANDName) MkANDName (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_11: root of subtree no. 1 can not be made a ANDName node ", 0, _coordref);
_currn->_desc2 = (_TSPANDRegions) MkANDRegions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE Rule_11: root of subtree no. 2 can not be made a ANDRegions node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_Rule_11;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_ANDSuperstate_persview0SavedPos", _TERM1);
#endif

#ifdef MONITOR
_dapto_term_VLPropertyPtr(((NODEPTR)_currn), "_Term_ANDSuperstate_persEndpoint", _TERM2);
#endif

#ifdef MONITOR
_dapto_term_VLString(((NODEPTR)_currn), "_Term_ANDSuperstate_persName", _TERM3);
#endif

#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
_currn->_desc2->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkRule_11 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkpANDRegion (POSITION *_coordref, NODEPTR _desc1, VLPointArray _TERM1, VLPointArray _TERM2)
#else
NODEPTR MkpANDRegion (_coordref,_desc1, _TERM1, _TERM2)
	POSITION *_coordref;
	NODEPTR _desc1;
	VLPointArray _TERM1;
	VLPointArray _TERM2;
#endif
{	_TPPpANDRegion _currn;
#ifdef __cplusplus
_currn = new _TPpANDRegion;
#else
_currn = (_TPPpANDRegion) TreeNodeAlloc (sizeof (struct _TPpANDRegion));
#endif
_currn->_prod = RULEpANDRegion;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSP_List_Substate) Mk_List_Substate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pANDRegion: root of subtree no. 1 can not be made a _List_Substate node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_pANDRegion;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_ANDRegion_persview0Pos", _TERM1);
#endif

#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_ANDRegion_persview0Size", _TERM2);
#endif

#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkpANDRegion */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRule_12 (POSITION *_coordref, VLPointArray _TERM1)
#else
NODEPTR MkRule_12 (_coordref, _TERM1)
	POSITION *_coordref;
	VLPointArray _TERM1;
#endif
{	_TPPRule_12 _currn;
#ifdef __cplusplus
_currn = new _TPRule_12;
#else
_currn = (_TPPRule_12) TreeNodeAlloc (sizeof (struct _TPRule_12));
#endif
_currn->_prod = RULERule_12;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_SETCOORD(_currn)
_TERMACT_Rule_12;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_TransitionLabel_persview0Pos", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* MkRule_12 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkpTransition (POSITION *_coordref, NODEPTR _desc1, VLPointArray _TERM1, VLPointArray _TERM2, VLPointArray _TERM3, VLPointArray _TERM4, VLPointArray _TERM5, VLPointArray _TERM6, VLPropertyPtr _TERM7, VLPropertyPtr _TERM8, VLString _TERM9)
#else
NODEPTR MkpTransition (_coordref,_desc1, _TERM1, _TERM2, _TERM3, _TERM4, _TERM5, _TERM6, _TERM7, _TERM8, _TERM9)
	POSITION *_coordref;
	NODEPTR _desc1;
	VLPointArray _TERM1;
	VLPointArray _TERM2;
	VLPointArray _TERM3;
	VLPointArray _TERM4;
	VLPointArray _TERM5;
	VLPointArray _TERM6;
	VLPropertyPtr _TERM7;
	VLPropertyPtr _TERM8;
	VLString _TERM9;
#endif
{	_TPPpTransition _currn;
#ifdef __cplusplus
_currn = new _TPpTransition;
#else
_currn = (_TPPpTransition) TreeNodeAlloc (sizeof (struct _TPpTransition));
#endif
_currn->_prod = RULEpTransition;
#if _NODE_UPREF
_currn->_father = NULLNODEPTR;
#endif
_currn->_desc1 = (_TSPTransitionLabel) MkTransitionLabel (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pTransition: root of subtree no. 1 can not be made a TransitionLabel node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_pTransition;
#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Transition_persview0Pos", _TERM1);
#endif

#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Transition_persview0FromOffset", _TERM2);
#endif

#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Transition_persview0ToOffset", _TERM3);
#endif

#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Transition_persview0SavedFromOffset", _TERM4);
#endif

#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Transition_persview0SavedToOffset", _TERM5);
#endif

#ifdef MONITOR
_dapto_term_VLPointArray(((NODEPTR)_currn), "_Term_Transition_persview0IntermediatePoints", _TERM6);
#endif

#ifdef MONITOR
_dapto_term_VLPropertyPtr(((NODEPTR)_currn), "_Term_Transition_persFrom", _TERM7);
#endif

#ifdef MONITOR
_dapto_term_VLPropertyPtr(((NODEPTR)_currn), "_Term_Transition_persTo", _TERM8);
#endif

#ifdef MONITOR
_dapto_term_VLString(((NODEPTR)_currn), "_Term_Transition_persEvent", _TERM9);
#endif

#if _NODE_UPREF
_currn->_desc1->_father = (NODEPTR) _currn;
#endif
return ( (NODEPTR) _currn);
}/* MkpTransition */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _MkSubstatep_List_Substate (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR _MkSubstatep_List_Substate (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	_TPPLST_Substatep_List_Substate _new;
#ifdef __cplusplus
	_new = new _TPLST_Substatep_List_Substate;
#else
	_new = (_TPPLST_Substatep_List_Substate) TreeNodeAlloc (sizeof (struct _TPLST_Substatep_List_Substate));
#endif
	_new->_prod = RULELST_Substatep_List_Substate;
#if _NODE_UPREF
	_new->_father = NULLNODEPTR;
#endif
	_new->_desc1 = (_TSPSubstate)elem;
	_new->_desc2 = (_TSPLST__List_Substate)_new;
	{_TPPLST_Substatep_List_Substate _currn = _new;
	 _SETCOORD(_currn)
	 _TERMACT_LST_Substatep_List_Substate;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _Elemp_List_Substate (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _Elemp_List_Substate (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (IsSymb (t, SYMBSubstate)) 
		return _MkSubstatep_List_Substate (_coordref, MkSubstate (_coordref, t));
	message (DEADLY, "a p_List_Substate list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mk2p_List_Substate (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR Mk2p_List_Substate (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (LHSMAP[l->_prod] != SYMBLST__List_Substate)
	l =_Elemp_List_Substate (_coordref, l);
if (LHSMAP[r->_prod] != SYMBLST__List_Substate)
	r = _Elemp_List_Substate (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((_TPPLST_0p_List_Substate)r)->_desc2);
((_TPPLST_0p_List_Substate)r)->_desc2 = ((_TPPLST_0p_List_Substate)l)->_desc2;
((_TPPLST_0p_List_Substate)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkp_List_Substate (POSITION *_coordref, NODEPTR t)
#else
NODEPTR Mkp_List_Substate (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; _TPPp_List_Substate root;
	_TPPLST_0p_List_Substate list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (_TPPLST_0p_List_Substate)NULLNODEPTR;
else {	if (LHSMAP[t->_prod] != SYMBLST__List_Substate)
		t = _Elemp_List_Substate (_coordref, t);
	remain = (_TPPLST_0p_List_Substate)(((_TPPLST_0p_List_Substate) t)->_desc2);
	((_TPPLST_0p_List_Substate) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new _TPLST_0p_List_Substate;
#else
list = (_TPPLST_0p_List_Substate) TreeNodeAlloc (sizeof (struct _TPLST_0p_List_Substate));
#endif
list->_prod = RULELST_0p_List_Substate;
#if _NODE_UPREF
list->_father = NULLNODEPTR;
#endif
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (_TPPLST_0p_List_Substate)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == RULELST_Substatep_List_Substate)
	{_TPPLST_Substatep_List_Substate _currn = _currn;
	 _TERMACT_LST_Substatep_List_Substate;
	} this code has been moved to _Elemp_List_Substate*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new _TPp_List_Substate;
#else
root = (_TPPp_List_Substate) TreeNodeAlloc (sizeof (struct _TPp_List_Substate));
#endif
root->_prod = RULEp_List_Substate;
#if _NODE_UPREF
root->_father = NULLNODEPTR;
#endif
root->_desc1 = (_TSPLST__List_Substate)list;
{ _TPPp_List_Substate _currn = root;
  _SETCOORD(_currn)
  _TERMACT_p_List_Substate;
}
_currn = (NODEPTR)root;
#if _NODE_UPREF
while(list) {
  list->_father = (NODEPTR) root;
  if(list->_desc1)
    list->_desc1->_father = (NODEPTR) root;
  list = (_TPPLST_0p_List_Substate) list->_desc2;
}
#endif
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _MkANDRegionp_List_ANDRegion (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR _MkANDRegionp_List_ANDRegion (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	_TPPLST_ANDRegionp_List_ANDRegion _new;
#ifdef __cplusplus
	_new = new _TPLST_ANDRegionp_List_ANDRegion;
#else
	_new = (_TPPLST_ANDRegionp_List_ANDRegion) TreeNodeAlloc (sizeof (struct _TPLST_ANDRegionp_List_ANDRegion));
#endif
	_new->_prod = RULELST_ANDRegionp_List_ANDRegion;
#if _NODE_UPREF
	_new->_father = NULLNODEPTR;
#endif
	_new->_desc1 = (_TSPANDRegion)elem;
	_new->_desc2 = (_TSPLST__List_ANDRegion)_new;
	{_TPPLST_ANDRegionp_List_ANDRegion _currn = _new;
	 _SETCOORD(_currn)
	 _TERMACT_LST_ANDRegionp_List_ANDRegion;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _Elemp_List_ANDRegion (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _Elemp_List_ANDRegion (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (IsSymb (t, SYMBANDRegion)) 
		return _MkANDRegionp_List_ANDRegion (_coordref, MkANDRegion (_coordref, t));
	message (DEADLY, "a p_List_ANDRegion list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mk2p_List_ANDRegion (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR Mk2p_List_ANDRegion (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (LHSMAP[l->_prod] != SYMBLST__List_ANDRegion)
	l =_Elemp_List_ANDRegion (_coordref, l);
if (LHSMAP[r->_prod] != SYMBLST__List_ANDRegion)
	r = _Elemp_List_ANDRegion (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((_TPPLST_0p_List_ANDRegion)r)->_desc2);
((_TPPLST_0p_List_ANDRegion)r)->_desc2 = ((_TPPLST_0p_List_ANDRegion)l)->_desc2;
((_TPPLST_0p_List_ANDRegion)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkp_List_ANDRegion (POSITION *_coordref, NODEPTR t)
#else
NODEPTR Mkp_List_ANDRegion (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; _TPPp_List_ANDRegion root;
	_TPPLST_0p_List_ANDRegion list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (_TPPLST_0p_List_ANDRegion)NULLNODEPTR;
else {	if (LHSMAP[t->_prod] != SYMBLST__List_ANDRegion)
		t = _Elemp_List_ANDRegion (_coordref, t);
	remain = (_TPPLST_0p_List_ANDRegion)(((_TPPLST_0p_List_ANDRegion) t)->_desc2);
	((_TPPLST_0p_List_ANDRegion) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new _TPLST_0p_List_ANDRegion;
#else
list = (_TPPLST_0p_List_ANDRegion) TreeNodeAlloc (sizeof (struct _TPLST_0p_List_ANDRegion));
#endif
list->_prod = RULELST_0p_List_ANDRegion;
#if _NODE_UPREF
list->_father = NULLNODEPTR;
#endif
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (_TPPLST_0p_List_ANDRegion)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == RULELST_ANDRegionp_List_ANDRegion)
	{_TPPLST_ANDRegionp_List_ANDRegion _currn = _currn;
	 _TERMACT_LST_ANDRegionp_List_ANDRegion;
	} this code has been moved to _Elemp_List_ANDRegion*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new _TPp_List_ANDRegion;
#else
root = (_TPPp_List_ANDRegion) TreeNodeAlloc (sizeof (struct _TPp_List_ANDRegion));
#endif
root->_prod = RULEp_List_ANDRegion;
#if _NODE_UPREF
root->_father = NULLNODEPTR;
#endif
root->_desc1 = (_TSPLST__List_ANDRegion)list;
{ _TPPp_List_ANDRegion _currn = root;
  _SETCOORD(_currn)
  _TERMACT_p_List_ANDRegion;
}
_currn = (NODEPTR)root;
#if _NODE_UPREF
while(list) {
  list->_father = (NODEPTR) root;
  if(list->_desc1)
    list->_desc1->_father = (NODEPTR) root;
  list = (_TPPLST_0p_List_ANDRegion) list->_desc2;
}
#endif
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _MkTransitionp_List_Transition (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR _MkTransitionp_List_Transition (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	_TPPLST_Transitionp_List_Transition _new;
#ifdef __cplusplus
	_new = new _TPLST_Transitionp_List_Transition;
#else
	_new = (_TPPLST_Transitionp_List_Transition) TreeNodeAlloc (sizeof (struct _TPLST_Transitionp_List_Transition));
#endif
	_new->_prod = RULELST_Transitionp_List_Transition;
#if _NODE_UPREF
	_new->_father = NULLNODEPTR;
#endif
	_new->_desc1 = (_TSPTransition)elem;
	_new->_desc2 = (_TSPLST__List_Transition)_new;
	{_TPPLST_Transitionp_List_Transition _currn = _new;
	 _SETCOORD(_currn)
	 _TERMACT_LST_Transitionp_List_Transition;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _Elemp_List_Transition (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _Elemp_List_Transition (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (IsSymb (t, SYMBTransition)) 
		return _MkTransitionp_List_Transition (_coordref, MkTransition (_coordref, t));
	message (DEADLY, "a p_List_Transition list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mk2p_List_Transition (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR Mk2p_List_Transition (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (LHSMAP[l->_prod] != SYMBLST__List_Transition)
	l =_Elemp_List_Transition (_coordref, l);
if (LHSMAP[r->_prod] != SYMBLST__List_Transition)
	r = _Elemp_List_Transition (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((_TPPLST_0p_List_Transition)r)->_desc2);
((_TPPLST_0p_List_Transition)r)->_desc2 = ((_TPPLST_0p_List_Transition)l)->_desc2;
((_TPPLST_0p_List_Transition)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkp_List_Transition (POSITION *_coordref, NODEPTR t)
#else
NODEPTR Mkp_List_Transition (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; _TPPp_List_Transition root;
	_TPPLST_0p_List_Transition list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (_TPPLST_0p_List_Transition)NULLNODEPTR;
else {	if (LHSMAP[t->_prod] != SYMBLST__List_Transition)
		t = _Elemp_List_Transition (_coordref, t);
	remain = (_TPPLST_0p_List_Transition)(((_TPPLST_0p_List_Transition) t)->_desc2);
	((_TPPLST_0p_List_Transition) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new _TPLST_0p_List_Transition;
#else
list = (_TPPLST_0p_List_Transition) TreeNodeAlloc (sizeof (struct _TPLST_0p_List_Transition));
#endif
list->_prod = RULELST_0p_List_Transition;
#if _NODE_UPREF
list->_father = NULLNODEPTR;
#endif
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (_TPPLST_0p_List_Transition)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == RULELST_Transitionp_List_Transition)
	{_TPPLST_Transitionp_List_Transition _currn = _currn;
	 _TERMACT_LST_Transitionp_List_Transition;
	} this code has been moved to _Elemp_List_Transition*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new _TPp_List_Transition;
#else
root = (_TPPp_List_Transition) TreeNodeAlloc (sizeof (struct _TPp_List_Transition));
#endif
root->_prod = RULEp_List_Transition;
#if _NODE_UPREF
root->_father = NULLNODEPTR;
#endif
root->_desc1 = (_TSPLST__List_Transition)list;
{ _TPPp_List_Transition _currn = root;
  _SETCOORD(_currn)
  _TERMACT_p_List_Transition;
}
_currn = (NODEPTR)root;
#if _NODE_UPREF
while(list) {
  list->_father = (NODEPTR) root;
  if(list->_desc1)
    list->_desc1->_father = (NODEPTR) root;
  list = (_TPPLST_0p_List_Transition) list->_desc2;
}
#endif
return (NODEPTR)_currn;
}
