
# by cs:
# In dieser Datei sind zusaetzliche nuetzliche Funktionen zur Arbeit mit
# der abstrakten Struktur implementiert.


namespace eval ::treeutil {
    variable data
    variable nextid 0
}


proc ::treeutil::exists {node} {
    # einfach eine Funktion aufrufen die schon merkt, ob der Knoten existiert
    return [expr ![catch {::vl::getCode $node}]]
}

proc ::treeutil::setAttr {node attrName attrValue} {
  #teste zuerst, ob Attribt existiert
  array set attrs [::vl::getAttrs $node]
  if [catch {set dummy $attrs($attrName)}] {
   ::log::log "::treeutil::setAttr: Attribute $attrName of node $node does not exist"
   error "see log"
  }
  ::vl::setAttrs $node [list $attrName $attrValue]
}



proc ::treeutil::attrExists {node attrName} {
    array set attrs [::vl::getAttrs $node]
    return [info exists attrs($attrName)]
}


proc ::treeutil::getAttr {node attrName} {
  array set attrs [::vl::getAttrs $node]
  if [catch {set attr $attrs($attrName)}] {
   ::log::log "::treeutil::getAttr: Attribute $attrName of node $node does not exist"
   error "see log"
  }
  return $attr
}

proc ::treeutil::getSubNode {node position} {
  set subnode [lindex [::vl::getDesc $node] $position]
  if {$subnode == ""} {
    ::log::log "::treeutil::getSubNode: Subnode $position of node $node does not exist"
    error "see log"
  }
  return $subnode
}


proc ::treeutil::insertNode {destNode listIndex newNode} {
  if {[catch {::vl::insert $destNode $listIndex $newNode} errmsg]} {
    ::vl::delete $newNode
    ::log::log "::treeutil::insertNode: Can not insert node at index $listIndex under $destNode:"
    ::log::log $::errorInfo
    error "see log"
  }
}

proc ::treeutil::getRemovableSuperNode {node} {
    while { ![::vl::isRemovable $node] } {
        set node [::vl::getFather $node]
    }
    return $node
}


proc ::treeutil::extractSuperNode {node} {
    set node [getRemovableSuperNode $node]
    ::vl::extract $node
}


proc ::treeutil::removeSuperNode {node} {
    set node [getRemovableSuperNode $node]
    ::treeutil::removeNode $node
}


proc ::treeutil::removeNode {node} {
  if { ![::vl::isRemovable $node] } {
    ::log::log "::treeutil::removeNode: Node $node ist not removable"
    error "see log"
  }
  ::vl::extract $node
  ::vl::delete $node
}


proc ::treeutil::setSubAttr {node attrName value} {
    if [catch {
        while { ![attrExists $node $attrName] } {
            set node [::treeutil::getSubNode $node 0]
        }
        ::treeutil::setAttr $node $attrName $value
    }] {
        ::log::log "setSubAttr: cant set $attrName"
        error "see log"
    }
}

proc ::treeutil::connect {symbolAttrName attrName2 symbol production} {
    set value [::treeutil::getAttr $symbol $symbolAttrName]
    ::treeutil::setSubAttr $production $attrName2 $value
}


proc ::treeutil::serializeSubtree {node} {
    lappend liste [list treecons_applyRule [::vl::rulename [::vl::getCode $node]] $node [::vl::getDesc $node]]  
  if [llength [::vl::getVLValues $node]] {
      lappend liste [list treecons_setAttrs $node [::vl::getVLValues $node]]
  }
  if [llength [::vl::getVLPropertyIds $node]] {
      lappend liste [list treecons_setConnections $node [::vl::getVLPropertyIds $node]]
  }
  foreach n [::vl::getDesc $node] {
      set liste [concat $liste [::treeutil::serializeSubtree $n]]
  }
  return $liste
}


proc ::treeutil::unserializeSubtree {liste} {
    treecons_init
    foreach command $liste {
        eval $command
    }
    return [treecons_done]
}


# ****************************************** Laden von Baeumen: treecons


# Zwischen treecons_init und treecons_done existiert eine Datenstruktur
# Zur Verwaltung der logischen Namen, die in der Datei vorkommen:
#
# treecons(root):          Zeiger auf die Wurzel des erzeugten Baums
# treecons(nodeptr,$p):    Zeiger auf den Knoten mit logischer Adresse $p
# treecons(parentptr,$p):  Zeiger auf den Vaterknoten des Knotens mit logischer Adresse $p
# treecons(index,$p):      Index des Knotens mit logischer Adresse $p im Vaterknoten
# treecons(key,$k):        Zeiger auf eine (beliebige) VLProperty mit logischem Key $k


proc treecons_init {} {
    global treecons
    #zur Sicherheit:
    catch {unset treecons}
}


proc treecons_done {} {
    global treecons
    set root $treecons(root)
    unset treecons
    return $root
}



proc treecons_applyRule {rulename nodename childnames} {
    global treecons

    #teste, ob den schon vorhandene Knoten verwenden werden kann
    if [info exists treecons(root)] {
        # set newnode [::vl::getDesc $treecons(parentptr,$nodename) $treecons(index,$nodename)]
        set newnode [lindex [::vl::getDesc $treecons(parentptr,$nodename)] $treecons(index,$nodename)]
        if { $newnode == {} } { #Listenknoten!
            unset newnode
        } else {
            if { [::vl::rulename [::vl::getCode $newnode]] != $rulename } {unset newnode}
        }
    }
    
    #falls notwendig fuege einen neuen Knoten ein
    if ![info exists newnode] {
        set newnode [::vl::create $::vl::prod($rulename)]
        if [info exists treecons(root)] {
            # puts "wende produktion $rulename an knoten $nodename an"
            ::vl::insert $treecons(parentptr,$nodename) $treecons(index,$nodename) $newnode 1         
        } else {
            set treecons(root) $newnode
        }
    }
            
    #speichere den Knotenzeiger
    set treecons(nodeptr,$nodename) $newnode
    for {set i 0} {$i<[llength $childnames]} {incr i} {
        set son [lindex $childnames $i]
        set treecons(parentptr,$son) $newnode
        set treecons(index,$son) $i
    }
}


proc treecons_setAttrs {nodename values} {
    global treecons
    ::vl::setAttrs $treecons(nodeptr,$nodename) $values
}


proc treecons_setConnections {nodename ids} {
    global treecons
    array set attr [::vl::getAttrs $treecons(nodeptr,$nodename)]
    foreach {conname keyname} $ids {
        if [info exists treecons(key,$keyname)] {
            ::vl::setAttrs $treecons(nodeptr,$nodename) [list $conname $treecons(key,$keyname)]
        } else {
            set treecons(key,$keyname) $attr($conname)
        }
    }
}

