# ----------------------------------------------------------------------
# Module Visual Drag
# ----------------------------------------------------------------------
# Procedures to create nonhierarchical relationships
#
# $Id: visual.tcl,v 3.4 1999/10/22 15:54:38 mjung Exp $

# Create our namespace.
namespace eval ::visual {
}

proc ::visual::drag_create {win} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    set data(tooldrag,inspoint) ""
    set data(tooldrag,inspoints) ""
    set data(tooldrag,inspointafter) ""

    # Bindings
    $data(workspace) bind drag <ButtonPress-1> "::visual::startobjdrag $win %x %y"
    $data(workspace) bind drag <B1-Motion> "::visual::doobjdrag $win %x %y"
    $data(workspace) bind drag <ButtonRelease-1> "::visual::endobjdrag $win %x %y"
    $data(workspace) bind drag <Enter> \
	    "$data(workspace) configure -cursor pencil"
    $data(workspace) bind drag <Leave> \
	    "$data(workspace) configure -cursor crosshair"

}

proc ::visual::startobjdrag {win x y} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    # Auf Canvaskoordinaten umrechnen (Scrolling)
    set x [$data(workspace) canvasx $x]
    set y [$data(workspace) canvasy $y]

    # If multiple dragobjekts exist for x,y, 
    # choose the selected one, if it exists.
    if {$data(selectedid) != ""} {
	set halo 2
	set ids [$data(workspace) find overlapping [expr $x - $halo] [expr $y - $halo] [expr $x + $halo] [expr $y + $halo]]
	foreach id $ids {
	    set t [$data(workspace) gettags $id]
	    if {[lsearch -exact $t "drag#$data(selectedid)"] >= 0} {
		set data(objdrag) $id
		break
	    }
	}
    } 

    if {[info exists data(objdrag)] == 0} {
	set data(objdrag) [$data(workspace) find withtag current] 
    }

    $data(workspace) itemconfigure $data(objdrag) -outline black

    set tags [$data(workspace) gettags $data(objdrag)]
    set kindind [lsearch $tags "dragkind,*"]
    set data(objdrag,kind) [string range [lindex $tags $kindind] 9 end]

    # No droptarget highlighted
    set data(objdrag,drop) ""
}

proc ::visual::doobjdrag {win x y} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {[info exists data(objdrag)] == 0} {
	# Not dragging.
	return
    }

    # Auf Canvaskoordinaten umrechnen (Scrolling)
    set x [$data(workspace) canvasx $x]
    set y [$data(workspace) canvasy $y]

    # Nchsten Punkt bestimmen
    set kind $data(objdrag,kind)
    set nearestdroptarget [getclosestdrag ::visual::$var $x $y $kind]
    set nearestdroptarget [lindex $nearestdroptarget 0]
    highlightdroptarget ::visual::$var $nearestdroptarget

}
	
proc ::visual::getclosestdrag {varname x y kind} {
    upvar #0 $varname data

    set closest ""
    set closestdistance 0

    set ids ""
    foreach k [split $kind ","] {
	eval lappend ids [$data(workspace) find withtag "drop,$k"]
    }

    foreach id $ids {
	# Calc distance from given point to closest point of the object with the given id
        set bbox [$data(workspace) bbox $id]
	set bx1 [lindex $bbox 0]
	set by1 [lindex $bbox 1]
	set bx2 [lindex $bbox 2]
	set by2 [lindex $bbox 3]

	if {$bx1 < $x && $x < $bx2} {
	    set dx 0
	} elseif {$x <= $bx1} {
	    set dx [expr $bx1 - $x]
	} else {
	    set dx [expr $x - $bx2]
	}

	if {$by1 < $y && $y < $by2} {
	    set dy 0
	} elseif {$y <= $by1} {
	    set dy [expr $by1 - $y]
	} else {
	    set dy [expr $y - $by2]
	}

        set distance [expr sqrt($dx * $dx + $dy * $dy)]

	set tags [$data(workspace) gettags $id]
	set depthindex [lsearch -glob $tags "depth_*"]
	set depth [string range [lindex $tags $depthindex] 6 end]

        if {$closest == ""} {
            set closest $id
	    set closestdepth $depth
            set closestdistance $distance
        } elseif {$distance < $closestdistance} {
            set closest $id
            set closestdistance $distance
	    set closestdepth $depth
        } elseif {$distance == $closestdistance && $depth > $closestdepth} {
            set closest $id
            set closestdistance $distance
	    set closestdepth $depth
	}
    }

    return $closest
}

proc ::visual::endobjdrag {win x y} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {[info exists data(objdrag)] == 0} {
	# Not dragging.
	return
    }

    if {$data(objdrag,drop) != ""} {
	set droptags [$data(workspace) gettags $data(objdrag,drop)]
	set dragtags [$data(workspace) gettags $data(objdrag)]

	set dragid [lsearch $dragtags "drag#*"]
	set dragid [string range [lindex $dragtags $dragid] 5 end]

	set attrname [lsearch $dragtags "dragattr,*"]
	set attrname [string range [lindex $dragtags $attrname] 9 end]

	set associatecmd [lsearch $dragtags "dragcmd,*"]
	if {$associatecmd >= 0} {
	    set associatecmd [string range [lindex $dragtags $associatecmd] 8 end]
	} else {
	    set associatecmd ""
	}

	set args1 ""
	if {[info exists data(vargs,$data(objdrag))]} {
	    set args1 $data(vargs,$data(objdrag))
	}

	set args2 ""
	if {[info exists data(vargs,$data(objdrag,drop))]} {
	    set args2 $data(vargs,$data(objdrag,drop))
	}

	set dropid ""
	set dropindex [lsearch $droptags "prop,*"]
	if {$dropindex >= 0} {
	    set dropid [string range [lindex $droptags $dropindex] 5 end]
	}
	
	if {$dropid != ""} {
	    set partindex [lsearch -glob $droptags "node,*"]
	    set idtag [lindex $droptags $partindex]
	    set partname_drop [lindex [split $idtag ","] 2]
	    set partindex [lsearch -glob $dragtags "drag,*"]
	    set idtag [lindex $droptags $partindex]
	    set partname_drag [lindex [split $idtag ","] 2]
	    setInsertContext objdrag $args1 $args2 [$data(workspace) bbox $data(objdrag,drop)] [list $x $y] $partname_drag $partname_drop

	    if {$associatecmd != ""} {
		eval $associatecmd {$dragid $attrname $dropid}
	    } else {
		if {[catch {::vl::setAttrs $dragid [list $attrname "" $attrname $dropid]}]} {
		    puts $::$errorInfo
		}
	    }
	}
    }
	
    # No highlighted drop target
    highlightdroptarget ::visual::$var ""

    # No highlighted drag source
    $data(workspace) itemconfigure $data(objdrag) -outline ""
    unset data(objdrag)
}

proc ::visual::highlightdroptarget {var obid} {
    variable visInfo
    upvar #0 $var data

    if {$data(objdrag,drop) == $obid} {
	return
    }

    if {$data(objdrag,drop) != ""} {
	$data(workspace) delete dragdropselect
	set data(objdrag,drop) ""
    }

    if {$obid != ""} {
	set bbox [eval $data(workspace) bbox $obid]
	$data(workspace) create rectangle \
		    [expr [lindex $bbox 0] - 1] [expr [lindex $bbox 1] - 1] \
		    [expr [lindex $bbox 2] + 1] [expr [lindex $bbox 3] + 1] \
		    -tags dragdropselect -outline "#0000ff" -width 1
	set data(objdrag,drop) $obid
    }
}



