# ----------------------------------------------------------------------
# Module Visual Insert
# ----------------------------------------------------------------------
# Code to Manage Insertion of Productions from the Toolbar
#
# $Id: visual.tcl,v 3.4 1999/10/22 15:54:38 mjung Exp $

# Create our namespace.
namespace eval ::visual {
}

proc ::visual::insert_init {} {
    variable visInfo
    set visInfo(ias-state) "start"

    bind Toolbarbutton <ButtonPress-1> "::visual::ias_handleEvent controlPressed %W %X %Y"
    bind Toolbarbutton <B1-Motion> "::visual::ias_handleEvent controlMotion %W %X %Y"
    bind Toolbarbutton <ButtonRelease-1> "::visual::ias_handleEvent controlReleased %W %X %Y"
    bind Toolbarbutton <Leave> "::visual::ias_handleEvent controlLeave %W %X %Y"
    bind all <Motion> "+::visual::ias_handleEvent mouseMotion %W %X %Y"
    bind all <ButtonPress-1> "+::visual::ias_handleEvent mousePressed %W %X %Y"

    bind Toolbarbutton <Enter> \
	    {+if {[%W cget -state] == "normal"} {%W configure -state active}}
    bind Toolbarbutton <Leave> \
	    {+if {[%W cget -state] == "active"} {%W configure -state normal}}

}

proc ::visual::insert_init_old {} {

    # Called in ::init to initialize Insertion-Routines

    # Add bindings for toolbar buttons
    bind Toolbarbutton <ButtonPress-1> "::visual::starttooldrag %W %X %Y"
    bind Toolbarbutton <B1-Motion> "::visual::dotooldrag %W %X %Y"
    bind Toolbarbutton <ButtonRelease-1> "::visual::endtooldrag %W %X %Y"
    bind Toolbarbutton <Enter> \
	    {if {[%W cget -state] == "normal"} {%W configure -state active}}
    bind Toolbarbutton <Leave> \
	    {if {[%W cget -state] == "active"} {%W configure -state normal}}

}

proc ::visual::ias_handleEvent {event control X Y} {
    variable visInfo
    set state $visInfo(ias-state)
    ias_$state $event $control $X $Y
    #puts "$event: $state --> $visInfo(ias-state)"
}

proc ::visual::ias_start {event control X Y} {
    variable visInfo
    switch -regexp $event {
	controlPressed {	
	    set visInfo(ias-state) "controlDown"
	    starttooldrag $control $X $Y
	}
    }
}

proc ::visual::ias_controlDown {event control X Y} {
    variable visInfo
    switch -regexp $event {
	controlReleased {	
	    set visInfo(ias-state) "controlSelected"
	    set visInfo(ias-selectedControl) $control
	}
	controlLeave {
	    set visInfo(ias-state) "controlDragging"
	}
    }
}

proc ::visual::ias_controlDragging {event control X Y} {
    variable visInfo
    switch -regexp $event {
	controlReleased {	
	    set visInfo(ias-state) "start"
	    endtooldrag $control $X $Y
	}
	controlMotion {
	    ::visual::dotooldrag $control $X $Y
	}
    }
}

proc ::visual::ias_controlSelected {event control X Y} {
    variable visInfo
    switch -regexp $event {
	mousePressed {	
	    set visInfo(ias-state) "start"
	    endtooldrag $visInfo(ias-selectedControl) $X $Y
	}
	mouseMotion {
	    ::visual::dotooldrag $visInfo(ias-selectedControl) $X $Y
	}
    }
}


proc ::visual::insert_create {win} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    # Called in ::create do per-window-initialization.
}

# node is the code of a production those application is initiated through the button.
proc ::visual::add_toolbutton {win node helpMsg args} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    set no 0
    while {[winfo exists $data(toolbar).b$no]} {
	incr no
    }

    set b [eval {button $data(toolbar).b$no -anchor c} $args]
    pack $b -padx 2 -pady 1 -side top
    set data(toolbutton,$b) $node
    balloon::defineMessage $b $helpMsg

    # Set Bindings for the button
    set t [bindtags $b]
    lappend t Toolbarbutton
    set ind [lsearch $t Button]
    bindtags $b [lreplace $t $ind $ind]

    # Store information about the variable for use by the Toolbarbutton binding routines
    set visInfo(tbwin-$b) $var
}

proc ::visual::starttooldrag {win X Y} {
    variable visInfo
    set var $visInfo(tbwin-$win)
    upvar #0 ::visual::$var data

    if {[$win cget -state] == "disabled"} {
	set data(tooldrag,ok) "no"
	return
    }

    # Raise the button as a visual feedback for the drag.
    set data(tooldrag,prevstate) [$win cget -relief]
    $win configure -relief sunken

    # Give the cursor another shape.
    $data(toolbar) configure -cursor crosshair

    # No insertion point is highlighted
    highlightinspoint ::visual::$var ""

    # Start automatic centering of highlighted insertion
    set data(tooldrag,autohighlight) [after $::cfg::visualscrolldelay "::visual::doscroll ::visual::$var"]

    # Ok to drag.
    set data(tooldrag,ok) "yes"
}

proc ::visual::dotooldrag {win X Y} {
    variable visInfo
    set var $visInfo(tbwin-$win)
    upvar #0 ::visual::$var data

    if {$data(tooldrag,ok) != "yes"} {
	return
    }

    set ppX [winfo rootx $data(workspace)]
    set ppY [winfo rooty $data(workspace)]
    set x [expr $X - $ppX]
    set y [expr $Y - $ppY]
    if {$x < 0 || $y < 0} {
	# Outside.
	highlightinspoint ::visual::$var ""
	return
    }
    if {$x > [winfo width $data(workspace)] || $y > [winfo height $data(workspace)]} {
	# Outside.
	highlightinspoint ::visual::$var ""
	return
    }	

    # Auf Canvaskoordinaten umrechnen (Scrolling)
    set x [$data(workspace) canvasx $x]
    set y [$data(workspace) canvasy $y]

    # Nchsten Punkt bestimmen
    set rule $data(toolbutton,$win)
    set nearestinspoint [getclosestmovable ::visual::$var $x $y $rule "insert"]
    if {$nearestinspoint == ""} {
	set nearestinspoint [getclosest ::visual::$var $x $y $rule "insert"]
    }
    highlightinspoint ::visual::$var $nearestinspoint
}

proc ::visual::endtooldrag {win X Y} {
    variable visInfo
    set var $visInfo(tbwin-$win)
    upvar #0 ::visual::$var data

    if {$data(tooldrag,ok) != "yes"} {
	return
    }

    set inspoint_xy [list 0 0]
    if {$data(tooldrag,inspoint) != ""} {
	set tags [$data(workspace) gettags $data(tooldrag,inspoint)]
	set cmdind [lsearch $tags "insertcmd,*"]
	set insertargs ""
	if {$cmdind >= 0} {
	    set insertcmd [lrange [split [lindex $tags $cmdind] ","] 1 end]
	    if {[info exists data(vargs,$data(tooldrag,inspoint))]} {
		set insertargs $data(vargs,$data(tooldrag,inspoint))
	    }
	} else {
	    set insertcmd "::vl::insert"
	}

	setInsertContext toolbar

	set bboxind [lsearch $tags "bbox,*"]
	if {$bboxind >= 0} {
	    set inspoint_xy [lrange [split [lindex $tags $bboxind] ","] 1 2]
	} else {
	    set inspoint_xy "0 0"
	}

	set lind [lsearch $tags "insargs,*"]
	if {$lind >= 0} {
	    set insarglist [lrange [split [lindex $tags $lind] ","] 1 end]
	    set newid [::vl::create $data(toolbutton,$win)]
	    set insarglist [linsert $insarglist 2 $newid]
	    if {[catch {eval $insertcmd $insarglist $insertargs} errmsg]} {
		::vl::delete $newid
		::CDlg::Information "Cannot insert here: $errmsg"
		# ::log::log $::errorInfo
	    } else {

                inline_setPosAttr
            }
                #cs:ISP

	    # win is here the window name of the button.
	    # Search the window name of the whole window.
	    set thewin $win
	    while {[info exists visInfo($thewin-data)] == 0} {
		set thewin [winfo parent $thewin]
	    }

	    # and give it the command to select the new node.
	    ::visual::select $thewin 
#           Not a good Idea: possibly selects a node that cannot be selected
#           normally because it has no own visualization. Additonally, it may
#           look unfamiliar.	   
#  	    after idle "
#  		update idletasks
#  		::visual::select $thewin $newid
#  	    "
	}
    }

    # Remove insertion point highlighting
    highlightinspoint ::visual::$var ""

    # Give the button it's normal shape.
    $win configure -relief $data(tooldrag,prevstate)

    # Restore the cursors.
    $data(toolbar) configure -cursor ""

    # cancel automatic centering 
    after cancel $data(tooldrag,autohighlight)
}

proc ::visual::highlightinspoint {varname ids {context "insert"}} {
    upvar #0 $varname data
    variable inspointkind

    if {$data(tooldrag,inspoints) == $ids} {
	return
    }

    if {$data(tooldrag,inspoint) != ""} {
	# Hide the last insertion point
	set kind $data(tooldrag,inspointkind)
	$inspointkind(onselect,$kind) $varname $data(tooldrag,inspoint) 0 $data(tooldrag,inspointcontext)
	set data(tooldrag,inspoint) ""
	set data(tooldrag,inspoints) ""
	set data(tooldrag,inspointkind) ""
	set data(tooldrag,inspointcontext) ""
	if {$data(tooldrag,inspointafter) != ""} {
	    after cancel $data(tooldrag,inspointafter)
	}
    }

    if {$ids != ""} {
	# And show the new one.
	set id [lindex $ids 0]
	set tags [$data(workspace) gettags $id]
	set kindindex [lsearch $tags "inskind,*"]
	set kind [string range [lindex $tags $kindindex] 8 end]

        #by cs: der "einfuegecursor", der an der Einfuegestelle gezeigt wird,
        #sollte spaeter auch fuer die Berechnung des Positionsattributs
        #benutzt werden
	set data(tooldrag,inscursor_position) [$inspointkind(onselect,$kind) $varname $id 1 $context]
	set data(tooldrag,inspoint) $id
	set data(tooldrag,inspoints) $ids
	set data(tooldrag,inspointkind) $kind
	set data(tooldrag,inspointcontext) $context

	if {[llength $ids] > 1} {
	    set data(tooldrag,inspointafter) [after 750 ::visual::rotateinspoint $varname]
	} else {
	    set data(tooldrag,inspointafter) ""
	}
    }
}	    

proc ::visual::rotateinspoint {varname} {
    upvar #0 $varname data
    variable inspointkind

    set l [llength $data(tooldrag,inspoints)]
    if {$l == 0} {
	return
    }
    
    set ind [lsearch -exact $data(tooldrag,inspoints) $data(tooldrag,inspoint)]
    incr ind
    if {$ind >= $l} {set ind 0}

    if {$data(tooldrag,inspoint) != ""} {
	# Hide the last insertion point
	set kind $data(tooldrag,inspointkind)
	$inspointkind(onselect,$kind) $varname $data(tooldrag,inspoint) 0 $data(tooldrag,inspointcontext)
	set data(tooldrag,inspoint) ""
	set data(tooldrag,inspointkind) ""
    }

    set id [lindex $data(tooldrag,inspoints) $ind]
    set tags [$data(workspace) gettags $id]
    set kindindex [lsearch $tags "inskind,*"]
    set kind [string range [lindex $tags $kindindex] 8 end]
    $inspointkind(onselect,$kind) $varname $id 1 $data(tooldrag,inspointcontext)
    set data(tooldrag,inspoint) $id
    set data(tooldrag,inspointkind) $kind
    
    set data(tooldrag,inspointafter) [after 750 ::visual::rotateinspoint $varname]
}

proc ::visual::visible {varname id} {
    upvar #0 $varname data
    
    # Calc upper left canvas coord of visible area
    set rootx 0
    set rooty 0
    set width [winfo width $data(workspace)]
    set height [winfo height $data(workspace)]

    set X1 [$data(workspace) canvasx $rootx]
    set Y1 [$data(workspace) canvasy $rooty]
    set X2 [$data(workspace) canvasx [expr $rootx + $width]]
    set Y2 [$data(workspace) canvasy [expr $rooty + $height]]
    
    set bbox [$data(workspace) bbox $id]

    if {[lindex $bbox 0] < $X1} {
	set scrollx -1
    } elseif {[lindex $bbox 2] > $X2} {
	set scrollx 1
    } else {
	set scrollx 0
    }

    if {[lindex $bbox 1] < $Y1} {
	set scrolly -1
    } elseif {[lindex $bbox 3] > $Y2} {
	set scrolly 1
    } else {
	set scrolly 0
    }

    return "$scrollx $scrolly"
}

proc ::visual::doscroll {varname} {
    upvar #0 $varname data

    if {$data(tooldrag,inspoint) != ""} {
	set scroll [visible $varname $data(tooldrag,inspoint)]
	if {[lindex $scroll 0] != 0} {
	    $data(workspace) xview scroll [lindex $scroll 0] units
	}
	if {[lindex $scroll 1] != 0} {
	    $data(workspace) yview scroll [lindex $scroll 1] units
	}
    }

    set data(tooldrag,autohighlight) [after $::cfg::visualscrolldelay "::visual::doscroll $varname"]
}

proc ::visual::isAllowedInspoint {varname tags id} {
    upvar #0 $varname data

    if {[lsearch -glob $tags "inspointmp,*"] < 0} {
	# Is no reposition inspoint
	set id_ind [lsearch -glob $tags "inspoint#*"]
	set inspoint_node [string range [lindex $tags $id_ind] 9 end]
	if {$id != $inspoint_node && [::vl::isAncestor $id $inspoint_node]} {
	    # cannot insert a node in itself
	    return 0
	}
    }

    set inscontext_ind [lsearch -glob $tags "inscontext,*"]
    if {$inscontext_ind >= 0} {
	set inscontext [split [lindex $tags $inscontext_ind] ","]
	set root [lindex $inscontext 2]
	if {[lindex $inscontext 1]} {    
	    # tree-parameter was != 0
	    if {![::vl::isAncestor $root $id]} {
		# This insertion-Point is illegal for this node
		return 0
	    }
	} else {
	    if {[::vl::getFather $id] != $root} {
		# This insertion-Point is illegal for this node
		return 0
	    }
	}
    }
    
    return 1
}

proc ::visual::getclosest {varname x y node action {movednode ""}} {
    upvar #0 $varname data

    set closest ""
    set closestdistance 0

    set ids [$data(workspace) find withtag "inspointfb,$node"]
    if {$action == "insert"} {
	eval lappend ids [$data(workspace) find withtag "inspointfi,$node"]
    } elseif {$action == "move"} {
	eval lappend ids [$data(workspace) find withtag "inspointfm,$node"]
	eval lappend ids [$data(workspace) find withtag "inspointfp,$node"]
    }

    foreach id $ids {
	if {$action == "move" && $movednode != ""} {
	    set tags [$data(workspace) gettags $id]
	    if {![isAllowedInspoint $varname $tags $movednode]} {
		continue
	    }
	}
		    
	# Calc distance from given point to closest point of the object with the given id
        set bbox [$data(workspace) bbox $id]
	set bx1 [lindex $bbox 0]
	set by1 [lindex $bbox 1]
	set bx2 [lindex $bbox 2]
	set by2 [lindex $bbox 3]
	if {$bx1 < $x && $x < $bx2} {
	    set dx 0
	} elseif {$x <= $bx1} {
	    set dx [expr $bx1 - $x]
	} else {
	    set dx [expr $x - $bx2]
	}

	if {$by1 < $y && $y < $by2} {
	    set dy 0
	} elseif {$y <= $by1} {
	    set dy [expr $by1 - $y]
	} else {
	    set dy [expr $y - $by2]
	}

        set distance [expr sqrt($dx * $dx + $dy * $dy)]

        if {$closest == ""} {
            set closest $id
            set closestdistance $distance
        } elseif {$distance < $closestdistance} {
            set closest $id
            set closestdistance $distance
        } elseif {$distance == $closestdistance} {
	    lappend closest $id
	}
    }

    return $closest
}

proc ::visual::getclosestmovable {varname x y node action {movednode ""}} {
    upvar #0 $varname data

    set closest ""
    set closestdistance 0
    set closestdepth -1

    set ids [$data(workspace) find withtag "inspointmb,$node"]
    if {$action == "insert"} {
	eval lappend ids [$data(workspace) find withtag "inspointmi,$node"]
    } elseif {$action == "move"} {
	eval lappend ids [$data(workspace) find withtag "inspointmm,$node"]
	eval lappend ids [$data(workspace) find withtag "inspointmp,$node"]
    }

    foreach id $ids {

	# Get Coords of Bounding Box of movable.

	set tags [$data(workspace) gettags $id]
	if {$action == "move" && $movednode != ""} {
	    if {![isAllowedInspoint $varname $tags $movednode]} {
		continue
	    }
	}
		    
	set bboxind [lsearch $tags "bbox,*"]
	if {$bboxind >= 0} {
	    set bbox [lrange [split [lindex $tags $bboxind] ","] 1 end]
	} else {
	    set bbox [eval list 0 0 [lrange [$data(workspace) cget -scrollregion] 2 end]]
	}
	set inspoint_bbox [$data(workspace) bbox $id]
	set inspoint_width [expr [lindex $inspoint_bbox 2] - [lindex $inspoint_bbox 0]]
	set inspoint_height [expr [lindex $inspoint_bbox 3] - [lindex $inspoint_bbox 1]]
	set bbox [lreplace $bbox 2 3 \
		[expr [lindex $bbox 2] - $inspoint_width] \
		[expr [lindex $bbox 3] - $inspoint_height]]

	set depthind [lsearch $tags "inspointdepth_*"]
	if {$depthind >= 0} {
	    set depth [lindex [split [lindex $tags $depthind] "_"] 1]
	} else {
	    set depth 0
	}

	# If x,y is inside, simply move insertion point to the given coord.
	if {[::visual::_inside $x $y $bbox]} {
	    set ibb [$data(workspace) bbox $id]
	    $data(workspace) move $id [expr $x - [lindex $ibb 0]] [expr $y - [lindex $ibb 1]]
	    set distance 0
	} else {
	    # Calc X and Y position to move inspoint to.
	    
	    set bx1 [lindex $bbox 0]
	    set by1 [lindex $bbox 1]
	    set bx2 [lindex $bbox 2]
	    set by2 [lindex $bbox 3]

	    if {$bx1 < $x && $x < $bx2} {
		set new_x $x
		set dx 0
	    } elseif {$x <= $bx1} {
	        set new_x $bx1
		set dx [expr $bx1 - $x]
	    } else {
		set new_x $bx2
		set dx [expr $x - $bx2]
	    }

	    if {$by1 < $y && $y < $by2} {
		set new_y $y
		set dy 0
	    } elseif {$y <= $by1} {
		set new_y $by1
		set dy [expr $by1 - $y]
	    } else {
		set new_y $by2
		set dy [expr $y - $by2]
	    }

	    set ibb [$data(workspace) bbox $id]
	    $data(workspace) move $id [expr $new_x - [lindex $ibb 0]] [expr $new_y - [lindex $ibb 1]]
	    set distance [expr sqrt($dx * $dx + $dy * $dy)]
	}

        if {$closest == ""} {
	    set closest $id
	    set closestdepth $depth
            set closestdistance $distance
        } elseif {$distance < $closestdistance} {
            set closest $id
	    set closestdepth $depth
            set closestdistance $distance
        } elseif {$distance == $closestdistance && $depth > $closestdepth} {
            set closest $id
	    set closestdepth $depth
	}
    }

    return $closest
}

proc ::visual::_inside {x y bbox} {
    set x1 [lindex $bbox 0]
    set y1 [lindex $bbox 1]
    set x2 [lindex $bbox 2]
    set y2 [lindex $bbox 3]

    return [expr {$x >= $x1 && $x <= $x2 && $y >= $y1 && $y <= $y2}]
}
