# ----------------------------------------------------------------------
# Module Visual
# ----------------------------------------------------------------------
# Procedures to visualize and edit the visualized data structure
#
# $Id: visual.tcl,v 3.4 1999/10/22 15:54:38 mjung Exp $

# Create our namespace.
namespace eval ::visual {

    variable resize_cursor

    switch $::tcl_platform(platform) {
	"unix" { 
	    array set resize_cursor {
		"normal" "crosshair"
		"0" "top_left_corner"
		"1" "top_side"
		"2" "top_right_corner"
		"3" "right_side"
		"4" "bottom_right_corner"
		"5" "bottom_side"
		"6" "bottom_left_corner"
		"7" "left_side"
                "8" "hand1"
	    }
	}
	
	"macintosh" { 
	    array set resize_cursor {
		"normal" "crosshair"
		"0" "top_left_corner"
		"1" "top_side"
		"2" "top_right_corner"
		"3" "right_side"
		"4" "bottom_right_corner"
		"5" "bottom_side"
		"6" "bottom_left_corner"
		"7" "left_side"
                "8" "hand1"
	    }
	}
	
	"windows" {
	    array set resize_cursor {
		"normal" "crosshair"
		"0" "size_nw_se"
		"1" "size_ns"
		"2" "size_ne_sw"
		"3" "size_we"
		"4" "size_nw_se"
		"5" "size_ns"
		"6" "size_ne_sw"
		"7" "size_we"
                "8" "hand1"
	    }
	}   
    }
}

proc ::visual::move_init {} {
    menu .visualcontextmenu -tearoff 0
}

proc ::visual::move_create {win} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    set data(selectedid) ""
    set data(selected_editid) ""
    set data(selected_removeid) ""

    bind $data(workspace) <ButtonPress-1> "::visual::leftclick $win %x %y"
    bind $data(workspace) <B1-Motion> "::visual::leftmotion $win %x %y %X %Y"
    bind $data(workspace) <ButtonRelease-1> "::visual::leftrelease $win %x %y %X %Y"
    bind $data(workspace) <ButtonPress-3> "::visual::rightclick $win %x %y"

    $data(workspace) bind focusitem <Key> "::visual::insertchar $win %A %K %s"
    bind $data(workspace) <Delete> "::visual::cutselection $win 0"
    
    $data(workspace) bind resizerselected <ButtonPress-1> "::visual::resize_begin $win %x %y"
    $data(workspace) bind resizerselected <B1-Motion> "::visual::resize_do $win %x %y"
    $data(workspace) bind resizerselected <ButtonRelease-1> "::visual::resize_end $win %x %y"

    $data(workspace) bind resizerselected <Enter> "::visual::resize_enter $win %x %y"
    $data(workspace) bind resizerselected <Leave> "::visual::resize_leave $win %x %y"


}

proc ::visual::leftclick {win x y} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {[info exists data(objdrag)]} {
	# Dragging, do not click.
	return
    }

    if {[info exists data(resizeid)]} {
	# Resizing, do not click.
	return
    }

    if { "$visInfo(ias-state)" != "start" } {
	# by cs: Inserting, do not click.
	return
    }

    if {[$data(workspace) focus] != ""} {
	commitedit $win 1
    }

    set x [$data(workspace) canvasx $x]
    set y [$data(workspace) canvasy $y]
    set halo 2

    # Find out about current context
    set ids [$data(workspace) find overlapping \
            [expr $x - $halo] [expr $y - $halo] [expr $x + $halo] [expr $y + $halo]]

    set iddepth ""
    foreach id $ids {
	set tags [$data(workspace) gettags $id]
	if {[lsearch -exact $tags "noselect"] >= 0} {
	    continue
	}

	set depthindex [lsearch -glob $tags "depth_*"]
	set idindex [lsearch -glob $tags "node#*"]
	if {$idindex < 0} {
	    continue
	}
	lappend iddepth [list $id [string range [lindex $tags $idindex] 5 end] \
		[string range [lindex $tags $depthindex] 6 end]]
    }

    if {$iddepth != ""} {
	set id [lindex [lsort -index 2 -integer $iddepth] end]
	set treeid [lindex $id 1]
	set canvid [lindex $id 0]

	if {[lsearch -exact [$data(workspace) gettags $canvid] edittext] >= 0} {
	    startedit $win $canvid $x $y
	} else {
	    commitedit $win 1
	}
	::visual::select $win $treeid
	set data(objmove) $data(selected_removeid)
	set data(objmove-coords) [list 1 $x $y]
	set data(objmove-inside) 1
	highlightinspoint ::visual::$var ""
	return
    }

    ::visual::select $win
}

proc ::visual::select {win {id ""}} {
    # id ist ein knotenzeiger!

    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    # by cs: dies muss auch im fall id=="" gemacht werden!
    $data(workspace) delete selection


    if {$data(selectedid) != ""} {
	set selectedresizers [$data(workspace) find withtag "resizerselected"]
	foreach resizer $selectedresizers {
	    set tags [$data(workspace) gettags $resizer]
	    set ind [lsearch -exact $tags "resizerselected"]
	    $data(workspace) itemconfigure $resizer -tags [lreplace $tags $ind $ind] -outline {} -fill {}
	}
    }

    if {$id == ""} {
	$win.cmd.cut configure -state disabled
	$win.cmd.copy configure -state disabled
	$win.cmd.properties configure -state disabled
	set data(selected_editid) ""
	set data(selected_removeid) ""
	set data(selected_bbox) ""
	set data(selectedid) ""
    } else {
	set data(selectedid) $id
	set ids [findobjects $win $data(selectedid)]

	set code [::vl::getCode $id]
	set canedit [::view::is_editable $id]
	set candelete [expr {[::vl::isRemovable $id] || [$data(workspace) find withtag inspointmp,$code] != ""}]
	set data(selected_editid) $id
	set data(selected_removeid) $id

	set outer_id $id
	while {!$canedit || !$candelete} {
  	    set fatherid [lindex [::vl::getFather $outer_id] 0]
  	    if {$fatherid == "" || $fatherid == $data(rootid)} {
  		break
  	    }

  	    set outer_id $fatherid
	    if {[hasselectableobjects $win $outer_id]} {
		break
	    }

	    if {!$canedit} {
		set canedit [::view::is_editable $outer_id]
		set data(selected_editid) $outer_id
	    }
	    if {!$candelete} {
		set outer_code [::vl::getCode $outer_id]
		set candelete [expr {[::vl::isRemovable $outer_id] || [$data(workspace) find withtag inspointmp,$outer_code] != ""}]
		set data(selected_removeid) $outer_id
	    }
	}

	set status "Selecting [::vl::symbname [::vl::lhs [::vl::getCode $data(selectedid)]]] "
	if {!$canedit} { 
	    set data(selected_editid) ""
	} else {
	    set status "${status}, on edit [::vl::symbname [::vl::lhs [::vl::getCode $data(selected_editid)]]] "
	}
	
	if {!$candelete} { 
	    set data(selected_removeid) ""
	} else {
	    set status "${status}, on remove [::vl::symbname [::vl::lhs [::vl::getCode $data(selected_removeid)]]]"
	}
	::log::info selection $status

	$win.cmd.cut configure -state [expr {$candelete ? "normal" : "disabled"}]
	$win.cmd.copy configure -state [expr {$candelete ? "normal" : "disabled"}]
	$win.cmd.properties configure -state [expr {$canedit ? "normal" : "disabled"}]

	set bboxids ""
	set thicklineids ""
	set resizerids ""

	foreach canvasid $ids {
	    set type [$data(workspace) type $canvasid]
	    if {[lsearch -exact [$data(workspace) gettags $canvasid] "resizer"] >= 0} {
		lappend resizerids $canvasid
	    } elseif {$type == "line"} {
		lappend thicklineids $canvasid
	    } else {
		lappend bboxids $canvasid
	    }
	}

	if {$bboxids != ""} {
	    set bbox [eval $data(workspace) bbox $bboxids]
	    $data(workspace) create rectangle \
		    [expr [lindex $bbox 0] - 1] [expr [lindex $bbox 1] - 1] \
		    [expr [lindex $bbox 2] + 1] [expr [lindex $bbox 3] + 1] \
		    -tags selection -outline $::cfg::defaultselectcolor -width 2
	    set data(selected_bbox) $bbox
	} else {
	    set data(selected_bbox) [eval $data(workspace) bbox $thicklineids]
	}

	foreach line $thicklineids {
	    eval {$data(workspace) create line} [$data(workspace) coords $line] {\
		    -tags selection -fill $::cfg::defaultselectcolor -width 4}
	}

	foreach resizer $resizerids {
	    set tags [$data(workspace) gettags $resizer]
	    set idind [lsearch $tags "node#*"]
	    if {[lindex $tags $idind] == "node#$data(selectedid)"} {
		lappend tags "resizerselected"
		$data(workspace) itemconfigure $resizer -tags $tags -outline $::cfg::defaultselectcolor -fill $::cfg::defaultresizerfillcolor
		$data(workspace) raise $resizer selection
	    }
	}

	catch {$data(workspace) raise inspoint}
	catch {$data(workspace) raise drag}
    }
}

proc ::visual::leftmotion {win x y X Y} {
    variable visInfo
    variable resize_cursor
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {$data(selectedid) == ""} {
	# Not selected anything, cannot move
	return
    }

    if {[info exists data(resizeid)]} {
	# Resizing, do not click.
	return
    }

    if {[info exists data(objmove)] == 0} {
	# Currently not moving
	return
    }

    set inside [::MDIFrame::IsInside [::MDIFrame::GetClient $win] $X $Y]
    if {$data(objmove-inside) && !$inside} {
	set data(objmove-droptarget) [::MDIFrame::FindDropTarget $X $Y]
	if {$data(objmove-droptarget) != ""} {
	    if {[eval $data(objmove-droptarget) inquire $data(selectedid) $X $Y]} {
		set data(objmove-id) $data(selectedid)
		$data(workspace) conf -cursor cross
	    } elseif {$data(selected_removeid) != "" && [eval $data(objmove-droptarget) inquire $data(selected_removeid) $X $Y]} {
		set data(objmove-id) $data(selected_removeid)
		$data(workspace) conf -cursor cross
	    } else {
		$data(workspace) conf -cursor pirate
	    }
		
	} else {
	    $data(workspace) conf -cursor pirate
	}
    }
    if {!$data(objmove-inside) && !$inside} {
	# Inside same other or another other window
	set dt [::MDIFrame::FindDropTarget $X $Y]
	if {$dt != $data(objmove-droptarget)} {
	    # Inside another window
	    if {$data(objmove-droptarget) != ""} {
		eval $data(objmove-droptarget) cancel $data(selectedid) $X $Y
	    }
	    set data(objmove-droptarget) $dt
	}

	if {$data(objmove-droptarget) != ""} {
	    if {[eval $data(objmove-droptarget) inquire $data(selectedid) $X $Y]} {
		$data(workspace) conf -cursor cross
		set data(objmove-id) $data(selectedid)
	    } elseif {$data(selected_removeid) != "" && [eval $data(objmove-droptarget) inquire $data(selected_removeid) $X $Y]} {
		$data(workspace) conf -cursor cross
		set data(objmove-id) $data(selected_removeid)
	    } else {
		$data(workspace) conf -cursor pirate
	    }
	} else {
	    $data(workspace) conf -cursor pirate
	}
    }
    if {!$data(objmove-inside) && $inside} {
	# Leave other windows
	if {$data(objmove-droptarget) != ""} {
	    eval $data(objmove-droptarget) cancel $data(selectedid) $X $Y
	}
	$data(workspace) conf -cursor $resize_cursor(normal)
    }
    set data(objmove-inside) $inside
    if {$data(objmove-inside) == 0} {
	highlightinspoint ::visual::$var ""
	return
    }

    set id $data(selected_removeid)
    if {$id == ""} {
	set id $data(selectedid)
    }

    set x [$data(workspace) canvasx $x]
    set y [$data(workspace) canvasy $y]

    if {[lindex $data(objmove-coords) 0]} {
	set orig_x [lindex $data(objmove-coords) 1]
	set orig_y [lindex $data(objmove-coords) 2]
	if {$orig_x - 3 < $x || $x > $orig_x + 3 || $orig_y - 3 < $y || $y > $orig_y + 3} {
	    set data(objmove-coords) [list 0 $orig_x $orig_y]
	} else {
	    return
	}
    }

    commitedit $win 0

    # Hier beginnt die Suche nach einer geeigneten Einfuegestelle
    # fuer ein "move" innerhalb des eigenen Fensters

    set rule [::vl::getCode $id]
    set nearestinspoint [getclosestmovable ::visual::$var $x $y $rule "move" $id]
    if {$nearestinspoint == ""} {
	set nearestinspoint [getclosest ::visual::$var $x $y $rule "move" $id]
    }
    
    set delta_x [expr $x - [lindex $data(objmove-coords) 1]]
    set delta_y [expr $y - [lindex $data(objmove-coords) 2]]
    set moved_bbox [list "move" \
	    [expr [lindex $data(selected_bbox) 0] + $delta_x] \
	    [expr [lindex $data(selected_bbox) 1] + $delta_y] \
	    [expr [lindex $data(selected_bbox) 2] + $delta_x] \
	    [expr [lindex $data(selected_bbox) 3] + $delta_y]]

    highlightinspoint ::visual::$var ""
    highlightinspoint ::visual::$var $nearestinspoint $moved_bbox
}

proc ::visual::add_interwindowdrop {win prodcode symbcode action} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    set data(interwindowdrop-$symbcode) $prodcode
    set data(interwindowdropaction-$symbcode) $action
}

proc ::visual::accept_interwindowdrop {win which node X Y} {
    # Which can have multiple values:
    # 'inquire':     Ask if drop will be accepted. If yes, this also
    #                can draw some indication (highlight insertion point)
    # 'cancel':      Remove indication (cursor left client area)
    # 'perform':     Perform action for drop.
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    set x [$data(workspace) canvasx [expr $X - [winfo rootx $data(workspace)]]]
    set y [$data(workspace) canvasy [expr $Y - [winfo rooty $data(workspace)]]]
    
    switch $which {
	inquire {
	    set symbol [::vl::lhs [::vl::getCode $node]]
	    if {[info exists data(interwindowdrop-$symbol)] == 0} {
		highlightinspoint ::visual::$var ""
		return 0
	    } else {
		set rule $data(interwindowdrop-$symbol)
	    }
	    
	    set nearestinspoint [getclosestmovable ::visual::$var $x $y $rule "insert"]
	    if {$nearestinspoint == ""} {
		set nearestinspoint [getclosest ::visual::$var $x $y $rule "insert"]
	    }
	    highlightinspoint ::visual::$var $nearestinspoint
	    return [expr {"" != "$nearestinspoint"}]
	}
		
	cancel {
	    highlightinspoint ::visual::$var ""
	}

	perform {
	    set symbol [::vl::lhs [::vl::getCode $node]]
	    if {[info exists data(interwindowdrop-$symbol)] == 0} {
		highlightinspoint ::visual::$var ""
		return 0
	    } else {
		set rule $data(interwindowdrop-$symbol)
	    }

	    set inspoint_xy [list 0 0]
	    if {$data(tooldrag,inspoint) != ""} {
		set tags [$data(workspace) gettags $data(tooldrag,inspoint)]
		set cmdind [lsearch $tags "insertcmd,*"]
		set insertargs ""
		setInsertContext dragdrop $node
		if {$cmdind >= 0} {
		    set insertcmd [lrange [split [lindex $tags $cmdind] ","] 1 end]
		    if {[info exists data(vargs,$data(tooldrag,inspoint))]} {
			set insertargs $data(vargs,$data(tooldrag,inspoint))
		    }
		} else {
		    set insertcmd "::vl::insert"
		}

		set bboxind [lsearch $tags "bbox,*"]
		if {$bboxind >= 0} {
		    set inspoint_xy [lrange [split [lindex $tags $bboxind] ","] 1 2]
		} else {
		    set inspoint_xy "0 0"
		}
		set lind [lsearch $tags "insargs,*"]
		if {$lind >= 0} {
		    set insarglist [lrange [split [lindex $tags $lind] ","] 1 end]
		    set newid [::vl::create $rule]
                    
                    # by cs:
                    if { $data(interwindowdropaction-$symbol) != {} } {
                        eval $data(interwindowdropaction-$symbol) [list $node $newid]
                    }

		    set insarglist [linsert $insarglist 2 $newid]
		    if {[catch {eval $insertcmd $insarglist $insertargs} errmsg]} {
			::vl::delete $newid
			::CDlg::Information "Cannot insert here: $errmsg"
			::log::log $::errorInfo
		    } else {
                        inline_setPosAttr
		    }
		    
		}
	    }

	    # Remove insertion point highlighting
	    highlightinspoint ::visual::$var ""

	}
    }
}

proc ::visual::leftrelease {win x y X Y} {
    variable visInfo
    variable resize_cursor
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {$data(selectedid) == ""} {
	return
    }
    
    if {[info exists data(resizeid)]} {
	# Resizing, do not click.
	return
    }

    if {[info exists data(objmove)] == 0} {
	# Currently not moving
	return
    }

    if {[lindex $data(objmove-coords) 0]} {
	# Not moved enough
	return
    }

    if {$data(objmove-inside) == 0} {
	if {$data(objmove-droptarget) != "" && [info exists data(objmove-id)]} {
	    eval $data(objmove-droptarget) perform $data(objmove-id) $X $Y
	}
	$data(workspace) conf -cursor $resize_cursor(normal)
	return
    }
    
    # Absolute Mouse Position.

    # Calc Relative Mouse Position in Selection
    set ids [findobjects $win $data(selectedid)]
    if {$ids != ""} {
	set bbox [eval {$data(workspace) bbox} $ids]
	set orig_x [lindex $data(objmove-coords) 1]
	set orig_y [lindex $data(objmove-coords) 2]
	set orig_x [expr int($orig_x - [lindex $bbox 0])]
	set orig_y [expr int($orig_y - [lindex $bbox 1])]
    } else {
	set orig_x 0
	set orig_y 0
    }

    unset data(objmove)
    unset data(objmove-coords)

    if {$data(tooldrag,inspoint) == ""} {
	return
    }

    set x [$data(workspace) canvasx $x]
    set y [$data(workspace) canvasy $y]

    set tags [$data(workspace) gettags $data(tooldrag,inspoint)]
    set cmdind [lsearch $tags "insertcmd,*"]
    if {$cmdind >= 0} {
	set insertcmd [lrange [split [lindex $tags $cmdind] ","] 1 end]
    } else {
	set insertcmd "::vl::insert"
    }

    # Extract action-code from insertion-point.
    set id $data(selected_removeid)
    if {$id == ""} {
	set id $data(selectedid)
    }

    set rule [::vl::getCode $id]
    set actionind [lsearch $tags "inspoint*,$rule"]
    if {$actionind < 0} {
	error "Internal. Could not find index of instag"
    }
    set action [string range [lindex $tags $actionind] 9 9]
    if {$action != "p" && $data(selected_removeid) == ""} {
	# Only p insertion points allowed for non-removable objects
	return
    }

    setInsertContext move $action

    set newid $data(selected_removeid)

    #Folgendes ist uplevel-Code
    inline_setPosAttr


    set lind [lsearch -glob $tags "insargs,*"]
    if {$lind >= 0 && $action != "p"} {
	set insarglist [lrange [split [lindex $tags $lind] ","] 1 end]

	if {$action != "p"} {
	    set insarglist [linsert $insarglist 2 $newid]
	
	    if {[catch {eval $insertcmd $insarglist} errormsg]} {
		::CDlg::Error "Can not insert here: $errormsg"
	    }
	}
    }

    # Remove insertion point highlighting
    highlightinspoint ::visual::$var ""
}

proc ::visual::findobjects {win id} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    set oblist ""
    eval lappend oblist [$data(workspace) find withtag "node#$id"]

    foreach subid [::vl::getDesc $id] {
	eval lappend oblist [findobjects $win $subid]
    }
    return $oblist
}

proc ::visual::hasselectableobjects {win id} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    set oblist ""
    eval lappend oblist [$data(workspace) find withtag "node#$id"]

    foreach ob $oblist {
	set tags [$data(workspace) gettags $ob]
	if {[lsearch -exact $tags "noselect"] < 0} {
	    return 1
	}
    }
    return 0
}

proc ::visual::cutselection {win {clipboard 1}} {
    # In a later Version, setting clipboard to 0 will only
    # perform a delete, without copying the node to the 
    # clipboard.

    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {[$data(workspace) focus] != ""} {
	# Editing a textfield.
	return
    }

    if {$data(selected_removeid) == ""} {
	bell
	return
    }
    
    if {! [::vl::isRemovable $data(selected_removeid)]} {
	bell
	return
    }

    # Unselect node first to avoid problems.
    set node $data(selected_removeid)
    ::visual::select $win

    ::vl::extract $node
    if {[::vl::getFather $node] == ""} {
	::vl::delete $node
    }
}

proc ::visual::editselection {win} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data
    
    if {$data(selected_editid) == ""} {
	bell
	return
    }

    # Clear context menu
    $win.contextmenu delete 0 last
    foreach win [winfo children $win.contextmenu] {
	::destroy $win
    }

    set numeditors 0
    foreach editor [::view::get_editors $data(selected_editid)] {
	incr numeditors
	$win.contextmenu add command -label [lindex $editor 1] -command "[lindex $editor 0] $data(selected_editid)"
    }

    if {$numeditors == 0} {
	bell
	return
    }

    if {$numeditors == 1} {
	$win.contextmenu invoke 0
	return
    }

    set X [winfo pointerx $win]
    set Y [winfo pointery $win]
    tk_popup $win.contextmenu $X $Y 0
}

proc ::visual::resize_enter {win x y} {
    variable visInfo
    variable resize_cursor
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data
    
    set tags [$data(workspace) gettags current]
    set cursor [tags_getproperty $tags "resizerCursor"]
    $data(workspace) configure -cursor $resize_cursor($cursor)

    #set cornerind [lsearch $tags "resizercorner_*"]
    #if {$cornerind >= 0} {
    #  set corner [string range [lindex $tags $cornerind] 14 end]
    #    catch {
    #	    $data(workspace) configure -cursor $resize_cursor($corner)
    #	}
    # }
}
    
proc ::visual::resize_leave {win x y} {
    variable visInfo
    variable resize_cursor

    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {[info exists data(resizeid)]} {
	# Resize in Progress. Do not change cursor
	return
    }

    $data(workspace) configure -cursor $resize_cursor(normal)
}

proc ::visual::stdresize_begin {params datavar x y} {
    upvar 1 $datavar data

    set data(resizecorner) [lindex $params 0]
    set bbox ""
    set selection [$data(workspace) find withtag selection]
    foreach selectionid $selection {
	if {[$data(workspace) type $selectionid] == "rectangle"} {
	    set bbox [$data(workspace) coords $selectionid]
	    break
	}
    }
    if {$bbox == ""} {
	# Selection enthlt keine Box. Dann nimm die Linien...
	set bbox [eval $data(workspace) bbox $selection]
    }

    set data(resizer_bboxid) [eval \
	    {$data(workspace) create rectangle} $bbox {-outline $::cfg::defaultselectcolor}]
    set data(resizer_bbox) $bbox
    set data(resize_orig) [list [$data(workspace) canvasx $x] [$data(workspace) canvasy $y]]
}


proc ::visual::resize_begin {win x y} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    set data(resizeid) [$data(workspace) find withtag current]
    set tags [$data(workspace) gettags $data(resizeid)]
    if {[lsearch $tags "resizer"] < 0} {
	unset data(resizeid)
	return
    }
    set procname [tags_getproperty $tags resizerProc]
    set params [tags_getproperty $tags resizerProcParam]
    ${procname}_begin $params data $x $y
}


proc ::visual::stdresize_do {params datavar x y} {
    upvar 1 $datavar data

    set dx [expr [$data(workspace) canvasx $x] - [lindex $data(resize_orig) 0]]
    set dy [expr [$data(workspace) canvasy $y] - [lindex $data(resize_orig) 1]]

    switch $data(resizecorner) {
	0 {set bbox [list \
		[expr [lindex $data(resizer_bbox) 0] + $dx] \
		[expr [lindex $data(resizer_bbox) 1] + $dy] \
		[lindex $data(resizer_bbox) 2] \
		[lindex $data(resizer_bbox) 3]]}
	1 {set bbox [list \
		[lindex $data(resizer_bbox) 0] \
		[expr [lindex $data(resizer_bbox) 1] + $dy] \
		[lindex $data(resizer_bbox) 2] \
		[lindex $data(resizer_bbox) 3]]}
	2 {set bbox [list \
		[lindex $data(resizer_bbox) 0] \
		[expr [lindex $data(resizer_bbox) 1] + $dy] \
		[expr [lindex $data(resizer_bbox) 2] + $dx] \
		[lindex $data(resizer_bbox) 3]]}
	3 {set bbox [list \
		[lindex $data(resizer_bbox) 0] \
		[lindex $data(resizer_bbox) 1] \
		[expr [lindex $data(resizer_bbox) 2] + $dx] \
		[lindex $data(resizer_bbox) 3]]}
	4 {set bbox [list \
		[lindex $data(resizer_bbox) 0] \
		[lindex $data(resizer_bbox) 1] \
		[expr [lindex $data(resizer_bbox) 2] + $dx] \
		[expr [lindex $data(resizer_bbox) 3] + $dy]]}
	5 {set bbox [list \
		[lindex $data(resizer_bbox) 0] \
		[lindex $data(resizer_bbox) 1] \
		[lindex $data(resizer_bbox) 2] \
		[expr [lindex $data(resizer_bbox) 3] + $dy]]}
	6 {set bbox [list \
		[expr [lindex $data(resizer_bbox) 0] + $dx] \
		[lindex $data(resizer_bbox) 1] \
		[lindex $data(resizer_bbox) 2] \
		[expr [lindex $data(resizer_bbox) 3] + $dy]]}
	7 {set bbox [list \
		[expr [lindex $data(resizer_bbox) 0] + $dx] \
		[lindex $data(resizer_bbox) 1] \
		[lindex $data(resizer_bbox) 2] \
		[lindex $data(resizer_bbox) 3]]}
    }

    eval {$data(workspace) coords $data(resizer_bboxid)} $bbox
}


proc ::visual::resize_do {win x y} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {[info exists data(resizeid)] == 0} {
	# Not resizing
	return
    }
    set tags [$data(workspace) gettags $data(resizeid)]
    set procname [tags_getproperty $tags resizerProc]
    set params [tags_getproperty $tags resizerProcParam]
    ${procname}_do $params data $x $y
}


proc ::visual::stdresize_end {params datavar x y} {
    upvar 1 $datavar data

    set bbox [$data(workspace) coords $data(resizer_bboxid)]
    set size [list [expr int([lindex $bbox 2] - [lindex $bbox 0])] \
	           [expr int([lindex $bbox 3] - [lindex $bbox 1])]]
    
    set attrname [lindex $params 1]
    set tags [$data(workspace) gettags $data(resizeid)]
    set idindex [lsearch -glob $tags "node#*"]
    set id [string range [lindex $tags $idindex] 5 end]
    ::vl::setAttrs $id [list $attrname $size]

    $data(workspace) delete $data(resizer_bboxid)
    unset data(resizer_bboxid)
    unset data(resizer_bbox)
    unset data(resizecorner)
    unset data(resize_orig)
}
    

proc ::visual::resize_end {win x y} {
    variable visInfo
    variable resize_cursor

    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {[info exists data(resizeid)] == 0} {
	# Not resizing
	return
    }
    set tags [$data(workspace) gettags $data(resizeid)]
    set procname [tags_getproperty $tags resizerProc]
    set params [tags_getproperty $tags resizerProcParam]
    ${procname}_end $params data $x $y

    unset data(resizeid)
    $data(workspace) configure -cursor $resize_cursor(normal)
}

proc ::visual::startedit {win id x y} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data
    
    $data(workspace) focus $id
    $data(workspace) icursor $id "@$x,$y"
    set tags [$data(workspace) gettags $id]
    lappend tags focusitem
    $data(workspace) itemconfigure $id -tags $tags

    # selektiere zu Beginn den gesamten Text
    set data(insertchar,selfrom) 0
    set data(insertchar,selto) [$data(workspace) index $id end]
    insertchar_selection update
}


proc ::visual::insertchar_selectclear {} {
    upvar data data
    catch {unset data(insertchar,selfrom)}
    $data(workspace) select clear
}


proc ::visual::insertchar_selection {action} {
    upvar data data
    upvar id id
    if ![info exists data(insertchar,selfrom)] return

    if { $data(insertchar,selfrom) < $data(insertchar,selto)} {
        set from $data(insertchar,selfrom)
        set to $data(insertchar,selto)
    } else {
        set from $data(insertchar,selto)
        set to $data(insertchar,selfrom)
    }
    switch -- $action {
      "delete" {
        $data(workspace) dchars $id $from [expr $to-1]
        insertchar_selectclear
      }
      "update" {
          if "$from == $to" {
              $data(workspace) select clear
          } else {
              $data(workspace) select from $id $from
              $data(workspace) select to $id [expr $to-1]
          }
      }
      default {
          error "Error"
      }
    }
}

proc ::visual::insertchar_repos {newpos} {
    upvar data data
    upvar id id
    upvar state state
    upvar insertpos insertpos

    $data(workspace) icursor $id $newpos

    if $state {
        if ![info exists data(insertchar,selfrom)] {
            set data(insertchar,selfrom) $insertpos
        }
        set data(insertchar,selto) $newpos
        insertchar_selection update
    } else {
        insertchar_selectclear
    }
}

proc ::visual::insertchar {win ascii keysym state} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    set id [$data(workspace) find withtag focusitem]

#    puts "Inserting $ascii Key $keysym to $id"

    set insertpos [$data(workspace) index $id insert]
    switch -- $keysym {
 	Left {
            insertchar_repos [expr $insertpos - 1]
	}
	Right {
            insertchar_repos [expr $insertpos + 1]
	}
	Home {
            insertchar_repos 0
	}
	End {
            insertchar_repos [$data(workspace) index $id end]
	}
        default {

            switch -- $keysym {
                Return {
                    commitedit $win
                }
                BackSpace {
                    if [info exists data(insertchar,selfrom)] {
                        insertchar_selection delete
                    } else {
                        $data(workspace) dchars $id [expr $insertpos - 1]
                    }
                }
                Delete {
                    if [info exists data(insertchar,selfrom)] {
                        insertchar_selection delete
                    } else {
                        $data(workspace) dchars $id $insertpos
                    }
                }
                Escape {
                    commitedit $win 0
                }
                Tab {
                }
                default {
                    if {$id != ""} {
                        if {$ascii == ""} {
                            # log::log "No binding for $keysym\n"
                        } else {
                            insertchar_selection delete
                            $data(workspace) insert $id insert $ascii
                        }
                    }
                }
            }
        }
    
    }
}

proc ::visual::commitedit {win {write 1}} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    insertchar_selectclear

    set id [$data(workspace) find withtag focusitem]
    if {$id != ""} {
	set tags [$data(workspace) gettags $id]
	
	# Remove tag "focusitem"
	set focind [lsearch -exact $tags focusitem]
	$data(workspace) itemconfigure $id -tags [lreplace $tags $focind $focind]

	# Search id
	set idindex [lsearch -glob $tags "node#*"]
	set node [string range [lindex $tags $idindex] 5 end]

	# Search attrname
	set attrind [lsearch -glob $tags "editattr,*"]
	set attr [string range [lindex $tags $attrind] 9 end]

	if {$node != "" && $attr != ""} {
	    if {$write} {
		::vl::setAttrs $node [list $attr [$data(workspace) itemcget $id -text]]
	    } else {
		::vl::setAttrs $node ""
	    }
	} 
    }

    # Remove icursor
    $data(workspace) focus ""

}
    
# Context-Menu

proc ::visual::rightclick {win x y} {
    variable visInfo
    set var $visInfo($win-data)
    upvar #0 ::visual::$var data

    if {[info exists data(objdrag)]} {
	# Dragging, do not click.
	return
    }

    if {[info exists data(resizeid)]} {
	# Resizing, do not click.
	return
    }

    if {[$data(workspace) focus] != ""} {
	commitedit $win 1
    }

    set x [$data(workspace) canvasx $x]
    set y [$data(workspace) canvasy $y]
    set halo 2

    # Find out about current context
    set ids [$data(workspace) find overlapping \
            [expr $x - $halo] [expr $y - $halo] [expr $x + $halo] [expr $y + $halo]]

    set iddepth ""
    foreach id $ids {
	set tags [$data(workspace) gettags $id]
	if {[lsearch -exact $tags "noselect"] >= 0} {
	    continue
	}

	set depthindex [lsearch -glob $tags "depth_*"]
	set idindex [lsearch -glob $tags "node#*"]
	if {$idindex < 0} {
	    continue
	}
	lappend iddepth [list $id [string range [lindex $tags $idindex] 5 end] \
		[string range [lindex $tags $depthindex] 6 end]]
    }

    if {$iddepth != ""} {
	set id [lindex [lsort -index 2 -integer $iddepth] end]
	set treeid [lindex $id 1]
	::visual::select $win $treeid
	highlightinspoint ::visual::$var ""
    } else {
	::visual::select $win
    }

    if {$data(selectedid) != ""} {
	.visualcontextmenu delete 0 last
	foreach subwin [winfo children .visualcontextmenu] {
	    ::destroy $subwin
	}

	if {$data(selected_editid) != ""} {
	    foreach editor [::view::get_editors $data(selected_editid)] {
		.visualcontextmenu add command -label [lindex $editor 1] -command "[lindex $editor 0] $data(selected_editid)"
	    }
	}
	
	#by cs: Zeige auch zusaetzliche Editierkontexte
	if {[info exists data(additionalEditContext,$data(selectedid))]} {
#	    puts "addEditContext"
	    foreach editContext $data(additionalEditContext,$data(selectedid)) {

	      foreach editor [::view::get_editors $editContext] {
		.visualcontextmenu add command -label [lindex $editor 1] -command "[lindex $editor 0] $editContext"
	      }
	    }
	}


	if {[info exists data(contextmenu,attr,$data(selectedid))]} {
#	    puts "ctxdata"
	    foreach {label attr value} $data(contextmenu,attr,$data(selectedid)) {
		.visualcontextmenu add command -label $label -command "::vl::setAttrs $data(selectedid) \"$attr $value\""
	    }
	}

	if {[info exists data(contextmenu,cmd,$data(selectedid))]} {
#	    puts "ctxdata"
	    foreach {label cmd} $data(contextmenu,cmd,$data(selectedid)) {
		.visualcontextmenu add command -label $label -command "$cmd $data(selectedid)"
	    }
	}


	if {$data(selected_removeid) != "" && [::vl::isRemovable $data(selected_removeid)]} {
#	    puts "Remove?"
	    .visualcontextmenu add command -label "Delete" -command "::visual::cutselection $win 0"
	}



	if {[.visualcontextmenu index last] != "none"} {
#	    puts "Show"
	    set X [winfo pointerx $win]
	    set Y [winfo pointery $win]
	    tk_popup .visualcontextmenu $X $Y
	}
    }
}



proc ::visual::stdsetposattr {newid cursor bboxprop posattrlist} {
    # bboxprop: gueltiger Einfuegebereich

    # berechne inspoint_xy = Koordinaten-Ursprung fuer Einfuegeoperation
    set inspoint_xy "0 0"
    if {$bboxprop != ""} {
	set inspoint_xy [lrange $bboxprop 0 1]
    }
    if {[llength $posattrlist] > 1} {
        set inspoint_xy [lrange $posattrlist 1 end]
    } else {
    }
    set inspoint_x [lindex $inspoint_xy 0]
    set inspoint_y [lindex $inspoint_xy 1]


    set posattrname [lindex $posattrlist 0]

    set x [expr int([lindex $cursor 0] - $inspoint_x)]
    set y [expr int([lindex $cursor 1] - $inspoint_y)]
#    puts "POS: $cursor     $bboxprop     $x $y     $posattrname"

    if {[catch {::treeutil::setSubAttr $newid $posattrname "$x $y"}]} {
        ::log::log $::errorInfo
    }
}



proc ::visual::linesetposattr {newid cursor bboxprop posattrlist} {
  set posattrname [lindex $posattrlist 0]
  set linestart [lindex $posattrlist 1]
  set lineend [lindex $posattrlist 2]
  set sizex [expr [lindex $cursor 2] - [lindex $cursor 0]]
  set sizey [expr [lindex $cursor 3] - [lindex $cursor 1]]

  set res [abscoord2linecoord $linestart $lineend "$sizex $sizey" [lrange $cursor 0 1]]

  if {[catch {::treeutil::setSubAttr $newid $posattrname $res}]} {
       ::log::log $::errorInfo
  }
}



proc ::visual::inline_setPosAttr {} {
  #by cs:
  #Das urspruengliche Codefragment war an drei verschiedenen Stellen enthalten!
  #Dies ist wartungstechnisch sehr problematisch.

  uplevel 1 {

    #cs:ISP
    #Setzen des Positionsattributs: Property inssetpos ist veraltet. 
    #Wird durch posAttrProc ueberschrieben.

        if { $data(tooldrag,inscursor_position) == ""} {
          set data(tooldrag,inscursor_position) [$data(workspace) bbox $data(tooldrag,inspoint)]
#          puts "bbox: $data(tooldrag,inscursor_position)"
        }
        if {[llength $data(tooldrag,inscursor_position)] != 4} {
          error "Schrott zurueckgeliefert!"
        }


    set posattrlist [tags_getproperty $tags "posAttrProc"]

    if {$posattrlist == ""} {
        set posattrlist [tags_getproperty $tags "inssetpos"]
        if {$posattrlist != ""} {
          set posattrlist [concat "stdsetposattr" $posattrlist]
        }
    }

    if {$posattrlist != ""} {
        set posattrcmd   [lindex $posattrlist 0]
        set posattrparam [lrange $posattrlist 1 end]
	set xy [lrange [$data(workspace) bbox $data(tooldrag,inspoint)] 0 1]
        set bboxprop [tags_getproperty $tags "bbox"]

        $posattrcmd $newid $data(tooldrag,inscursor_position) $bboxprop $posattrparam
        #$posattrcmd $newid [lindex $xy 0] [lindex $xy 1] $orig_x $orig_y $bboxprop $posattrparam
    }
  }
}



