
namespace eval ::abstree {
}

proc ::abstree::wincmd_closeAfterConfirmation {win} {
    variable data
    ::file::closeAfterConfirmation $data($win-fileid)
    return 0
}    

proc ::abstree::wincmd_save {win} {
    variable data
    ::file::save $data($win-fileid)
}

proc ::abstree::wincmd_saveAs {win} {
    variable data
    ::file::saveAs $data($win-fileid)
}

proc ::abstree::wincmd_showErrWindow {win} {
    variable data
    ::file::openErrorList $data($win-fileid)
}

proc ::abstree::wincmd_showAbstree {win} {
    # Do nothing. This is already the abstree!
}

proc ::abstree::wincmd_process {win processor} {
    variable data
    ::file::process $data($win-fileid) $processor
}


proc ::abstree::updateTitle {win} {
    variable data
    set fileid $data($win-fileid)
    set title "AST ([file::modifiedMarker $fileid][::file::getPath $fileid])"
    ::MDIFrame::SetTitle [::MDIFrame::GetClient $win] $title
}


proc ::abstree::open {fileid} {
    variable data
    set root [::file::getRoot $fileid]
    #set title "Abstract Grammar Tree - [::file::getFilename $fileid]"
    set title "temp title"
    set window [::MDIFrame::New_Window $title ""]
    set contents [::hierlist::create $window.win]
    pack $contents -expand yes -fill both
    set procs(nodename)         ::abstree::nodename
    set procs(nodechildren)     ::abstree::nodechildren
    set procs(nodemenu)         ::abstree::nodemenu
    set procs(nodeicon)         ""
    set procs(nodeselect)       ""
    set procs(nodeedit)         ::abstree::nodeedit
    set procs(nodehaschildren)  ""
    ::hierlist::data $contents $root procs tree
    ::hierlist::display $contents
    ::event::add_listener tree_changed "::abstree::changed"

    set data($window-fileid) $fileid
    bind $window <Destroy> "if \{\"%W\" == \"$window\"\} \{::abstree::destroy $window\}"


    ::oop::setClass $window abstree
    ::oop::setAlias $window [::MDIFrame::GetClient $window]
    ::file::addWindow $fileid $window
    # ::MDIFrame::setPreWindowDeleteAction $window "::file::closeAfterConfirmation $fileid"

    updateTitle $window
    return [::MDIFrame::GetClient $window]
}

proc ::abstree::closeWindow {win} {
    ::MDIFrame::CloseClient [::MDIFrame::GetClient $win] 1
}

proc ::abstree::destroy {win} {
    variable data
    # ::file::close $data($win-fileid)
    file::forgetWindow $data($win-fileid) $win
}


proc ::abstree::nodename {id} {
    set ruletype [::vl::getCode $id]
    return [::vl::symbname [::vl::lhs $ruletype]]
}

proc ::abstree::nodechildren {id} {
    return [::vl::getDesc $id]
}

proc ::abstree::nodemenu {id menu} {
    set editors [lreplace [::view::get_editors $id] 0 0]

    foreach editor $editors {
	set edname [lindex $editor 0]
	set edtitle [lindex $editor 1]

	$menu add command \
		-label "Open $edtitle" \
		-command "$edname $id"
    }

    set rule [::vl::getCode $id]
    set nodetype [::vl::ruletype $rule]
    if {$nodetype == 1} {
	# Listed Node

	set elemsym [::vl::listelem [::vl::lhs $rule]]
	set insertable [::vl::possible_rules $elemsym]
	foreach insrule $insertable {
	    $menu add command \
		    -label "Insert Son [::vl::rulename $insrule]" \
		    -command "::abstree::insnode $id 0 $insrule"
	}
    }
    
    set father [::vl::getFather $id]
    if {$father != ""} {
	set myindex [lsearch -exact [::vl::getDesc $father] $id]
	set fatherrule [::vl::getCode $father]
	set fathertype [::vl::ruletype $fatherrule]
	if {$myindex >= 0 && $fathertype == 1} {
	    set elemsym [::vl::listelem [::vl::lhs $fatherrule]]
	    set insertable [::vl::possible_rules $elemsym]
	    foreach insrule $insertable {
		$menu add command \
			-label "Insert Brother [::vl::rulename $insrule]" \
			-command "::abstree::insnode $father $myindex $insrule"
	    }
	}

	if {[::vl::isRemovable $id] == 1} {
	    $menu add command \
		    -label "Delete" \
		    -command "::abstree::delnode $id"
	}
    } else {
	$menu add command \
		-label "Count" \
		-command "::abstree::count $id"
    }
}

proc ::abstree::count {id} {
    set count [docount $id]
    
    ::CDlg::Information "Tree has $count nodes" "Node count"
}

proc ::abstree::docount {id} {
    set nodes [::vl::getDesc $id]
    set count [llength $nodes]
    foreach n $nodes {
	set count [expr $count + [::abstree::docount $n]]
    }
    return $count
}

proc ::abstree::delnode {id} {
    global errorInfo

    set father [::vl::getFather $id]
    ::vl::extract $id
    ::vl::delete $id
}
	
    
proc ::abstree::insnode {father index type} {
    global errorInfo

    set insnode [::vl::create $type]
    if {[catch {::vl::insert $father $index $insnode} result]} {
	::vl::delete $insnode
	error $result $errorInfo
    }
}

proc ::abstree::on_treeModified {win node} {
    # dummy-Methode; wird von file aufgerufen
    updateTitle $win
}

proc ::abstree::changed {node} {
    set father [::vl::getFather $node]
    if {$father != ""} {
	::hierlist::update $father tree
    } else {
	::hierlist::update $node tree
    }
}

proc ::abstree::nodeedit {which id} {
    set editor [lindex [::view::get_editors $id] 0]

    set edname [lindex $editor 0]
    set edtitle [lindex $editor 1]

    if {$which == "inquire"} {
	return [expr {"$edname" != ""}]
    } elseif {$editor != ""} {
	$edname $id
    }
}

